<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppAdminController
{
    public static function createPrestAppAccount($params)
    {
        if (!isset($params) || !$params) {
            return 'ERROR';
        }

        if (!$params['employee'] && !$params['employeeData']) {
            return 'ERROR';
        }

        $email = null;
        $firstname = null;
        $lastname = null;

        if ($params['employee']) {
            $employee = new Employee($params['employee']);
            $email = $employee->email;
            $firstname = $employee->firstname;
            $lastname = $employee->lastname;
        } elseif ($params['employeeData']) {
            $email = $params['employeeData']['email'];
            $firstname = $params['employeeData']['firstname'];
            $lastname = $params['employeeData']['lastname'];
        }

        $country = "fr";

        $country_conf_value = Configuration::get('PS_SHOP_COUNTRY_ID');
        if ($country_conf_value) {
            $country_object = new Country($country_conf_value);
            if ($country_object && $country_object->iso_code) {
                $country = Tools::strtolower($country_object->iso_code);
            }
        }

        $language_conf_value = Configuration::get('PS_LANG_DEFAULT');
        if ($language_conf_value) {
            $language_object = new Language($language_conf_value);
            if ($language_object && $language_object->iso_code) {
                $language = Tools::strtolower($language_object->iso_code);
            }
        }

        $timezone_conf_value = Configuration::get('PS_TIMEZONE');

        $curl = curl_init();

        $api_url = "https://webapi.prestapp.eu/autolog";

        $params = array(
            "email" => $email,
            "firstName" => $firstname,
            "lastName" => $lastname,
            "address" => Configuration::get('PS_SHOP_ADDR1'),
            "postalCode" => Configuration::get('PS_SHOP_CODE'),
            "city" => Configuration::get('PS_SHOP_CITY'),
            "language" => $language,
            "country" => $country,
            "timezone" => $timezone_conf_value,
            "phone1" => Configuration::get('PS_SHOP_PHONE')
        );

        $params = json_encode($params);

        curl_setopt_array($curl, array(
            CURLOPT_URL => $api_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => $params,
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        $is_ok = false;

        if ($response) {
            $data = json_decode($response);
            if ($data->code == 'CREATED') {
                $is_ok = true;
            } elseif ($data->code == 'EXISTS') {
                return 'EXISTS';
            } elseif ($data->code == 'MISSING') {
                return array(
                    'code' => 'MISSING',
                    'missing'=> $data->missing
                );
            } else {
                return 'ERROR';
            }
        } else {
            return 'ERROR';
        }

        if ($is_ok) {
            $response_data = $data->data;
            $autolog_key = $response_data->autologKey;
            return 'CREATED';
        }
    }
}
