<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppCartController
{
    /**
     * getCartSummary - Return the summary of a cart from its id.
     *
     * @param int     $id_cart
     * @param Context $context
     *
     * @return array
     */
    public static function getCartSummary($id_cart, $delete, $context)
    {
        //get cart from id
        $cart = new Cart($id_cart);

        if (null === $cart->id) {
            return false;
        } elseif ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            if ($cart->id_lang != (int)$context->language->id) {
                $cart->id_lang = (int)$context->language->id;
                $cart->save();
            }

            $time_start = microtime(true);

            if ($delete) {
                $temp_summary = $cart->getSummaryDetails($context->language->id);
                foreach ($temp_summary['products'] as $key => &$product) {
                    if (($product['quantity_available'] <= 0 && !Product::isAvailableWhenOutOfStock($product['out_of_stock'])) || !(int) $product['active']) {
                        $cart->deleteProduct($product['id_product'], $product['id_product_attribute'], $product['id_customization']);
                    }
                }
            }

            $summary = $cart->getSummaryDetails($context->language->id);

            if (version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
                $summary['discounts'] = PrestAppCartController::getCartDiscounts($cart, $context);
            }

            $summary['id_guest'] = $cart->id_guest;

            foreach ($summary['products'] as &$product) {
                $id_image = explode('-', $product['id_image']);

                $image = new Image($id_image[1]);

                $product['is_available_out_of_stock'] = Product::isAvailableWhenOutOfStock($product['out_of_stock']) ? true : false;

                $urls = PrestAppImageController::getImageUrl((int) $image->id, $context);
                $product['image'] = $urls;

                if ($product['price_without_reduction'] == $product['price_with_reduction']) {
                    $product['reduction_type'] = null;
                }

                if (!empty($product['unity']) && $product['unit_price_ratio'] > 0.000000) {
                    $product['unit_price'] = Tools::ps_round(($product['unit_price_ratio'] != 0 ? $product['price_wt'] / $product['unit_price_ratio'] : 0), 2);
                    $product['unit_price_unity'] = $product['unity'];
                }

                $specific_price_output = null;

                $product['price_without_reduction_without_tax'] = Product::getPriceStatic(
                    $product['id_product'],
                    false,
                    $product['id_product_attribute'],
                    6,
                    null,
                    false,
                    false,
                    1,
                    false,
                    null,
                    null,
                    null,
                    $specific_price_output,
                    true,
                    true,
                    $context
                );
            }

            foreach ($summary['gift_products'] as &$gift_product) {
                $id_image = explode('-', $gift_product['id_image']);

                $image = new Image($id_image[1]);

                $urls = PrestAppImageController::getImageUrl((int) $image->id, $context);
                $gift_product['image'] = $urls;
            }

            $customized_datas = Product::getAllCustomizedDatas($id_cart);

            if ($customized_datas) {
                foreach ($summary['products'] as &$product) {
                    if (isset($customized_datas[$product['id_product']]) && $customized_datas[$product['id_product']]) {
                        if (isset($customized_datas[$product['id_product']][$product['id_product_attribute']]) && $customized_datas[$product['id_product']][$product['id_product_attribute']]) {
                            if (isset($customized_datas[$product['id_product']][$product['id_product_attribute']][$product['id_address_delivery']]) && $customized_datas[$product['id_product']][$product['id_product_attribute']][$product['id_address_delivery']]) {
                                if (isset($customized_datas[$product['id_product']][$product['id_product_attribute']][$product['id_address_delivery']][$product['id_customization']]) && $customized_datas[$product['id_product']][$product['id_product_attribute']][$product['id_address_delivery']][$product['id_customization']]) {
                                    $product['customized_data'] = $customized_datas[$product['id_product']][$product['id_product_attribute']][$product['id_address_delivery']][$product['id_customization']];
                                }
                            }
                        }
                    }
                }
            }

            $currency = Currency::getCurrency((int)$context->cart->id_currency);
            $minimal_purchase = Tools::convertPrice((float)Configuration::get('PS_PURCHASE_MINIMUM'), $currency);

            $summary['minimal_purchase'] = $minimal_purchase;

            $priceDisplay = Product::getTaxCalculationMethod((int) $cart->id_customer);
            $summary['taxExcl'] = $priceDisplay ? 1 : 0;

            $summary['guest_allowed_checkout'] = (boolean)Configuration::get('PS_GUEST_CHECKOUT_ENABLED');

            // var_dump(json_encode($customized_datas));exit();

            // $summary['customized_datas'] =

            return $summary;
        } else {
            return false;
        }
    }

    public static function getCartDiscounts($cart, $context)
    {
        $taxConfiguration = new TaxConfiguration();
        $cartVouchers = $cart->getCartRules();
        $vouchers = array();
        $cartHasTax = null === $cart->id ? false : $cart::getTaxesAverageUsed($cart);

        foreach ($cartVouchers as $cartVoucher) {
            $tempVoucher = array();

            $tempVoucher['id_cart_rule'] = $cartVoucher['id_cart_rule'];
            $tempVoucher['description'] = $cartVoucher['name'];

            if ($cartHasTax && $cartVoucher['reduction_tax'] == '0') {
                $cartVoucher['reduction_amount'] = $cartVoucher['reduction_amount'] * (1 + $cartHasTax / 100);
            }

            $tempVoucher['reduction_amount'] = $cartVoucher['reduction_amount'];

            if (array_key_exists('gift_product', $cartVoucher) && $cartVoucher['gift_product']) {
                $cartVoucher['reduction_amount'] = $cartVoucher['value_real'];
            }

            if (isset($cartVoucher['reduction_percent']) && $cartVoucher['reduction_amount'] == '0.00') {
                $value = $taxConfiguration->includeTaxes() ? $cartVoucher['value_real'] : $cartVoucher['value_tax_exc'];
                $cartVoucher['reduction_formatted'] = $value;
            } elseif (isset($cartVoucher['reduction_amount']) && $cartVoucher['reduction_amount'] > 0) {
                $value = $taxConfiguration->includeTaxes() ? $cartVoucher['reduction_amount'] : $cartVoucher['value_tax_exc'];
                $cartVoucher['reduction_formatted'] = $value;
            }

            $tempVoucher['value_real'] = $cartVoucher['reduction_formatted'] ? $cartVoucher['reduction_formatted'] : 0;

            $vouchers[] = $tempVoucher;
        }

        return $vouchers;
    }

    /**
     * deleteProductInCart - delete all quantity of a specific product in cart.
     *
     * @param int     $id_cart
     * @param int     $id_product
     * @param int     $id_combination
     * @param int     $id_customization
     * @param Context $context
     *
     * @return bool
     */
    public static function deleteProductInCart($id_cart, $id_product, $context, $id_combination = null, $id_customization = false)
    {
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $cart->deleteProduct($id_product, $id_combination, $id_customization);

            return $cart;
        } else {
            return false;
        }
    }

    /**
     * FindOrCreateCart - If user is authenticated, we try to get the last cart non orderer, else we create another one.
     *
     * @param Context $context
     *
     * @return Cart or false
     */
    public static function findOrCreateCart($context)
    {
        if ($context->customer && $context->customer->id) {
            $id_cart = Cart::lastNoneOrderedCart($context->customer->id);
        } else {
            $id_cart = null;
        }

        $cart = new Cart($id_cart);

        // check if cart is older than the cookie session
        $a = new DateTime($cart->date_add);
        $b = new DateTime();

        $cart_date_add = $a->getTimestamp();
        $current_date = $b->getTimestamp();

        //get the max age for a cart. get the value in the Prestashop BO. 48 hours by default.
        $max_cart_age = 60 * 60 * (Configuration::get('PS_COOKIE_LIFETIME_FO') ? Configuration::get('PS_COOKIE_LIFETIME_FO') : 48);
        $diff = $current_date - $cart_date_add;

        if (null === $cart->id || $diff > $max_cart_age) {
            $cart = new Cart();
            $cart->id_currency = $context->currency->id;
            $cart->id_lang = $context->language->id;

            if ((boolean)Configuration::get('PS_GUEST_CHECKOUT_ENABLED')) {
                $guest = new Guest((isset($context->customer->id) && $context->customer->id) ? Guest::getFromCustomer((int) ($context->customer->id)) : null);
                $guest->userAgent();
                $guest->save();
    
                $cart->id_guest = (int) ($guest->id);
            } else {
                $cart->id_guest = 999999999;
            }

            if ($context->customer) {
                $cart->id_customer = $context->customer->id;
            }

            $cart->save();
        }

        return $cart;
    }

    /**
     * updateProductQuantity - allow to update all quantity in cart (remove and add specific product quantity).
     *
     * @param int     $id_cart
     * @param int     $id_product
     * @param int     $quantity
     * @param int     $id_product_attribute
     * @param bool    $id_customization
     * @param string  $operator             UP|DOWN
     * @param Context $context
     *
     * @return bool
     */
    public static function updateProductQuantity($id_cart, $id_product, $context, $quantity = 1, $id_product_attribute = null, $id_customization = false, $operator = 'up')
    {
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            return $cart->updateQty($quantity, $id_product, $id_product_attribute, $id_customization, $operator);
        } else {
            return false;
        }
    }

    /**
     * claimOwnership - Allow to take the ownership of a cart if its not belongs to someone else.
     *
     * @param int     $id_cart
     * @param Context $context
     *
     * @return bool
     */
    public static function claimOwnership($id_cart, $context)
    {
        $cart = new Cart($id_cart);

        if (null !== $context->customer && !(int) $cart->id_customer) {
            $cart->id_customer = (int) $context->customer->id;
            $cart->id_guest = 999999999;
            $cart->save();

            return true;
        } else {
            return false;
        }
    }

    /**
     * addCartRule - add cart rule from the code and check validity before adding it to cart.
     *
     * @param int $id_cart
     * @param int $id_cart_rule
     * @param int $context
     *
     * @return bool
     */
    public static function addCartRule($id_cart, $id_cart_rule, $context)
    {
        $cart_rule = new CartRule($id_cart_rule);
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            if ($cart_rule->checkValidity($context, false, false, true)) {
                return $cart->addCartRule($id_cart_rule);
            } else {
                return false;
            }
        }

        return false;
    }

    /**
     * removeCartRule - remove cart rule from its id in cart.
     *
     * @param int     $id_cart
     * @param int     $id_cart_rule
     * @param Context $context
     *
     * @return bool
     */
    public static function removeCartRule($id_cart, $id_cart_rule, $context)
    {
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            return $cart->removeCartRule($id_cart_rule);
        } else {
            return false;
        }
    }

    /**
     * updateAddressDeliveryId Update the delivery address id of the cart.
     *
     * @param int $id_cart        Current cart id to update
     * @param int $id_address     Current address id to change
     * @param int $id_address_new New address id
     */
    public static function updateAddressDeliveryId($id_cart, $id_address, $id_address_new, $context)
    {
        $cart = new Cart($id_cart);

        $new_address = new Address($id_address_new);
        if (!Validate::isLoadedObject($new_address)) {
            return false;
        } elseif (null !== $context->customer && ((int) $new_address->id_customer !== (int) $context->customer->id)) {
            return false;
        }

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $cart->id_address_delivery = $id_address_new;

            if (!$cart->update()) {
                return false;
            } else {
                if (!$cart->isMultiAddressDelivery()) {
                    $cart->setNoMultishipping();
                }
                return true;
            }
        }

        return false;
    }

    /**
     * updateAddressInvoiceId Update the invoice address id of the cart.
     *
     * @param int $id_cart        Current cart id to update
     * @param int $id_address     Current address id to change
     * @param int $id_address_new New address id
     */
    public static function updateAddressInvoiceId($id_cart, $id_address, $id_address_new, $context)
    {
        $cart = new Cart($id_cart);

        $new_address = new Address($id_address_new);
        if (!Validate::isLoadedObject($new_address)) {
            return false;
        } elseif (null !== $context->customer && ((int) $new_address->id_customer !== (int) $context->customer->id)) {
            return false;
        }

        $cart->id_address_invoice = $id_address_new;

        if (!$cart->update()) {
            return false;
        } else {
            return true;
        }

        return false;
    }

    public static function updateAddressDeliveryInvoiceId($id_cart, $id_address_delivery, $id_address_invoice, $id_address_new_delivery, $id_address_new_invoice, $context)
    {
        $cart = new Cart($id_cart);

        $new_address_delivery = new Address($id_address_new_delivery);
        if (!Validate::isLoadedObject($new_address_delivery)) {
            return false;
        } elseif (null !== $context->customer && ((int) $new_address_delivery->id_customer !== (int) $context->customer->id)) {
            return false;
        }
        $new_address_invoice = new Address($id_address_new_invoice);
        if (!Validate::isLoadedObject($new_address_invoice)) {
            return false;
        } elseif (null !== $context->customer && ((int) $new_address_invoice->id_customer !== (int) $context->customer->id)) {
            return false;
        }

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $cart->id_address_delivery = $id_address_new_delivery;
            $cart->id_address_invoice = $id_address_new_invoice;

            if (!$cart->update()) {
                return false;
            } else {
                if (!$cart->isMultiAddressDelivery()) {
                    $cart->setNoMultishipping();
                }
                return true;
            }
        }

        return false;
    }

    public static function updateAddressDefaultId($id_cart, $context)
    {
        if (!Validate::isLoadedObject($context->customer)) {
            return false;
        }

        $cart = new Cart($id_cart);

        $customer_addresses = $context->customer->getAddresses((int) $context->language->id);

        if (!$customer_addresses || count($customer_addresses) == 0) {
            $address = PrestAppAddressController::createOrUpdateAddress(
                $context->customer->lastname,
                $context->customer->firstname,
                '',
                'Street name',
                '',
                'City name',
                99999,
                '',
                '',
                8,
                '',
                '',
                'default_address_' . $context->customer->id . '_' . time(),
                '',
                '',
                $context,
                null
            );

            $new_address = new Address($address->id);
        } else {
            $new_address = new Address($customer_addresses[0]['id_address']);
        }

        if (!Validate::isLoadedObject($new_address)) {
            return false;
        } elseif (null !== $context->customer && ((int) $new_address->id_customer !== (int) $context->customer->id)) {
            return false;
        }

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $cart->id_address_delivery = $new_address->id;
            $cart->id_address_invoice = $new_address->id;
            if (!$cart->update()) {
                return false;
            } else {
                if (!$cart->isMultiAddressDelivery()) {
                    $cart->setNoMultishipping();
                }
                return true;
            }
        }

        return false;
    }

    /**
     * getDeliveryOptionList Get carriers list available for this cart.
     *
     * @param int $id_cart Current cart id to update
     */
    public static function getDeliveryOptionList($id_cart, $context)
    {
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $carriers = $cart->getDeliveryOptionList();
            $formattedCarriers = array();


            foreach ($carriers[$context->cart->id_address_delivery] as $key => $carrier) {
                $c = (object) array();
                $c->id = (int) array_values($carrier['carrier_list'])[0]['instance']->id;
                $c->referenceId = (int) array_values($carrier['carrier_list'])[0]['instance']->id_reference;
                $c->name = array_values($carrier['carrier_list'])[0]['instance']->name;
                $c->priceWithTax = $carrier['total_price_with_tax'];
                $c->priceWithoutTax = $carrier['total_price_without_tax'];
                $c->isFree = $carrier['is_free'];
                $c->isGeneric = 0 === (int) array_values($carrier['carrier_list'])[0]['instance']->is_module;
                $c->moduleName = array_values($carrier['carrier_list'])[0]['instance']->external_module_name;
                $c->deliveryOptionId = $key;
                $c->deliveryOptionAddress = (int) $context->cart->id_address_delivery;
                $c->shopLogo = array_values($carrier['carrier_list'])[0]['logo'] ? Tools::getHttpHost(true) . __PS_BASE_URI__ . array_values($carrier['carrier_list'])[0]['logo'] : null;
                // delay

                $delay = array_values($carrier['carrier_list'])[0]['instance']->delay;
                if (isset($delay[$context->language->id])) {
                    $delay = $delay[$context->language->id];
                } else {
                    $delay = $delay[(int)Configuration::get('PS_LANG_DEFAULT')];
                }

                $c->delay = $delay;

                array_push($formattedCarriers, $c);
            }

            $formattedCarriers = PrestAppShippingController::completeDeliveryOptionList($formattedCarriers, $context);

            return $formattedCarriers;
        }

        return false;
    }

    public static function getExtraCarriers($id_cart, $carriers, $context)
    {
        $carriers_ids = json_decode($carriers);

        if (count($carriers_ids) > 0) {
            $extra_carriers = array();
            foreach ($carriers_ids as $carrier_id) {
                $extra_carriers[$carrier_id] = array();
                if (PrestAppCustomizationController::checkModuleAvailability('prestadrive')) {
                    $prestadrive = new PrestAppPrestadriveCustomizationController();

                    $carrier = new Carrier($carrier_id);

                    $extra_carriers[$carrier_id]['prestadrive'] = array(
                        'is_enabled' => $prestadrive->isEnabledForCarrier($carrier_id),
                    );

                    if ($extra_carriers[$carrier_id]['prestadrive']['is_enabled']) {
                        $extra_carriers[$carrier_id]['prestadrive']['cart_data'] = $prestadrive->getCartValues($id_cart);
                        $extra_carriers[$carrier_id]['prestadrive']['params_data'] = $prestadrive->getData($context);
                        $extra_carriers[$carrier_id]['prestadrive']['params_data']['aposttimevalue'] = $carrier->external_module_name ? $extra_carriers[$carrier_id]['prestadrive']['params_data']['aposttimevalue'] : $extra_carriers[$carrier_id]['prestadrive']['params_data']['aposttimevalue_store'];
                    }
                }
            }
            return $extra_carriers;
        } else {
            return false;
        }
    }

    /**
     * setDeliveryOption Get carriers list available for this cart.
     *
     * @param int   $id_cart         Current cart id to update
     * @param array $delivery_option Delivery option choose
     */
    public static function setDeliveryOption($id_cart, $delivery_option, $context)
    {
        $cart = new Cart($id_cart);

        if ((null === $context->customer && 0 === (int) $cart->id_customer) || (null !== $context->customer && ((int) $cart->id_customer === (int) $context->customer->id))) {
            $cart->setDeliveryOption($delivery_option);
            if ($cart->save()) {
                return true;
            }
        }

        return false;
    }

    /**
     * getPaymentModulesList Get payment modules list available for this context.
     */
    public static function getPaymentModulesList($context)
    {
        if (isset($context->cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')}) && $context->cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')}) {
            $infos = Address::getCountryAndState((int)$context->cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')});
            $country = new Country((int)$infos['id_country']);
            $context->country = $country;
        }

        if (version_compare(_PS_VERSION_, '1.7', '<') === true) {
            $modules = Hook::getHookModuleExecList('displayPayment');
        } else {
            $modules = Hook::getHookModuleExecList('paymentOptions');
        }

        foreach ($modules as $key => $value) {
            if (isset($value) && isset($value['module']) && $value['module']) {
                $modules[$key]['name'] = $value['module'];
                unset($modules[$key]['module']);
                $module_object = Module::getInstanceByName($modules[$key]['name']);
                $modules[$key]['version'] = $module_object->version;
            }
        }

        return $modules;
    }
}
