<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppContactController
{
    /**
     * getActiveLanguages - Return the languages list.
     *
     * @return array Language
     */
    public static function sendContactForm($context, $message, $email, $id_contact, $id_order, $id_product)
    {
        $errors = array();

        if (!$email || !Validate::isEmail($email)) {
            $errors['email'] = 'Invalid email address.';
        } elseif (!$message) {
            $errors['message'] = 'The message cannot be blank.';
        } elseif (!Validate::isCleanHtml($message)) {
            $errors['messageFormat'] = 'Invalid message';
        } elseif (!$id_contact || !(Validate::isLoadedObject($contact = new Contact($id_contact, $context->language->id)))) {
            $errors['id_contact'] = 'Please select a subject from the list provided.';
        } else {
            $customer = $context->customer;
            if (!$customer->id) {
                $customer = new Customer();
                $customer->getByEmail($email);
            }

            $id_order = getOrder($id_order);

            $file_attachment = array();

            if (!((
                ($id_customer_thread = (int) Tools::getValue('id_customer_thread'))
                && (int) Db::getInstance()->getValue('
                    SELECT cm.id_customer_thread FROM ' . _DB_PREFIX_ . 'customer_thread cm
                    WHERE cm.id_customer_thread = ' . (int) $id_customer_thread . ' AND cm.id_shop = ' . (int) $context->shop->id . ' AND token = \'' . pSQL(Tools::getValue('token')) . '\'')
            ) || (
                $id_customer_thread = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($email, $id_order)
            ))) {
                $fields = Db::getInstance()->executeS('
					SELECT cm.id_customer_thread, cm.id_contact, cm.id_customer, cm.id_order, cm.id_product, cm.email
					FROM ' . _DB_PREFIX_ . 'customer_thread cm
					WHERE email = \'' . pSQL($email) . '\' AND cm.id_shop = ' . (int) $context->shop->id . ' AND (' .
                    ($customer->id ? 'id_customer = ' . (int) $customer->id . ' OR ' : '') . '
						id_order = ' . (int) $id_order . ')');
                $score = 0;
                foreach ($fields as $key => $row) {
                    $tmp = 0;
                    if ((int) $row['id_customer'] && $row['id_customer'] != $customer->id && $row['email'] != $email) {
                        continue;
                    }
                    if ($row['id_order'] != 0 && $id_order != $row['id_order']) {
                        continue;
                    }
                    if ($row['email'] == $email) {
                        $tmp += 4;
                    }
                    if ($row['id_contact'] == $id_contact) {
                        $tmp++;
                    }
                    if (Tools::getValue('id_product') != 0 && $row['id_product'] == Tools::getValue('id_product')) {
                        $tmp += 2;
                    }
                    if ($tmp >= 5 && $tmp >= $score) {
                        $score = $tmp;
                        $id_customer_thread = $row['id_customer_thread'];
                    }
                }
            }
            $old_message = Db::getInstance()->getValue('
                SELECT cm.message FROM ' . _DB_PREFIX_ . 'customer_message cm
                LEFT JOIN ' . _DB_PREFIX_ . 'customer_thread cc on (cm.id_customer_thread = cc.id_customer_thread)
                WHERE cc.id_customer_thread = ' . (int) $id_customer_thread . ' AND cc.id_shop = ' . (int) $context->shop->id . '
                ORDER BY cm.date_add DESC');

            if ($old_message == $message) {
                $context->smarty->assign('alreadySent', 1);
                $contact->email = '';
                $contact->customer_service = 0;
            }

            if ($contact->customer_service) {
                if ((int) $id_customer_thread) {
                    $ct = new CustomerThread($id_customer_thread);
                    $ct->status = 'open';
                    $ct->id_lang = (int) $context->language->id;
                    $ct->id_contact = (int) $id_contact;
                    $ct->id_order = (int) $id_order;
                    if ($id_product = (int) Tools::getValue('id_product')) {
                        $ct->id_product = $id_product;
                    }
                    $ct->update();
                } else {
                    $ct = new CustomerThread();
                    if (isset($customer->id)) {
                        $ct->id_customer = (int) $customer->id;
                    }
                    $ct->id_shop = (int) $context->shop->id;
                    $ct->id_order = (int) $id_order;
                    if ($id_product = (int) Tools::getValue('id_product')) {
                        $ct->id_product = $id_product;
                    }
                    $ct->id_contact = (int) $id_contact;
                    $ct->id_lang = (int) $context->language->id;
                    $ct->email = $email;
                    $ct->status = 'open';
                    $ct->token = Tools::passwdGen(12);
                    $ct->add();
                }

                if ($ct->id) {
                    $cm = new CustomerMessage();
                    $cm->id_customer_thread = $ct->id;
                    $cm->message = $message;
                    if (isset($file_attachment['rename']) && !empty($file_attachment['rename']) && rename($file_attachment['tmp_name'], _PS_UPLOAD_DIR_ . basename($file_attachment['rename']))) {
                        $cm->file_name = $file_attachment['rename'];
                        @chmod(_PS_UPLOAD_DIR_ . basename($file_attachment['rename']), 0664);
                    }
                    $cm->ip_address = (int) ip2long(Tools::getRemoteAddr());
                    $cm->user_agent = $_SERVER['HTTP_USER_AGENT'];
                    if (!$cm->add()) {
                        $errors['unknown'] = 'An error occurred while sending the message.';
                    }
                } else {
                    $errors['unknown'] = 'An error occurred while sending the message.';
                }
            }

            $contactform = Module::getInstanceByName('contactform');

            if (class_exists('Contactform')) {
                try {
                    $sendConfirmationEmail = Configuration::get($contactform::SEND_CONFIRMATION_EMAIL);
                    $sendNotificationEmail = Configuration::get($contactform::SEND_NOTIFICATION_EMAIL);
                    $confirmationMessage = $contactform::MESSAGE_PLACEHOLDER_FOR_OLDER_VERSION;
                } catch (Error $e) {
                    $sendConfirmationEmail = true;
                    $sendNotificationEmail = true;
                    $confirmationMessage = Tools::nl2br(Tools::stripslashes($message));
                }
            } else {
                $sendNotificationEmail = true;
            }

            if (!count($errors) && ($sendConfirmationEmail || $sendNotificationEmail)
            ) {
                $var_list = [
                    '{order_name}' => '-',
                    '{attached_file}' => '-',
                    '{message}' => Tools::nl2br(Tools::stripslashes($message)),
                    '{email}' =>  $email,
                    '{product_name}' => '',
                ];

                if (isset($file_attachment['name'])) {
                    $var_list['{attached_file}'] = $file_attachment['name'];
                }
                // $id_product = (int)Tools::getValue('id_product');
                if (isset($ct) && Validate::isLoadedObject($ct) && $ct->id_order) {
                    $order = new Order((int)$ct->id_order);
                    $var_list['{order_name}'] = $order->getUniqReference();
                    $var_list['{id_order}'] = (int)$order->id;
                }

                if ($id_product) {
                    $product = new Product((int)$id_product);

                    if (Validate::isLoadedObject($product) &&
                        isset($product->name[Context::getContext()->language->id])
                    ) {
                        $var_list['{product_name}'] = $product->name[Context::getContext()->language->id];
                    }
                }

                if ($sendNotificationEmail) {
                    if (empty($contact->email) || !Mail::Send(
                        $context->language->id,
                        'contact',
                        method_exists(Context::getContext(), 'getTranslator') ? PrestAppContactController::trans('Message from contact form', [], 'Emails.Subject').' [no_sync]' : Mail::l('Message from contact form').' [no_sync]',
                        $var_list,
                        $contact->email,
                        $contact->name,
                        null,
                        null,
                        $file_attachment,
                        null,
                        _PS_MAIL_DIR_,
                        false,
                        null,
                        null,
                        $email
                    )) {
                    }
                }

                if ($sendConfirmationEmail && $confirmationMessage) {
                    $var_list['{message}'] = $confirmationMessage;

                    if (!Mail::Send(
                        $context->language->id,
                        'contact_form',
                        ((isset($ct) && Validate::isLoadedObject($ct)) ? PrestAppContactController::trans(
                            'Your message has been correctly sent #ct%thread_id% #tc%thread_token%',
                            [
                                '%thread_id%' => $ct->id,
                                '%thread_token%' => $ct->token
                            ],
                            'Emails.Subject'
                        ) : PrestAppContactController::trans('Your message has been correctly sent', [], 'Emails.Subject')),
                        $var_list,
                        $email,
                        null,
                        null,
                        null,
                        $file_attachment,
                        null,
                        _PS_MAIL_DIR_,
                        false,
                        null,
                        null,
                        $contact->email
                    )) {
                    }
                }
            }
        }

        if (count($errors) > 0) {
            return $errors;
        } else {
            return true;
        }
    }

    public static function sendOrderContactForm($context, $message, $id_order, $id_product)
    {
        $order = new Order($id_order);
        if (Validate::isLoadedObject($order) && $order->id_customer == $context->customer->id) {
            $id_customer_thread = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($context->customer->email, $order->id);
            $id_product = (int) $id_product;
            $cm = new CustomerMessage();

            if (!$id_customer_thread) {
                $ct = new CustomerThread();
                $ct->id_contact = 0;
                $ct->id_customer = (int) $order->id_customer;
                $ct->id_shop = (int) $context->shop->id;
                if ($id_product && $order->orderContainProduct($id_product)) {
                    $ct->id_product = $id_product;
                }
                $ct->id_order = (int) $order->id;
                $ct->id_lang = (int) $context->language->id;
                $ct->email = $context->customer->email;
                $ct->status = 'open';
                $ct->token = Tools::passwdGen(12);
                $ct->add();
            } else {
                $ct = new CustomerThread((int) $id_customer_thread);
                $ct->status = 'open';
                $ct->update();
            }

            $cm->id_customer_thread = $ct->id;
            $cm->message = $message;
            $client_ip_address = Tools::getRemoteAddr();
            $cm->ip_address = (int) ip2long($client_ip_address);
            $cm->add();

            if (!Configuration::get('PS_MAIL_EMAIL_MESSAGE')) {
                $to = (string) Configuration::get('PS_SHOP_EMAIL');
            } else {
                $to = new Contact((int) Configuration::get('PS_MAIL_EMAIL_MESSAGE'));
                $to = (string) $to->email;
            }
            $toName = (string) Configuration::get('PS_SHOP_NAME');
            $customer = $context->customer;

            $product = new Product($id_product);
            $product_name = '';
            if (Validate::isLoadedObject($product) && isset($product->name[(int) $context->language->id])) {
                $product_name = $product->name[(int) $context->language->id];
            }

            if (Validate::isLoadedObject($customer)) {
                Mail::Send(
                    $context->language->id,
                    'order_customer_comment',
                    PrestAppContactController::trans(
                        'Message from a customer',
                        [],
                        'Emails.Subject'
                    ),
                    [
                        '{lastname}' => $customer->lastname,
                        '{firstname}' => $customer->firstname,
                        '{email}' => $customer->email,
                        '{id_order}' => (int) $order->id,
                        '{order_name}' => $order->getUniqReference(),
                        '{message}' => Tools::nl2br(Tools::htmlentitiesUTF8($message)),
                        '{product_name}' => $product_name,
                    ],
                    $to,
                    $toName,
                    (string) Configuration::get('PS_SHOP_EMAIL'),
                    $customer->firstname . ' ' . $customer->lastname,
                    null,
                    null,
                    _PS_MAIL_DIR_,
                    false,
                    null,
                    null,
                    $customer->email
                );
            }

            return true;
        } else {
            return false;
        }
    }

    public static function getContactTypes($context)
    {
        return Contact::getContacts($context->language->id);
    }

    public static function getTranslator()
    {
        return Context::getContext()->getTranslator();
    }

    protected static function trans($id, array $parameters = array(), $domain = null, $locale = null)
    {

        if (_PS_VERSION_ < '1.7') {
            return Mail::l($id);
        } else {
            $parameters['legacy'] = 'htmlspecialchars';
            return PrestAppContactController::getTranslator()->trans($id, $parameters, $domain, $locale);
        }
    }
}

function getOrder($id)
{
    $id_order = false;
    if (!is_numeric($reference = $id)) {
        $reference = ltrim($reference, '#');
        $orders = Order::getByReference($reference);
        if ($orders) {
            foreach ($orders as $order) {
                $id_order = (int) $order->id;
                break;
            }
        }
    } elseif (Order::getCartIdStatic((int) $id)) {
        $id_order = (int) $id;
    }
    return (int) $id_order;
}
