<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppOrderController
{
    /**
     * getOrderByOrderId - Return the order object from its id.
     *
     * @param Context $context
     *
     * @return array
     */
    public static function getOrderByOrderId($id_order, $context)
    {
        $order = new Order($id_order);

        if (!Validate::isLoadedObject($order)) {
            return 'ORDER_NOT_FOUND';
        } elseif ($order->id_customer && null == $context->customer) {
            return 'NOT_AUTHORIZED';
        } elseif ($order->id_customer && $order->id_customer != $context->customer->id) {
            return 'NOT_AUTHORIZED';
        } else {
            $ProductDetailObject = new OrderDetail();
            $CarrierDetailObject = new Carrier($order->id_carrier, (int) $context->language->id);
            $AddressDeliveryDetailObject = new Address($order->id_address_delivery);
            $AddressInvoiceDetailObject = new Address($order->id_address_invoice);
            $CurrencyDetailObject = new Currency($order->id_currency);
            $product_detail = $ProductDetailObject->getList($id_order);
            $order->products = $product_detail;
            $order->carrier = $CarrierDetailObject;
            $order->currency = $CurrencyDetailObject;
            $order->address_delivery = $AddressDeliveryDetailObject;
            $order->address_invoice = $AddressInvoiceDetailObject;
            $order->cart_rules = $order->getCartRules();
            $order->state = $order->getCurrentStateFull((int) $context->language->id);
            $order->history = $order->getHistory((int) $context->language->id, false, true);
            $order->customer_messages = CustomerMessage::getMessagesByOrderId((int) $order->id, false);

            // if (class_exists('PrestAppMondialrelayModuleController')) {
            //     $mondialrelay = new PrestAppMondialrelayModuleController();
            //     $mondialrelay_trackinginformations = $mondialrelay->getTrackingInformations($order->id_carrier, $order->id_cart);

            //     if ($mondialrelay_trackinginformations) {
            //         $order->tracking_informations = $mondialrelay_trackinginformations;
            //     }
            // }

            if (!isset($order->tracking_informations)) {
                $order_shipping = $order->getShipping();

                if ($order_shipping) {
                    if (count($order_shipping) > 0) {
                        $order->tracking_informations = array(
                            'tracking_id' => $order_shipping[0]['tracking_number'],
                            'tracking_url' => $order_shipping[0]['url'],
                        );
                    }
                }
            }

            // specific tracking system

            try {
                if (class_exists('Mondialrelay') && (Mondialrelay::isMondialRelayCarrier($order->id_carrier))) {
                    $sql = '
                        SELECT s.`id_mr_selected`, s.`MR_Selected_LgAdr1`, s.`MR_Selected_LgAdr2`, s.`MR_Selected_LgAdr3`, s.`MR_Selected_LgAdr4`,
                        s.`MR_Selected_CP`, s.`MR_Selected_Ville`, s.`MR_Selected_Pays`, s.`MR_Selected_Num`, s.`url_suivi`, s.`exp_number`, m.dlv_mode 
                        FROM `'._DB_PREFIX_.'mr_selected` s
                        INNER JOIN  '._DB_PREFIX_.'mr_method m ON m.id_mr_method = s.id_method 
                        WHERE s.`id_cart` = '.(int)$order->id_cart;
                    $res = Db::getInstance()->getRow($sql);

                    if ($res) {
                        $order->tracking_informations = array(
                            'tracking_id' => $res['exp_number'],
                            'tracking_url' => $res['url_suivi'],
                        );

                        $other_results = Db::getInstance()->executeS('
                            SELECT * FROM '._DB_PREFIX_.'mr_exp_number_history 
                            WHERE id_mr_selected ='.(int)($res['id_mr_selected']));

                        if ($other_results) {
                            $others_data = array();

                            foreach ($other_results as $other_exp) {
                                if ($other_exp['exp_number'] != $res['exp_number']) {
                                    array_push($others_data, array(
                                        'exp_url' => $other_exp['url_suivi'],
                                        'exp_number' => $other_exp['exp_number'],
                                        'exp_date' => $other_exp['date_add']
                                    ));
                                }
                            }

                            $order->tracking_informations['tracking_additional'] = $others_data;
                        }
                    }
                }

                if (file_exists(_PS_MODULE_DIR_.'/colisprive/classes/models/ColisPriveParcel.php')) {
                    require_once(_PS_MODULE_DIR_.'/colisprive/classes/models/ColisPriveParcel.php');

                    if (class_exists('ColisPriveParcel') && $id_parcel = (int) ColisPriveParcel::isColisPriveOrder($order->id)) {
                        $delivery_address = new Address($order->id_address_delivery);
                        if (isset($delivery_address) && $delivery_address->postcode) {
                            $postcode = $delivery_address->postcode;
                        } else {
                            $postcode = "";
                        }

                        $parcel = new ColisPriveParcel($id_parcel);

                        if ($parcel) {
                            $order->tracking_informations = array(
                                'tracking_id' => $parcel->cppclcode . $postcode,
                                'tracking_url' => ColisPriveParcel::getTrackingUrl($id_parcel, false) . $postcode,
                            );

                            $parcels = ColisPriveParcel::getAllParcelFromIdOrder($order->id);

                            if ($parcels && count($parcels)) {
                                $others_data = array();
                                foreach ($parcels as $parcel) {
                                    if ($parcel->id != $id_parcel) {
                                        $other_parcel = new ColisPriveParcel($parcel->id);
                                        if ($other_parcel) {
                                            $tracking = ColisPriveParcel::getTrackingUrl($other_parcel->id, false);
                                            if ($tracking) {
                                                array_push($others_data, array(
                                                    'exp_url' => $tracking . $postcode,
                                                    'exp_number' => $other_parcel->cppclcode . $postcode,
                                                    'exp_date' => $other_parcel->date_add
                                                ));
                                            }
                                        }
                                    }
                                }

                                $order->tracking_informations['tracking_additional'] = $others_data;
                            }
                        }
                    }
                }
            } catch (Error $e) {
            }

            // specific to prestadrive delivery

            if (PrestAppCustomizationController::checkModuleAvailability('prestadrive')) {
                $prestadrive_order = PrestAppPrestadriveCustomizationController::getOrder($order->id);
                if ($prestadrive_order) {
                    $prestadrive_data = array();
                    if ($prestadrive_order['id_deliverer'] && $prestadrive_order['delivered']) {
                        $prestadrive_data['status'] = 'done';
                    } elseif ($prestadrive_order['id_deliverer']) {
                        $prestadrive_data['status'] = 'in_progress';
                        $coordinates = PrestAppPrestadriveCustomizationController::getDelivererCoordinates($prestadrive_order['id_deliverer']);
                        if ($coordinates) {
                            $prestadrive_data['coordinates'] = $coordinates;
                        }
                    } else {
                        $prestadrive_data['status'] = 'waiting';
                    }
                }
                if (isset($prestadrive_data) && $prestadrive_data) {
                    $order->live_tracking = $prestadrive_data;
                }

                if (PrestAppCustomizationController::checkModuleAvailability('pdlg_livreurs_api')) {
                    $pdlg_livreurs_api = new PrestAppPdlgLivreursApiCustomizationController();
                    $isRatingEnabled = $pdlg_livreurs_api->isRatingEnabled($order->id);
                    $order->rating_enabled = $isRatingEnabled;
                    if ($isRatingEnabled) {
                        $order->rating_module = 'pdlg_livreurs_api';
                    }
                }
            }

            // specific to addvance integration
            if (PrestAppCustomizationController::checkModuleAvailability('addvance')) {
                $addvanceOrderReferencePrefix = Configuration::get('ADDVANCE_ORDERPREFIX');
                $order->addvance = array(
                    'qr_code_data' => array(
                        'DO_Piece' =>  $addvanceOrderReferencePrefix . '-' . $order->id,
                        'DO_Ref' => $order->reference . '-' . $order->id,
                        'mode' => 'auto'
                    )
                );
            }

            $order->extra = array();

            ///////////////////////////
            //    ets_marketplace    //
            ///////////////////////////

            if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
                $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
                $order->extra['ets_marketplace'] = $ets_marketplace->getOrderRelatedData($context, $order);
            }

            // specific expedition history system

            try {
                $show_exp = false;
                $product_late = false;

                $products = $order->getProducts();

                $exp_histories = array();

                foreach ($products as $key => $value) {
                    if ((int) $value['product_quantity_refunded'] > 0) {
                        $quant = (int) $value['product_quantity'] - (int) $value['product_quantity_refunded'];
                        if ($quant <= 0) {
                            continue;
                        }
                    }
                    $show_exp = true;
                    $order_detail_id = $products[$key]['id_order_detail'];
                    $req_history = 'SELECT * FROM `'._DB_PREFIX_.'order_exp_history` WHERE id_order_detail = '.$order_detail_id;
                    $res_history = Db::getInstance()->ExecuteS($req_history);

                    $exp_history = array(
                        'product' => $value,
                        'histories' => array()
                    );

                    if (file_exists(_PS_MODULE_DIR_.'privatesales/models/SaleCore.php')) {
                        include_once(_PS_MODULE_DIR_.'privatesales/models/SaleCore.php');
                        $privatesale_id = SaleCore::getPrivateSaleIdFromProductId($value['product_id']);
                        if ($privatesale_id) {
                            $privatesale = new SaleCore($privatesale_id);

                            $privatesale->express = (int)$privatesale->express;
                            $privatesale->expe_7j = (int)$privatesale->expe_7j;
                            $privatesale->expe_72h = (int)$privatesale->expe_72h;
                            $privatesale->livraison_noel = (int)$privatesale->livraison_noel;

                            $exp_histories[] = array(
                                'product' => $value,
                                'histories' => $res_history,
                                'privatesale' => $privatesale
                            );
                        }
                    }
                }

                if ($exp_histories) {
                    $order->exp_history = array(
                        'exp_history' => $exp_histories,
                        'strings' => array(
                            'christmas_text' => 'Livraison garantie avant Noël',
                            'in_progress_text' => 'Votre article est en cours d\'expédition',
                            'single_late_text' => 'Retard de livraison, voir le détail ci-dessous.',
                            'date_range_text' => 'Expédition prévue entre le @ et le @',
                            'late_text' => '<TO_REPLACE>'
                        )
                    );
                }
            } catch (Error $e) {
            }

            $customized_datas = Product::getAllCustomizedDatas($order->id_cart);

            if ($customized_datas) {
                foreach ($order->products as &$product) {
                    $customization = Db::getInstance()->getRow('
                        SELECT `id_customization`
                        FROM `'._DB_PREFIX_.'customization`
                        WHERE `id_cart` = '.(int)$order->id_cart.' AND `id_product_attribute` = '.(int)$product['product_attribute_id'].' AND `id_product` = '.(int)$product['product_id'].'
                    ');

                    if ($customization && $customization['id_customization']) {
                        $product['id_customization'] = $customization['id_customization'];
                    }

                    if (isset($customized_datas[$product['product_id']]) && $customized_datas[$product['product_id']]) {
                        if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']]) && $customized_datas[$product['product_id']][$product['product_attribute_id']]) {
                            if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']][$order->id_address_delivery]) && $customized_datas[$product['product_id']][$product['product_attribute_id']][$order->id_address_delivery]) {
                                if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']][$order->id_address_delivery][$product['id_customization']]) && $customized_datas[$product['product_id']][$product['product_attribute_id']][$order->id_address_delivery][$product['id_customization']]) {
                                    $product['customized_data'] = $customized_datas[$product['product_id']][$product['product_attribute_id']][$order->id_address_delivery][$product['id_customization']];
                                }
                            }
                        }
                    }
                }
            }

            $order->customized_datas = $customized_datas;

            return $order;
        }
    }

    /**
     * getOrderByCartId - Return the order object from its cart id.
     *
     * @param Context $context
     *
     * @return array
     */
    public static function getOrderByCartId($id_cart, $context)
    {
        $order_id = Order::getOrderByCartId((int) ($id_cart));
        $global_order = new Order($order_id);

        if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

            if (file_exists(_PS_MODULE_DIR_ . 'ets_marketplace/classes/seller.php')) {
                require_once _PS_MODULE_DIR_ . 'ets_marketplace/classes/seller.php';
            }

            $orders = array();
            $orders[] = new Order($order_id);

            $other_orders = Db::getInstance()->executeS('SELECT id_order FROM `'._DB_PREFIX_.'orders` WHERE id_order!="'.(int)$order_id.'" AND id_cart="'.(int)$id_cart.'"');
            
            if($other_orders) {

                foreach($other_orders as $other_order) {
                    $class_order = new Order($other_order['id_order']);
                    $orders[] = $class_order;
                    $global_order->total_paid += $class_order->total_paid;
                    $global_order->total_paid_real += $class_order->total_paid_real;
                    $global_order->total_paid_tax_incl += $class_order->total_paid_tax_incl;
                    $global_order->total_paid_tax_excl += $class_order->total_paid_tax_excl;
                    $global_order->total_discounts_tax_incl += $class_order->total_discounts_tax_incl;
                    $global_order->total_discounts_tax_excl += $class_order->total_discounts_tax_excl;
                    $global_order->total_shipping_tax_incl += $class_order->total_shipping_tax_incl;
                    $global_order->total_shipping_tax_excl += $class_order->total_shipping_tax_excl;
                    $global_order->total_wrapping_tax_incl += $class_order->total_wrapping_tax_incl;
                    $global_order->total_wrapping_tax_excl += $class_order->total_wrapping_tax_excl;
                    $global_order->total_products_wt += $class_order->total_products_wt;
                    $global_order->total_products += $class_order->total_products;
                }
            }

            foreach($orders as &$uniq_order) {

                $sql = 'SELECT o.id_customer FROM `'._DB_PREFIX_.'ets_mp_seller_order` o
                INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (o.id_customer = seller.id_customer)
                WHERE o.id_order=' . (int) $uniq_order->id . ' AND seller.active != 0';
                $id_customer = (int)Db::getInstance()->getValue($sql);

                if($id_customer && file_exists(_PS_MODULE_DIR_ . 'ets_marketplace/classes/seller.php')) {
                    $seller= Ets_mp_seller::_getSellerByIdCustomer($id_customer, $context->language->id);
                    $uniq_order->seller = $seller;
                }
            }
        }

        if (!Validate::isLoadedObject($global_order)) {
            return array(
                'order' => 'ORDER_NOT_FOUND',
            );
        } elseif ($global_order->id_customer && null == $context->customer) {
            return array(
                'order' => 'NOT_AUTHORIZED',
            );
        } elseif ($global_order->id_customer && $global_order->id_customer != $context->customer->id) {
            return array(
                'order' => 'NOT_AUTHORIZED',
            );
        } else {

            if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
                $product_detail = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                    SELECT od.*,ps.id_product
                    FROM `' . _DB_PREFIX_ . 'order_detail` od
                    INNER JOIN `'._DB_PREFIX_.'orders` o ON (od.id_order=o.id_order)
                    LEFT JOIN `' . _DB_PREFIX_ . 'product` p ON (p.id_product = od.product_id)
                    LEFT JOIN `' . _DB_PREFIX_ . 'product_shop` ps ON (ps.id_product = p.id_product AND ps.id_shop = od.id_shop)
                    WHERE o.`id_cart` = ' . (int) $id_cart);  
            } else {
                $ProductDetailObject = new OrderDetail();
                $product_detail = $ProductDetailObject->getList($order_id);
            }

            $global_order->products = $product_detail;
            $global_order->address_delivery = new Address($global_order->id_address_delivery);
            $global_order->address_invoice = new Address($global_order->id_address_invoice);
            $global_order->carrier = new Carrier($global_order->id_carrier, (int) $context->language->id);
            $global_order->currency = new Currency($global_order->id_currency);
            $global_order->cart_rules = $global_order->getCartRules();
            $global_order->state = $global_order->getCurrentStateFull((int) $context->language->id);

            $customized_datas = Product::getAllCustomizedDatas($global_order->id_cart);

            if ($customized_datas) {
                foreach ($global_order->products as &$product) {
                    $customization = Db::getInstance()->getRow('
                        SELECT `id_customization`
                        FROM `'._DB_PREFIX_.'customization`
                        WHERE `id_cart` = '.(int)$global_order->id_cart.' AND `id_product_attribute` = '.(int)$product['product_attribute_id'].' AND `id_product` = '.(int)$product['product_id'].'
                    ');

                    if ($customization && $customization['id_customization']) {
                        $product['id_customization'] = $customization['id_customization'];
                    }

                    if (isset($customized_datas[$product['product_id']]) && $customized_datas[$product['product_id']]) {
                        if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']]) && $customized_datas[$product['product_id']][$product['product_attribute_id']]) {
                            if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']][$global_order->id_address_delivery]) && $customized_datas[$product['product_id']][$product['product_attribute_id']][$global_order->id_address_delivery]) {
                                if (isset($customized_datas[$product['product_id']][$product['product_attribute_id']][$global_order->id_address_delivery][$product['id_customization']]) && $customized_datas[$product['product_id']][$product['product_attribute_id']][$global_order->id_address_delivery][$product['id_customization']]) {
                                    $product['customized_data'] = $customized_datas[$product['product_id']][$product['product_attribute_id']][$global_order->id_address_delivery][$product['id_customization']];
                                }
                            }
                        }
                    }
                }
            }

            // specific to addvance integration
            if (PrestAppCustomizationController::checkModuleAvailability('addvance')) {
                $addvanceOrderReferencePrefix = Configuration::get('ADDVANCE_ORDERPREFIX');
                $global_order->addvance = array(
                    'qr_code_data' => array(
                        'DO_Piece' =>  $addvanceOrderReferencePrefix . '-' . $global_order->id,
                        'DO_Ref' => $global_order->reference . '-' . $global_order->id,
                        'mode' => 'auto'
                    )
                );
            }

            $return_data = array(
                'order' => $global_order,
            );

            if ($orders) {
                $return_data['orders'] = $orders;
            }

            return $return_data;
        }
    }

    public static function getAnalyticsData($order, $orders, $context)
    {
        $parameters = Configuration::getMultiple(array('PS_LANG_DEFAULT'));
        $deliveryAddress = new Address((int)$order->id_address_delivery);
        $conversion_rate = 1;
        if ($order->id_currency != Configuration::get('PS_CURRENCY_DEFAULT')) {
            $currency = new Currency((int)$order->id_currency);
            $conversion_rate = (float)$currency->conversion_rate;
        }

        $trans = array(
            'id' => (int)$order->id, // order ID - required
            'store' => htmlentities(Configuration::get('PS_SHOP_NAME')), // affiliation or store name
            'revenue' => Tools::ps_round((float)$order->total_paid / (float)$conversion_rate, 2), // total - required
            'tax' => '0', // tax
            'shipping' => Tools::ps_round((float)$order->total_shipping / (float)$conversion_rate, 2), // shipping
            'city' => addslashes($deliveryAddress->city), // city
            'state' => '', // state or province
            'country' => addslashes($deliveryAddress->country), // country
        );

        $items = array();

        if ($orders) {
            foreach($orders as $order) {
                $products = $order->getProducts();
                foreach ($products as $product) {
                    $category = Db::getInstance()->getRow('
                        SELECT name FROM `' . _DB_PREFIX_ . 'category_lang` , ' . _DB_PREFIX_ . 'product
                        WHERE `id_product` = ' . (int)$product['product_id'] . ' AND `id_category_default` = `id_category`
                        AND `id_lang` = ' . (int)$parameters['PS_LANG_DEFAULT']);

                    $items[] = array(
                        'id' => (int)$order->id, // order ID - required
                        'sku' => addslashes($product['product_id']), // SKU/code - required
                        'name' => addslashes($product['product_name']), // product name
                        'category' => addslashes($category['name']), // category or variation
                        'price' => Tools::ps_round((float)$product['product_price_wt'] / (float)$conversion_rate, 2), // unit price - required
                        'quantity' => addslashes((int)$product['product_quantity']), //quantity - required
                    );
                }
            }
        } else {
            $products = $order->getProducts();
            foreach ($products as $product) {
                $category = Db::getInstance()->getRow('
                    SELECT name FROM `' . _DB_PREFIX_ . 'category_lang` , ' . _DB_PREFIX_ . 'product
                    WHERE `id_product` = ' . (int)$product['product_id'] . ' AND `id_category_default` = `id_category`
                    AND `id_lang` = ' . (int)$parameters['PS_LANG_DEFAULT']);
    
                $items[] = array(
                    'id' => (int)$order->id, // order ID - required
                    'sku' => addslashes($product['product_id']), // SKU/code - required
                    'name' => addslashes($product['product_name']), // product name
                    'category' => addslashes($category['name']), // category or variation
                    'price' => Tools::ps_round((float)$product['product_price_wt'] / (float)$conversion_rate, 2), // unit price - required
                    'quantity' => addslashes((int)$product['product_quantity']), //quantity - required
                );
            }
        }

        return array(
            'transaction' => $trans,
            'items' => $items,
        );
    }

    /**
     * getOrderHistory - Return the list of orders.
     *
     * @param Context $context
     *
     * @return array
     */
    public static function getOrderHistory($context)
    {
        if ($orders = Order::getCustomerOrders($context->customer->id)) {
            foreach ($orders as &$order) {
                $myOrder = new Order((int) $order['id_order']);
                if (Validate::isLoadedObject($myOrder)) {
                    $order['virtual'] = $myOrder->isVirtual(false);
                }
            }

            // if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
            //     foreach ($orders as &$order) {
            //         $order['ets_marketplace'] = array(
            //             'seller' => PrestAppEtsMarketPlaceCustomizationController::getSellerByOrderId($order['id_order'], $context)
            //         );
            //     }
            // }

            if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
                $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
                foreach ($orders as &$order) {
                    $myOrder = new Order((int) $order['id_order']);
                    $order['extra']['ets_marketplace'] = $ets_marketplace->getOrderRelatedData($context, $myOrder);
                }
            }
        }

        return $orders;
    }

    /**
     * getCurrency - Return currency from id.
     *
     * @param Context $context
     *
     * @return array
     */
    public static function getCurrency($id, $context)
    {
        $currency = new Currency($id, $context->language->id, $context->shop->id);

        return $currency;
    }

    public static function logOrder($id_order, $context)
    {
        if (isset($id_order) && (int) $id_order) {
            $sql = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'prestapp_orders` (
                `id_prestapp_order` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `id_ps_order` INT( 11 ) UNSIGNED NOT NULL,
                `date_add` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (`id_prestapp_order`)
              ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

            if (Db::getInstance()->Execute($sql)) {
                $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'prestapp_orders` (
                    `id_ps_order`
                    ) values (' . (int) ($id_order) . ')';

                Db::getInstance()->Execute($sql);
            }
        }

        return;
    }

    public static function getOrderCountByCustomerID($id_customer)
    {
        if ($orders = Order::getCustomerOrders($id_customer)) {
            return count($orders);
        } else {
            return 0;
        }
    }

    public static function rateOrder($id_order, $rating_module, $params, $context)
    {
        if (!$id_order || !$rating_module) {
            return false;
        } else {
            $order = new Order($id_order);

            if ($order->id_customer != $context->customer->id) {
                return false;
            }

            switch ($rating_module) {
                case 'pdlg_livreurs_api':
                    if (PrestAppCustomizationController::checkModuleAvailability('pdlg_livreurs_api')) {
                        $delivery_rating = $params['delivery_rating'];
                        $quality_rating = $params['quality_rating'];
                        $simplicity_rating = $params['simplicity_rating'];
                        $comment = $params['comment'];
                        if ($delivery_rating && $quality_rating && $simplicity_rating) {
                            $pdlg_livreurs_api = new PrestAppPdlgLivreursApiCustomizationController();
                            if ($pdlg_livreurs_api->getRating($id_order)) {
                                return false;
                            }

                            $result = $pdlg_livreurs_api->addReview($id_order, $delivery_rating, $quality_rating, $simplicity_rating, $comment);

                            return $result;
                        }
                    } else {
                        return false;
                    }
                    break;
                default:
                    return false;
                    break;
            }
        }
    }
}
