<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppSpecialController
{
    public static function getPriceDrop($page, $column, $order, $populate, $filters, $context)
    {
        $product_per_page = 30;

        if ('' == $column) {
            $column = 'position';
        }

        if ('' == $order) {
            $order = 'ASC';
        }

        if (PrestAppCustomizationController::checkModuleAvailability('pm_advancedsearch4')) {
            $pm_advancedsearch4 = new PrestAppPmadvancedsearch4CustomizationController($page, $product_per_page, $column, $order);
            if (isset($filters) && $filters) {
                $layer_response = $pm_advancedsearch4->getProducts($context, $populate, $filters, null, 'prices-drop');
                $result_products = $layer_response['products'];
                $layer = $layer_response['layer'];
                $sort_orders = $layer_response['sort_orders'];
                $filter_nbr_products = $layer_response['nb_products'];
            } else {
                $layer_response = $pm_advancedsearch4->getFilters($context, null, 'prices-drop');
                $layer = $layer_response;
                $products = Product::getPricesDrop($context->language->id, $page - 1, (int)$product_per_page, false, $column, $order);
                $result_products = array();
                foreach ($products as $product) {
                    $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                    if ($product && $product != 404 && $product != 403) {
                        array_push($result_products, $product);
                    }
                }
            }
        } else {
            $products = Product::getPricesDrop($context->language->id, $page - 1, (int)$product_per_page, false, $column, $order);
            $result_products = array();
            foreach ($products as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($result_products, $product);
                }
            }
        }

        $nbProducts = Product::getPricesDrop($context->language->id, null, null, true);

        if ($result_products) {
            $data = array(
                'nbr_products' => $nbProducts,
                'products' => $result_products
            );

            if (isset($layer) && $layer) {
                $data['layer'] = $layer;
                if (isset($filter_nbr_products)) {
                    $data['layer']['filters_products_number'] = (int) $filter_nbr_products;
                }
            }

            if (isset($sort_orders) && $sort_orders) {
                $data['sort_orders'] = $sort_orders;
            }

            return $data;
        } else {
            return array(
                'nbr_products' => 0,
                'products' => array()
            );
        }
    }

    public static function getManufacturersList($context)
    {
        $data = Manufacturer::getManufacturers(false, $context->language->id, true, false, false, false);
        return $data;
    }

    public static function getManufacturerProducts($manufacturer_id, $page, $column, $order, $populate, $filters, $context)
    {
        $product_per_page = 30;

        if ('' == $column) {
            $column = 'position';
        }

        if ('' == $order) {
            $order = 'ASC';
        }

        $manufacturer = new Manufacturer($manufacturer_id);

        if (PrestAppCustomizationController::checkModuleAvailability('pm_advancedsearch4')) {
            $pm_advancedsearch4 = new PrestAppPmadvancedsearch4CustomizationController($page, $product_per_page, $column, $order);
            if (isset($filters) && $filters) {
                $_POST['id_manufacturer'] = $manufacturer_id;
                $layer_response = $pm_advancedsearch4->getProducts($context, $populate, $filters, null, 'manufacturer');
                $result_products = $layer_response['products'];
                $layer = $layer_response['layer'];
                $sort_orders = $layer_response['sort_orders'];
                $filter_nbr_products = $layer_response['nb_products'];
            } else {
                $_POST['id_manufacturer'] = $manufacturer_id;
                $layer_response = $pm_advancedsearch4->getFilters($context, null, 'manufacturer');
                $layer = $layer_response;
                $products = $manufacturer->getProducts($manufacturer->id, $context->language->id, (int)$page, (int)$product_per_page, $column, $order);
                $result_products = array();
                foreach ($products as $product) {
                    $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                    if ($product && $product != 404 && $product != 403) {
                        array_push($result_products, $product);
                    }
                }
            }
        } else {
            $products = $manufacturer->getProducts($manufacturer->id, $context->language->id, (int)$page, (int)$product_per_page, $column, $order);
            $result_products = array();
            foreach ($products as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($result_products, $product);
                }
            }
        }

        $nbProducts = $manufacturer->getProducts($manufacturer->id, null, null, null, $column, $order, true);

        if ($result_products) {
            $data = array(
                'nbr_products' => $nbProducts,
                'products' => $result_products,
                'name' => $manufacturer->name
            );

            if (isset($layer) && $layer) {
                $data['layer'] = $layer;
                if (isset($filter_nbr_products)) {
                    $data['layer']['filters_products_number'] = (int) $filter_nbr_products;
                }
            }

            if (isset($sort_orders) && $sort_orders) {
                $data['sort_orders'] = $sort_orders;
            }

            return $data;
        } else {
            return array(
                'nbr_products' => 0,
                'products' => array(),
                'name' => $manufacturer->name
            );
        }
    }
}
