<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('addressregister') && Module::isEnabled('addressregister')) {

    class PrestAppAddressregisterCustomizationController
    {
        protected $addressregister;

        public function __construct()
        {
            $this->addressregister = Module::getInstanceByName('addressregister');
        }

        public function getFields($context)
        {

            $country = $context->country;

            if (Configuration::get('PS_RESTRICT_DELIVERED_COUNTRIES')) {
                $availableCountries = Carrier::getDeliveredCountries($context->language->id, true, true);
            } else {
                $availableCountries = Country::getCountries($context->language->id, true);
            }

            $addressFormatter = new CustomerAddressFormatter(
                $country,
                $this->addressregister->getTranslator(),
                $availableCountries
            );

            $formFields = $addressFormatter->getFormat();
            $formFields['id_country']->setType('select');

            $fields = array();

            $to_unset = ['firstname', 'lastname', 'alias', 'back', 'id_customer', 'id_address', 'token'];

            foreach($formFields as $formField) {

                $found = array_search($formField->getName(), $to_unset);

                if ($found === false) {
                    $fields[] = array(
                        'name' => $formField->getName(),
                        'type' => $formField->getType(),
                        'label' => $formField->getLabel(),
                        'value' => $formField->getValue(),
                        'required' => $formField->isRequired(),
                        'options' => $formField->getAvailableValues()
                    );
                }
            }

            $translator = $this->addressregister->getTranslator();

            $fields[] = array(
                'name' => 'id_state',
                'type' => 'select',
                'label' => $translator->trans('State', [], 'Shop.Forms.Labels'),
                'value' => null,
                'required' => false
            );

            return array(
                'status' => true,
                'data' => array(
                    'fields' => $fields
                )
            );
        }

        public function createAddress($customer, $params_values, $context) {

            if (Configuration::get('PS_RESTRICT_DELIVERED_COUNTRIES')) {
                $availableCountries = Carrier::getDeliveredCountries($context->language->id, true, true);
            } else {
                $availableCountries = Country::getCountries($context->language->id, true);
            }

            $persister = new CustomerAddressPersister(
                $customer,
                $context->cart,
                null // the token is not defined for new user.
            );

            $addressFormatter = new CustomerAddressFormatter(
                $context->country,
                $this->addressregister->getTranslator(),
                $availableCountries
            );

            $form = new CustomerAddressForm(
                $context->smarty,
                $context->language,
                $this->addressregister->getTranslator(),
                $persister,
                $addressFormatter
            );

            if (isset($params_values['addressregister']) && count($params_values['addressregister'])) {
                $params_values['addressregister']['firstname'] = $customer->firstname;
                $params_values['addressregister']['lastname'] = $customer->lastname;
                $form->fillWith($params_values['addressregister']);
                $form->submit();
            }
        }
    }
}
