<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('allinone_rewards') && Module::isEnabled('allinone_rewards')) {
    include_once _PS_MODULE_DIR_ . 'allinone_rewards/allinone_rewards.php';
    // include_once(_PS_MODULE_DIR_.'allinone_rewards/models/RewardsSpon.php');

    class PrestAppAllInOneRewardsCustomizationController extends allinone_rewards
    {
        protected $allinone_rewards;

        public function __construct()
        {
            $this->allinone_rewards = Module::getInstanceByName('allinone_rewards');
        }

        public function getData()
        {
            $data = new StdClass();
            $data->name = 'all_in_one_rewards';
            $data->sponsorship_active = $this->isSponsorshipActive();
            $data->loyalty_active = $this->isLoyaltyActive();
            return $data;
        }

        public function getCustomerData($context)
        {
            $rewards_sponsorship = new allinone_rewards();

            $id_template = (int)MyConf::getIdTemplate('sponsorship', $context->customer->id);
            $orderQuantityS = (int) MyConf::get('RSPONSORSHIP_ORDER_QUANTITY_S', null, $id_template);
            $stats = $context->customer->getStats();
            $canSendInvitations = false;
            if ((int) ($stats['nb_orders']) >= $orderQuantityS) {
                $canSendInvitations = true;
            }

            $data = array();

            $data['rules'] = MyConf::get('RSPONSORSHIP_RULES_TXT', $context->language->id, (int) MyConf::getIdTemplate('sponsorship', $context->customer->id));
            $data['orderQuantityS'] = $orderQuantityS;
            $data['canSendInvitations'] = $canSendInvitations;
            $data['pendingFriends'] = RewardsSponsorshipModel::getSponsorFriends((int) $context->customer->id, 'pending');
            $data['subscribeFriends'] = RewardsSponsorshipModel::getSponsorFriends((int) $context->customer->id, 'subscribed');
            $data['discount_gc'] = $rewards_sponsorship->getDiscountReadyForDisplay((int) MyConf::get('RSPONSORSHIP_DISCOUNT_TYPE_GC', null, $id_template), (int) MyConf::get('RSPONSORSHIP_FREESHIPPING_GC', null, $id_template), (float) MyConf::get('RSPONSORSHIP_VOUCHER_VALUE_GC_' . (int) $context->currency->id, null, $id_template));

            return $data;
        }

        public function getCustomerCredits($context)
        {
            if (version_compare($this->allinone_rewards->version, '4', '>=')) {
                return $this->getCustomerCreditsV4($context);
            } else {
                return $this->getCustomerCreditsAll($context);
            }
        }

        public function getCustomerCreditsV4($context)
        {
            $nbpagination = 10;

            $id_template = (int)MyConf::getIdTemplate('core', $context->customer->id);
            $payment_ratio = (float)MyConf::get('REWARDS_PAYMENT_RATIO', null, $id_template);

            // récupère le nombre de crédits convertibles
            $totals = RewardsModel::getAllTotalsByCustomer((int)$context->customer->id);
            $totalGlobal = isset($totals['total']) ? (float)$totals['total'] : 0;
            $totalConverted = isset($totals[RewardsStateModel::getConvertId()]) ? (float)$totals[RewardsStateModel::getConvertId()] : 0;
            $totalAvailable = isset($totals[RewardsStateModel::getValidationId()]) ? (float)$totals[RewardsStateModel::getValidationId()] : 0;
            $totalPending = (isset($totals[RewardsStateModel::getDefaultId()]) ? (float)$totals[RewardsStateModel::getDefaultId()] : 0) + (isset($totals[RewardsStateModel::getReturnPeriodId()]) ? $totals[RewardsStateModel::getReturnPeriodId()] : 0);
            $totalWaitingPayment = isset($totals[RewardsStateModel::getWaitingPaymentId()]) ? (float)$totals[RewardsStateModel::getWaitingPaymentId()] : 0;
            $totalPaid = isset($totals[RewardsStateModel::getPaidId()]) ? (float)$totals[RewardsStateModel::getPaidId()] : 0;
            $totalForPaymentDefaultCurrency = round($totalAvailable * $payment_ratio / 100, 2);

            $totalAvailableUserCurrency = RewardsModel::getCurrencyValue($totalAvailable, $context->currency->id);
            $voucherMininum = (float)MyConf::get('REWARDS_VOUCHER_MIN_VALUE_'.(int)$context->currency->id, null, $id_template) > 0 ? (float)MyConf::get('REWARDS_VOUCHER_MIN_VALUE_'.(int)$context->currency->id, null, $id_template) : 0;
            $paymentMininum = (float)MyConf::get('REWARDS_PAYMENT_MIN_VALUE_'.(int)$context->currency->id, null, $id_template) > 0 ? (float)MyConf::get('REWARDS_PAYMENT_MIN_VALUE_'.(int)$context->currency->id, null, $id_template) : 0;

            $giftAllowed = self::isCustomerAllowedForGiftProduct();
            $voucherAllowed = self::isCustomerAllowedForVoucher();
            $paymentAllowed = self::isCustomerAllowedForPayment();

            $link = $context->link->getModuleLink('allinone_rewards', 'rewards', array(), true);
            $rewards = RewardsModel::getAllByIdCustomer((int)$context->customer->id);
            $displayrewards = RewardsModel::getAllByIdCustomer((int)$context->customer->id, false, false, true, $nbpagination, (int)Tools::getValue('page') > 0 ? (int)Tools::getValue('page') : 1, $context->currency->id, true);

            $data = array(
                'return_days' => (Configuration::get('REWARDS_WAIT_RETURN_PERIOD') && Configuration::get('PS_ORDER_RETURN') && (int)Configuration::get('PS_ORDER_RETURN_NB_DAYS') > 0) ? (int)Configuration::get('PS_ORDER_RETURN_NB_DAYS') : 0,
                'rewards' => $rewards,
                'cart_rules' => RewardsModel::getCartRulesFromRewards((int)$context->customer->id),
                'cart_rules_available' => RewardsModel::getCartRulesFromRewards((int)$context->customer->id, true),
                'order_process' => Configuration::get('PS_ORDER_PROCESS_TYPE') ? 'order-opc' : 'order',
                'rewards_virtual' => (int)MyConf::get('REWARDS_VIRTUAL', null, $id_template),
                'show_link' => $giftAllowed && (int)MyConf::get('REWARDS_GIFT_SHOW_LINK', null, $id_template),
                'activeTab' => Tools::getValue('page') ? 'history' : '',
                'displayrewards' => $displayrewards,
                'pagination_link' => $link . (strpos($link, '?') !== false ? '&' : '?'),
                'totalGlobal' => $totalGlobal,
                'totalConverted' => $totalConverted,
                'totalAvailable' => $totalAvailable,
                'totalAvailableCurrency' => Tools::displayPrice($totalAvailableUserCurrency, $context->currency),
                'totalPending' => $totalPending,
                'totalWaitingPayment' => $totalWaitingPayment,
                'totalPaid' => $totalPaid,
                'convertColumns' => ($voucherAllowed || $totalConverted > 0) ? true : false,
                'paymentColumns' => ($paymentAllowed || $totalPaid > 0 || $totalWaitingPayment > 0) ? true : false,
                'totalForPaymentDefaultCurrency' => Tools::displayPrice($totalForPaymentDefaultCurrency, (int)Configuration::get('PS_CURRENCY_DEFAULT')),
                'voucher_allowed' => $voucherAllowed,
                'voucher_min' => $voucherAllowed ? $voucherMininum : 0,
                'voucher_minimum_allowed' => $voucherAllowed && $voucherMininum > 0 ? true : false,
                'voucher_button_allowed' => $voucherAllowed && $totalAvailableUserCurrency >= $voucherMininum && $totalAvailableUserCurrency > 0,
                'paymentMinimum' => $paymentMininum,
                'payment_minimum_allowed' => $paymentAllowed && $paymentMininum > 0 ? true : false,
                'payment_button_allowed' => $paymentAllowed && $totalAvailableUserCurrency >= $paymentMininum && $totalForPaymentDefaultCurrency > 0,
                'payment_txt' => MyConf::get('REWARDS_PAYMENT_TXT', (int)$context->language->id, $id_template),
                'general_txt' => MyConf::get('REWARDS_GENERAL_TXT', (int)$context->language->id, $id_template),
                'payment_details' => Tools::getValue('payment_details'),
                'payment_invoice' => (int)MyConf::get('REWARDS_PAYMENT_INVOICE', null, $id_template),
                'pagination' => (int)Tools::getValue('page') > 0 && (int)Tools::getValue('page') <= floor(sizeof($rewards) / $nbpagination) ? (int)Tools::getValue('page') : 1,
                'error' => false
            );

            return $data;
        }

        public function getCustomerCreditsAll($context)
        {
            $rewards_sponsorship = new allinone_rewards();

            $id_template = (int) MyConf::getIdTemplate('core', $context->customer->id);

            $totals = RewardsModel::getAllTotalsByCustomer((int) $context->customer->id);
            $totalGlobal = isset($totals['total']) ? (float) $totals['total'] : 0;
            $totalConverted = isset($totals[RewardsStateModel::getConvertId()]) ? (float) $totals[RewardsStateModel::getConvertId()] : 0;
            $totalAvailable = isset($totals[RewardsStateModel::getValidationId()]) ? (float) $totals[RewardsStateModel::getValidationId()] : 0;
            $totalPending = (isset($totals[RewardsStateModel::getDefaultId()]) ? (float) $totals[RewardsStateModel::getDefaultId()] : 0) + (isset($totals[RewardsStateModel::getReturnPeriodId()]) ? $totals[RewardsStateModel::getReturnPeriodId()] : 0);
            $totalWaitingPayment = isset($totals[RewardsStateModel::getWaitingPaymentId()]) ? (float) $totals[RewardsStateModel::getWaitingPaymentId()] : 0;
            $totalPaid = isset($totals[RewardsStateModel::getPaidId()]) ? (float) $totals[RewardsStateModel::getPaidId()] : 0;
            $totalForPaymentDefaultCurrency = round($totalAvailable * MyConf::get('REWARDS_PAYMENT_RATIO', null, $id_template) / 100, 2);
            $currency = Currency::getCurrency((int) $context->currency->id);
            $totalAvailableUserCurrency = Tools::convertPrice($totalAvailable, $currency);
            $voucherMininum = (float) MyConf::get('REWARDS_VOUCHER_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) > 0 ? (float) MyConf::get('REWARDS_VOUCHER_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) : 0;
            $paymentMininum = (float) MyConf::get('REWARDS_PAYMENT_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) > 0 ? (float) MyConf::get('REWARDS_PAYMENT_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) : 0;

            $voucherAllowed = RewardsModel::isCustomerAllowedForVoucher((int) $context->customer->id);
            $paymentAllowed = RewardsModel::isCustomerAllowedForPayment((int) $context->customer->id);

            if ($voucherAllowed && Tools::getValue('transform-credits') == 'true' && $totalAvailableUserCurrency >= $voucherMininum) {
                RewardsModel::createDiscount($totalAvailable);
            }

            $link = $context->link->getModuleLink('allinone_rewards', 'rewards', array(), true);
            $rewards = RewardsModel::getAllByIdCustomer((int) $context->customer->id);
            $displayrewards = RewardsModel::getAllByIdCustomer((int) $context->customer->id, false, false, true, ((int) (Tools::getValue('n')) > 0 ? (int) (Tools::getValue('n')) : 10), ((int) (Tools::getValue('p')) > 0 ? (int) (Tools::getValue('p')) : 1), $context->currency->id);

            $data = array();

            $data['return_days'] = (Configuration::get('REWARDS_WAIT_RETURN_PERIOD') && Configuration::get('PS_ORDER_RETURN') && (int) Configuration::get('PS_ORDER_RETURN_NB_DAYS') > 0) ? (int) Configuration::get('PS_ORDER_RETURN_NB_DAYS') : 0;
            $data['rewards_duration'] = (int) Configuration::get('REWARDS_DURATION');
            $data['rewards'] = $rewards;
            $data['displayrewards'] = $displayrewards;
            $data['pagination_link'] = $link . (strpos($link, '?') !== false ? '&' : '?');
            $data['totalGlobal'] = round(Tools::convertPrice($totalGlobal, $currency), 2);
            $data['totalConverted'] = round(Tools::convertPrice($totalConverted, $currency), 2);
            $data['totalAvailable'] = round(Tools::convertPrice($totalAvailable, $currency), 2);
            $data['totalPending'] = round(Tools::convertPrice($totalPending, $currency), 2);
            $data['totalWaitingPayment'] = round(Tools::convertPrice($totalWaitingPayment, $currency), 2);
            $data['totalPaid'] = round(Tools::convertPrice($totalPaid, $currency), 2);
            $data['totalForPaymentDefaultCurrency'] = $totalForPaymentDefaultCurrency;
            $data['payment_currency'] = Configuration::get('PS_CURRENCY_DEFAULT');
            $data['voucher_min'] = $voucherAllowed ? $voucherMininum : 0;
            $data['voucher_allowed'] = $voucherAllowed;
            $data['voucher_button_allowed'] = $voucherAllowed && $totalAvailableUserCurrency >= $voucherMininum && $totalAvailableUserCurrency > 0;
            $data['payment_min'] = $paymentAllowed ? $paymentMininum : 0;
            $data['payment_allowed'] = $paymentAllowed;
            $data['payment_button_allowed'] = $paymentAllowed && $totalAvailableUserCurrency >= $paymentMininum && $totalForPaymentDefaultCurrency > 0;
            $data['payment_txt'] = MyConf::get('REWARDS_PAYMENT_TXT', (int) $context->language->id, $id_template);
            $data['general_txt'] = MyConf::get('REWARDS_GENERAL_TXT', (int) $context->language->id, $id_template);
            $data['payment_invoice'] = (int) MyConf::get('REWARDS_PAYMENT_INVOICE', null, $id_template);
            $data['page'] = ((int) (Tools::getValue('p')) > 0 ? (int) (Tools::getValue('p')) : 1);
            $data['nbpagination'] = ((int) (Tools::getValue('n') > 0) ? (int) (Tools::getValue('n')) : 10);
            $data['nArray'] = array(10, 20, 50);
            $data['max_page'] = floor(sizeof($rewards) / ((int) (Tools::getValue('n') > 0) ? (int) (Tools::getValue('n')) : 10));

            return $data;
        }

        public static function isCustomerAllowedForVoucher()
        {
            return self::isCustomerAllowed('REWARDS_VOUCHER');
        }

        public static function isCustomerAllowedForGiftProduct()
        {
            return self::isCustomerAllowed('REWARDS_GIFT');
        }

        public static function isCustomerAllowedForPayment()
        {
            return self::isCustomerAllowed('REWARDS_PAYMENT');
        }

        private static function isCustomerAllowed($key)
        {
            $context = Context::getContext();
            if ($context->customer->logged) {
                $id_template = (int)MyConf::getIdTemplate('core', $context->customer->id);
                // if the customer is linked to a template, then it overrides the groups setting
                $stats = $context->customer->getStats();
                if (MyConf::get($key, null, $id_template) && (int)$stats['nb_orders'] >= (int)MyConf::get($key.'_NB_ORDERS', null, $id_template)) {
                    if ($id_template) {
                        return true;
                    }

                    $allowed_groups = explode(',', Configuration::get($key.'_GROUPS'));
                    $customer_groups = $context->customer->getGroups();

                    return sizeof(array_intersect($allowed_groups, $customer_groups)) > 0;
                }
            }
            return false;
        }

        public function sendInvitation($emails, $context)
        {
            $rewards_sponsorship = new allinone_rewards();

            $emails = json_decode($emails);

            if ($emails) {
                $is_error = false;

                foreach ($emails as $key => $friendEmail) {
                    if (empty($friendEmail) || !Validate::isEmail($friendEmail)) {
                        $is_error = true;
                    }
                }

                if ($is_error) {
                    return 'INVALID';
                } else {
                    $mails_exists = [];

                    foreach ($emails as $key => $friendEmail) {
                        if (RewardsSponsorshipModel::isEmailExists($friendEmail) || Customer::customerExists($friendEmail)) {
                            array_push($mails_exists, $friendEmail);
                        } else {
                            $sponsorship = new RewardsSponsorshipModel();
                            $sponsorship->id_sponsor = (int) $context->customer->id;
                            $sponsorship->channel = 1;
                            $sponsorship->email = $friendEmail;

                            if ($sponsorship->save()) {
                                $discount_gc = $rewards_sponsorship->getDiscountReadyForDisplay((int) MyConf::get('RSPONSORSHIP_DISCOUNT_TYPE_GC', null, $id_template), (int) MyConf::get('RSPONSORSHIP_FREESHIPPING_GC', null, $id_template), (float) MyConf::get('RSPONSORSHIP_VOUCHER_VALUE_GC_' . (int) $context->currency->id, null, $id_template));
                                $id_template = (int) MyConf::getIdTemplate('sponsorship', $context->customer->id);
                                $template = 'sponsorship-invitation-novoucher';

                                if ((int) MyConf::get('RSPONSORSHIP_DISCOUNT_GC', null, $id_template) == 1) {
                                    if ((int) MyConf::get('RSPONSORSHIP_DISCOUNT_TYPE_GC', null, $id_template) != 0) {
                                        $template = 'sponsorship-invitation';
                                    } else {
                                        $template = 'sponsorship-invitation-freeshipping';
                                    }
                                }

                                $vars = array(

                                    '{email}' => $context->customer->email,

                                    '{lastname}' => $context->customer->lastname,

                                    '{firstname}' => $context->customer->firstname,

                                    '{email_friend}' => $friendEmail,

                                    '{link}' => $context->link->getPageLink('index', true, $context->language->id, 's=' . $sponsorship->getSponsorshipMailLink()),

                                    '{nb_discount}' => (int) MyConf::get('RSPONSORSHIP_QUANTITY_GC', null, $id_template),

                                    '{discount}' => $discount_gc);

                                $rewards_sponsorship->sendMail((int) $context->language->id, $template, $rewards_sponsorship->getL('invitation'), $vars, $friendEmail, '' . ' ' . '');
                            }
                        }
                    }

                    $data = array();

                    $data['exists'] = $mails_exists;
                    $data['status'] = 'OK';

                    return $data;
                }
            } else {
                return 'UNKNOWN';
            }
        }

        public function revive($id_sponsorship, $context)
        {
            $rewards_sponsorship = new allinone_rewards();
            $sponsorship = new RewardsSponsorshipModel((int) $id_sponsorship);

            $discount_gc = $rewards_sponsorship->getDiscountReadyForDisplay((int) MyConf::get('RSPONSORSHIP_DISCOUNT_TYPE_GC', null, $id_template), (int) MyConf::get('RSPONSORSHIP_FREESHIPPING_GC', null, $id_template), (float) MyConf::get('RSPONSORSHIP_VOUCHER_VALUE_GC_' . (int) $context->currency->id, null, $id_template));
            $id_template = (int) MyConf::getIdTemplate('sponsorship', $context->customer->id);
            $template = 'sponsorship-invitation-novoucher';

            if ((int) MyConf::get('RSPONSORSHIP_DISCOUNT_GC', null, $id_template) == 1) {
                if ((int) MyConf::get('RSPONSORSHIP_DISCOUNT_TYPE_GC', null, $id_template) != 0) {
                    $template = 'sponsorship-invitation';
                } else {
                    $template = 'sponsorship-invitation-freeshipping';
                }
            }

            $vars = array(

                '{email}' => $context->customer->email,

                '{lastname}' => $context->customer->lastname,

                '{firstname}' => $context->customer->firstname,

                '{email_friend}' => $sponsorship->email,

                '{link}' => $context->link->getPageLink('index', true, $context->language->id, 's=' . $sponsorship->getSponsorshipMailLink()),

                '{nb_discount}' => (int) MyConf::get('RSPONSORSHIP_QUANTITY_GC', null, $id_template),

                '{discount}' => $discount_gc,

            );

            $sponsorship->save();
            $rewards_sponsorship->sendMail((int) $context->language->id, $template, $rewards_sponsorship->getL('invitation'), $vars, $sponsorship->email, $sponsorship->firstname . ' ' . $sponsorship->lastname);

            return 'OK';
        }

        public function askVoucher($context)
        {
            $id_template = (int) MyConf::getIdTemplate('core', $context->customer->id);

            $totals = RewardsModel::getAllTotalsByCustomer((int) $context->customer->id);
            $currency = Currency::getCurrency((int) $context->currency->id);
            $totalAvailable = isset($totals[RewardsStateModel::getValidationId()]) ? (float) $totals[RewardsStateModel::getValidationId()] : 0;
            if (version_compare($this->allinone_rewards->version, '4', '>=')) {
                $voucherAllowed = self::isCustomerAllowedForVoucher();
            } else {
                $voucherAllowed = RewardsModel::isCustomerAllowedForVoucher((int) $context->customer->id);
            }

            $totalAvailableUserCurrency = Tools::convertPrice($totalAvailable, $currency);
            $voucherMininum = (float) MyConf::get('REWARDS_VOUCHER_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) > 0 ? (float) MyConf::get('REWARDS_VOUCHER_MIN_VALUE_' . (int) $context->currency->id, null, $id_template) : 0;


            if ($voucherAllowed && $totalAvailableUserCurrency >= $voucherMininum && $totalAvailableUserCurrency > 0) {
                RewardsModel::createDiscount($totalAvailable);
                return 'OK';
            } else {
                return 'ERROR';
            }
        }

        public function isSponsorshipActive()
        {
            $allinone_rewards = new allinone_rewards();
            $rewards_sponsorship = new RewardsSponsorshipPlugin($allinone_rewards);
            return $rewards_sponsorship->isActive();
        }

        public function isLoyaltyActive()
        {
            $allinone_rewards = new allinone_rewards();
            $rewards_loyalty = new RewardsLoyaltyPlugin($allinone_rewards);
            return (bool) $rewards_loyalty->isActive();
        }

        // loyalty block

        public function getProductLoyalty($id_product, $id_product_attribute, $context)
        {
            if ($this->isLoyaltyActive()) {
                $product = new Product($id_product);
                $customer = new Customer($context->customer->id);

                if ($product->available_for_order && $this->_isCustomerAllowed($customer, true) && Validate::isLoadedObject($product) && $this->_isProductAllowed($product->id, $context)) {
                    $id_template = (int)MyConf::getIdTemplate('loyalty', $context->customer->id);
                    $rewards_on_total = (int)MyConf::get('RLOYALTY_TYPE', null, $id_template) == 2 ? false : true;

                    $product = new Product((int)$id_product);
                    $product->id_product_attribute = $id_product_attribute;

                    if (Validate::isLoadedObject($context->cart)) {
                        if ($rewards_on_total) {
                            $total_before = $this->_getCartTotalForReward($context);
                            $total_after = $this->_getCartTotalForReward($context, $product);
                            $credits_before = (float)$this->_getNbCreditsByPrice($context->customer->id, $total_before, $context->currency->id);
                            $credits_after = (float)($this->_getNbCreditsByPrice($context->customer->id, $total_after, $context->currency->id));
                        } else {
                            $cart = new Cart($context->cart->id);
                            $credits_before = $this->_getCartRewardByProduct($cart, $context);
                            $credits_after = $this->_getCartRewardByProduct($cart, $context, $product);
                        }
                        $credits = (float)($credits_after - $credits_before);
                    } else {
                        if (!(int)(MyConf::get('RLOYALTY_DISCOUNTED_ALLOWED', null, $id_template)) && RewardsModel::isDiscountedProduct($product->id)) {
                            $credits = $credits_before = $credits_after = 0;
                        // $this->context->smarty->assign('no_pts_discounted', 1);
                        } else {
                            $credits_before = 0;
                            if ($rewards_on_total) {
                                $total_after = $this->_getCartTotalForReward($context, $product);
                                $credits_after = (float)($this->_getNbCreditsByPrice($context->customer->id, $total_after, $context->currency->id));
                            } else {
                                $credits_after = $this->_getCartRewardByProduct(null, $context, $product);
                            }
                            $credits = $credits_after;
                        }
                    }

                    // si pas de crédit, pas un produit discount, et pas en mode tranche, on affiche rien
                    if ($credits == 0 && (int)MyConf::get('RLOYALTY_TYPE', null, $id_template) != 0) {
                        return false;
                    }

                    return array(
                        'text1' => $this->allinone_rewards->l('Buying this product you will collect ', 'PRODUCT'),
                        'text2' => $this->allinone_rewards->l(' with our loyalty program.', 'PRODUCT'),
                        'text3' => $this->allinone_rewards->l('Your cart will total', 'PRODUCT'),
                        'text4' => $this->allinone_rewards->l('that can be converted into a voucher for a future purchase.', 'PRODUCT'),
                        'display_credits' => ((float)$credits > 0) ? true : false,
                        'credits' => $this->allinone_rewards->getRewardReadyForDisplay((float)$credits, (int)$context->currency->id, null, false),
                        'total_credits' => $this->allinone_rewards->getRewardReadyForDisplay((float)$credits_after, (int)$context->currency->id, null, false),
                        'minimum' => Tools::displayPrice(round(Tools::convertPrice(MyConf::get('RLOYALTY_POINT_RATE', null, $id_template), (int)$context->currency->id), 2), (int)$context->currency->id)
                    );
                } else {
                    return false;
                }
            } else {
                return false;
            }
        }

        public function getCartLoyalty($context)
        {
            $customer = new Customer($context->customer->id);
            $cart = new Cart($context->cart->id);

            if ($this->_isCustomerAllowed($customer, true)) {
                if (Validate::isLoadedObject($cart)) {
                    $id_template = (int)MyConf::getIdTemplate('loyalty', $context->customer->id);
                    if ((int)MyConf::get('RLOYALTY_TYPE', null, $id_template) != 2) {
                        $total = $this->_getCartTotalForReward($context);
                        $total = RewardsModel::getCurrencyValue($total, Configuration::get('PS_CURRENCY_DEFAULT'));
                        $credits = $this->_getNbCreditsByPrice($context->customer->id, $total, $context->currency->id);
                    } else {
                        $credits = $this->_getCartRewardByProduct($cart);
                    }

                    return array(
                        'text1' => $this->allinone_rewards->l('By checking out this shopping cart you will collect ', 'SHOPPING-CART'),
                        'text2' => $this->allinone_rewards->l('that can be converted into a voucher for a future purchase.', 'SHOPPING-CART'),
                        'text_guest' => $this->allinone_rewards->l('Not available for Instant checkout order', 'SHOPPING-CART'),
                        'text_empty' => $this->allinone_rewards->l('Add some products to your shopping cart to collect some loyalty credits.', 'SHOPPING-CART'),
                        'display_credits' => ((float)$credits > 0) ? true : false,
                        'credits' => $this->allinone_rewards->getRewardReadyForDisplay((float)$credits, (int)$context->currency->id, null, false),
                        'guest_checkout' => (int)Configuration::get('PS_GUEST_CHECKOUT_ENABLED')
                    );
                } else {
                    return array(
                        'display_credits' => false
                    );
                }
            } else {
                return false;
            }
        }

        private function _getCartRewardByProduct($cart, $context, $newProduct = null)
        {
            $total = 0;
            $cartProducts = array();
            $taxesEnabled = Product::getTaxCalculationMethod();
            $currency = new Currency($context->currency->id);
            $cart_currency = $currency;
            $id_template = 0;

            if (Validate::isLoadedObject($cart)) {
                $cartProducts = $cart->getProducts();
                $taxesEnabled = Product::getTaxCalculationMethod((int)$cart->id_customer);
                $cart_currency = new Currency((int)$cart->id_currency);
                $id_template = (int)MyConf::getIdTemplate('loyalty', (int)$cart->id_customer);
            }

            if (isset($newProduct) && !empty($newProduct->id)) {
                $found = false;
                foreach ($cartProducts as $key => $product) {
                    if ($newProduct->id == $product['id_product'] && $newProduct->id_product_attribute == $product['id_product_attribute']) {
                        $found = true;
                        $cartProducts[$key]['cart_quantity']++;
                    }
                }
                if (!$found) {
                    $cartProductsNew = array();
                    $cartProductsNew['id_product'] = (int)$newProduct->id;
                    $cartProductsNew['id_product_attribute'] = $newProduct->id_product_attribute ? (int)$newProduct->id_product_attribute : (int)$newProduct->getIdProductAttributeMostExpensive();
                    $cartProductsNew['price'] = number_format($newProduct->getPrice(false, $cartProductsNew['id_product_attribute']), 2, '.', '');
                    if ($taxesEnabled != PS_TAX_EXC && MyConf::get('RLOYALTY_TAX', null, $id_template)) {
                        $cartProductsNew['price_wt'] = number_format($newProduct->getPrice(true, $cartProductsNew['id_product_attribute']), 2, '.', '');
                    }
                    $cartProductsNew['cart_quantity'] = 1;
                    $cartProducts[] = $cartProductsNew;
                }
            }

            $gifts = array();
            if (Validate::isLoadedObject($cart)) {
                foreach ($cart->getCartRules(CartRule::FILTER_ACTION_GIFT) as $rule) {
                    $cart_rule = new CartRule($rule['id_cart_rule']);
                    $gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute] = isset($gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute]) ? $gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute] + 1 : 1;
                }
            }

            foreach ($cartProducts as $product) {
                if ((!MyConf::get('RLOYALTY_DISCOUNTED_ALLOWED', null, $id_template) && RewardsModel::isDiscountedProduct($product['id_product'], (int)$product['id_product_attribute']))) {
                    if (is_object($newProduct) && $product['id_product'] == $newProduct->id && $product['id_product_attribute'] == $newProduct->id_product_attribute) {
                        // $this->context->smarty->assign('no_pts_discounted', 1);
                        continue;
                    }
                }

                $quantity = (int)$product['cart_quantity'] - (isset($gifts[$product['id_product'].'_'.$product['id_product_attribute']]) ? $gifts[$product['id_product'].'_'.$product['id_product_attribute']] : 0);
                $price = $taxesEnabled == PS_TAX_EXC || !MyConf::get('RLOYALTY_TAX', null, $id_template) ? $product['price'] : $product['price_wt'];
                $total += (float)RewardsProductModel::getProductReward((int)$product['id_product'], $price, $quantity, $cart_currency->id, $id_template, 'loyalty');
            }

            if ($total < 0) {
                $total = 0;
            }

            return $total;
        }

        private function _getNbCreditsByPrice($id_customer, $price, $idCurrencyFrom, $idCurrencyTo = null, $extraParams = array())
        {
            $id_template = (int)MyConf::getIdTemplate('loyalty', $id_customer);
            if (!isset($idCurrencyTo)) {
                $idCurrencyTo = $idCurrencyFrom;
            }

            if (Configuration::get('PS_CURRENCY_DEFAULT') != $idCurrencyFrom) {
                // converti de la devise du client vers la devise par défaut
                $price = Tools::convertPrice($price, Currency::getCurrency($idCurrencyFrom), false);
            }
            /* Prevent division by zero */
            $credits = 0;
            if ((int)MyConf::get('RLOYALTY_TYPE', null, $id_template) == 0) {
                $credits = floor(number_format($price, 2, '.', '') / (float)MyConf::get('RLOYALTY_POINT_RATE', null, $id_template)) * (float)MyConf::get('RLOYALTY_POINT_VALUE', null, $id_template);
            } elseif ((int)MyConf::get('RLOYALTY_TYPE', null, $id_template) == 1) {
                $credits = number_format($price, 2, '.', '') * (float)MyConf::get('RLOYALTY_PERCENTAGE', null, $id_template) / 100;
            }
            return round(Tools::convertPrice($credits, Currency::getCurrency($idCurrencyTo)), 2);
        }

        private function _getCartTotalForReward($context, $newProduct = null)
        {
            $benefits = false;
            $total = 0;
            $cartProducts = array();
            $taxesEnabled = Product::getTaxCalculationMethod();
            $cart_currency = new Currency($context->currency->id);
            $cart = new Cart($context->cart->id);
            $id_template = 0;
            $allowedCategories = $this->_getAllowedCategories($context);

            if (Validate::isLoadedObject($cart)) {
                $cartProducts = $cart->getProducts();
                $taxesEnabled = Product::getTaxCalculationMethod((int)$cart->id_customer);
                $cart_currency = new Currency((int)$cart->id_currency);
                $id_template = (int)MyConf::getIdTemplate('loyalty', (int)$cart->id_customer);
            }

            if (isset($newProduct) && !empty($newProduct->id)) {
                $found = false;
                foreach ($cartProducts as $key => $product) {
                    if ($newProduct->id == $product['id_product'] && $newProduct->id_product_attribute == $product['id_product_attribute']) {
                        $found = true;
                        $cartProducts[$key]['cart_quantity']++;
                    }
                }
                if (!$found) {
                    $cartProductsNew = array();
                    $cartProductsNew['id_product'] = (int)$newProduct->id;
                    $cartProductsNew['id_product_attribute'] = $newProduct->id_product_attribute ? (int)$newProduct->id_product_attribute : (int)$newProduct->getIdProductAttributeMostExpensive();
                    $cartProductsNew['price'] = number_format($newProduct->getPrice(false, $cartProductsNew['id_product_attribute']), 2, '.', '');
                    if ($taxesEnabled != PS_TAX_EXC && MyConf::get('RLOYALTY_TAX', null, $id_template)) {
                        $cartProductsNew['price_wt'] = number_format($newProduct->getPrice(true, $cartProductsNew['id_product_attribute']), 2, '.', '');
                    }
                    $cartProductsNew['cart_quantity'] = 1;
                    if ($benefits) {
                        $product_attribute = $newProduct->getAttributeCombinationsById($cartProductsNew['id_product_attribute'], (int)(Configuration::get('PS_LANG_DEFAULT')));
                        $cartProductsNew['wholesale_price'] = isset($product_attribute[0]['wholesale_price']) && (float)($product_attribute[0]['wholesale_price']) > 0 ? (float) $product_attribute[0]['wholesale_price'] : (float)$newProduct->wholesale_price;
                    }
                    $cartProducts[] = $cartProductsNew;
                }
            }

            $gifts = array();
            if (Validate::isLoadedObject($cart)) {
                foreach ($cart->getCartRules(CartRule::FILTER_ACTION_GIFT) as $rule) {
                    $cart_rule = new CartRule($rule['id_cart_rule']);
                    $gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute] = isset($gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute]) ? $gifts[$cart_rule->gift_product.'_'.$cart_rule->gift_product_attribute] + 1 : 1;
                }
            }

            foreach ($cartProducts as $product) {
                if ((!MyConf::get('RLOYALTY_DISCOUNTED_ALLOWED', null, $id_template) && RewardsModel::isDiscountedProduct($product['id_product'], (int)$product['id_product_attribute'])) || (is_array($allowedCategories) && !Product::idIsOnCategoryId($product['id_product'], $allowedCategories))) {
                    if (is_object($newProduct) && $product['id_product'] == $newProduct->id && $product['id_product_attribute'] == $newProduct->id_product_attribute) {
                        // $this->context->smarty->assign('no_pts_discounted', 1);
                        continue;
                    }
                }

                $quantity = (int)$product['cart_quantity'] - (isset($gifts[$product['id_product'].'_'.$product['id_product_attribute']]) ? $gifts[$product['id_product'].'_'.$product['id_product_attribute']] : 0);
                if ($benefits) {
                    $total += ($product['price'] - ((float)$product['wholesale_price'] * (float)$cart_currency->conversion_rate)) * $quantity;
                } else {
                    $total += ($taxesEnabled == PS_TAX_EXC || !MyConf::get('RLOYALTY_TAX', null, $id_template) ? $product['price'] : $product['price_wt']) * $quantity;
                }
            }

            if (Validate::isLoadedObject($cart)) {
                foreach ($cart->getCartRules(CartRule::FILTER_ACTION_REDUCTION) as $cart_rule) {
                    $total -= $benefits || $taxesEnabled == PS_TAX_EXC || !MyConf::get('RLOYALTY_TAX', null, $id_template) ? $cart_rule['value_tax_exc'] : $cart_rule['value_real'];
                }
            }
            if ($total < 0) {
                $total = 0;
            }

            return $total;
        }

        private function _getAllowedCategories($context)
        {
            $customer = new Customer($context->customer->id);

            $id_template=0;
            if (isset($customer)) {
                $id_template = (int)MyConf::getIdTemplate('loyalty', $customer->id);
            }
            if (MyConf::get('RLOYALTY_ALL_CATEGORIES', null, $id_template)) {
                return null;
            } else {
                $allowed_categories = array();
                $categories = explode(',', MyConf::get('RLOYALTY_CATEGORIES', null, $id_template));
                foreach ($categories as $category) {
                    $allowed_categories[] = array('id_category' => $category);
                }
                return $allowed_categories;
            }
        }

        private function _isCustomerAllowed($customer, $bCheckDefault = false)
        {
            $allowed_groups = explode(',', Configuration::get('RLOYALTY_GROUPS'));
            if (Validate::isLoadedObject($customer)) {
                if ((int)MyConf::getIdTemplate('loyalty', $customer->id)) {
                    return true;
                }
                $customer_groups = $customer->getGroups();
                return sizeof(array_intersect($allowed_groups, $customer_groups)) > 0;
            } elseif ($bCheckDefault && in_array(1, $allowed_groups)) {
                return true;
            } else {
                return false;
            }
        }

        private function _isProductAllowed($id_product, $context)
        {
            $id_template = (int)MyConf::getIdTemplate('loyalty', $context->customer->id);
            if ((int)MyConf::get('RLOYALTY_TYPE', null, $id_template) == 0 || (int)MyConf::get('RLOYALTY_TYPE', null, $id_template) == 1) {
                if (MyConf::get('RLOYALTY_ALL_CATEGORIES', null, $id_template)) {
                    return true;
                }
                return Product::idIsOnCategoryId($id_product, $this->_getAllowedCategories($context));
            } else {
                return RewardsProductModel::isProductRewarded($id_product, $id_template, 'loyalty');
            }
        }
    }
}
