<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('blockwishlist') && Module::isEnabled('blockwishlist')) {
    
    class PrestAppBlockwishlistCustomizationController
    {
        protected $blockwishlist;

        public function __construct()
        {
            $this->blockwishlist = Module::getInstanceByName('blockwishlist');
        }

        public function addProductToWishlist($id_product, $id_product_attribute, $context)
        {
            $idWishList = WishList::getDefault($context->customer->id);

            $product = new Product($id_product);
            if (!Validate::isLoadedObject($product)) {
                return array(
                    'status' => false,
                    'message' => 'PRODUCT_NOT_FOUND'
                );
            }

            $quantity = $product->minimal_quantity;

            if (!$id_product_attribute) {
                $id_product_attribute = 0;
            }

            $id_product_attribute_exists = Db::getInstance()->getValue(
                'SELECT pas.`id_product_attribute` ' .
                'FROM `' . _DB_PREFIX_ . 'product_attribute` pa ' .
                'INNER JOIN `' . _DB_PREFIX_ . 'product_attribute_shop` pas ON (pas.id_product_attribute = pa.id_product_attribute) ' .
                'WHERE pas.id_shop =' . (int) $context->shop->id . ' AND pa.`id_product` = ' . (int) $id_product . ' ' .
                'AND pas.id_product_attribute = ' . (int) $id_product_attribute
            );

            if (false === $id_product_attribute_exists && isset($id_product_attribute) && $id_product_attribute !== 0) {
                return array(
                    'status' => false,
                    'message' => 'PRODUCT_ATTRIBUTE_NOT_FOUND'
                );
            }

            $wishlist = new WishList($idWishList);

            if ($wishlist->id_customer !== $context->customer->id) {
                return array(
                    'status' => false,
                    'message' => 'WISHLIST_ACCESS_UNAUTHORIZED'
                );
            }

            $productIsAdded = $wishlist->addProduct(
                $idWishList,
                $context->customer->id,
                $id_product,
                $id_product_attribute,
                $quantity
            );

            $newStat = new Statistics();
            $newStat->id_product = $id_product;
            $newStat->id_product_attribute = $id_product_attribute;
            $newStat->id_shop = $context->shop->id;
            $newStat->save();

            if (false === $productIsAdded) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            return array(
                'status' => true
            );
        }

        public function removeProductFromWishlist($id_product, $id_product_attribute, $context)
        {
            $idWishList = WishList::getDefault($context->customer->id);

            $wishlist = new WishList($idWishList);

            if ($wishlist->id_customer !== $context->customer->id) {
                return array(
                    'status' => false,
                    'message' => 'WISHLIST_ACCESS_UNAUTHORIZED'
                );
            }

            if (!$id_product_attribute) {
                $id_product_attribute = 0;
            }

            $isDeleted = WishList::removeProduct(
                $idWishList,
                $context->customer->id,
                $id_product,
                $id_product_attribute
            );

            if (false === $isDeleted) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            return array(
                'status' => true
            );
        }

        public function getDefaultWishlist($populate, $context)
        {
            $idWishList = WishList::getDefault($context->customer->id);
            $wishlist = new WishList($idWishList);

            if (false === Validate::isLoadedObject($wishlist)) {
                return array(
                    'status' => false,
                    'message' => 'WISHLIST_NOT_FOUND'
                );
            }

            if ($wishlist->id_customer !== $context->customer->id) {
                return array(
                    'status' => false,
                    'message' => 'WISHLIST_ACCESS_UNAUTHORIZED'
                );
            }

            $wishlistProducts = Db::getInstance()->executeS('
                SELECT `id_product`, `id_product_attribute`, `quantity`
                FROM `' . _DB_PREFIX_ . 'wishlist_product`
                WHERE `id_wishlist` = ' . (int) $wishlist->id . '
                And quantity > 0'
            );

            foreach ($wishlistProducts as $key => $value) {
                $product = PrestAppProductController::getProduct((int) $wishlistProducts[$key]['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    $wishlistProducts[$key]['data'] = $product;
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'products' => $wishlistProducts
                )
            );
        }
    }
}
