<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('ets_marketplace') && Module::isEnabled('ets_marketplace')) {
    
    class PrestAppEtsMarketPlaceCustomizationController
    {
        protected $ets_marketplace;

        public function __construct()
        {
            $this->ets_marketplace = Module::getInstanceByName('ets_marketplace');
        }

        public function test($context)
        {
            return 'lalaal';
        }

        // Publish a review for the seller following an order
        public function addSellerReview($title, $content, $grade, $id_order, $context)
        {
            if (!Validate::isLoadedObject($context->customer)) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            $order = new Order($id_order);

            if (!$title || !$content || !$grade || !$id_order)
            {
                return array(
                    'status' => false,
                    'message' => 'MISSING_PARAMETERS'
                );
            }

            if (!$order || $order->id_customer != $context->customer->id)
            {
                return array(
                    'status' => false,
                    'message' => 'ORDER_DO_NOT_BELONG'
                );
            }

            if (!class_exists('Ets_mp_review'))
            {
                return array(
                    'status' => false
                );
            }

            $review_id = Ets_mp_review::getSellerOrderReviewIdByIdOrder($order->id);

            if ($review_id)
            {
                return array(
                    'status' => false,
                    'message' => 'REVIEW_EXISTS'
                );
            }

            if (Ets_mp_review::addSellerOrderReview($order->id, $title, $content, $grade)) {
                return array(
                    'status' => true
                );
            }

            return array(
                'status' => false
            );
        }

        // Get seller reviews
        public function getSellerReviews($id_seller, $page, $limit, $context)
        {
            $seller = new Ets_mp_seller($id_seller, $context->language->id);

            if (!Validate::isLoadedObject($seller)) {
                return array(
                    'status' => false,
                    'message' => 'SELLER_NOT_FOUND'
                );
            }

            if (!class_exists('Ets_mp_review'))
            {
                return array(
                    'status' => false
                );
            }

            $all_reviews = Ets_mp_review::getSellerReviews($seller->id_customer);

            $reviews = array();

            for($i = 0; $i < count($all_reviews); $i++) {
                if ($i >= (($page * $limit) - $limit) && ($i < ($page * $limit))) {
                    $order = new Order($all_reviews[$i]['id_order']);
                    $customer = new Customer($order->id_customer);
                    $all_reviews[$i]['customer_name'] = Tools::ucfirst($customer->firstname) . ' ' . Tools::ucfirst(Tools::substr($customer->lastname, 0, 1)) . '.';
                    $reviews[] = $all_reviews[$i];
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'page' => $page,
                    'pages_count' => ceil(count($all_reviews) / $limit),
                    'reviews' => $reviews
                )
            );
        }

        public function followSeller($id_seller, $context)
        {

            if (!Validate::isLoadedObject($context->customer)) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            if(!Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller="'.(int)$id_seller.'" AND id_customer="'.(int)$context->customer->id.'"'))
            {
                Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'ets_mp_seller_customer_follow`(id_seller,id_customer) VALUES("'.(int)$id_seller.'","'.(int)$context->customer->id.'")');
            }

            $total_follow = Db::getInstance()->getValue('SELECT COUNT(id_customer) FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller='.(int)$id_seller);
        
            return array(
                'status' => true,
                'data' => array(
                    'total_follow' => $total_follow,
                    'follow' => true
                )
            );
            
        }

        public function unfollowSeller($id_seller, $context)
        {

            if (!Validate::isLoadedObject($context->customer)) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller="'.(int)$id_seller.'" AND id_customer="'.(int)$context->customer->id.'"');
            
            $total_follow = Db::getInstance()->getValue('SELECT COUNT(id_customer) FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller='.(int)$id_seller);
        
            return array(
                'status' => true,
                'data' => array(
                    'total_follow' => $total_follow,
                    'follow' => false
                )
            );
            
        }

        public function getSeller($id_seller, $context)
        {

            $seller = new Ets_mp_seller($id_seller, $context->language->id);

            $data = [];

            if (Validate::isLoadedObject($seller) && $seller->active == 1)
            {
                $reviews = $seller->getAVGReviewProduct();
                $total_reviews = isset($reviews['avg_grade']) ? $reviews['avg_grade'] : 0;
                $count_reviews = isset($reviews['count_grade']) ? $reviews['count_grade'] : 0;

                $total_messages = $this->ets_marketplace->_getOrderMessages('', null, null, null, true, $seller->id);

                if ($total_messages)
                {
                    $total_messages_reply = Db::getInstance()->getValue('SELECT COUNT(DISTINCT cm.id_customer_thread) FROM `'._DB_PREFIX_.'customer_message` cm INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_customer_message` scm ON (scm.id_customer_message=cm.id_customer_message AND scm.id_customer="'.(int)$seller->id_customer.'")') + Db::getInstance()->getValue('SELECT COUNT(DISTINCT id_contact) FROM `'._DB_PREFIX_.'ets_mp_seller_contact_message` WHERE id_seller='.(int)$seller->id);
                    $response_rate = Tools::ps_round($total_messages_reply*  100 / $total_messages, 2);
                }

                if (Configuration::get('ETS_MP_DISPLAY_PRODUCT_SOLD'))
                {
                    $total_product_sold = $seller->_getTotalNumberOfProductSold();
                }

                $is_follow = false;

                if($context->customer->logged)
                {
                    if (Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller="'.(int)$seller->id.'" AND id_customer="'.(int)$context->customer->id.'"'))
                    {
                        $is_follow = true;
                    }
                }

                $data = array(
                    'seller' => $seller,
                    'total_all_products' => $seller->getProducts(false,0,0,false,true,true),
                    'total_new_products' => $seller->getNewProducts(false,0,0,false,true),
                    'total_best_seller_products' => $seller->getBestSellerProducts(false,false,false,false,true),
                    'total_special_products' => $seller->getSpecialProducts(false,false,false,false,true),
                    'total_reviews' => Tools::ps_round($total_reviews,1),
                    'total_follow' => Db::getInstance()->getValue('SELECT COUNT(id_customer) FROM `'._DB_PREFIX_.'ets_mp_seller_customer_follow` WHERE id_seller='.(int)$seller->id),
                    'total_reviews_int' => (int)$total_reviews,
                    'count_reviews' => $count_reviews,
                    'total_products' => (int)$seller->getProducts(false,false,false,false,true,true,false),
                    'total_product_sold' => isset($total_product_sold) ? $total_product_sold: false,
                    'response_rate' => isset($response_rate) ? $response_rate :false,
                    'follow' => $is_follow,
                    'reported' => $context->customer->logged ? Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_mp_seller_report` WHERE id_customer="'.(int)$context->customer->id.'" AND id_seller="'.(int)$seller->id.'" AND id_product=0'):false
                );

                return array(
                    'status' => true,
                    'data' => $data
                );
            } else {
                return array(
                    'status' => false,
                    'message' => 'SELLER_NOT_FOUND'
                );
            }
        }

        public function getSellerProducts($id_seller, $tab, $page, $limit, $populate, $context)
        {

            $order_by = 'p.id_product desc';

            $seller = new Ets_mp_seller($id_seller, $context->language->id);

            $products_base = array();
            $products = array();

            if($seller && Validate::isLoadedObject($seller))
            {
                $current_tab = $tab;

                switch ($current_tab) {
                    case 'all':
                        $products_base =  $seller->getProducts('', $page, $limit, $order_by, false, true, false);
                    case 'new_product':
                        $products_base =  $seller->getNewProducts('',$page,$limit,$order_by,false,false);
                    case 'best_seller':
                        $products_base =  $seller->getBestSellerProducts('',$page,$limit,$order_by,false,false);
                    case 'special':
                        $products_base =  $seller->getSpecialProducts('',$page,$limit,$order_by,false,false);
                    default:
                        $products_base =  $seller->getProducts('',$page,$limit,$order_by,false,false);
                } 
            }

            foreach ($products_base as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($products, $product);
                }
            }

            return array(
                'status' => true,
                'data' => $products
            );
        }

        public function reportSeller($id_seller, $title, $content, $context)
        {
            if (!$title) {
                return array(
                    'status' => false,
                    'message' => 'MISSING_TITLE'
                );
            } else if (!Validate::isCleanHtml($title)) {
                return array(
                    'status' => false,
                    'message' => 'INVALID_TITLE'
                );
            } else if (Tools::strlen($title) > 100) {
                return array(
                    'status' => false,
                    'message' => 'TOO_LONG_TITLE'
                );
            } else if (!$content) {
                return array(
                    'status' => false,
                    'message' => 'MISSING_CONTENT'
                );
            } else if (!Validate::isCleanHtml($content)) {
                return array(
                    'status' => false,
                    'message' => 'INVALID_CONTENT'
                );
            } else if (Tools::strlen($content) > 300) {
                return array(
                    'status' => false,
                    'message' => 'TOO_LONG_CONTENT'
                );
            } else if (!$id_seller) {
                return array(
                    'status' => false,
                    'message' => 'SELLER_NOT_FOUND'
                );
            }
            
            $seller = new Ets_mp_seller($id_seller);
            
            if (!Validate::isLoadedObject($seller)) {
                return array(
                    'status' => false,
                    'message' => 'SELLER_NOT_FOUND'
                );
            }

            $report = new Ets_mp_report();

            $report->id_seller = (int) $id_seller;
            $report->id_customer = $context->customer->id;
            $report->title = $title;
            $report->content = $content;

            if($report->add()) {
                return array(
                    'status' => true
                );
            } else {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }
        }

        public function getProductRelatedData($context, $id_product) 
        {

            // related shop products

            $temp_populate = array('description', 'price', 'images');

            $data = array();

            $product = new Product($id_product, true, (int) $context->language->id, $context->shop->id, $context);

            if (Validate::isLoadedObject($product)) {
                if ($id_customer = (int)Db::getInstance()->getValue('SELECT id_customer FROM `'._DB_PREFIX_.'ets_mp_seller_product` WHERE id_product='.(int)$id_product))
                {
                    $seller = Ets_mp_seller::_getSellerByIdCustomer($id_customer, $context->language->id);

                    if (Configuration::get('ETS_MP_DISPLAY_OTHER_PRODUCT'))
                    {
                        $formatted_products = array();
                        $other_products = $seller->getProductOther($product);
                        foreach ($other_products as $other_product) {
                            $formatted_product = PrestAppProductController::getProduct((int) $other_product['id_product'], json_encode($temp_populate), $context);
                            $formatted_products[] = $formatted_product;
                        }

                        $data['OTHER_PRODUCT'] = $formatted_products;
                    }
                    if (Configuration::get('ETS_MP_DISPLAY_OTHER_SELLER_PRODUCT'))
                    {
                        $formatted_other_seller_products = array();
                        $other_seller_products = $seller->getProductSellerOther($product);
                        foreach ($other_seller_products as $product) {
                            $formatted_other_seller_product = PrestAppProductController::getProduct((int) $product['id_product'], json_encode($temp_populate), $context);
                            $formatted_other_seller_products[] = $formatted_other_seller_product;
                        }

                        $data['OTHER_SELLER_PRODUCT'] = $formatted_other_seller_products;
                    }

                    $reviews = $seller->getAVGReviewProduct();
                    $total_reviews = isset($reviews['avg_grade']) ? $reviews['avg_grade'] : 0;
                    $count_reviews = isset($reviews['count_grade']) ? $reviews['count_grade'] : 0;

                    $seller->total_reviews = $total_reviews;
                    $seller->count_reviews = $count_reviews;

                    $data['SELLER'] = $seller;
                }
            }

            return $data;
        }

        public function getOrderRelatedData($context, $order) {

            $data = array();

            $sql ='SELECT sor.id FROM `'._DB_PREFIX_.'ets_mp_seller_order_review` sor
            WHERE sor.id_order='.(int) $order->id;

            $id_review = (int)Db::getInstance()->getValue($sql);

            if (!$id_review && $order->current_state == 4) {
                $data['show_review'] = true;
            } else {
                $data['show_review'] = false;
            }

            $data['seller'] = self::getSellerByOrderId($order->id, $context);

            return $data;
        }

        public function getSellerMessagesNotifications($context)
        {
            $customer_id = $context->customer->id ? $context->customer->id : 0;

            $sql = 'SELECT cm.customer_read
                FROM `'._DB_PREFIX_.'ets_mp_seller_contact` contact
                INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_contact_message` cm ON (contact.id_contact=cm.id_contact)
                WHERE cm.customer_read = 0 AND contact.id_customer = ' . $customer_id . '
            ';
            
            $result = count(Db::getInstance()->executeS($sql));

            return array(
                'status' => true,
                'data' => array(
                    'messages_count' => $result
                )
            );
        }

        public function getSellerMessagesList($context)
        {
            $sort = 'date_add desc';

            $sql = 'SELECT o.reference, contact.id_order,contact.id_contact,cm.message,cm.id_employee,cm.id_seller,cm.date_add,cm.customer_read,CONCAT(customer.firstname," ",customer.lastname) as seller_name,if(contact.id_customer!=0,CONCAT(c.firstname," ",c.lastname),contact.name) as customer_name, CONCAT(e.firstname," ",e.lastname) as employee_name
            FROM `'._DB_PREFIX_.'ets_mp_seller_contact` contact
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_contact_message` cm ON (contact.id_contact=cm.id_contact)
            INNER JOIN (SELECT id_contact,max(id_message) as id_message_max FROM `'._DB_PREFIX_.'ets_mp_seller_contact_message` GROUP BY id_contact) cmmax ON (cmmax.id_message_max = cm.id_message AND cmmax.id_contact= contact.id_contact)
            LEFT JOIN `'._DB_PREFIX_.'customer` c ON (c.id_customer= contact.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_seller=cm.id_seller)
            LEFT JOIN `'._DB_PREFIX_.'customer` customer ON (customer.id_customer=seller.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'employee` e ON (e.id_employee=cm.id_employee)
            LEFT JOIN `'._DB_PREFIX_.'orders` o ON (o.id_order=contact.id_order)
            WHERE contact.id_customer="'.(int)$context->customer->id.'"';

            $sql .= ($sort ? ' ORDER BY '. $sort : '');

            $messages = Db::getInstance()->executeS($sql);

            if($messages)
            {
                foreach($messages as &$message)
                {
                    $message['read'] = (int) $message['customer_read']; 

                    if($message['id_employee'])
                    {
                        if($message['seller_name']) {
                            $message['author'] = $message['seller_name'].' ('.$this->ets_marketplace->l('Seller','contactseller').')';
                        }
                        else {
                            $message['author'] = $message['employee_name'].' ('.$this->ets_marketplace->l('Admin','contactseller').')';
                        }
                    } else {
                        $message['author'] = $message['customer_name'];
                    }
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'messages' => $messages
                )
            );
        }

        public function getSellerOrders($id_seller, $context)
        {

            $seller = new Ets_mp_seller($id_seller, $context->language->id);

            if ($context->customer && $context->customer->id) {
                $sql = 'SELECT o.reference FROM `'._DB_PREFIX_.'orders` o 
                INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_order` seller_order ON (seller_order.id_order=o.id_order)
                WHERE o.id_customer="'.(int)$context->customer->id.'" AND seller_order.id_customer="'.(int)$seller->id_customer.'"';

                $order_references = Db::getInstance()->executeS($sql);

                return array(
                    'status' => true,
                    'data' => array(
                        'order_references' => $order_references
                    )
                );
            } else {
                return array(
                    'status' => true,
                    'data' => array(
                        'order_references' => array()
                    )
                );
            }
        }

        public function sendSellerMessage($id_contact, $id_seller, $id_product, $email, $reference, $message, $context)
        {
            if ($id_contact && Validate::isUnsignedId($id_contact)) {
                $contact = new Ets_mp_contact($id_contact);
                if(!Validate::isLoadedObject($contact) || $contact->id_customer != $context->customer->id) {
                    return array(
                        'status' => false,
                        'message' => 'DISCUSSION_DONT_BELONG_USER'
                    );
                }
                if (!$message) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_MISSING'
                    );
                }
                if (!Validate::isCleanHtml($message)) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_ERROR'
                    );
                }
                if (Tools::strlen($message) > 300) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_TOO_LONG'
                    );
                }

                $contact_message = new Ets_mp_contact_message();
                $contact_message->id_customer = (int) $context->customer->id;
                $contact_message->message = $message;
                $contact_message->id_contact = (int) $id_contact;
                $contact_message->customer_read = 1;

                if ($contact_message->add())
                {
                    return array(
                        'status' => true,
                        'data' => array(
                            'message' => $contact_message
                        )
                    );
                } else {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            }

            if ($id_seller) {

                if (!$message) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_MISSING'
                    );
                }
                if (!Validate::isCleanHtml($message)) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_ERROR'
                    );
                }
                if (Tools::strlen($message) > 300) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_TOO_LONG'
                    );
                }

                $seller = new Ets_mp_seller($id_seller, $context->language->id);

                if (!Validate::isLoadedObject($seller)) {
                    return array(
                        'status' => false,
                        'message' => 'SELLER_NOT_FOUND'
                    );
                }

                if ($reference && $context->customer->logged) {
                    $sql = 'SELECT o.id_order FROM `'._DB_PREFIX_.'orders` o 
                        INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_order` seller_order ON (seller_order.id_order=o.id_order)
                        WHERE o.id_customer="'.(int)$context->customer->id.'" AND seller_order.id_customer="'.(int)$seller->id_customer.'" AND o.reference = "'.pSQL($reference).'"
                    ';

                    $id_order = (int)Db::getInstance()->getValue($sql);
                }

                $contact = new Ets_mp_contact();
                $contact->id_customer = (int) $context->customer->id;
                $contact->id_seller = (int) $id_seller;
                if ($id_product) {
                    $contact->id_product = (int) $id_product;
                }
                if ($id_order) {
                    $contact->id_order = (int) $id_order;
                }

                $contact->email = $email;

                if($contact->add()) {
                    $id_contact = $contact->id;

                    $contact_message = new Ets_mp_contact_message();
                    $contact_message->id_contact = (int) $id_contact;
                    $contact_message->id_customer = 0;
                    $contact_message->id_seller = 0;
                    $contact_message->id_employee = 0;
                    $contact_message->title = $context->customer->firstname . ' ' . $context->customer->lastname;
                    $contact_message->message = $message;
                    $contact_message->customer_read = 1;

                    if ($contact_message->add())
                    {
                        return array(
                            'status' => true,
                            'data' => array(
                                'message' => $contact_message
                            )
                        );
                    } else {
                        return array(
                            'status' => false,
                            'message' => 'ERROR'
                        );
                    }
                } else {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            }

            return array(
                'status' => false,
                'message' => 'ERROR'
            );
        }

        public function deleteSellerMessage($id_contact, $context)
        {
            if(Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_mp_seller_contact` WHERE id_contact="'.(int)$id_contact.'" AND id_customer="'.(int)$context->customer->id.'"'))
            {
                $contact = new Ets_mp_contact($id_contact);
                if($contact->delete())  {
                    return array(
                        'status' => true
                    );
                } else {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            } else {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }
        }

        public function getSellerMessage($id_contact, $context)
        {

            $data = array();

            if (!Validate::isUnsignedId($id_contact)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            $contact = new Ets_mp_contact($id_contact);

            if (!$contact) {
                return array(
                    'status' => false,
                    'message' => 'CONTACT_NOT_FOUND'
                );
            }

            if(!Validate::isLoadedObject($contact) || $contact->id_customer != $context->customer->id) {
                return array(
                    'status' => false,
                    'message' => 'CONTACT_NOT_FOUND'
                );
            }

            $messages = $contact->getMessages();

            $seller = new Ets_mp_seller($contact->id_seller, $context->language->id);
            $contact->seller = $seller;

            Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'ets_mp_seller_contact_message` SET customer_read=1 WHERE id_contact="'.(int)$contact->id.'"');
        
            if($contact->id_product) {
                $id_image = Db::getInstance()->getValue('SELECT id_image FROM `'._DB_PREFIX_.'image` WHERE id_product='.(int) $contact->id_product.' AND cover=1');
                if(!$id_image) {
                    $id_image = Db::getInstance()->getValue('SELECT id_image FROM `'._DB_PREFIX_.'image` WHERE id_product='.(int) $contact->id_product);
                }

                $product = new Product($contact->id_product, false ,$context->language->id);
                
                if($this->ets_marketplace->is17) {
                    $type_image = ImageType::getFormattedName('small');
                } else {
                    $type_image = ImageType::getFormatedName('small');
                }

                $data['product'] = $product;
                $data['link_image'] = $id_image ? $context->link->getImageLink($product->link_rewrite, $id_image, $type_image) : '';
            }
        
            if($contact->id_order)
            {
                $data['order_message'] = new Order($contact->id_order);
            }

            $data['contact'] = $contact;
            $data['messages'] = $messages;

            return array(
                'status' => true,
                'data' => $data
            );
        }

        public function getTopSellers($limit, $context)
        {

            $sellers = Ets_mp_seller::_getSellers(' AND seller_sale.total_sale >0','seller_sale.total_sale DESC', 0, $limit);

            return array(
                'status' => true,
                'data' => array(
                    'sellers' => $sellers
                )
            );
        }

        public function getFollowedSellers($limit, $context)
        {

            if (!$context->customer->logged) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            $sellers = Db::getInstance()->executeS('SELECT seller.id_customer FROM 
            `'._DB_PREFIX_.'ets_mp_seller_customer_follow` scl
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_seller=scl.id_seller AND seller.active=1)
            WHERE scl.id_customer="'.(int)$context->customer->id.'"');

            $id_sellers = array();
            foreach($sellers as $seller)
            {
                $id_sellers[] = $seller['id_customer'];
            }

            $sql = 'SELECT s.*,CONCAT(c.firstname," ", c.lastname) as customer_name,sl.shop_name,sl.shop_address,sl.shop_description,top_order_seller.total_order,seller_product.total_product 
            FROM `'._DB_PREFIX_.'ets_mp_seller` s
                LEFT JOIN (
                    SELECT seller.id_seller as id_seller,COUNT(DISTINCT seller_order.id_order) as total_order
                    FROM `'._DB_PREFIX_.'ets_mp_seller_order` seller_order
                    INNER JOIN `'._DB_PREFIX_.'orders` o ON(o.id_order=seller_order.id_order)
                    INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller_order.id_customer=seller.id_customer)
                    WHERE o.id_shop="'.(int)$context->shop->id.'" GROUP BY seller.id_seller
                ) as top_order_seller ON (top_order_seller.id_seller=s.id_seller)
                LEFT JOIN (
                    SELECT seller.id_seller,count(sp.id_product) as total_product FROM `'._DB_PREFIX_.'ets_mp_seller_product` sp
                    INNER JOIN `'._DB_PREFIX_.'product` p ON (sp.id_product= p.id_product AND p.active=1)
                    INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON(seller.id_customer=sp.id_customer)
                    GROUP BY seller.id_seller
                ) as seller_product ON (seller_product.id_seller=s.id_seller)
                LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller_lang` sl ON (s.id_seller = sl.id_seller AND sl.id_lang ="'.(int)$context->language->id.'")
                LEFT JOIN `'._DB_PREFIX_.'customer` c ON (s.id_customer=c.id_customer)
                WHERE s.id_shop="'.(int)$context->shop->id.'" AND s.active=1 AND s.id_customer IN ('.implode(',',array_map('intval',$id_sellers)).') ORDER BY top_order_seller.total_order DESC LIMIT 0,'.(int)$limit;
            
            $sellers = Db::getInstance()->executeS($sql);

            return array(
                'status' => true,
                'data' => array(
                    'sellers' => $sellers
                )
            );
        }

        public function getFollowedSellersProducts($limit, $populate, $context)
        {

            if (!$context->customer->logged) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            $sellers = Db::getInstance()->executeS('SELECT seller.id_customer FROM 
            `'._DB_PREFIX_.'ets_mp_seller_customer_follow` scl
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_seller=scl.id_seller AND seller.active=1)
            WHERE scl.id_customer="'.(int)$context->customer->id.'"');

            $id_sellers = array();
            foreach($sellers as $seller)
            {
                $id_sellers[] = $seller['id_customer'];
            }

            $front = true;
            $nb_days_new_product = Configuration::get('PS_NB_DAYS_NEW_PRODUCT');
            $id_lang = (int) $context->language->id;
            if (!Validate::isUnsignedInt($nb_days_new_product)) {
                $nb_days_new_product = 20;
            }
            $prev_version = version_compare(_PS_VERSION_, '1.6.1.0', '<');

            $sql ='SELECT DISTINCT p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) AS stock_quantity' . ($prev_version? ' ,IFNULL(product_attribute_shop.id_product_attribute, 0)':' ,MAX(product_attribute_shop.id_product_attribute)') . ' id_product_attribute, pl.`description`, pl.`description_short`, pl.`available_now`,
                    pl.`available_later`, pl.`link_rewrite`, pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`, pl.`name`, i.`id_image`,
                    il.`legend` as legend, m.`name` AS manufacturer_name,cl.name as default_category,
                    DATEDIFF(product_shop.`date_add`, DATE_SUB("' . date('Y-m-d') . ' 00:00:00",
                    INTERVAL ' . (int)$nb_days_new_product . ' DAY)) > 0 AS new, product_shop.price AS orderprice,sp.approved';
            $sql .= ' FROM `'._DB_PREFIX_.'product` p
                    '.Shop::addSqlAssociation('product', 'p').
                    (!$prev_version?
                        'LEFT JOIN `'._DB_PREFIX_.'product_attribute` pa ON (pa.id_product = p.id_product)'.Shop::addSqlAssociation('product_attribute', 'pa', false, 'product_attribute_shop.default_on=1').'':
                        'LEFT JOIN `'._DB_PREFIX_.'product_attribute_shop` product_attribute_shop ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop='.(int)$context->shop->id.')'
                    )
                    .Product::sqlStock('p', 0, false, $context->shop).'
                    LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller_product` sp ON (sp.id_product=p.id_product)
                    LEFT JOIN `'._DB_PREFIX_.'category` c ON (c.id_category=p.id_category_default)
                    LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (cp.id_product=p.id_product)
                    LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (c.id_category = cl.id_category AND cl.id_lang="'.(int)$id_lang.'")
                    LEFT JOIN `' . _DB_PREFIX_ . 'product_lang` pl ON (p.`id_product` = pl.`id_product` AND pl.`id_lang` = ' . (int)$id_lang . Shop::addSqlRestrictionOnLang('pl') . ')'.'
                    LEFT JOIN `' . _DB_PREFIX_ . 'image` i ON (i.`id_product` = p.`id_product` AND i.cover=1)
                    LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (i.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$id_lang.')	
                    LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer` m ON m.`id_manufacturer` = p.`id_manufacturer`
                    WHERE product_shop.active=1 AND sp.id_customer IN ('.implode(',',array_map('intval',$id_sellers)).') AND product_shop.`id_shop` = ' . (int) $context->shop->id.($front ? ' AND product_shop.`visibility` IN ("both", "catalog")' : ''); 
            $sql .= ' GROUP BY p.id_product ORDER BY RAND() LIMIT 0,'.(int)$limit;

            $products_base = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql, true, true);

            $products = array();

            foreach ($products_base as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($products, $product);
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'products' => $products
                )
            );
        }

        public function getPopularProducts($limit, $populate, $context)
        {
            $products_base = Ets_mp_algorithm::getPopularProducts($limit);

            $products = array();

            foreach ($products_base as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($products, $product);
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'products' => $products
                )
            );
        }

        public function getPopularCategories($limit, $limit_categories, $populate, $context)
        {
            $categories_base = Ets_mp_algorithm::getPopularCategories($limit, $limit_categories);

            foreach ($categories_base as &$category) {

                $products = array();

                foreach ($category['products'] as $product) {
                    $product = PrestAppProductController::getProduct((int) $product['id'], $populate, $context);
                    if ($product && $product != 404 && $product != 403) {
                        array_push($products, $product);
                    }
                }

                $category['products'] = $products;
            }

            return array(
                'status' => true,
                'data' => array(
                    'categories' => $categories_base
                )
            );
        }

        //////////////////////////////////////////////////////////////////////
        /////////////////////////// ADMIN ////////////////////////////////////
        //////////////////////////////////////////////////////////////////////

        public function getAdminMessagesList($context)
        {
            if(!Configuration::get('ETS_MP_ENABLE_CONTACT_SHOP')) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            $seller = $this->ets_marketplace->_getSeller();

            if (!$seller) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            $sort = 'date_add desc';

            $sql1 ='SELECT o.id_order,"" as id_contact,"" as title_contact,o.reference,cm.read,cm.message,cm.id_employee,"0" as id_seller,cm.date_add,CONCAT(manager.firstname," ",manager.lastname) as manager_name,CONCAT(customer.firstname," ",customer.lastname) as seller_name,CONCAT(c.firstname," ",c.lastname) as customer_name,CONCAT(e.firstname," ",e.lastname) as employee_name,
            cm_min.id_employee as id_employee_min,CONCAT(manager_min.firstname," ",manager_min.lastname) as manager_min_name,CONCAT(customer_min.firstname," ",customer_min.lastname) as seller_min_name,CONCAT(e_min.firstname," ",e_min.lastname) as employee_min_name
            FROM `'._DB_PREFIX_.'customer_thread` ct
            INNER JOIN `'._DB_PREFIX_.'customer_message` cm ON (cm.id_customer_thread=ct.id_customer_thread)
            INNER JOIN (SELECT id_customer_thread,max(id_customer_message) as id_customer_message_max FROM `'._DB_PREFIX_.'customer_message` group by id_customer_thread) last_message ON (last_message.id_customer_message_max=cm.id_customer_message AND last_message.id_customer_thread=ct.id_customer_thread)
            INNER JOIN `'._DB_PREFIX_.'orders` o ON (ct.id_order=o.id_order)
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_order` so ON (o.id_order=so.id_order)  
            LEFT JOIN `'._DB_PREFIX_.'customer` c ON (c.id_customer=ct.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'employee` e ON (e.id_employee=cm.id_employee)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller_customer_message` scm ON (scm.id_customer_message=cm.id_customer_message)
            LEFT JOIN `'._DB_PREFIX_.'customer` manager ON (manager.id_customer=scm.id_manager)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_customer=scm.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'customer` customer ON (customer.id_customer=seller.id_customer)
            LEFT JOIN (
                SELECT id_customer_thread,MIN(id_customer_message) as id_customer_message_min FROM `'._DB_PREFIX_.'customer_message` group by id_customer_thread
            ) first_message ON (first_message.id_customer_thread=ct.id_customer_thread)
            LEFT JOIN `'._DB_PREFIX_.'customer_message` cm_min ON (first_message.id_customer_message_min=cm_min.id_customer_message)
            LEFT JOIN `'._DB_PREFIX_.'employee` e_min ON (e_min.id_employee=cm_min.id_employee)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller_customer_message` scm_min ON (scm_min.id_customer_message=cm_min.id_customer_message)
            LEFT JOIN `'._DB_PREFIX_.'customer` manager_min ON (manager_min.id_customer=scm_min.id_manager)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller` seller_min ON (seller_min.id_customer=scm_min.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'customer` customer_min ON (customer_min.id_customer=seller_min.id_customer)   
            WHERE so.id_customer="'.(int)$seller->id_customer.'" GROUP BY o.id_order';

            $sql2 = 'SELECT contact.id_order,contact.id_contact,cm.title as title_contact,o.reference,cm.read,cm.message,cm.id_employee,cm.id_seller,cm.date_add,CONCAT(manager.firstname," ",manager.lastname) as manager_name,CONCAT(customer.firstname," ",customer.lastname) as seller_name,if(contact.id_customer!=0,CONCAT(c.firstname," ",c.lastname),contact.name) as customer_name, CONCAT(e.firstname," ",e.lastname) as employee_name,
            "" as id_employee_min,"" as manager_min_name,"" as seller_min_name,"" as employee_min_name
            FROM `'._DB_PREFIX_.'ets_mp_seller_contact` contact
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller_contact_message` cm ON (contact.id_contact=cm.id_contact)
            INNER JOIN (SELECT id_contact,max(id_message) as id_message_max FROM `'._DB_PREFIX_.'ets_mp_seller_contact_message` GROUP BY id_contact) cmmax ON (cmmax.id_message_max = cm.id_message AND cmmax.id_contact= contact.id_contact)
            LEFT JOIN `'._DB_PREFIX_.'customer` c ON (c.id_customer= contact.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_seller=cm.id_seller)
            LEFT JOIN `'._DB_PREFIX_.'customer` customer ON (customer.id_customer=seller.id_customer)
            LEFT JOIN `'._DB_PREFIX_.'customer` manager ON (manager.id_customer=cm.id_manager)
            LEFT JOIN `'._DB_PREFIX_.'employee` e ON (e.id_employee=cm.id_employee)
            LEFT JOIN `'._DB_PREFIX_.'orders` o ON (o.id_order=contact.id_order)
            WHERE contact.id_seller="'.(int)$seller->id.'"';

            $sql = "SELECT * FROM (($sql1) UNION ALL ($sql2)) as tb WHERE 1";

            $sql .= ($sort ? ' ORDER BY '. $sort : '');

            $messages = Db::getInstance()->executeS($sql);

            if ($messages)
            {
                foreach ($messages as &$message)
                {
                    if ($message['id_employee_min'])
                    {
                        if ($message['manager_min_name']) {
                            $message['author'] = $message['manager_min_name'].' ('.$this->ets_marketplace->l('Seller manager','messages').')';
                        } else if ($message['seller_min_name']) {
                            $message['author'] = $message['seller_min_name'].' ('.$this->ets_marketplace->l('Seller','messages').')';
                        } else {
                            $message['author'] = $message['employee_min_name'].' ('.$this->ets_marketplace->l('Admin','messages').')';
                        }
                    } else {
                        $message['author'] = $message['customer_name'].' ('.$this->ets_marketplace->l('Customer','messages').')';; 
                    }

                    if ($message['id_employee'] || $message['id_seller'])
                    {
                        $message['read'] = 1; 
                        if ($message['manager_name']) {
                            $message['author_message'] = $message['manager_name'].' ('.$this->ets_marketplace->l('Seller manager','messages').')';
                        } else if ($message['seller_name']) {
                            $message['author_message'] = $message['seller_name'].' ('.$this->ets_marketplace->l('Seller','messages').')';
                        } else {
                            $message['author_message'] = $message['employee_name'].' ('.$this->ets_marketplace->l('Admin','messages').')';
                        }
                    } else {
                        $message['author_message'] = $message['customer_name']; 
                    }

                    if (!$message['read']) {
                        $message['read'] = 0;
                    }
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'messages' => $messages
                )
            );
        }

        public function sendAdminMessage($id_contact, $id_order, $id_customer, $message, $context)
        {
            $seller = $this->ets_marketplace->_getSeller(true);

            if ($id_order) {

                $order = new Order($id_order);

                if(!$seller->checkHasOrder($id_order) || !Validate::isLoadedObject($order)) {
                    return array(
                        'status' => false,
                        'message' => 'ORDER_NOT_FOUND'
                    );
                }

                $customer = new Customer($id_customer);

                if (!Validate::isLoadedObject($customer)) {
                    return array(
                        'status' => false,
                        'message' => 'CUSTOMER_NOT_FOUND'
                    );
                }

                if (!$message) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_MISSING'
                    );
                }
                if (!Validate::isCleanHtml($message)) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_ERROR'
                    );
                }
                if (Tools::strlen($message) > 1600) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_TOO_LONG'
                    );
                }

                $id_customer_thread = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($customer->email, $order->id);

                if (!$id_customer_thread) {
                    $customer_thread = new CustomerThread();
                    $customer_thread->id_contact = 0;
                    $customer_thread->id_customer = (int) $order->id_customer;
                    $customer_thread->id_shop = (int) $context->shop->id;
                    $customer_thread->id_order = (int) $order->id;
                    $customer_thread->id_lang = (int) $context->language->id;
                    $customer_thread->email = $customer->email;
                    $customer_thread->status = 'open';
                    $customer_thread->token = Tools::passwdGen(12);
                    $customer_thread->add();
                } else {
                    $customer_thread = new CustomerThread((int) $id_customer_thread);
                }

                $customer_message = new CustomerMessage();
                $customer_message->id_customer_thread = $customer_thread->id;
                $customer_message->id_employee = 1;
                $customer_message->message = $message;
                $customer_message->private = false;
                $add = true;

                if (!$customer_message->add()) {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }

                $message = $customer_message->message;

                if($add) {
                    Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'ets_mp_seller_customer_message`(id_customer,id_manager,id_customer_message) VALUES("'.(int)$seller->id_customer.'","'.($context->customer->id != $seller->id_customer ? $context->customer->id : 0).'","'.(int)$customer_message->id.'")');
                }

                return array(
                    'status' => true,
                    'data' => array(
                        'message' => $customer_message
                    )
                );
            }

            if ($id_contact) {
                $contact = new Ets_mp_contact($id_contact);

                if (!Validate::isLoadedObject($contact) || $contact->id_seller != $seller->id) {
                    return array(
                        'status' => false,
                        'message' => 'DISCUSSION_DONT_BELONG_USER'
                    );
                }
                if (!$message) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_MISSING'
                    );
                }

                if (!Validate::isCleanHtml($message)) {
                    return array(
                        'status' => false,
                        'message' => 'MESSAGE_ERROR'
                    );
                }

                $contact_message = new Ets_mp_contact_message();
                $contact_message->id_seller = (int) $seller->id;
                $contact_message->id_manager = $seller->id_customer != $context->customer->id ? (int) $context->customer->id : 0;
                $contact_message->message = $message;
                $contact_message->id_contact = $id_contact;
                $contact_message->read = 1;

                if ($contact_message->add())
                {
                    return array(
                        'status' => true,
                        'data' => array(
                            'message' => $contact_message
                        )
                    );
                } else {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            }

            return array(
                'status' => false,
                'message' => 'ERROR'
            );
        }

        public function getAdminMessage($id_contact, $id_order, $context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            if ($id_order) {
                if (!Validate::isUnsignedId($id_order)) {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }

                $order = new Order($id_order);

                if(!$seller->checkHasOrder($id_order) || !Validate::isLoadedObject($order)) {
                    return array(
                        'status' => false,
                        'message' => 'ORDER_NOT_FOUND'
                    );
                }

                $id_customer_thread = (int)Db::getInstance()->getValue('SELECT id_customer_thread FROM `'._DB_PREFIX_.'customer_thread` WHERE id_order="'.(int)$id_order.'" ');
                Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'customer_message` SET `read`=1 WHERE id_customer_thread ="'.(int)$id_customer_thread.'"');
                $messages = $this->ets_marketplace->getCustomerMessagesOrder($order->id_customer,$order->id,10);
                
                return array(
                    'status' => true,
                    'data' => array(
                        'order' => $order,
                        'messages' => $messages,
                        'ets_mp_customer' => new Customer($order->id_customer),
                        'customer_thread_message' => CustomerThread::getCustomerMessages($order->id_customer, null, $order->id),
                        'orderMessages' => OrderMessage::getOrderMessages($order->id_lang)
                    )
                );
            }

            if ($id_contact) {

                $data = array();

                if (!Validate::isUnsignedId($id_contact)) {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }

                $contact = new Ets_mp_contact($id_contact);

                if(!Validate::isLoadedObject($contact) || $contact->id_seller != $seller->id) {
                    return array(
                        'status' => false,
                        'message' => 'CONTACT_NOT_FOUND'
                    );
                }

                if($contact->id_customer)
                {
                    $customer = new Customer($contact->id_customer);
                    $contact->name = $customer->firstname.' '.$customer->lastname;
                    $contact->email = $customer->email;
                }

                Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'ets_mp_seller_contact_message` SET `read`=1 WHERE id_contact="'.(int) $id_contact.'"');
                $messages = $contact->getMessages();

                if ($contact->id_product) {
                    $id_image = Db::getInstance()->getValue('SELECT id_image FROM `'._DB_PREFIX_.'image` WHERE id_product='.(int)$contact->id_product.' AND cover=1');
                    if(!$id_image) {
                        $id_image = Db::getInstance()->getValue('SELECT id_image FROM `'._DB_PREFIX_.'image` WHERE id_product='.(int)$contact->id_product);
                    }
                    $product = new Product($contact->id_product, false, $context->language->id);
                    if($this->ets_marketplace->is17) {
                        $type_image = ImageType::getFormattedName('small');
                    } else {
                        $type_image = ImageType::getFormatedName('small');
                    }

                    $data['product'] = $product;
                    $data['link_image'] = $id_image ? $context->link->getImageLink($product->link_rewrite, $id_image, $type_image) : '';
                }

                if($contact->id_order) {
                    $data['order_message'] = new Order($contact->id_order);
                }

                $data['contact'] = $contact;
                $data['messages'] = $messages;

                return array(
                    'status' => true,
                    'data' => $data
                );
            }

            return array(
                'status' => false,
                'message' => 'ERROR'
            );
        }

        public function addProduct($context)
        {
            if (!$this->_checkValidateProduct()) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            $id_product = $this->_submitSaveProduct($context);

            if (!$id_product) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            if (!$this->_completeProduct($id_product, $context)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            if (!$this->_createCombination($id_product, $context)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            if (!$this->_createProductImage($id_product, $context)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            return array(
                'status' => true
            );
        }

        public function addManufacturer($context)
        {
            if (!$this->_submitSaveManufacturer($context)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            return array(
                'status' => true
            );
        }

        public function editManufacturer($context)
        {
            if (!$this->_submitSaveManufacturer($context)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            return array(
                'status' => true
            );
        }

        public function setVacationMode($context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            if(!Configuration::get('ETS_MP_VACATION_MODE_FOR_SELLER')) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }
            if(!$context->customer->logged || !$seller) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }
            if(!$this->ets_marketplace->_checkPermissionPage($seller)) {
                return array(
                    'status' => false,
                    'message' => 'UNAUTHORIZED'
                );
            }

            $valueFieldPost = array();
            $languages = Language::getLanguages(false);

            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            $vacation_mode = (int)Tools::getValue('vacation_mode');
            $vacation_type = Tools::getValue('vacation_type');
            $date_vacation_start = trim(Tools::getValue('date_vacation_start'));
            $date_vacation_end = trim(Tools::getValue('date_vacation_end'));
            if(!in_array($vacation_type,array('show_notifications','disable_product','disable_product_and_show_notifications','disable_shopping','disable_shopping_and_show_notifications'))) {
                return array(
                    'status' => false,
                    'message' => 'TYPE_NOT_VALID'
                );
            }
            $valueFieldPost['vacation_type'] = $vacation_type;
            $valueFieldPost['vacation_mode'] = $vacation_mode;

            if ($vacation_mode) {
                $vacation_notifications_default = Tools::getValue('vacation_notifications_'.$id_lang_default);
                $vacation_notifications = array();
                if(Tools::strpos($vacation_type,'show_notifications') !== false && !$vacation_notifications_default) {
                    return array(
                        'status' => false,
                        'message' => 'NOTIFICATIONS_REQUIRED'
                    );
                }
                foreach($languages as $language) {
                    if(($vacation_notifications[$language['id_lang']] = Tools::getValue('vacation_notifications_'.$language['id_lang'],$seller->vacation_notifications[$language['id_lang']])) && Tools::strpos($vacation_type,'show_notifications')!==false && !Validate::isCleanHtml($vacation_notifications[$language['id_lang']])) {
                        return array(
                            'status' => false,
                            'message' => 'ERROR'
                        );
                    }
                }
            }

            if($date_vacation_start && !Validate::isDate($date_vacation_start)) {
                return array(
                    'status' => false,
                    'message' => 'START_DATE_INVALID'
                );
            }
            if($date_vacation_end && !Validate::isDate($date_vacation_end)) {
                return array(
                    'status' => false,
                    'message' => 'END_DATE_INVALID'
                );
            }
            if($date_vacation_end && $date_vacation_start && strtotime($date_vacation_start) >= strtotime($date_vacation_end)) {
                return array(
                    'status' => false,
                    'message' => 'END_DATE_MUST_BE_SUPERIOR'
                );
            }
            foreach($languages as $language) {
                $valueFieldPost['vacation_notifications'][$language['id_lang']] = Tools::getValue('vacation_notifications_'.$language['id_lang'], $seller->vacation_notifications[$language['id_lang']]);
            }

            $seller->vacation_mode = (int) $vacation_mode;
            if($seller->vacation_mode)
            {
                $seller->vacation_type = $vacation_type;
                $seller->date_vacation_start = $date_vacation_start;
                $seller->date_vacation_end = $date_vacation_end;
                if(Tools::strpos($vacation_type,'show_notifications') !== false)
                {
                    foreach($languages as $language)
                    {
                        $seller->vacation_notifications[$language['id_lang']] = $vacation_notifications[$language['id_lang']] ? : $vacation_notifications_default;
                    }
                }
            }
            if($seller->update()) {
                return array(
                    'status' => true
                );
            } else {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }
        }

        public function editProfile($context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            $seller_fields = array(
                'seller_name' => $this->ets_marketplace->l('Seller name'),
                'seller_email' => $this->ets_marketplace->l('Seller email'),
                'shop_name' => $this->ets_marketplace->l('Shop name'),
                'shop_description' => $this->ets_marketplace->l('Shop description'),
                'shop_address' => $this->ets_marketplace->l('Shop address'),
                'vat_number' => $this->ets_marketplace->l('VAT number'),
                'shop_phone' => $this->ets_marketplace->l('Shop phone number'),
                'shop_logo' => $this->ets_marketplace->l('Shop logo'),
                'shop_banner' => $this->ets_marketplace->l('Shop banner'),
                'banner_url' => $this->ets_marketplace->l('Banner URL'),
                'link_facebook' => $this->ets_marketplace->l('Facebook link'),
                'link_google' => $this->ets_marketplace->l('Google link'),
                'link_instagram' => $this->ets_marketplace->l('Instagram link'),
                'link_twitter' => $this->ets_marketplace->l('Twitter link'),
                'latitude' => $this->ets_marketplace->l('Latitude'),
                'longitude' => $this->ets_marketplace->l('Longitude'),
                'id_shop_category' => $this->ets_marketplace->l('Shop category'),
            );

            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            $languages = Language::getLanguages(false);

            $shop_name_default = Tools::getValue('shop_name_'.$id_lang_default);
            if(!$shop_name_default) {
                return array(
                    'status' => false,
                    'message' => 'SHOP_NAME_REQUIRED'
                );
            }
            
            $shop_description_default = Tools::getValue('shop_description_'.$id_lang_default);
            if(!$shop_description_default) {
                return array(
                    'status' => false,
                    'message' => 'SHOP_DESCRIPTION_REQUIRED'
                );
            }
            $shop_address_default = Tools::getValue('shop_address_'.$id_lang_default);
            if(!$shop_address_default) {
                return array(
                    'status' => false,
                    'message' => 'SHOP_ADDRESS_REQUIRED'
                );
            }

            if(!($shop_phone = Tools::getValue('shop_phone'))) {
                return array(
                    'status' => false,
                    'message' => 'SHOP_PHONE_REQUIRED'
                );
            } else if ($shop_phone && !Validate::isPhoneNumber($shop_phone)) {
                return array(
                    'status' => false,
                    'message' => 'SHOP_PHONE_INVALID'
                );
            }

            foreach($seller_fields as $key=> $seller_field) {
                $value = Tools::getValue($key);
                if(in_array($key,array('shop_name','shop_description','shop_address','banner_url'))) {
                    foreach($languages as $language) {
                        if(($value= Tools::getValue($key.'_'.$language['id_lang'])) && ($key =='banner_url' ? !Ets_marketplace::isLink($value) : !Validate::isCleanHtml($value))) {
                            return array(
                                'status' => false,
                                'message' => 'SHOP_FIELD_INVALID'
                            );
                        }
                        $valueFieldPost[$key][$language['id_lang']] = Tools::getValue($key.'_'.$language['id_lang']);
                    }
                } else {
                    if(in_array($key,array('link_facebook','link_google','link_instagram','link_twitter'))) {
                        if($value && !Ets_marketplace::isLink($value)) {
                            return array(
                                'status' => false,
                                'message' => 'SHOP_FIELD_INVALID'
                            );
                        }
                    } else if (in_array($key,array('longitude','latitude'))) {
                        if($value && !Validate::isCoordinate($value)) {
                            return array(
                                'status' => false,
                                'message' => 'SHOP_FIELD_INVALID'
                            );
                        }
                    } else if ($key!='shop_logo' && $key!='shop_banner' && $key!='seller_email' && $key!='vat_number' && $value && !Validate::isCleanHtml($value)) {
                        return array(
                            'status' => false,
                            'message' => 'SHOP_FIELD_INVALID'
                        );
                    } else if ( $key=='seller_email' && $value && !Validate::isEmail($value)) {
                        return array(
                            'status' => false,
                            'message' => 'SHOP_FIELD_INVALID'
                        );
                    } else if ($key=='vat_number' && $value && !Validate::isGenericName($value)) {
                        return array(
                            'status' => false,
                            'message' => 'SHOP_FIELD_INVALID'
                        );
                    }

                    $valueFieldPost[$key] = $value;
                }
            }

            if(isset($seller) && $seller->id) {
                $seller = new Ets_mp_seller($seller->id);
            } else {
                $seller = new Ets_mp_seller();
                $seller->date_add = date('Y-m-d H:i:s');
                $seller->id_customer = $context->customer->id;
                $seller->id_shop = $context->shop->id;
                $seller->date_add = date('Y-m-d H:i:s');
                $seller->id_group = (int)Configuration::get('ETS_MP_SELLER_GROUP_DEFAULT');
                $seller->auto_enabled_product='default';

                if($seller->getFeeType()!='no_fee') {
                    $seller->active = -1;
                    $seller->payment_verify=-1;
                } else {
                    $seller->payment_verify =0;
                    if(Configuration::get('ETS_MP_ENABLED_IF_NO_FEE')) {
                        $seller->active = 1;
                    } else {
                        $seller->active =- 1;
                    }
                }
                $seller->shop_logo = Db::getInstance()->getValue('SELECT shop_logo FROM `'._DB_PREFIX_.'ets_mp_registration` WHERE id_customer='.(int)$context->customer->id);
                foreach($languages as $language) {
                    $seller->shop_banner[$language['id_lang']] = Db::getInstance()->getValue('SELECT shop_banner FROM `'._DB_PREFIX_.'ets_mp_registration` WHERE id_customer='.(int)$context->customer->id);
                }
            }

            $seller->date_upd = date('Y-m-d H:i:s');

            foreach(array_keys($seller_fields) as $field)
            {
                if(in_array($field,array('shop_name','shop_description','shop_address','banner_url'))) {
                    $field_value_default = Tools::getValue($field.'_'.$id_lang_default);
                    foreach($languages as $language)
                    {
                        $field_value = Tools::getValue($field.'_'.$language['id_lang']);
                        $seller->{$field}[$language['id_lang']] = $field_value ?: $field_value_default;
                    }
                } else {
                    $field_value = Tools::getValue($field);
                    if($field!='shop_logo' && $field!='shop_banner')
                    {
                        if(Tools::isSubmit($field)) {
                            $seller->{$field} = $field_value;
                        }
                    } else {
                        if($field=='shop_logo')
                        {
                            if(isset($_FILES['shop_logo']['name'])&& $_FILES['shop_logo']['name'])
                            {
                                $logo = $this->ets_marketplace->uploadFile('shop_logo',$errors);
                                if($logo)
                                {
                                    $logo_old = $seller->shop_logo;
                                    $seller->shop_logo = $logo;
                                }
                            }
                            elseif(!$seller->shop_logo) {
                                return array(
                                    'status' => false,
                                    'message' => 'SHOP_LOGO_REQUIRED'
                                );
                            }
                        }
                        if($field=='shop_banner')
                        {
                            $shop_banner_news = array();
                            $shop_banner_olds = array();
                            foreach($languages as $language)
                            {
                                if(isset($_FILES['shop_banner_'.$language['id_lang']]['name'])&& $_FILES['shop_banner_'.$language['id_lang']]['name'])
                                {
                                    $shop_banner_news[$language['id_lang']] = $this->ets_marketplace->uploadFile('shop_banner_'.$language['id_lang'],$errors);
                                    if($shop_banner_news[$language['id_lang']])
                                    {
                                        $shop_banner_olds[$language['id_lang']] = $seller->shop_banner[$language['id_lang']];
                                        $seller->shop_banner[$language['id_lang']] = $shop_banner_news[$language['id_lang']];
                                    }
                                }
                            }
                            foreach($languages as $language)
                            {
                                if(!$seller->shop_banner[$language['id_lang']])
                                    $seller->shop_banner[$language['id_lang']] = $seller->shop_banner[$id_lang_default];
                            }
                        }
                    }
                }
            }

            if($seller->id)
            {
                if($seller->update(true)) {
                    $registration = Ets_mp_registration::_getRegistration($seller->id_customer);
                    if(isset($logo_old) && $logo_old && (!$registration || $registration->shop_logo!=$logo_old ))
                        @unlink(_PS_IMG_DIR_.'mp_seller/'.$logo_old);
                    if(isset($shop_banner_olds) && $shop_banner_olds)
                    {
                        foreach($shop_banner_olds as $shop_banner_old)
                        {
                            if(!in_array($shop_banner_old,$seller->shop_banner) && (!$registration || $registration->shop_banner!=$shop_banner_old) && file_exists(_PS_IMG_DIR_.'mp_seller/'.$shop_banner_old))
                                @unlink(_PS_IMG_DIR_.'mp_seller/'.$shop_banner_old);
                        }
                    }
                } else {
                    if(isset($logo) && $logo && file_exists(_PS_IMG_DIR_.'mp_seller/'.$logo))
                        @unlink(_PS_IMG_DIR_.'mp_seller/'.$logo);
                    if(isset($shop_banner_news) && $shop_banner_news)
                    {
                        foreach($shop_banner_news as $shop_banner)
                        {
                            if(file_exists(_PS_IMG_DIR_.'mp_seller/'.$shop_banner))
                                @unlink(_PS_IMG_DIR_.'mp_seller/'.$shop_banner);
                        }
                    }
                    
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            } else {
                if($seller->add(true,true))
                {
                    $registration = Ets_mp_registration::_getRegistration($seller->id_customer);
                    if(isset($logo_old) && $logo_old && $registration->shop_logo!=$logo_old)
                    {
                        @unlink(_PS_IMG_DIR_.'mp_seller/'.$logo_old);
                    }
                    if(isset($shop_banner_olds) && $shop_banner_olds)
                    {
                        foreach($shop_banner_olds as $idLang => $shop_banner_old)
                        {
                            if(!in_array($shop_banner_old,$seller->shop_banner) && $seller->shop_banner[$idLang]!=$shop_banner_old && file_exists(_PS_IMG_DIR_.'mp_seller/'.$shop_banner_old))
                                @unlink(_PS_IMG_DIR_.'mp_seller/'.$shop_banner_old);
                        }
                    }
                    $fee_type = $seller->getFeeType();
                    if($fee_type!='no_fee')
                    {
                        $billing = new Ets_mp_billing();
                        $billing->id_customer = $seller->id_customer;
                        $billing->amount = (float)$seller->getFeeAmount();
                        $billing->amount_tax = $this->ets_marketplace->getFeeIncludeTax($billing->amount,$seller);
                        $billing->active = 0;
                        $billing->date_from = date('Y-m-d');
                        if($fee_type=='monthly_fee')
                            $billing->date_to = date("Y-m-d", strtotime(date('Y-m-d')."+1 month"));
                        elseif($fee_type=='quarterly_fee')
                            $billing->date_to = date("Y-m-d", strtotime(date('Y-m-d')."+3 month"));
                        elseif($fee_type=='yearly_fee')
                            $billing->date_to = date("Y-m-d", strtotime(date('Y-m-d')."+1 year"));
                        else
                            $billing->date_to ='';
                        $billing->fee_type = $fee_type;
                        if($billing->add(true,true))
                        {
                            $seller->id_billing = $billing->id;
                            $seller->update();
                        }
                        $message = Configuration::get('ETS_MP_MESSAGE_CREATED_SHOP_FEE_REQUIRED',$context->language->id)?: $this->ets_marketplace->l('Thanks for creating your shop. Please send the fee [fee_amount] right now to activate your shop and click on the button "I have just sent the fee" after making payment. [payment_information_manager]');
                        $str_search = array(
                            '[fee_amount]',
                            '[manager_email]',
                            '[payment_information_manager]',
                            '[manager_phone]',
                        );
                        $str_replace =array(
                            Tools::displayPrice($billing->amount_tax,new Currency(Configuration::get('PS_CURRENCY_DEFAULT'))).' ('.$this->ets_marketplace->l('Tax incl').')',
                            Configuration::get('ETS_MP_EMAIL _ADMIN_NOTIFICATION')?:Configuration::get('PS_SHOP_EMAIL'),
                            $this->ets_marketplace->_replaceTag(Configuration::get('ETS_MP_SELLER_PAYMENT_INFORMATION',$context->language->id)),
                            Configuration::get('PS_SHOP_PHONE'),
                        );
                        $message = str_replace($str_search,$str_replace,$message);
                    }
                    else
                    {
                        if($seller->active==1)
                            $message = Configuration::get('ETS_MP_ MESSAGE_SHOP_ACTIVED',$context->language->id)?:$this->ets_marketplace->l('Congratulations! Your shop has been activated. You can upload products and start selling them','create');
                        else
                            $message = Configuration::get('ETS_MP_MESSAGE_CREATED_SHOP_NO_FEE',$context->language->id)?:$this->l('Thanks for creating your shop. Our team are reviewing it. We will get back to you soon','create');
                    
                    } 

                    if(Configuration::get('ETS_MP_EMAIL_ADMIN_SHOP_CREATED'))
                    {
                        $data = array(
                            '{seller_name}'=>  $context->customer->firstname.' '.$context->customer->lastname,
                            '{shop_seller_name}' => $seller->shop_name[$context->language->id],
                            '{shop_description}' => $seller->shop_description[$context->language->id],
                            '{shop_address}' => $seller->shop_address[$context->language->id],
                            '{shop_phone}' => $seller->shop_phone,
                        );
                        $subjects = array(
                            'translation' => $this->l('New shop has been created'),
                            'origin'=>'New shop has been created',
                            'specific'=>'create'
                        );
                        Ets_marketplace::sendMail('to_admin_shop_created',$data,'',$subjects);
                        
                    }
                }
                else
                {
                    if(isset($logo) && $logo && file_exists(_PS_IMG_DIR_.'mp_seller/'.$logo))
                        @unlink(_PS_IMG_DIR_.'mp_seller/'.$logo);
                    if(isset($shop_banner_news) && $shop_banner_news)
                    {
                        foreach($shop_banner_news as $shop_banner)
                        {
                            if(file_exists(_PS_IMG_DIR_.'mp_seller/'.$shop_banner))
                                @unlink(_PS_IMG_DIR_.'mp_seller/'.$shop_banner);
                        }
                    }

                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            }

            return array(
                'status' => true
            );
        }

        public function getOrdersList($page, $limit, $context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            $orderStates = Db::getInstance()->executeS(
                'SELECT os.*,osl.name FROM `'._DB_PREFIX_.'order_state` os
                LEFT JOIN `'._DB_PREFIX_.'order_state_lang` osl ON (os.id_order_state = osl.id_order_state AND osl.id_lang="'.(int)$context->language->id.'")'
            );

            $sort = 'date_add desc';
            $filter = '';

            $orders = $seller->getOrders($filter,($page * $limit) - $limit, $limit, $sort, false);

            if($orders)
            {
                foreach($orders as &$order)
                {
                    $order['total_paid_tax_incl'] = Tools::displayPrice($order['total_paid_tax_incl'],new Currency($order['id_currency']));
                    $order['current_state'] = $this->ets_marketplace->displayOrderState($order['current_state']);  
                    $order['child_view_url'] = $context->link->getModuleLink($this->ets_marketplace->name,'orders',array('id_order'=> $order['id_order']));    
                    $order['commission'] = Tools::displayPrice($order['commission'],new Currency(Configuration::get('PS_CURRENCY_DEFAULT')));  
                    $order['products'] = $this->getProductList($order['id_order'], $context);            
                }
            }

            return array(
                'status' => true,
                'data' => array(
                    'orders' => $orders
                )
            );
        }

        public function getOrderDetails($id_order, $context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            if(!$context->customer->logged || !($seller)) {
                return array(
                    'status' => false,
                    'message' => 'ERROR'
                );
            }

            if(($id_order = (int) $id_order) && Validate::isUnsignedId($id_order))
            {
                if(!$seller->checkHasOrder($id_order)) {
                    return array(
                        'status' => false,
                        'message' => 'ERROR'
                    );
                }
            }

            $data = array();

            $order = new Order($id_order);
            if (!Validate::isLoadedObject($order)) {
                return array(
                    'status' => false,
                    'message' => 'ORDER_NOT_FOUND'
                );
            }

            $customer = new Customer($order->id_customer);
            $carrier = new Carrier($order->id_carrier);
            $products = $this->getProducts($order);
            $currency = new Currency((int)$order->id_currency);

            // Carrier module call
            $carrier_module_call = null;
            if ($carrier->is_module) {
                $module = Module::getInstanceByName($carrier->external_module_name);
                if (method_exists($module, 'displayInfoByCart')) {
                    $carrier_module_call = call_user_func(array($module, 'displayInfoByCart'), $order->id_cart);
                }
            }

            // Retrieve addresses information
            $addressInvoice = new Address($order->id_address_invoice, $context->language->id);
            if (Validate::isLoadedObject($addressInvoice) && $addressInvoice->id_state) {
                $invoiceState = new State((int)$addressInvoice->id_state);
            }

            if ($order->id_address_invoice == $order->id_address_delivery) {
                $addressDelivery = $addressInvoice;
                if (isset($invoiceState)) {
                    $deliveryState = $invoiceState;
                }
            } else {
                $addressDelivery = new Address($order->id_address_delivery, $context->language->id);
                if (Validate::isLoadedObject($addressDelivery) && $addressDelivery->id_state) {
                    $deliveryState = new State((int)($addressDelivery->id_state));
                }
            }

            $warehouse_list = null;

            $order_details = $order->getOrderDetailList();
            foreach ($order_details as $order_detail) {
                $product = new Product($order_detail['product_id']);

                if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT')
                    && $product->advanced_stock_management) {
                    $warehouses = Warehouse::getWarehousesByProductId($order_detail['product_id'], $order_detail['product_attribute_id']);
                    foreach ($warehouses as $warehouse) {
                        if (!isset($warehouse_list[$warehouse['id_warehouse']])) {
                            $warehouse_list[$warehouse['id_warehouse']] = $warehouse;
                        }
                    }
                }
            }

            $payment_methods = array();
            foreach (PaymentModule::getInstalledPaymentModules() as $payment) {
                $module = Module::getInstanceByName($payment['name']);
                if (Validate::isLoadedObject($module) && $module->active) {
                    $payment_methods[] = $module->displayName;
                }
            }

            // display warning if there are products out of stock
            $display_out_of_stock_warning = false;
            $current_order_state = $order->getCurrentOrderState();
            if (Configuration::get('PS_STOCK_MANAGEMENT') && (!Validate::isLoadedObject($current_order_state) || ($current_order_state->delivery != 1 && $current_order_state->shipped != 1))) {
                $display_out_of_stock_warning = true;
            }

            // products current stock informations (from stock_available)
            $stockLocationIsAvailable = false;
            foreach ($products as &$product) {
                // Get total customized quantity for current product
                $customized_product_quantity = 0;

                if (Ets_marketplace::validateArray($product['customizedDatas'])) {
                    foreach ($product['customizedDatas'] as $customizationPerAddress) {
                        foreach ($customizationPerAddress as $customization) {
                            $customized_product_quantity += (int)$customization['quantity'];
                        }
                    }
                }
                $product['customized_product_quantity'] = $customized_product_quantity;
                $product['current_stock'] = StockAvailable::getQuantityAvailableByProduct($product['product_id'], $product['product_attribute_id'], $product['id_shop']);
                $resume = OrderSlip::getProductSlipResume($product['id_order_detail']);
                $product['quantity_refundable'] = $product['product_quantity'] - $resume['product_quantity'];
                $product['amount_refundable'] = $product['total_price_tax_excl'] - $resume['amount_tax_excl'];
                $product['amount_refundable_tax_incl'] = $product['total_price_tax_incl'] - $resume['amount_tax_incl'];
                $product['amount_refund'] = $order->getTaxCalculationMethod() ? Tools::displayPrice($resume['amount_tax_excl'], $currency) : Tools::displayPrice($resume['amount_tax_incl'], $currency);
                $product['refund_history'] = OrderSlip::getProductSlipDetail($product['id_order_detail']);
                $product['return_history'] = OrderReturn::getProductReturnDetail($product['id_order_detail']);

                // if the current stock requires a warning
                if ($product['current_stock'] <= 0 && $display_out_of_stock_warning) {
                    $this->ets_marketplace->displayWarning($this->ets_marketplace->l('This product is out of stock: ','orders') . ' ' . $product['product_name']);
                }
                if ($product['id_warehouse'] != 0) {
                    $warehouse = new Warehouse((int)$product['id_warehouse']);
                    $product['warehouse_name'] = $warehouse->name;
                    $warehouse_location = WarehouseProductLocation::getProductLocation($product['product_id'], $product['product_attribute_id'], $product['id_warehouse']);
                    if (!empty($warehouse_location)) {
                        $product['warehouse_location'] = $warehouse_location;
                    } else {
                        $product['warehouse_location'] = false;
                    }
                } else {
                    $product['warehouse_name'] = '--';
                    $product['warehouse_location'] = false;
                }

                if (!empty($product['location'])) {
                    $stockLocationIsAvailable = true;
                }
            }

            // Package management for order
            foreach ($products as &$product) {
                $pack_items = $product['cache_is_pack'] ? Pack::getItemTable($product['id_product'], $context->language->id, true) : array();
                foreach ($pack_items as &$pack_item) {
                    $pack_item['current_stock'] = StockAvailable::getQuantityAvailableByProduct($pack_item['id_product'], $pack_item['id_product_attribute'], $pack_item['id_shop']);
                    $this->setProductImageInformations($pack_item);
                    if ($pack_item['image'] != null) {
                        $name = 'product_mini_' . (int)$pack_item['id_product'] . (isset($pack_item['id_product_attribute']) ? '_' . (int)$pack_item['id_product_attribute'] : '') . '.jpg';
                        // generate image cache, only for back office
                        $pack_item['image_tag'] = ImageManager::thumbnail(_PS_IMG_DIR_ . 'p/' . $pack_item['image']->getExistingImgPath() . '.jpg', $name, 45, 'jpg');
                        if (file_exists(_PS_TMP_IMG_DIR_ . $name)) {
                            $pack_item['image_size'] = getimagesize(_PS_TMP_IMG_DIR_ . $name);
                        } else {
                            $pack_item['image_size'] = false;
                        }
                    }
                }
                $product['pack_items'] = $pack_items;
            }

            $gender = new Gender((int)$customer->id_gender, $context->language->id);

            $history = $order->getHistory($context->language->id);

            foreach ($history as &$order_state) {
                $order_state['text-color'] = Tools::getBrightness($order_state['color']) < 150 ? 'white' : 'black';
            }

            $shipping_refundable_tax_excl = $order->total_shipping_tax_excl;
            $shipping_refundable_tax_incl = $order->total_shipping_tax_incl;
            $slips = OrderSlip::getOrdersSlip($customer->id, $order->id);
            foreach ($slips as $slip) {
                $shipping_refundable_tax_excl -= $slip['total_shipping_tax_excl'];
                $shipping_refundable_tax_incl -= $slip['total_shipping_tax_incl'];
            }
            $shipping_refundable_tax_excl = max(0, $shipping_refundable_tax_excl);
            $shipping_refundable_tax_incl = max(0, $shipping_refundable_tax_incl);
            if(Module::isInstalled('ets_payment_with_fee') && Module::isEnabled('ets_payment_with_fee'))
            {
                if($method_order = Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_paymentmethod_order` WHERE id_order ='.(int)$order->id))
                {
                    $price = $method_order['fee'];
                    if($price)
                    {
                        $priceFormatter = new PrestaShop\PrestaShop\Adapter\Product\PriceFormatter();
                        $data['payment_fee'] = $priceFormatter->format($price, Currency::getCurrencyInstance((int)$order->id_currency));
                    }
                }
            }

            $data = array_merge($data, array(
                'order' => $order,
                'order_state' => new OrderState($order->current_state, $context->language->id),
                'cart' => new Cart($order->id_cart),
                'order_customer' => $customer,
                'gender' => $gender,
                'customer_addresses' => $customer->getAddresses($context->language->id),
                'addresses' => array(
                    'delivery' => $addressDelivery,
                    'deliveryState' => isset($deliveryState) ? $deliveryState : null,
                    'invoice' => $addressInvoice,
                    'invoiceState' => isset($invoiceState) ? $invoiceState : null,
                ),
                'customerStats' => $customer->getStats(),
                'products' => $products,
                'can_edit' => false,
                'stock_management' => true,
                'discounts' => $order->getCartRules(),
                'orders_total_paid_tax_incl' => $order->getOrdersTotalPaid(), // Get the sum of total_paid_tax_incl of the order with similar reference
                'total_paid' => $order->getTotalPaid(),
                'returns' => OrderReturn::getOrdersReturn($order->id_customer, $order->id),
                'shipping_refundable_tax_excl' => $shipping_refundable_tax_excl,
                'shipping_refundable_tax_incl' => $shipping_refundable_tax_incl,
                'customer_thread_message' => CustomerThread::getCustomerMessages($order->id_customer, null, $order->id),
                'orderMessages' => OrderMessage::getOrderMessages($order->id_lang),
                'messages' => $this->ets_marketplace->is17 ? CustomerThread::getCustomerMessagesOrder($order->id_customer, $order->id) : Message::getMessagesByOrderId($order->id, true),
                'carrier' => new Carrier($order->id_carrier),
                'history' => $history,
                'states' => $this->getOrderStates($context->language->id,$order->current_state, $context),
                'warehouse_list' => $warehouse_list,
                'currentState' => $order->getCurrentOrderState(),
                'currency' => new Currency($order->id_currency),
                'currencies' => Currency::getCurrenciesByIdShop($order->id_shop),
                'previousOrder' => (int)Db::getInstance()->getValue('SELECT id_order FROM `'._DB_PREFIX_.'ets_mp_seller_order` WHERE id_customer="'.(int)$seller->id_customer.'" AND id_order < "'.(int)$id_order.'" ORDER BY id_order DESC'),
                'nextOrder' => (int)Db::getInstance()->getValue('SELECT id_order FROM `'._DB_PREFIX_.'ets_mp_seller_order` WHERE id_customer="'.(int)$seller->id_customer.'" AND id_order > "'.(int)$id_order.'" ORDER BY id_order DESC'),
                'carrierModuleCall' => $carrier_module_call,
                'iso_code_lang' => $context->language->iso_code,
                'id_lang' => $context->language->id,
                'current_id_lang' => $context->language->id,
                'invoices_collection' => $order->getInvoicesCollection(),
                'not_paid_invoices_collection' => $order->getNotPaidInvoicesCollection(),
                'payment_methods' => $payment_methods,
                'invoice_management_active' => Configuration::get('PS_INVOICE', null, null, $order->id_shop),
                'display_warehouse' => (int)Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT'),
                'carrier_list' => $this->getCarrierList($order),
                'recalculate_shipping_cost' => (int)Configuration::get('PS_ORDER_RECALCULATE_SHIPPING'),
                'stock_location_is_available' => $stockLocationIsAvailable
            ));

            return array(
                'status' => true,
                'data' => $data
            );
        }

        ///////////// Add manufacturer functions

        private function _submitSaveManufacturer($context)
        {

            $seller = $this->ets_marketplace->_getSeller(true);

            $languages = Language::getLanguages(false);
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if(($id_manufacturer = (int)Tools::getValue('id_manufacturer')) && Validate::isUnsignedId($id_manufacturer)) {
                $manufacturer = new Manufacturer($id_manufacturer);
                if(!Validate::isLoadedObject($manufacturer) || !$seller->checkHasManufacturer($id_manufacturer,false)) {
                    return false;
                }
            } else {
                $manufacturer = new Manufacturer();
            }

            if(!($name= Tools::getValue('name'))) {
                return false;
            } elseif($name && !Validate::isCatalogName($name)) {
                return false;
            } else {
                $manufacturer->name = $name;
            }

            foreach($languages as $language)
            {
                if (($description = Tools::getValue('description_'.$language['id_lang'])) && !Validate::isCleanHtml($description)) {
                    return false;
                } else if ($description) {
                    $manufacturer->description[$language['id_lang']] = $description;
                } else {
                    $manufacturer->description[$language['id_lang']] = Tools::getValue('description_'.$id_lang_default);
                }

                if (($short_description= Tools::getValue('short_description_'.$language['id_lang'])) && !Validate::isCleanHtml($short_description)) {
                    return false;
                } else if ($short_description) {
                    $manufacturer->short_description[$language['id_lang']] = $short_description;
                } else {
                    $manufacturer->short_description[$language['id_lang']] = Tools::getValue('short_description_'.$id_lang_default);
                } 

                if (($meta_title= Tools::getValue('meta_title_'.$language['id_lang'])) && !Validate::isGenericName($meta_title)) {
                    return false;
                } else if ($meta_title) {
                    $manufacturer->meta_title[$language['id_lang']] = $meta_title;
                } else {
                    $manufacturer->meta_title[$language['id_lang']] = Tools::getValue('meta_title_'.$id_lang_default);
                } 

                if (($meta_description= Tools::getValue('meta_description_'.$language['id_lang'])) && !Validate::isGenericName($meta_description)) {
                    return false;
                } else if ($meta_description) {
                    $manufacturer->meta_description[$language['id_lang']] = $meta_description;
                } else {
                    $manufacturer->meta_description[$language['id_lang']] = Tools::getValue('meta_description_'.$id_lang_default);
                } 

                if (($meta_keywords= Tools::getValue('meta_keywords_'.$language['id_lang'])) && !Validate::isGenericName($meta_keywords)) {
                    return false;
                } else if ($meta_keywords) {
                    $manufacturer->meta_keywords[$language['id_lang']] = $meta_keywords;
                } else {
                    $manufacturer->meta_keywords[$language['id_lang']] = Tools::getValue('meta_keywords_'.$id_lang_default);
                }
            }

            $manufacturer->active = 1;

            if($manufacturer->id) {
                if($manufacturer->update()) {
                    return true;
                } else {
                    return false;
                }
            } else if ($manufacturer->add()) {
                Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'ets_mp_manufacturer_seller`(id_customer,id_manufacturer) VALUES("'.(int)$seller->id_customer.'","'.(int)$manufacturer->id.'")');
                return true;
            } else {
                return false;
            }
        }

        ///////////// Add product functions

        private function _checkValidateProduct()
        {

            $seller_product_information = explode(',', Configuration::get('ETS_MP_SELLER_ALLOWED_INFORMATION_SUBMISSION'));
            $seller = $this->ets_marketplace->_getSeller(true);

            if(($id_product = Tools::getValue('id_product')) && Validate::isUnsignedId($id_product))
            {
                if(($seller_product = $seller->checkHasProduct($id_product)) && isset($seller_product['id_product']) && ($id_product = $seller_product['id_product']))
                {
                    $product = new Product($id_product);
                    // if(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED') && Configuration::get('ETS_MP_EDIT_PRODUCT_APPROVE_REQUIRED') && $seller_product['approved']==1 && ($fields = Configuration::get('ETS_MP_FIELD_PRODUCT_APPROVE_REQUIRED')))
                    // {
                    //     $this->list_field_approve = explode(',',$fields);
                    //     $this->mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    // }
                    // else
                    //     $this->mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                } else {
                    return false;
                }
                $product->loadStockData();
            } else {
                $product = new Product();
            }

            $id_lang_default = Configuration::get('PS_LANG_DEFAULT');
            $languages = Language::getLanguages(true);
            $name_default = Tools::getValue('name_' . $id_lang_default);
            if(!trim($name_default)) {
                return false;
            } else {
                foreach($languages as $language)
                {
                    if(($name = Tools::getValue('name_' . $language['id_lang'])) && !Validate::isCatalogName($name)) {
                        return false;
                    } else if (Tools::strlen($name) > 128) {
                        return false;
                    }
                }
            }

            if(in_array('seo', $seller_product_information)) {
                $link_rewrite_default = Tools::getValue('link_rewrite_' . $id_lang_default);
                if(!trim($link_rewrite_default)) {
                    return false;
                } else {
                    foreach($languages as $language)
                    {
                        if(($link_rewrite = Tools::getValue('link_rewrite_' . $language['id_lang'])) && !Validate::isLinkRewrite($link_rewrite)) {
                            return false;
                        } else if (Tools::strlen($link_rewrite) > 128) {
                            return false;
                        }
                    }
                }
            }

            if(!($id_categories = Tools::getValue('id_categories'))) {
                return false;
            } else if (!Ets_marketplace::validateArray($id_categories, 'isInt')) {
                return false;
            } else if ($seller_categories = $seller->getApplicableProductCategories()) {
               foreach($id_categories as $id_category)
               {
                    if(!in_array($id_category, $seller_categories)) {
                        return false;
                    }
               }
            }

            foreach($languages as $language)
            {
                if(in_array('short_description', $seller_product_information))
                {
                    if(($description_short = Tools::getValue('description_short_'.$language['id_lang'])) && !Validate::isCleanHtml($description_short,Configuration::get('PS_ALLOW_HTML_IFRAME'))) {
                        return false;
                    }
                    $short_description_limit= Configuration::get('PS_PRODUCT_SHORT_DESC_LIMIT') ? : 800;
                    if(Tools::strlen(strip_tags($description_short)) > $short_description_limit) {
                        return false;
                    }
                }
                if(($description = Tools::getValue('description_'.$language['id_lang'])) && !Validate::isCleanHtml($description,Configuration::get('PS_ALLOW_HTML_IFRAME')))
                    return false;
                if(Tools::strlen(strip_tags($description))>21844)
                    return false;
                if( ($meta_title = Tools::getValue('meta_title_'.$language['id_lang'])) && !Validate::isCleanHtml($meta_title))
                    return false;
                elseif(Tools::strlen($meta_title) >128)
                    return false;
                if( ($meta_description=  Tools::getValue('meta_description_'.$language['id_lang'])) && !Validate::isCleanHtml($meta_description))
                    return false;
                elseif(Tools::strlen($meta_description) >512)
                    return false;
                if( ($delivery_in_stock = Tools::getValue('delivery_in_stock_'.$language['id_lang'])) && !Validate::isCleanHtml($delivery_in_stock))
                    return false;
                elseif(Tools::strlen($delivery_in_stock) >255)
                    return false;
                if(($delivery_out_stock = Tools::getValue('delivery_out_stock_'.$language['id_lang'])) && !Validate::isCleanHtml($delivery_out_stock))
                    return false;
                elseif(Tools::strlen($delivery_out_stock) >255)
                    return false;
                if ($tags = Tools::getValue('tags_'.$language['id_lang'])) {
                    foreach(explode(',',$tags) as $tag) {
                        if($tag && !Validate::isGenericName($tag)) {
                            return false;
                        }
                    }
                }       
            }

            $price_excl = Tools::getValue('price_excl');
            if(trim($price_excl) === '') {
                return false;
            }
            elseif(!Validate::isUnsignedFloat($price_excl))
                return false;
            $width = Tools::getValue('width');
            if(trim($width)!=='' && (!Validate::isUnsignedFloat($width) || $width==0))
                return false;
            $height = Tools::getValue('height');
            if(trim($height)!=='' && (!Validate::isUnsignedFloat($height) || $height==0))
                return false;
            $depth = Tools::getValue('depth');
            if(trim($depth)!=='' && (!Validate::isUnsignedFloat($depth) || $depth ==0 ))
                return false;
            $weight = Tools::getValue('weight');
            if(trim($weight)!=='' &&  (!Validate::isUnsignedFloat($weight) || $weight==0 ))
                return false;
            $additional_shipping_cost = Tools::getValue('additional_shipping_cost');
            if(trim($additional_shipping_cost)!=='' &&  (!Validate::isUnsignedFloat($additional_shipping_cost) || $additional_shipping_cost==0 ))
                return false;
            if(($condition = Tools::getValue('condition')) && !Validate::isGenericName($condition))
                return false;
            if($this->ets_marketplace->is17)
            {
                if(($show_condition = Tools::getValue('show_condition')) && !Validate::isBool($show_condition)) {
                    return false;
                }
            }
            if(($isbn = Tools::getValue('isbn')) && !Validate::isIsbn($isbn))
                return false;
            elseif(Tools::strlen($isbn) >32)
                return false;
            if(($ean13 = Tools::getValue('ean13')) && !Validate::isEan13($ean13))
                return false;
            elseif(Tools::strlen($ean13) >13)
                return false;
            if(($upc = Tools::getValue('upc')) && !Validate::isUpc($upc))
                return false;
            elseif(Tools::strlen($upc) >12)
                return false;
            if(($custom_fields = Tools::getValue('custom_fields')) && Ets_marketplace::validateArray($custom_fields)) {
                foreach($custom_fields as $custom_field) {
                    if(!$custom_field['label'][$id_lang_default]) {
                        return false;
                        break;
                    }
                    if(!Validate::isUnsignedInt($custom_field['type'])) {
                        return false;
                        break;
                    }
                    foreach($languages as $language) {
                        if($custom_field['label'][$language['id_lang']] && !Validate::isCleanHtml($custom_field['label'][$language['id_lang']])) {
                            return false;
                            break;
                        }
                    }
                    if($id_customization_field = (int)$custom_field['id_customization_field']) {
                        $customizationField = new CustomizationField($id_customization_field);
                        if(!Validate::isLoadedObject($customizationField) || $customizationField->id_product != $product->id) {
                            return false;
                        }
                    }
                }
            }

            $product_type = (int) Tools::getValue('product_type');
            if($product_type == 1) {
                if(!($inputPackItems = Tools::getValue('inputPackItems')) || !Ets_marketplace::validateArray($inputPackItems)) {
                    return false;
                } else {
                    foreach($inputPackItems as $inputPackItem) {
                        $packItem = explode('x', $inputPackItem);
                        if(!isset($packItem[0]) || !isset($packItem[1]) || !isset($packItem[2])) {
                            return false;
                        } else {
                            if(!$id_product_item = $packItem[0])
                                return false;
                            elseif(($productPackItem = new Product($id_product_item)) && (!Validate::isLoadedObject($productPackItem) || !$seller->checkHasProduct($id_product_item)))
                                return false;
                            if(($id_product_attribute_item = $packItem[1]) && ($combination = new Combination($id_product_attribute_item)) && (!Validate::isLoadedObject($combination) || $combination->id_product!=$id_product_item))
                                return false;
                            if(!$quantity_item = $packItem[2])
                                return false;
                            elseif($quantity_item && !Validate::isUnsignedInt($quantity_item))
                                return false;
                        }
                        
                    }
                }
            }

            if($id_manufacturer = (int)Tools::getValue('id_manufacturer'))
            {
                $manufacture = new Manufacturer($id_manufacturer);
                if(!Validate::isLoadedObject($manufacture) || !$seller->checkHasManufacturer($manufacture->id))
                   return false;
            }
            if(($id_features = Tools::getValue('id_features')) && Ets_marketplace::validateArray($id_features,'isInt') && ($id_feature_values = Tools::getValue('id_feature_values')) && Ets_marketplace::validateArray($id_feature_values,'isInt')) {
                foreach($id_features as $index=> $id_feature) {
                    if($id_feature && ($feature = new Feature($id_feature)) && (!Validate::isLoadedObject($feature) || !$seller->checkHasFeature($id_feature)))
                        return false;
                    elseif($id_feature && isset($id_feature_values[$index]) && ($id_feature_value = $id_feature_values[$index])) {
                        if(($featureValue = new FeatureValue($id_feature_value)) && (!Validate::isLoadedObject($featureValue) || $featureValue->id_feature!= $id_feature))
                            return false;
                    }
                }
            }

            if(Tools::isSubmit('submitCreateCombination') && ($attribute_options = Tools::getValue('attribute_options')))
            {
                $check_attribute = true;
                if(!Ets_marketplace::validateArray($attribute_options))
                    return false;
                else {
                    foreach($attribute_options as $id_attribute_group => $id_attributes) {
                        if(!$id_attribute_group || !$id_attributes) {
                            return false;
                            $check_attribute= false;
                            break;
                        }
                    }
                    if($check_attribute) {
                        foreach($attribute_options as $id_attribute_group => $id_attributes) {
                            foreach($id_attributes as $id_attribute) {
                                if(($attributeGroup = new AttributeGroup($id_attribute_group)) && ($attribute = new Attribute($id_attribute)) && (!Validate::isLoadedObject($attributeGroup) || !Validate::isLoadedObject($attribute) || $attribute->id_attribute_group!= $id_attribute_group || !$seller->checkHasAttributeGroup($id_attribute_group)))
                                    return false;
                            }   
                        }
                    }
                }
            }

            if($product_type !=2 && ($selectedCarriers = Tools::getValue('selectedCarriers')))
            {
                if(!Ets_marketplace::validateArray($selectedCarriers,'isInt'))
                    return false;
                else {
                    foreach($selectedCarriers as $selectedCarrier) {
                        if($selectedCarrier && ($carrier = Carrier::getCarrierByReference($selectedCarrier)) && (!Validate::isLoadedObject($carrier) || !$seller->getListCarriersUser(0,$selectedCarrier)))
                            return false;
                    }
                }
            }

            if(($id_suppliers = Tools::getValue('id_suppliers')) && Ets_marketplace::validateArray($id_suppliers,'isInt') && ($product_supplier_reference = Tools::getValue('product_supplier_reference')) && Ets_marketplace::validateArray($product_supplier_reference) && ($product_supplier_price = Tools::getValue('product_supplier_price')) && Ets_marketplace::validateArray($product_supplier_price) && ($product_supplier_price_currency = Tools::getValue('product_supplier_price_currency')) && Ets_marketplace::validateArray($product_supplier_price_currency))
            {
                foreach($id_suppliers as $id_supplier)
                {
                    $supplier = new Supplier($id_supplier);
                    if(!Validate::isLoadedObject($supplier) || !$seller->checkHasSupplier($supplier->id,true))
                    {
                        return false;
                    }
                    else
                    {
                        if(isset($product_supplier_reference[$id_supplier]) && ($references = $product_supplier_reference[$id_supplier]))
                        {
                            foreach($references as $reference)
                                if($reference && !Validate::isReference($reference))
                                    return false;
                        }
                        if(isset($product_supplier_price[$id_supplier]) && ($prices = $product_supplier_price[$id_supplier]))
                        {
                            foreach($prices as $price)
                            {
                                if($price && !Validate::isPrice($price))
                                   return false;
                            }
                        }
                        if(isset($product_supplier_price_currency[$id_supplier]) && ($currencies = $product_supplier_price_currency[$id_supplier]))
                        {
                            foreach($currencies as $id_currency)
                            {
                                $currency_class = new Currency($id_currency);
                                if(!Validate::isLoadedObject($currency_class))
                                    return false;
                            }
                        }
                    }
                }
            }
            if(($related_products = Tools::getValue('related_products')) && Ets_marketplace::validateArray($related_products,'isInt'))
            {
                foreach($related_products as $related_product)
                {
                    $related_product_obj = new Product($related_product);
                    if(!Validate::isLoadedObject($related_product_obj) || !$seller->checkHasProduct($related_product))
                    {
                        return false;
                        break;
                    }    
                }
            }
             
            return true;
        }

        private function _submitSaveProduct($context)
        {
            $seller = $this->ets_marketplace->_getSeller(true);
            $seller_product_information = explode(',', Configuration::get('ETS_MP_SELLER_ALLOWED_INFORMATION_SUBMISSION'));

            if(($id_product = Tools::getValue('id_product')) && Validate::isUnsignedId($id_product))
            {
                if(($seller_product = $seller->checkHasProduct($id_product)) && isset($seller_product['id_product']) && ($id_product = $seller_product['id_product']))
                {
                    $product = new Product($id_product);
                    if(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED') && Configuration::get('ETS_MP_EDIT_PRODUCT_APPROVE_REQUIRED') && $seller_product['approved']==1 && ($fields = Configuration::get('ETS_MP_FIELD_PRODUCT_APPROVE_REQUIRED')))
                    {
                        $list_field_approve = explode(',', $fields);
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                    else {
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                } else {
                    return false;
                }
                $product->loadStockData();
            } else {
                $product = new Product();
            }

            $languages = Language::getLanguages(false);
            $seller_categories = $seller->getApplicableProductCategories();
            $id_lang_default =Configuration::get('PS_LANG_DEFAULT');
            $fields_changed = array();
            if($languages)
            {
                foreach($languages as $language)
                {

                    $name = Tools::getValue('name_'.$language['id_lang']) ? : Tools::getValue('name_'.$id_lang_default);
                    if($mpProduct && in_array('name', $list_field_approve))
                    {
                        if(trim($product->name[$language['id_lang']]) != trim($name)) {
                            $fields_changed[] = 'name';
                        }
                        $mpProduct->name[$language['id_lang']] = $name;
                    }    
                    else
                    {
                        if($mpProduct) {
                            $mpProduct->name[$language['id_lang']] = '';
                        }
                        $product->name[$language['id_lang']] = $name;
                    }
    
                    if(in_array('short_description', $seller_product_information))
                    {
                        $description_short = ($description_short = Tools::getValue('description_short_'.$language['id_lang']))  && Tools::strpos($description_short,'etsmp_imThePlaceholder')===false ? $description_short : ( ($description_short_default = Tools::getValue('description_short_'.$id_lang_default))&& Tools::strpos($description_short_default,'etsmp_imThePlaceholder')===false ? $description_short_default:'');
                        if($mpProduct && in_array('description_short', $list_field_approve))
                        {
                            $mpProduct->description_short[$language['id_lang']] = $description_short;
                            if(strcmp(str_replace(' ','',$product->description_short[$language['id_lang']]),str_replace(' ','',$description_short))!==0)
                            {  
                                $fields_changed[] = 'description_short';                         
                            }
                        }
                        else
                            $product->description_short[$language['id_lang']] = $description_short;
                    }  
                    $description = Tools::getValue('description_'.$language['id_lang']) ? : Tools::getValue('description_'.$id_lang_default);  
                    if($mpProduct && in_array('description',$list_field_approve))
                    {
                        $mpProduct->description[$language['id_lang']] = $description;
                        if(strcmp(str_replace(' ','',$product->description[$language['id_lang']]),str_replace(' ','',$description))!==0)
                            $fields_changed[] = 'description';
                    }
                    else
                        $product->description[$language['id_lang']] = $description;
                    if(in_array('seo',$seller_product_information))
                    {
                        $meta_title = Tools::getValue('meta_title_'.$language['id_lang']) ? : Tools::getValue('meta_title_'.$id_lang_default);
                        if($mpProduct && in_array('meta_title',$list_field_approve))
                        {
                            $mpProduct->meta_title[$language['id_lang']] = $meta_title;
                            if(strcmp($product->meta_title[$language['id_lang']],$meta_title)!==0)
                                $fields_changed[] = 'meta_title';
                        }    
                        else
                            $product->meta_title[$language['id_lang']] = $meta_title;
                        $meta_description = Tools::getValue('meta_description_'.$language['id_lang']) ? : Tools::getValue('meta_description_'.$id_lang_default);
                        if($mpProduct && in_array('meta_description',$list_field_approve))
                        {
                            $mpProduct->meta_description[$language['id_lang']] =$meta_description;
                            if(strcmp($product->meta_description[$language['id_lang']],$meta_description)!==0)
                                $fields_changed[]= 'meta_description';
                        }
                        else
                            $product->meta_description[$language['id_lang']] = $meta_description;
                        $link_rewrite = Tools::getValue('link_rewrite_'.$language['id_lang']) ?: Tools::getValue('link_rewrite_'.$id_lang_default);
                        if($mpProduct && in_array('link_rewrite',$list_field_approve))
                        {
                            $mpProduct->link_rewrite[$language['id_lang']] = $link_rewrite;
                            if(strcmp($product->link_rewrite[$language['id_lang']],$link_rewrite)!==0)
                            {
                                $fields_changed[] ='link_rewrite';
                            }
                        }
                        else
                            $product->link_rewrite[$language['id_lang']] = $link_rewrite;
                    }
                    elseif(!isset($product->link_rewrite[$language['id_lang']]) || !$product->link_rewrite[$language['id_lang']])
                        $product->link_rewrite[$language['id_lang']] = Tools::link_rewrite($name);
                    $delivery_in_stock = Tools::getValue('delivery_in_stock_'.$language['id_lang']);
                    if($mpProduct && in_array('delivery_in_stock',$list_field_approve))
                    {
                        $mpProduct->delivery_in_stock[$language['id_lang']] = $delivery_in_stock;
                        if(trim($product->delivery_in_stock[$language['id_lang']]) != trim($delivery_in_stock))
                            $fields_changed[] ='delivery_in_stock';
                    }
                    else 
                        $product->delivery_in_stock[$language['id_lang']] = $delivery_in_stock;
                    $delivery_out_stock = Tools::getValue('delivery_out_stock_'.$language['id_lang']);
                    if($mpProduct && in_array('delivery_out_stock',$list_field_approve))
                    {
                        $mpProduct->delivery_out_stock[$language['id_lang']] = $delivery_out_stock;
                        if(trim($product->delivery_out_stock[$language['id_lang']]) != trim($delivery_out_stock))
                            $fields_changed[] ='delivery_out_stock';
                    }
                    else
                        $product->delivery_out_stock[$language['id_lang']] = $delivery_out_stock;
                }
            }
            $product_location = '';
            $product_type = Tools::getValue('product_type');
            $inputPackItems = Tools::getValue('inputPackItems');
            if($product_type==2)
                $product->is_virtual=1;
            else
                $product->is_virtual=0;
            if($product_type==1 && $inputPackItems)
                $product->cache_is_pack=1;
            else
                $product->cache_is_pack=0;
            if(in_array('product_reference',$seller_product_information))    
            {
                $reference = Tools::getValue('reference');
                if($mpProduct && in_array('reference',$list_field_approve))
                {
                    $mpProduct->reference = $reference;
                    if(trim($product->reference)!= trim($reference))
                        $fields_changed[] = 'reference';
                }
                else
                    $product->reference = $reference;
            }
            $price_excl = (float)Tools::getValue('price_excl');
            $id_tax_rules_group = (int)Tools::getValue('id_tax_rules_group');
            if($mpProduct && in_array('price',$list_field_approve))
            {
                $mpProduct->price = $price_excl;
                if(in_array('tax',$seller_product_information))
                    $mpProduct->id_tax_rules_group = (int)$id_tax_rules_group;
                if($product->price!=$price_excl || (in_array('tax',$seller_product_information) && $product->id_tax_rules_group!= $id_tax_rules_group))
                    $fields_changed[] ='price';
            }
            else
            {
                if($mpProduct)
                    $mpProduct->price = 0;
                $product->price = $price_excl;
                if(in_array('tax',$seller_product_information))
                    $product->id_tax_rules_group = (int)$id_tax_rules_group;
            }
            $width = (float)Tools::getValue('width');
            $height = (float)Tools::getValue('height');
            $depth = (float)Tools::getValue('depth');
            $weight = (float)Tools::getValue('weight');
            if($mpProduct && in_array('width',$list_field_approve))
            {
                $mpProduct->width = (float)$width;
                if($product->width!= (float)$width)
                    $fields_changed[] ='width';
            }
            else
                $product->width =(float)$width;
            if($mpProduct && in_array('height',$list_field_approve))
            {
                $mpProduct->height = (float)$height;
                if($product->height != (float)$height)
                    $fields_changed[] = 'height';
            }
            else
                $product->height = (float)$height;
            if($mpProduct && in_array('depth',$list_field_approve))
            {
                $mpProduct->depth = (float)$depth;
                if($product->depth!= (float)$depth)
                    $fields_changed[] = 'depth';
            }
            else
                $product->depth = (float)$depth;
            if($mpProduct && in_array('weight',$list_field_approve))
            {
                $mpProduct->weight = (float)$weight;
                if($product->weight!=(float)$weight)
                    $fields_changed[] ='weight';
            }
            else
                $product->weight =(float)$weight;
            if($this->ets_marketplace->is17)
                $product->additional_delivery_times = Tools::getValue('additional_delivery_times');
            $product->additional_shipping_cost =(float)Tools::getValue('additional_shipping_cost');
            // $this->product->id_category_default = (int)Tools::getValue('id_category_default');
            $product->id_category_default = 2;
            $product->minimal_quantity =(int)Tools::getValue('product_minimal_quantity');
            $product->location = $product_location;
            if($this->ets_marketplace->is17)
            {
                $product->low_stock_threshold = (int)Tools::getValue('product_low_stock_threshold');
                $product->low_stock_alert = (int)Tools::getValue('product_low_stock_alert');
            }
            if(in_array('out_of_stock_behavior',$seller_product_information))
                $product->out_of_stock = (int)Tools::getValue('out_of_stock');
            $product->id_manufacturer = (int)Tools::getValue('id_manufacturer');
            $product->redirect_type ='404';
            $product->condition = Tools::getValue('condition');
            if($this->ets_marketplace->is17)
                $product->show_condition = (int)Tools::getValue('show_condition');
            $isbn = Tools::getValue('isbn');
            $ean13 = Tools::getValue('ean13');
            $upc = Tools::getValue('upc');
            if($mpProduct && in_array('isbn',$list_field_approve))
            {
                $mpProduct->isbn = $isbn;
                if(trim($product->isbn) !=trim($isbn))
                    $fields_changed[] = 'isbn';
            }
            else
                $product->isbn = $isbn;
            if($mpProduct && in_array('ean13',$list_field_approve))
            {
                $mpProduct->ean13 = $ean13;
                if(trim($this->product->ean13) != trim($ean13))
                    $fields_changed[] ='ean13';
            }
            else
                $product->ean13 = $ean13;
            if($mpProduct && in_array('upc',$list_field_approve))
            {
                $mpProduct->upc = $upc;
                if(trim($product->upc)!=trim($upc))
                    $fields_changed[] ='upc';
            }
            else
                $product->upc = $upc;
            if($mpProduct && in_array('image',$list_field_approve))
            {
                if(Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'ets_mp_product_image`  WHERE id_product='.(int)$product->id))
                    $fields_changed[] ='image';
            }
            $custom_fields = Tools::getValue('custom_fields');
            if($custom_fields && Ets_marketplace::validateArray($custom_fields) && isset($product->customizable))
            {
                $product->customizable=1;
            }
            if(($id_supplier_default = Tools::getValue('id_supplier_default')) && Validate::isUnsignedId($id_supplier_default) && Validate::isLoadedObject(new Supplier($id_supplier_default)) && ($id_suppliers = Tools::getValue('id_suppliers')) && in_array($id_supplier_default,$id_suppliers))
                $product->id_supplier = $id_supplier_default;
            else
                $product->id_supplier=0;
            $active = (int)Tools::getValue('active');
            $available_for_order = 1;
            $show_price = $available_for_order ? 1 : 1;
            if(!$seller->checkVacation() || Tools::strpos($seller->vacation_type,'disable')===false)
            {
                $product->available_for_order = $available_for_order;
                $product->show_price = $show_price;
            }
            if(!$product->id)
            {
                if($seller->checkVacation() && Tools::strpos($seller->vacation_type,'disable_shopping')!==false)
                {
                    $product->available_for_order =0;
                }
                if(Tools::isSubmit('active') && Ets_mp_product::checkSellerUpdateStatusProduct($product->id) && (!$seller->checkVacation() || Tools::strpos($seller->vacation_type,'disable_product')===false))
                {
                    $product->active=(int)$active;
                    $approved=1;
                }
                else
                {
                    if($seller->auto_enabled_product=='yes')
                    {
                        $product->active=1;
                        $approved=1;
                    }
                    elseif($seller->auto_enabled_product=='no')
                    {
                        $product->active=0;
                        $approved=0;
                    }
                    elseif(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED'))
                    {
                        $product->active=0;
                        $approved=0;
                    }
                    else
                    {
                        $product->active=1;
                        $approved=1;
                    }
                    if($seller->checkVacation() && Tools::strpos($seller->vacation_type,'disable_product')!==false)
                        $product->active=0;
                }
                $product->indexed =0;
                if($product->add())
                {

                    $product_quantity = 1;
                    StockAvailable::setQuantity($product->id, 0, $product_quantity);

                    Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'ets_mp_seller_product`(id_customer,id_product,approved,active) VALUES("'.(int)$seller->id_customer.'","'.(int)$product->id.'","'.(int)$approved.'","'.(int)$product->active.'")');
                    if (Configuration::get('PS_SEARCH_INDEXATION')) {
                        Search::indexation(false, $product->id);
                    }
                    if(Configuration::get('ETS_MP_EMAIL_ADMIN_NEW_PRODUCT_UPLOADED'))
                    {
                        $data= array(
                        '{seller_name}' => $seller->seller_name,
                        '{seller_email}' => $seller->seller_email,
                        '{shop_seller}'=> $seller->shop_name[$context->language->id],
                        '{shop_seller_url}' => $this->ets_marketplace->getShopLink(array('id_seller'=>$seller->id)),
                        '{product_id}' => $product->id,
                        '{product_name}' => $product->name[$context->language->id],
                        '{product_link}' => $context->link->getProductLink($product->id),
                        );
                        $subjects = array(
                            'translation' => $this->ets_marketplace->l('A new product has been uploaded','products'),
                            'origin'=>'A new product has been uploaded',
                            'specific'=>'products'
                        );
                        Ets_marketplace::sendMail('to_admin_new_product_uploaded',$data,'',$subjects);
                    }
                }
            }
            else
            {
                if(Tools::isSubmit('active') && Ets_mp_product::checkSellerUpdateStatusProduct($product->id) && (!$seller->checkVacation() || Tools::strpos($seller->vacation_type,'disable_product')===false))
                {
                    $product->active = (int)$active;
                }
                if($mpProduct)
                {
                    if($fields_changed)
                    {
                        $mpProduct->filed_change = implode(',',$fields_changed);
                        $mpProduct->status =-1;
                        if($mpProduct->id)
                            $mpProduct->update();
                        else
                        {
                            $mpProduct->id_product = $product->id;
                            $mpProduct->add();
                        }
                        if(Configuration::get('ETS_MP_EMAIL_ADMIN_PRODUCT_UPDATED'))
                        {
                            $data= array(
                            '{shop_seller}'=> $seller->shop_name[$context->language->id],
                            '{shop_seller_url}' => $this->ets_marketplace->getShopLink(array('id_seller'=>$seller->id)),
                            '{product_name}' => $product->name[$context->language->id],
                            '{product_link}' => $context->link->getProductLink($product->id),
                            );
                            $subjects = array(
                                'translation' => $this->ets_marketplace->l('There is a new product update','products'),
                                'origin'=>'There is a new product update',
                                'specific'=>'products'
                            );
                            Ets_marketplace::sendMail('to_admin_when_seller_update_product',$data,'',$subjects);
                        }
                    }
                    elseif($mpProduct->id)
                        $mpProduct->delete();
                    
                }
                $product->update();
            }
            if($product->id)
            {
                StockAvailable::setProductOutOfStock($product->id,$product->out_of_stock);
                if($this->ets_marketplace->is17)
                    Ets_mp_product::setLocation($product->id,$product_location);
                Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'category_product` WHERE id_product='.(int)$product->id.(isset($seller_categories) && $seller_categories ? ' AND id_category IN ('.implode(',',array_map('intval',$seller_categories)).')':''));
                Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'category_product` (id_product,id_category,position) VALUES("'.(int)$product->id.'","'.(int)"2".'","1")');
                if(($id_categories = Tools::getValue('id_categories')) && Ets_marketplace::validateArray($id_categories,'isInt'))
                {
                    foreach($id_categories as $id_category)
                    {
                        if(Validate::isUnsignedId($id_category) && Validate::isLoadedObject(new Category($id_category)))
                        {
                            if(!isset($seller_categories) || !$seller_categories ||  (isset($seller_categories) && $seller_categories && in_array($id_category,$seller_categories)))
                                Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'category_product` (id_product,id_category,position) VALUES("'.(int)$product->id.'","'.(int)$id_category.'","1")');
                        }
                    }
                }
                if(($related_products = Tools::getValue('related_products')) && Ets_marketplace::validateArray($related_products,'isInt'))
                {
                    foreach($related_products as $related_product)
                    {
                        if($related_product!=$product->id && Validate::isLoadedObject(new Product($related_product)) && !Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'accessory` WHERE id_product_1="'.(int)$product->id.'" AND id_product_2="'.(int)$related_product.'"'))
                        {
                            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'accessory`(id_product_1,id_product_2) VALUES("'.(int)$product->id.'","'.(int)$related_product.'")');
                        }    
                    }
                    Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'accessory` WHERE id_product_1="'.(int)$product->id.'" AND id_product_2 NOT IN ('.implode(',',array_map('intval',$related_products)).')');
                }
                else
                    Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'accessory` WHERE id_product_1="'.(int)$product->id.'"');
                if($this->ets_marketplace->_use_feature)
                {
                    Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'feature_product` WHERE id_product='.(int)$product->id);
                    if(($id_features = Tools::getValue('id_features')) && Ets_marketplace::validateArray($id_features,'isInt') && ($id_feature_values = Tools::getValue('id_feature_values')) && Ets_marketplace::validateArray($id_feature_values,'isInt') && ($feature_value_custom = Tools::getValue('feature_value_custom')) && Ets_marketplace::validateArray($feature_value_custom))
                    {
                        foreach($id_features as $key=> $id_feature)
                        {
                            if($id_feature)
                            {
                                if(isset($id_feature_values[$key]) && $id_feature_values[$key])
                                {
                                    if(!Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'feature_product` WHERE id_product="'.(int)$product->id.'" AND id_feature = "'.(int)$id_feature.'" AND id_feature_value="'.(int)$id_feature_values[$key].'"'))
                                    {
                                        Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'feature_product`(id_product,id_feature,id_feature_value) VALUES("'.(int)$product->id.'","'.(int)$id_feature.'","'.(int)$id_feature_values[$key].'")');
                                    }
                                }
                                elseif(isset($feature_value_custom[$key]) && $feature_value_custom[$key])
                                {
                                    $feature_value = new FeatureValue();
                                    $feature_value->id_feature = $id_feature;
                                    $feature_value->custom=1;
                                    foreach($languages as $language)
                                        $feature_value->value[$language['id_lang']] = $feature_value_custom[$key];
                                    if($feature_value->add())
                                        Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'feature_product`(id_product,id_feature,id_feature_value) VALUES("'.(int)$product->id.'","'.(int)$id_feature.'","'.(int)$feature_value->id.'")');
                                }
                            }
                        }
                    }
                }
                if($languages)
                {
                    foreach($languages as $language)
                    {
                        if($product->link_rewrite[$language['id_lang']] && Ets_mp_product::checkExistLinkWrite($product->link_rewrite[$language['id_lang']],$language['id_lang'],$product->id))
                        {
                            $product->link_rewrite[$language['id_lang']] .= '-'.$product->id;
                        }
                        if(($tags = Tools::getValue('tags_'.$language['id_lang'])) && Validate::isCleanHtml($tags))
                        {
                            Ets_mp_product::deleteProducttag($product->id,$language['id_lang']);
                            Tag::addTags($language['id_lang'],$product->id,$tags);
                        }
                        else
                            Ets_mp_product::deleteProducttag($product->id,$language['id_lang']);
                    }
                    
                }

                if ($product->update()) {
                    $product->id;
                } else {
                    return false;
                }
            }    

            return $product->id;
        }

        private function _completeProduct($id_product, $context)
        {

            $product_type = Tools::getValue('product_type');
            $seller = $this->ets_marketplace->_getSeller(true);
            $seller_product_information = explode(',', Configuration::get('ETS_MP_SELLER_ALLOWED_INFORMATION_SUBMISSION'));

            if(($id_product) && Validate::isUnsignedId($id_product))
            {
                if(($seller_product = $seller->checkHasProduct($id_product)) && isset($seller_product['id_product']) && ($id_product = $seller_product['id_product']))
                {
                    $product = new Product($id_product);
                    if(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED') && Configuration::get('ETS_MP_EDIT_PRODUCT_APPROVE_REQUIRED') && $seller_product['approved']==1 && ($fields = Configuration::get('ETS_MP_FIELD_PRODUCT_APPROVE_REQUIRED')))
                    {
                        $list_field_approve = explode(',', $fields);
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                    else {
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                } else {
                    return false;
                }
                $product->loadStockData();
            } else {
                $product = new Product();
            }

            $languages = Language::getLanguages(false);
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            $combinations_id_product_attribute = Tools::getValue('combinations_id_product_attribute');
            $show_variations = Tools::getValue('show_variations');

            if($product_type == 0 && $show_variations && $combinations_id_product_attribute && Ets_marketplace::validateArray($combinations_id_product_attribute,'isInt'))
            {
                $combinations_attribute_default = Tools::getValue('combinations_attribute_default') ?: array();
                $combinations_attribute_quantity = Tools::getValue('combinations_attribute_quantity');
                $combinations_attribute_available_date = Tools::getValue('combinations_attribute_available_date');
                $combinations_attribute_minimal_quantity= Tools::getValue('combinations_attribute_minimal_quantity');
                $combinations_attribute_reference = Tools::getValue('combinations_attribute_reference');
                $combinations_attribute_location = Tools::getValue('combinations_attribute_location');
                if($this->ets_marketplace->is17) {
                    $combinations_attribute_low_stock_threshold = Tools::getValue('combinations_attribute_low_stock_threshold');
                } else {
                    $combinations_attribute_low_stock_threshold = array();
                }
                $combinations_attribute_wholesale_price= Tools::getValue('combinations_attribute_wholesale_price');
                $combinations_attribute_price = Tools::getValue('combinations_attribute_price');
                $combinations_attribute_unity = Tools::getValue('combinations_attribute_unity');
                $combinations_attribute_weight = Tools::getValue('combinations_attribute_weight');
                $combinations_attribute_isbn = Tools::getValue('combinations_attribute_isbn');
                $combinations_attribute_ean13= Tools::getValue('combinations_attribute_ean13');
                $combinations_attribute_upc = Tools::getValue('combinations_attribute_upc');
                $combination_id_image_attr = Tools::getValue('combination_id_image_attr') ?: array();
                $combination_attribute_low_stock_alert = Tools::getValue('combination_attribute_low_stock_alert')?:array();
                if(Ets_marketplace::validateArray($combinations_attribute_low_stock_threshold) && Ets_marketplace::validateArray($combination_id_image_attr) && Ets_marketplace::validateArray($combinations_attribute_upc) && Ets_marketplace::validateArray($combinations_attribute_ean13) && Ets_marketplace::validateArray($combinations_attribute_isbn) && Ets_marketplace::validateArray($combinations_attribute_weight) && Ets_marketplace::validateArray($combinations_attribute_unity) && Ets_marketplace::validateArray($combinations_attribute_price) && Ets_marketplace::validateArray($combinations_attribute_wholesale_price) && Ets_marketplace::validateArray($combination_attribute_low_stock_alert) &&  Ets_marketplace::validateArray($combinations_attribute_default) && Ets_marketplace::validateArray($combinations_attribute_quantity) && Ets_marketplace::validateArray($combinations_attribute_available_date) && Ets_marketplace::validateArray($combinations_attribute_minimal_quantity) && Ets_marketplace::validateArray($combinations_attribute_reference) && Ets_marketplace::validateArray($combinations_attribute_location))
                {
                    foreach($combinations_id_product_attribute as $id_product_attribute)
                    {
                        $combination = new Combination($id_product_attribute);
                        
                        if(Validate::isLoadedObject($combination) && $combination->id_product == $product->id)
                        {
                            if(isset($combinations_attribute_default[$id_product_attribute]))
                            {
                                $combination->default_on = (int)$combinations_attribute_default[$id_product_attribute];
                                if($combination->default_on)
                                    Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'product_attribute` SET default_on=0 WHERE default_on=1 AND id_product="'.(int)$this->product->id.'" AND id_product_attribute!="'.(int)$id_product_attribute.'"');
                            }
                            else
                                $combination->default_on=0; 
                            if(isset($combinations_attribute_quantity[$id_product_attribute]) && Validate::isUnsignedInt($combinations_attribute_quantity[$id_product_attribute]))
                                $combination->quantity = (int)$combinations_attribute_quantity[$id_product_attribute];
                            else
                                $combination->quantity=0;
                            if(isset($combinations_attribute_available_date[$id_product_attribute]) && Validate::isDate($combinations_attribute_available_date[$id_product_attribute]))
                                $combination->available_date = $combinations_attribute_available_date[$id_product_attribute];
                            else
                                $combination->available_date ='0000-00-00';
                            if(isset($combinations_attribute_minimal_quantity[$id_product_attribute]) && Validate::isUnsignedInt($combinations_attribute_minimal_quantity[$id_product_attribute]))
                                $combination->minimal_quantity = (int)$combinations_attribute_minimal_quantity[$id_product_attribute];
                            else
                                $combination->minimal_quantity=0;
                            if(isset($combinations_attribute_reference[$id_product_attribute]) && Validate::isReference($combinations_attribute_reference[$id_product_attribute]))
                                $combination->reference = $combinations_attribute_reference[$id_product_attribute];
                            else
                                $combination->reference='';
                            if(isset($combinations_attribute_location[$id_product_attribute]) && Validate::isGenericName($combinations_attribute_location[$id_product_attribute]))
                                $combination->location = $combinations_attribute_location[$id_product_attribute];
                            else
                                $combination->location='';
                            if($this->ets_marketplace->is17)
                            {
                                if(isset($combinations_attribute_low_stock_threshold[$id_product_attribute]) && Validate::isInt($combinations_attribute_low_stock_threshold[$id_product_attribute]))
                                    $combination->low_stock_threshold = (int)$combinations_attribute_low_stock_threshold[$id_product_attribute];
                                else
                                    $combination->low_stock_threshold =0;
                                if(isset($combination_attribute_low_stock_alert[$id_product_attribute]) && Validate::isInt($combination_attribute_low_stock_alert[$id_product_attribute]))
                                    $combination->low_stock_alert = (int)$combination_attribute_low_stock_alert[$id_product_attribute];
                                else
                                    $combination->low_stock_alert=0;
                            }
                            if(isset($combinations_attribute_wholesale_price[$id_product_attribute]) && Validate::isPrice($combinations_attribute_wholesale_price[$id_product_attribute]))
                                $combination->wholesale_price = (float)$combinations_attribute_wholesale_price[$id_product_attribute];
                            else
                                $combination->wholesale_price =0;
                            if(isset($combinations_attribute_price[$id_product_attribute]) && Validate::isNegativePrice($combinations_attribute_price[$id_product_attribute]))
                                $combination->price = (float)$combinations_attribute_price[$id_product_attribute];
                            else
                                $combination->price = 0;
                            if(isset($combinations_attribute_unity[$id_product_attribute]) && Validate::isNegativePrice($combinations_attribute_unity[$id_product_attribute]))
                                $combination->unit_price_impact = $combinations_attribute_unity[$id_product_attribute];
                            else
                                $combination->unit_price_impact = 0; 
                            if(isset($combinations_attribute_weight[$id_product_attribute]) && Validate::isUnsignedFloat($combinations_attribute_weight[$id_product_attribute]))
                                $combination->weight = (float)$combinations_attribute_weight[$id_product_attribute]; 
                            else
                                $combination->weight =0; 
                            if(isset($combination->isbn) && isset($combinations_attribute_isbn[$id_product_attribute]) && Validate::isIsbn($combinations_attribute_isbn[$id_product_attribute]))
                                $combination->isbn = $combinations_attribute_isbn[$id_product_attribute];
                            else
                                $combination->isbn=''; 
                            if(isset($combinations_attribute_ean13[$id_product_attribute]) && Validate::isEan13($combinations_attribute_ean13[$id_product_attribute]))
                                $combination->ean13 = $combinations_attribute_ean13[$id_product_attribute];
                            else
                                $combination->ean13='';
                            if(isset($combinations_attribute_upc[$id_product_attribute]) && Validate::isUpc($combinations_attribute_upc[$id_product_attribute]))
                                $combination->upc = $combinations_attribute_upc[$id_product_attribute];
                            else
                                $combination->upc = '';
                            
                            if($combination->update())
                            {
                                StockAvailable::setQuantity($product->id, (int)$id_product_attribute, $combination->quantity);
                                if($this->ets_marketplace->is17)
                                    Ets_mp_product::setLocation($product->id,$combination->location,null,$combination->id);
                                Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'product_attribute_image` WHERE id_product_attribute='.(int)$id_product_attribute);
                                if(isset($combination_id_image_attr[$id_product_attribute]) && $combination_id_image_attr[$id_product_attribute])
                                {
                                    foreach($combination_id_image_attr[$id_product_attribute] as $id_image)
                                    {
                                        if(!Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'product_attribute_image` WHERE id_product_attribute="'.(int)$id_product_attribute.'" AND id_image="'.(int)$id_image.'"'))
                                            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'product_attribute_image`(id_product_attribute,id_image) VALUES("'.(int)$id_product_attribute.'","'.(int)$id_image.'")');
                                    }
                                }
                            }               
                        }
                    } 
                }
            }
            else
            {
                $product->deleteProductAttributes();
            }
            if($product_type == 1)
            {
                Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'pack` WHERE id_product_pack='.(int)$product->id);
                if(($inputPackItems = Tools::getValue('inputPackItems')) && Ets_marketplace::validateArray($inputPackItems))
                {
                    foreach($inputPackItems as $inputPackItem)
                    {
                        $packItem = explode('x',$inputPackItem);
                        $id_product_item = $packItem[0];
                        $id_product_attribute_item = $packItem[1];
                        $quantity_item = $packItem[2];
                        if(!Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'pack` WHERE id_product_pack="'.(int)$product->id.'" AND id_product_item="'.(int)$id_product_item.'" AND id_product_attribute_item="'.(int)$id_product_attribute_item.'"'))
                        {
                            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'pack`(id_product_pack,id_product_item,id_product_attribute_item,quantity) VALUES("'.(int)$product->id.'","'.(int)$id_product_item.'","'.(int)$id_product_attribute_item.'","'.(int)$quantity_item.'")');
                        }
                    }
                }

            }
            else
            {
                Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'pack` WHERE id_product_pack='.(int)$product->id);
            }
            // if($product_type==2)
            // {
            //     $virtual = $this->updateDownloadProduct($product);
            // }
            if($product_download = Db::getInstance()->getRow('SELECT id_product_download,filename FROM `'._DB_PREFIX_.'product_download` WHERE id_product="'.(int)$product->id.'"'))
            {
                $productDownload = new ProductDownload($product_download['id_product_download']);
                $productDownload->delete($product_download['filename']? true:false);
            }
            Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'product_carrier` WHERE id_product='.(int)$product->id);
            if($product_type!=2 && ($selectedCarriers = Tools::getValue('selectedCarriers')) && Ets_marketplace::validateArray($selectedCarriers,'isInt'))
            {
                foreach($selectedCarriers as $selectedCarrier)
                {
                    if($selectedCarrier)
                    {
                        Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'product_carrier`(id_product,id_carrier_reference,id_shop) VALUES("'.(int)$product->id.'","'.(int)$selectedCarrier.'","'.(int)$context->shop->id.'")');
                    }
                }
            }
            $id_customization_fields = array();
            $uploadable_files =0;
            $text_fields = 0;
            if(($custom_fields = Tools::getValue('custom_fields')) && Ets_marketplace::validateArray($custom_fields))
            {
                Configuration::updateValue('PS_CUSTOMIZATION_FEATURE_ACTIVE',1);
                foreach($custom_fields as $custom_field)
                {
                    if($id_customization_field = $custom_field['id_customization_field'])
                        $customizationField = new CustomizationField($id_customization_field);
                    else
                    {
                        $customizationField = new CustomizationField();
                        $customizationField->id_product=  $product->id;
                    }
                    foreach($languages as $language)
                    {
                        $customizationField->name[$language['id_lang']] = $custom_field['label'][$language['id_lang']] ? : $custom_field['label'][$id_lang_default];
                    }
                    $customizationField->type = (int)$custom_field['type'];
                    if($customizationField->type==1)
                        $text_fields +=1;
                    else
                        $uploadable_files +=1;
                    if(isset($custom_field['required']))
                        $customizationField->required = $custom_field['required'];
                    else    
                        $customizationField->required = 0;
                    if($customizationField->id)
                    {
                        $id_customization_fields[] = $customizationField->id;
                        $customizationField->update();
                    }
                    elseif($customizationField->add())
                        $id_customization_fields[] = $customizationField->id;
                }
            }
            if(!$this->ets_marketplace->is17 && ($product->uploadable_files!=$uploadable_files || $product->text_fields!= $text_fields))
            {
                $product->uploadable_files = $uploadable_files;
                $product->text_fields = $text_fields;
                $product->update();
            }
            if($this->ets_marketplace->is17)
                Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'customization_field` SET is_deleted=1 WHERE id_product="'.(int)$product->id.'"'.($id_customization_fields ? ' AND id_customization_field NOT IN ('.implode(',',array_map('intval',$id_customization_fields)).')':''));
            else
            {
               $ids =  Db::getInstance()->executeS('SELECT id_customization_field FROM `'._DB_PREFIX_.'customization_field` WHERE id_product="'.(int)$product->id.'"'.($id_customization_fields ? ' AND id_customization_field NOT IN ('.implode(',',array_map('intval',$id_customization_fields)).')':''));
               if($ids)
               {
                    foreach($ids as $id)
                    {
                        $customizationField = new CustomizationField($id['id_customization_field']);
                        $customizationField->delete();
                    }
               }
            }
            if($seller->auto_enabled_product=='yes')
                $allow_active=1;
            elseif($seller->auto_enabled_product=='no')
                $allow_active=0;
            elseif(!Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED'))
                $allow_active=1;
            else
                $allow_active=0;
            // $this->_submitProductSupplier();
            $adtoken = Tools::getAdminToken(
                'AdminProducts'
                . (int) Tab::getIdFromClassName('AdminProducts')
                . $context->customer->id
            );

            return true;
            // die(
            //     Tools::jsonEncode(
            //         array(
            //             'success' => $this->product->active==0 && !$allow_active ? $this->module->l('Your new product has just been submitted successfully. It is waiting to be approved by Administrator','products'): (($mpproduct =  Ets_mp_product::getMpProductByIdProduct($this->product->id)) && Validate::isLoadedObject($mpproduct) ? $this->module->l('Submitted successfully. It is waiting to be approved by Administrator','products') : $this->module->l('Submitted successfully','products') ),
            //             'virtual' => isset($virtual) && $virtual ? array('link_download_file' => $this->context->link->getModuleLink($this->module->name,'products',array('id_product'=>$this->product->id,'downloadfileproduct'=>1)),'link_delete_file' => $this->context->link->getModuleLink($this->module->name,'products',array('id_product'=>$this->product->id,'deletefileproduct'=>1)),):false,
            //             'list_combinations' => $this->displayListCombinations(),
            //             'id_product'=> $this->product->id,
            //             'link_product' => $this->product->active ? $this->context->link->getProductLink($this->product->id) : $this->context->link->getProductLink($this->product->id,null,null,null,null,null,null,false,false,false,array('adtoken'=>$adtoken,'id_employee'=>$this->context->customer->id,'preview'=>1)),
            //             'preview_text' => $this->module->l('Preview','products'),
            //             'save_text' => $this->module->l('Save','products'),
            //             'html_form_supplier' => $this->renderFormSupplier(),                            
            //         )
            //     )
            // );
        }

        private function _createCombination($id_product, $context)
        {
            $seller = $this->ets_marketplace->_getSeller(true);
            $seller_product_information = explode(',', Configuration::get('ETS_MP_SELLER_ALLOWED_INFORMATION_SUBMISSION'));

            if(($id_product) && Validate::isUnsignedId($id_product))
            {
                if(($seller_product = $seller->checkHasProduct($id_product)) && isset($seller_product['id_product']) && ($id_product = $seller_product['id_product']))
                {
                    $product = new Product($id_product);
                    if(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED') && Configuration::get('ETS_MP_EDIT_PRODUCT_APPROVE_REQUIRED') && $seller_product['approved']==1 && ($fields = Configuration::get('ETS_MP_FIELD_PRODUCT_APPROVE_REQUIRED')))
                    {
                        $list_field_approve = explode(',', $fields);
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                    else {
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                } else {
                    return false;
                }
                $product->loadStockData();
            } else {
                $product = new Product();
            }

            $product->is_virtual = 0;
            $product->update();
            if($product_download = Db::getInstance()->getRow('SELECT id_product_download,filename FROM `'._DB_PREFIX_.'product_download` WHERE id_product="'.(int)$product->id.'"'))
            {
                $productDownload = new ProductDownload($product_download['id_product_download']);
                $productDownload->delete($product_download['filename']? true:false);
            }

            $attribute_options = Tools::getValue('attribute_options');

            if (!Ets_marketplace::validateArray($attribute_options)) {
                return false;
            } else {
                $tab = array_values($attribute_options);

                if (count($tab) && Validate::isLoadedObject($product)) {
                    
                    self::setAttributesImpacts($product->id, $tab);
                    $combinations = array_values(Ets_marketplace::createCombinations($tab));
                    $values = array_values(array_map(function ($attributes, $price = 0, $weight = 0) use ($product) {
                        $quantity = Tools::getValue('quantity');
                        $reference = Tools::getValue('reference');
                        if ($product->id) {
                            return array(
                                'id_product' => (int)$product->id,
                                'price' => (float)$price,
                                'weight' => (float)$weight,
                                'ecotax' => 0,
                                'quantity' => $quantity ? (int) $quantity : 0,
                                'reference' => Validate::isReference($reference) ? $reference:'',
                                'default_on' => 0,
                                'available_date' => '0000-00-00'
                            );
                        }
                        unset($attributes);
                        return array();
                    }, $combinations));

                    // @since 1.5.0
                    if ($this->ets_marketplace->is15 && $product->depends_on_stock == 0) {
                        $attributes = Product::getProductAttributesIds($product->id, true);
                        foreach ($attributes as $attribute) {
                            StockAvailable::removeProductFromStockAvailable($product->id, $attribute['id_product_attribute'], $context->shop);
                        }
                    }
    
                    SpecificPriceRule::disableAnyApplication();
    
                    //$this->product->deleteProductAttributes();
                    if(!$this->ets_marketplace->is17)
                    {
                        Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'product_attribute` SET default_on=NULL WHERE id_product="'.(int)$product->id.'" AND default_on=1');
                        Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'product_attribute_shop` SET default_on=NULL WHERE id_product="'.(int)$product->id.'" AND default_on=1');
                    }    
                    $product->generateMultipleCombinations($values, $combinations, false);
                    // Reset cached default attribute for the product and get a new one
                    Product::getDefaultAttribute($product->id, 0, true);
                    Product::updateDefaultAttribute($product->id);
                    // @since 1.5.0
                    // if ($this->ets_marketplace->is15 && $product->depends_on_stock == 0) {
                    //     $attributes = Product::getProductAttributesIds($product->id, true);
                    //     $quantity = (int)Tools::getValue('quantity');
                    //     foreach ($attributes as $attribute) {
                    //         if (Shop::getContext() == Shop::CONTEXT_ALL) {
                    //             $shops_list = Shop::getShops();
                    //             if (is_array($shops_list)) {
                    //                 foreach ($shops_list as $current_shop) {
                    //                     if (isset($current_shop['id_shop']) && (int)$current_shop['id_shop'] > 0) {
                    //                         StockAvailable::setQuantity($product->id, (int)$attribute['id_product_attribute'], $quantity, (int)$current_shop['id_shop']);
                    //                     }
                    //                 }
                    //             }
                    //         } else {
                    //             StockAvailable::setQuantity($product->id, (int)$attribute['id_product_attribute'], $quantity);
                    //         }
                    //     }
                    // } else {
                    //     StockAvailable::synchronize($product->id);
                    // }

                    $attributes = Product::getProductAttributesIds($product->id, true);
                    $quantity = (int)1;
                    foreach ($attributes as $attribute) {
                        if (Shop::getContext() == Shop::CONTEXT_ALL) {
                            $shops_list = Shop::getShops();
                            if (is_array($shops_list)) {
                                foreach ($shops_list as $current_shop) {
                                    if (isset($current_shop['id_shop']) && (int)$current_shop['id_shop'] > 0) {
                                        StockAvailable::setQuantity($product->id, (int)$attribute['id_product_attribute'], $quantity, (int)$current_shop['id_shop']);
                                    }
                                }
                            }
                        } else {
                            StockAvailable::setQuantity($product->id, (int)$attribute['id_product_attribute'], $quantity);
                        }
                    }
    
                    SpecificPriceRule::enableAnyApplication();
                    SpecificPriceRule::applyAllRules(array((int)$product->id));
                    $id_product_attribute = Product::getDefaultAttribute((int)$product->id);
                    Db::getInstance()->execute('update `'._DB_PREFIX_.'cart_product`  set id_product_attribute="'.(int)$id_product_attribute.'" where id_product="'.(int)$product->id.'" AND id_product_attribute=0');
                
                    return true;
                } else {
                    return false;
                }
            }
        }

        private function _createProductImage($id_product, $context)
        {
            $seller = $this->ets_marketplace->_getSeller(true);
            $seller_product_information = explode(',', Configuration::get('ETS_MP_SELLER_ALLOWED_INFORMATION_SUBMISSION'));

            if(($id_product) && Validate::isUnsignedId($id_product))
            {
                if(($seller_product = $seller->checkHasProduct($id_product)) && isset($seller_product['id_product']) && ($id_product = $seller_product['id_product']))
                {
                    $product = new Product($id_product);
                    if(Configuration::get('ETS_MP_SELLER_PRODUCT_APPROVE_REQUIRED') && Configuration::get('ETS_MP_EDIT_PRODUCT_APPROVE_REQUIRED') && $seller_product['approved']==1 && ($fields = Configuration::get('ETS_MP_FIELD_PRODUCT_APPROVE_REQUIRED')))
                    {
                        $list_field_approve = explode(',', $fields);
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                    else {
                        $mpProduct = Ets_mp_product::getMpProductByIdProduct($id_product);
                    }
                } else {
                    return false;
                }
                $product->loadStockData();
            } else {
                $product = new Product();
            }

            $inputFileName = 'upload_image';

            $image_uploader = new HelperImageUploader($inputFileName);

            $file_errors = [];

            $this->ets_marketplace->validateFile($_FILES[$inputFileName]['name'],$_FILES[$inputFileName]['size'], $file_errors,array('jpeg', 'gif', 'png', 'jpg'),Configuration::get('PS_LIMIT_UPLOAD_IMAGE_VALUE')*1024*1024);
            
            $image_uploader->setAcceptTypes(array('jpeg', 'gif', 'png', 'jpg'))->setMaxSize(null);
            $files = $image_uploader->process();

            foreach ($files as &$file) {

                $file_errors = [];

                $image = new Image();
                if($file['error']) {
                    $file_errors[] = $file['error'];
                    continue;
                } else {
                    if($list_field_approve && in_array('image', $list_field_approve))
                    {
                        $image->id_product =0;
                        $image->cover =0;
                        $image->position =1+ (int)Db::getInstance()->getValue('SELECT MAX(i.position) FROM `'._DB_PREFIX_.'image` i 
                        LEFT JOIN `'._DB_PREFIX_.'ets_mp_product_image` mpi ON(i.id_image = mpi.id_image)
                        WHERE i.id_product="'.(int)$product->id.'" AND mpi.id_product="'.(int)$product->id.'"');
                        $image->cover = 0;
           
                    } else {
                        $image->id_product = (int) ($product->id);
                        $image->position = Image::getHighestPosition($product->id) + 1;
                        if (!Ets_mp_product::getCover($image->id_product)) {
                            $image->cover = 1;
                        } else {
                            $image->cover = 0;
                        }
                    }
                    if (($validate = $image->validateFieldsLang(false, true)) !== true) {
                        $file_errors[] = $validate;
                    }
        
                    if ($file_errors) {
                        continue;
                    }
                    if (!$image->add()) {
                        $file_errors[] = $this->ets_marketplace->l('An error occurred while creating additional image','products');
                    } else {
                        if($image->id_product==0)
                        {
                            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'ets_mp_product_image`(id_product,id_image) VALUES("'.(int)$product->id.'","'.(int)$image->id.'")');
                            $mpProduct->status=-1;
                            $fields_changed = $mpProduct->filed_change ? explode(',',$mpProduct->filed_change):array();
                            if(!in_array('image',$fields_changed))
                            {
                                $fields_changed[] ='image';
                                $mpProduct->filed_change = implode(',',$fields_changed);
                            }
                            
                            if($mpProduct->id)
                                $mpProduct->update();
                            else
                            {
                                $mpProduct->id_product = $product->id;
                                $mpProduct->add();
                            }
                        }
                        if (!$new_path = $image->getPathForCreation()) {
                            $file_errors[] = $this->ets_marketplace->l('An error occurred while attempting to create a new folder.','products');
                            continue;
                        }
                        $error = 0;
                        if (!ImageManager::resize($file['save_path'], $new_path . '.' . $image->image_format, null, null, 'jpg', false, $error)) {
                            switch ($error) {
                                case ImageManager::ERROR_FILE_NOT_EXIST:
                                    $file_errors[] = $this->ets_marketplace->l('An error occurred while copying image, the file does not exist anymore.','products');
                                    break;
                                case ImageManager::ERROR_FILE_WIDTH:
                                    $file_errors[] = $this->ets_marketplace->l('An error occurred while copying image, the file width is 0px.','products');
                                    break;
                                case ImageManager::ERROR_MEMORY_LIMIT:
                                    $file_errors[] = $this->ets_marketplace->l('An error occurred while copying image, check your memory limit.','products');
                                    break;
                                default:
                                    $file_errors[] = $this->ets_marketplace->l('An error occurred while copying the image.','products');
                                    break;
                            }
        
                            continue;
                        } else {
                            $imagesTypes = ImageType::getImagesTypes('products');
                            $generate_hight_dpi_images = (bool) Configuration::get('PS_HIGHT_DPI');
                            foreach ($imagesTypes as $imageType) {
                                if (!ImageManager::resize($file['save_path'], $new_path . '-' . Tools::stripslashes($imageType['name']) . '.' . $image->image_format, $imageType['width'], $imageType['height'], $image->image_format)) {
                                    $file_errors[] =$this->ets_marketplace->l('An error occurred while copying this image:','products').' ' . Tools::stripslashes($imageType['name']);
                                    continue;
                                }
        
                                if ($generate_hight_dpi_images) {
                                    if (!ImageManager::resize($file['save_path'], $new_path . '-' . Tools::stripslashes($imageType['name']) . '2x.' . $image->image_format, (int) $imageType['width'] * 2, (int) $imageType['height'] * 2, $image->image_format)) {
                                        $file_errors[] = $this->ets_marketplace->l('An error occurred while copying this image:','products') . ' ' . Tools::stripslashes($imageType['name']);
                                        continue;
                                    }
                                }
                            }
                        }
        
                        unlink($file['save_path']);
                        unset($file['save_path']);
                        Hook::exec('actionWatermark', array('id_image' => $image->id, 'id_product' => $product->id));
                        if (!$image->update()) {
                            $this->ets_marketplace->l('An error occurred while updating the status.','products');
                            continue;
                        }
                        $shops = Shop::getContextListShopID();
                        $image->associateTo($shops);
                        $json_shops = array();
                        foreach ($shops as $id_shop) {
                            $json_shops[$id_shop] = true;
                        }
                        $file['status'] = 'ok';
                        $file['id'] = $image->id;
                        $file['position'] = $image->position;
                        $file['cover'] = $image->cover;
                        $file['legend'] = $image->legend;
                        $file['path'] = $image->getExistingImgPath();
                        $file['shops'] = $json_shops;
                        @unlink(_PS_TMP_IMG_DIR_ . 'product_' . (int) $product->id . '.jpg');
                        @unlink(_PS_TMP_IMG_DIR_ . 'product_mini_' . (int) $product->id . '_' . $context->shop->id . '.jpg');
                        @unlink(_PS_TMP_IMG_DIR_ . 'product_mini_' . (int) $product->id . '_0.jpg');
                    }
                }
            }

            return true;
        }

        protected static function setAttributesImpacts($id_product, $tab)
        {
            $attributes = array();
            foreach ($tab as $group) {
                foreach ($group as $attribute) {
                    $price = 0;
                    $weight = 0;
                    $attributes[] = '('.(int)$id_product.', '.(int)$attribute.', '.(float)$price.', '.(float)$weight.')';
                }
            }

            return Db::getInstance()->execute('
            INSERT INTO `'._DB_PREFIX_.'attribute_impact` (`id_product`, `id_attribute`, `price`, `weight`)
            VALUES '.implode(',', $attributes).'
            ON DUPLICATE KEY UPDATE `price` = VALUES(price), `weight` = VALUES(weight)');
        }

        public function getProductList($id_order, $context)
        {
            $products = Db::getInstance()->executeS('SELECT product_name,product_id,product_attribute_id,sum(product_quantity) as product_quantity FROM `'._DB_PREFIX_.'order_detail` WHERE id_order='.(int)$id_order.' GROUP BY product_id,product_attribute_id ORDER BY id_order_detail ASC' );
            if($products)
            {
                if(version_compare(_PS_VERSION_, '1.7', '>='))
                    $type_image= ImageType::getFormattedName('small');
                else
                    $type_image= ImageType::getFormatedName('small');
                foreach($products as &$product)
                {
                    $product_class = new Product($product['product_id'],false,$context->language->id);
                    if(Db::getInstance()->getRow('SELECT id_product FROM `'._DB_PREFIX_.'product` WHERE id_product='.(int)$product['product_id'].' AND active=1'))
                    {
                        $image=false;
                        if($product['product_attribute_id'])
                        {
                            $sql = 'SELECT * FROM `'._DB_PREFIX_.'product_attribute_image` pai
                            INNER JOIN `'._DB_PREFIX_.'image` i ON pai.id_image=i.id_image WHERE pai.id_product_attribute='.(int)$product['product_attribute_id'];
                            if(!$image = Db::getInstance()->getRow($sql.' AND i.cover=1'))
                                $image  = Db::getInstance()->getRow($sql);
                        }
                        if(!$image)
                        {
                            $sql = 'SELECT i.id_image FROM `'._DB_PREFIX_.'image` i';
                            if($product['product_attribute_id'])
                                $sql .= ' LEFT JOIN `'._DB_PREFIX_.'product_attribute_image` pai ON (i.id_image=pai.id_image AND pai.id_product_attribute="'.(int)$product['product_attribute_id'].'")';
                            $sql .= ' WHERE i.id_product="'.(int)$product['product_id'].'"';
                            if(!$image = Db::getInstance()->getRow($sql.' AND i.cover=1'))
                            {
                                $image = Db::getInstance()->getRow($sql);
                            }
                        }
                        
                        if($image)
                        {
                            $product['image'] =  $context->link->getImageLink($product_class->link_rewrite,$image['id_image'],$type_image);
                        }
                        else
                        {
                            $product['image'] = '';
                        }
                    }
                    else
                        $product['deleted'] = true;
                    
                }
            }
            
            return $products;
        }

        protected function getProducts($order)
        {
            $products = $order->getProducts();
            foreach ($products as &$product) {
                if ($product['image'] != null) {
                    $name = 'product_mini_' . (int)$product['product_id'] . (isset($product['product_attribute_id']) ? '_' . (int)$product['product_attribute_id'] : '') . '.jpg';
                    // generate image cache, only for back office
                    $product['image_tag'] = ImageManager::thumbnail(_PS_IMG_DIR_ . 'p/' . $product['image']->getExistingImgPath() . '.jpg', $name, 45, 'jpg');
                    if (file_exists(_PS_TMP_IMG_DIR_ . $name)) {
                        $product['image_size'] = getimagesize(_PS_TMP_IMG_DIR_ . $name);
                    } else {
                        $product['image_size'] = false;
                    }
                }
            }
            ksort($products);
            return $products;
        }

        protected function setProductImageInformations(&$pack_item)
        {
            if (isset($pack_item['id_product_attribute']) && $pack_item['id_product_attribute']) {
                $id_image = Db::getInstance()->getValue('
                    SELECT `image_shop`.id_image
                    FROM `' . _DB_PREFIX_ . 'product_attribute_image` pai' .
                    Shop::addSqlAssociation('image', 'pai', true) . '
                    WHERE id_product_attribute = ' . (int) $pack_item['id_product_attribute']);
            }

            if (!isset($id_image) || !$id_image) {
                $id_image = Db::getInstance()->getValue(
                    '
                    SELECT `image_shop`.id_image
                    FROM `' . _DB_PREFIX_ . 'image` i' .
                    Shop::addSqlAssociation('image', 'i', true, 'image_shop.cover=1') . '
                    WHERE i.id_product = ' . (int) $pack_item['id_product']
                );
            }

            $pack_item['image'] = null;
            $pack_item['image_size'] = null;

            if ($id_image) {
                $pack_item['image'] = new Image($id_image);
            }
        }

        public function getOrderStates($id_lang=false,$currentState, $context)
        {
            if(!$id_lang)
                $id_lang = $context->language->id;
            if(Configuration::get('ETS_MP_SELLER_ALLOWED_STATUSES'))
            {
                $status = explode(',',Configuration::get('ETS_MP_SELLER_ALLOWED_STATUSES'));
                $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                SELECT *
                FROM `' . _DB_PREFIX_ . 'order_state` os
                LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON (os.`id_order_state` = osl.`id_order_state` AND osl.`id_lang` = ' . (int) $id_lang . ')
                WHERE deleted = 0 AND (os.id_order_state = "'.(int)$currentState.'" || os.id_order_state IN ('.implode(',',array_map('intval',$status)).'))
                ORDER BY `name` ASC');
                return $result;
            }
            else
                return array();
        }

        protected function getCarrierList($order)
        {
            $cart = new Cart($order->id_cart);
            $address = new Address((int)$cart->id_address_delivery);
            return Carrier::getCarriersForOrder(Address::getZoneById((int)$address->id), null, $cart);
        }

        public static function getSellersByName($name)
        {
            $db = Db::getInstance(_PS_USE_SQL_SLAVE_);

            $query_string = $name;
    
            $sql = 'SELECT DISTINCT s.*, sl.*, seller_product.* ' .
            'FROM ' . _DB_PREFIX_ . 'ets_mp_seller s ' .
            'INNER JOIN ' . _DB_PREFIX_ . 'ets_mp_seller_lang sl ON sl.shop_name LIKE "%' . $query_string . '%" AND sl.id_seller = s.id_seller ' .
            ' LEFT JOIN (
                SELECT seller.id_seller,count(sp.id_product) as total_product FROM `'._DB_PREFIX_.'ets_mp_seller_product` sp
                INNER JOIN `'._DB_PREFIX_.'product` p ON (sp.id_product= p.id_product AND p.active=1)
                INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON(seller.id_customer=sp.id_customer)
                GROUP BY seller.id_seller
            ) as seller_product ON (seller_product.id_seller=s.id_seller)';
    
            $sellers_result = $db->executeS($sql, true, false);

            return $sellers_result;
        }

        public static function getSellerByOrderId($id_order, $context)
        {

            $order = new Order((int) $id_order);

            if (!$order || !Validate::isLoadedObject($order)) {
                return false;
            }

            $sql ='SELECT seller.id_seller FROM `'._DB_PREFIX_.'ets_mp_seller_order` o
            INNER JOIN `'._DB_PREFIX_.'customer` customer ON (o.id_customer = customer.id_customer)
            INNER JOIN `'._DB_PREFIX_.'ets_mp_seller` seller ON (seller.id_customer=customer.id_customer)
            WHERE o.id_order='.(int)$order->id;

            $id_seller = (int) Db::getInstance()->getValue($sql);

            if(!$id_seller) {
                return false;
            }

            if (file_exists(_PS_MODULE_DIR_ . 'ets_marketplace/classes/seller.php')) {
                require_once _PS_MODULE_DIR_ . 'ets_marketplace/classes/seller.php';
            } else {
                return false;
            }

            $seller = new Ets_mp_seller($id_seller, $context->language->id);

            return $seller;
        }
    }
}
