<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('freedeliverymanager') && Module::isEnabled('freedeliverymanager')) {
    require_once _PS_MODULE_DIR_ . 'freedeliverymanager/freedeliverymanager.php';

    class PrestAppFreedeliverymanagerCustomizationController
    {
        protected $freedeliverymanager;

        public function __construct()
        {
            $this->freedeliverymanager = Module::getInstanceByName('freedeliverymanager');
        }

        public function getInfos($context)
        {
            if (isset($context->cart)) {
                $cart = new Cart($context->cart->id);
                if ($cart->isVirtualCart()) {
                    return false;
                }
            }

            $show_bloc = (int) Configuration::get('FDM_BLOC_CONDITION');
            if (!isset($context->cart) || $context->cart->getProducts() == 0 || (int) $show_bloc == 0) {
                return false;
            }

            $id_group = Group::getCurrent()->id;
            if (isset($context->customer->id) && (int) $context->customer->id) {
                $customer = new Customer((int) $context->customer->id);
                $id_group = $customer->id_default_group;
            }

            $id_zoneaddress = null;

            if (isset($context->cookie->id_country) && $context->cookie->id_country > 0) {
                $id_country = (int) $context->cookie->id_country;
            }

            if (!isset($id_country)) {
                $id_country = (isset($context->customer->geoloc_id_country) ? (int) $context->customer->geoloc_id_country : (int) Configuration::get('PS_COUNTRY_DEFAULT'));
            }

            if (isset($cart->id_address_delivery) && (int) $cart->id_address_delivery > 0) {
                $id_zoneaddress = Address::getZoneById($cart->id_address_delivery);
            } else {
                $id_zoneaddress = Country::getIdZone($id_country);
            }

            $id_state = 0;

            if (isset($context->cookie->id_state) && $context->cookie->id_state > 0) {
                $id_state = (int) $context->cookie->id_state;
            }

            if (!isset($id_state)) {
                $id_state = (isset($context->customer->geoloc_id_state) ? (int) $context->customer->geoloc_id_state : 0);
            }

            $zipcode = '';

            if (isset($context->cookie->postcode) && $context->cookie->postcode > 0) {
                $zipcode = Tools::safeOutput($context->cookie->postcode);
            }

            if (!isset($zipcode) || empty($zipcode)) {
                $zipcode = (isset($context->customer->geoloc_postcode) ? $context->customer->geoloc_postcode : '');
            }

            $city = '';

            if (isset($context->cookie->city) && !empty($context->cookie->city)) {
                $city = Tools::htmlentitiesDecodeUTF8($context->cookie->city);
            }

            $grid = $this->freedeliverymanager->getGridFreeDelivery(
                $id_country,
                $id_state,
                $zipcode,
                $city,
                $id_zoneaddress,
                $cart,
                $id_group
            );

            $country = new Country($id_country, (int) $context->cookie->id_lang);
            $state = null;
            if ((int) $id_state) {
                $state = new State($id_state);
            }

            $grid['country_select'] = $country;
            $grid['city'] = $city;
            $grid['state'] = $state;
            $grid['id_state'] = $id_state;
            $grid['zipcode'] = $zipcode;
            $grid['block_text'] = $this->freedeliverymanager->l('Enjoy free shipping cost', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC');
            $grid['translation'] = $this->getTranslations();

            return $grid;
        }

        public function changeAddress($context, $id_country, $id_state = 0, $zipcode = 0, $city = '')
        {
            $id_zone = 0;
            if ($id_state != 0) {
                $id_zone = State::getIdZone($id_state);
            }
            if (!$id_zone) {
                $id_zone = Country::getIdZone($id_country);
            }
            $id_group = Group::getCurrent()->id;
            if (isset($context->customer->id) && (int) $context->customer->id) {
                $customer = new Customer((int) $context->customer->id);
                $id_group = $customer->id_default_group;
            }

            $grid = $this->freedeliverymanager->getGridFreeDelivery(
                $id_country,
                $id_state,
                $zipcode,
                $city,
                $id_zone,
                null,
                $id_group
            );

            $state = null;
            if ((int) $id_state) {
                $state = new State($id_state);
            }

            $grid['translation'] = $this->getTranslations();
            $grid['state'] = $state;
            $grid['city'] = $city;
            $grid['zipcode'] = $zipcode;

            return $grid;
        }

        private function getTranslations()
        {
            return array(
                'More than' => $this->freedeliverymanager->l('More than', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Tax excluded' => $this->freedeliverymanager->l('Tax excluded', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'purchase' => $this->freedeliverymanager->l('purchase', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'or' => $this->freedeliverymanager->l('or', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'of product' => $this->freedeliverymanager->l('of product', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'to enjoy free delivery' => $this->freedeliverymanager->l('to enjoy free delivery', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'The amount of your purchases must exceed the minimum' => $this->freedeliverymanager->l('The amount of your purchases must exceed the minimum', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'the weight of your products must exceed the minimum' => $this->freedeliverymanager->l('the weight of your products must exceed the minimum', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'For maximum weight of' => $this->freedeliverymanager->l('For maximum weight of', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'weight of your shopping cart :' => $this->freedeliverymanager->l('weight of your shopping cart :', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Shipping offered' => $this->freedeliverymanager->l('Shipping offered', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Grid shipping offered by carrier' => $this->freedeliverymanager->l('Grid shipping offered by carrier', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Free!' => $this->freedeliverymanager->l('Free!', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'However, the following costs may not be offered :' => $this->freedeliverymanager->l('However, the following costs may not be offered :', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'corresponding to the handling charges.' => $this->freedeliverymanager->l('corresponding to the handling charges.', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'corresponding to the additional shipping costs associated with some products in the cart.' => $this->freedeliverymanager->l('corresponding to the additional shipping costs associated with some products in the cart.', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Please see shipping tab to control if carrier available' => $this->freedeliverymanager->l('Please see shipping tab to control if carrier available', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Sorry, not applicable for this selection' => $this->freedeliverymanager->l('Sorry, not applicable for this selection', 'FREEDELIVERYMANAGER-CARRIER-GRILLE-CLASSIC'),
                'Address :' => $this->freedeliverymanager->l('Address :', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'Change' => $this->freedeliverymanager->l('Change', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'Update' => $this->freedeliverymanager->l('Update', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'Country' => $this->freedeliverymanager->l('Country', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'State' => $this->freedeliverymanager->l('State', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'Zip Code' => $this->freedeliverymanager->l('Zip Code', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'City' => $this->freedeliverymanager->l('City', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC'),
                'Enjoy free shipping cost' => $this->freedeliverymanager->l('Enjoy free shipping cost', 'FREEDELIVERYMANAGER-CARRIER-CLASSIC')
            );
        }
    }
}
