<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('idxcartgift') && Module::isEnabled('idxcartgift')) {
    require_once _PS_MODULE_DIR_ . 'idxcartgift/idxcartgift.php';

    class PrestAppIdxcartgiftCustomizationController
    {
        protected $idxcartgift;

        public function __construct()
        {
            $this->idxcartgift = Module::getInstanceByName('idxcartgift');
        }

        public function getGifts($context)
        {
            if ($this->idxcartgift->active) {
                $this->idxcartgift->log->insert_line('Se muestra el footer del carrito: '.$context->cart->id);

                $currencyData = Currency::getCurrency($context->currency->id);
                $total_cart = $context->cart->getOrderTotal(true, 4);
                $isRangeAvailable = false;
                $sql = new DbQuery();
                $sql->select('dcg.*');
                $sql->from('idxcartgift_ranges', 'dcg');
                $sql->where('dcg.`id_shop` = ' . (int) $context->shop->id);
                $rangesSaved = Db::getInstance()->executeS($sql);

                $id_range = 0;
                $productsAvailableAsGift = array();
                foreach ($rangesSaved as $key => $range) {
                    if ($total_cart >= ($range['limit_1']*$currencyData['conversion_rate'])) {
                        $prod = explode(',', $range['products']);
                        $productsAvailableAsGift[$key] = $this->idxcartgift->getProductsGiftInfo($prod, $range['id_idxcartgift']);
                        $isRangeAvailable = true;
                        $id_range = $range['id_idxcartgift'];
                    }
                }

                foreach ($productsAvailableAsGift as &$gift) {
                    foreach ($gift as &$product) {
                        $product['image'] = $context->link->getImageLink($product['link_rewrite'], $product['id_image'], ImageType::getFormatedName('medium'));
                    }
                }

                $suggestedProducts = array();
                $next_limit = 0;

                $suggestedProductQuery = 'SELECT * FROM `' . _DB_PREFIX_ . 'idxcartgift_ranges`
                WHERE id_idxcartgift > ' . (int) $id_range . '
                AND `id_shop` = ' . (int) $context->shop->id . '
                ORDER BY `id_idxcartgift`';

                $suggested = array();
                $existSuggestedProduct = array();
                if ($query = Db::getInstance()->executeS($suggestedProductQuery)) {
                    $existSuggestedProduct = $query;
                } else {
                    $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'idxcartgift_ranges`
                    WHERE id_idxcartgift >
                        (SELECT MAX(id_idxcartgift)
                        FROM `' . _DB_PREFIX_ . 'idxcartgift_ranges`  WHERE `id_shop` = ' . (int) $context->shop->id . ')
                    AND `id_shop` = ' . (int) $context->shop->id . '
                    ORDER BY `id_idxcartgift`';
                    $existSuggestedProduct = Db::getInstance()->executeS($sql);
                }

                if ($existSuggestedProduct) {
                    foreach ($existSuggestedProduct as $key => $g) {
                        $suggested[$key]['next_limit'] = $g['limit_1']*$currencyData['conversion_rate'];
                        $idSuggestedProduct = explode(',', $g['products']);
                        $suggested[$key]['all_products'] = $this->idxcartgift->getProductsGiftInfo($idSuggestedProduct);
                    }
                }

                foreach ($suggested as &$suggestion) {
                    foreach ($suggestion['all_products'] as &$product) {
                        $product['image'] = $context->link->getImageLink($product['link_rewrite'], $product['id_image'], ImageType::getFormatedName('medium'));
                    }
                }

                return array(
                    'productsAvailableAsGift' => $productsAvailableAsGift,
                    'all_suggestion' => $suggested,
                    'combinations' => (bool) Configuration::get('IDXCARTGIFT_ALLOW_COMBI'),
                    'v_count' => count($context->cart->getProducts()),
                );
            } else {
                return false;
            }
        }

        public function addGift($id_product, $id_product_attribute, $id_range, $id_cart, $rand, $context)
        {
            $idProduct = $id_product;
            $idProductAttribute = $id_product_attribute;
            $idCart = $context->cart->id;

            $this->idxcartgift->log->insert_line('Se procesa un regalo en el carrito '.$idCart);

            $product = new Product($idProduct);
            $id_lang = $context->language->id;
            $cart_rule_name = 'cart_gift_' . $idCart . '_' . $id_range;

            if ($product->available_for_order && Configuration::get('PS_STOCK_MANAGEMENT')) {
                if ($product->advanced_stock_management) {
                    $warehouses = Warehouse::getWarehousesByProductId($idProduct, $idProductAttribute);
                    $stock = 0;
                    foreach ($warehouses as $warehouse) {
                        $stock += Product::getRealQuantity($idProduct, $idProductAttribute, $warehouse['id_warehouse']);
                    }
                    if ($stock <= 0) {
                        return 'nonstock';
                    }
                } else {
                    if (Product::getRealQuantity($idProduct, $idProductAttribute) <= 0 && !Configuration::get('PS_ORDER_OUT_OF_STOCK')) {
                        return 'nonstock';
                    }
                }
            }

            $cartrules = $this->idxcartgift->mygetCartRules($idCart, $id_lang);

            $resultdelete = "";
            if (count($cartrules) > 0) {
                foreach ($cartrules as $myrule) {
                    if (strpos($myrule['name'], "cart_gift_")!== false) {
                        $id_cart_rule = $myrule['id_cart_rule'];
                        $idProductToRemove = (int)$myrule['gift_product'];
                        $idProductAttributeToRemove = (int) $myrule['gift_product_attribute'];
                        $cartRule = new CartRule($id_cart_rule);
                        $resultdelete = $this->idxcartgift->deleteGiftProduct($idProductToRemove, $idProductAttributeToRemove, $context->cart->id);
                        $cartRule->delete();
                    }
                }
            }

            $cart_rule_clone = new CartRule();
            $language_ids = Language::getIDs(false);
            foreach ($language_ids as $idforlang) {
                $thisLang = new Language($idforlang);
                $cart_rule_clone->name[$idforlang] = $cart_rule_name;
            }

            $cart_rule_clone->free_shipping = false;

            $cart_rule_clone->id_customer = $context->customer->id;
            $cart_rule_clone->quantity = 1;
            $cart_rule_clone->quantity_per_user = 1;
            $cart_rule_clone->gift_product = $idProduct;

            if ($idProductAttribute != 'null') {
                $cart_rule_clone->gift_product_attribute = $idProductAttribute;
            }

            $cart_rule_clone->date_from = date('Y-m-d H:i:s', time());
            $cart_rule_clone->date_to = date('Y-m-d H:i:s', time() + 24 * 36000);
            $cart_rule_clone->quantity = 1;
            $cart_rule_clone->quantity_per_user = 1;
            $cart_rule_clone->code = $this->idxcartgift->gencode(8);
            $cart_rule_clone->active = 1;
            $id_cart_rule = 0;

            /*modification double cadeaux*/
            $cartrule = $this->idxcartgift->obtainCartRules();

            if (count($cartrule) > 0) {
                foreach ($cartrule as $myrule) {
                    $isCartgiftRule = false;
                    if (strpos($myrule['name'], "art_gift_".$idCart) !== false) {
                        return false;
                    }
                }
            }
            /*Fin de modification*/

            if ($cart_rule_clone->add()) {
                $id_cart_rule = $cart_rule_clone->id;
            }

            if ($context->cart->addCartRule($id_cart_rule)) {
                return array(
                    'success' => 1,
                    'error' => false,
                    'cartid' => $idCart,
                    'delete' => $resultdelete
                );
            } else {
                return false;
            }
        }
    }
}
