<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('netreviews') && Module::isEnabled('netreviews')) {
    include_once _PS_MODULE_DIR_ . 'netreviews/netreviews.php';

    class PrestAppNetreviewsCustomizationController
    {
        protected $netreviews;

        public function __construct()
        {
            $this->netreviews = Module::getInstanceByName('netreviews');
        }

        public function getReviews($id_product, $context)
        {
            $display_prod_reviews = configuration::get('AV_DISPLAYPRODREVIEWS' . $this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);

            $o_av = new NetReviewsModel();
            $lang_id = (isset($context->language->id) && !empty($context->language->id)) ? (int) $context->language->id : 1;

            if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                $product = new Product((int) $id_product, false, $lang_id);
            } else {
                $product = new Product((int) $id_product);
            }

            if (!$this->netreviews->all_multishops_reviews && !$this->netreviews->all_languages_reviews) {
                $reviews = $o_av->getStatsProduct($id_product, $this->netreviews->group_name, $this->netreviews->current_shop_id);
                $num_reviews = !empty($reviews['nb_reviews']) ? $reviews['nb_reviews'] : 0;
                if ($num_reviews < 1 || $display_prod_reviews != 'yes') {
                    return false;
                }
            } else {
                $language_group = ($this->netreviews->all_languages_reviews) ? null : $this->netreviews->group_name;
                $shop_id_group = ($this->netreviews->all_multishops_reviews) ? null : $this->netreviews->current_shop_id;
                $reviews = $o_av->getProductReviews($id_product, $language_group, $shop_id_group, 0, 1, 'horodate_DESC', 0, false);
                $num_reviews = count($reviews);
                if ($num_reviews < 1 || $display_prod_reviews != 'yes') {
                    return false;
                }
                $somme_rate = 0;
                foreach ($reviews as $review) {
                    $somme_rate += $review['rate'];
                }
                $reviews['rate'] = $somme_rate / $num_reviews;
            }

            $average_rate_percent = array();
            $average_rate_percent['floor'] = floor($reviews['rate']) - 1;
            $average_rate_percent['decimals'] = ($reviews['rate'] - floor($reviews['rate'])) * 20;

            $customized_star_color = (Configuration::get('AV_STARCOLOR', null, null, $this->netreviews->current_shop_id)) ? Configuration::get('AV_STARCOLOR', null, null, $this->netreviews->current_shop_id) : "FFCD00";

            return array(
                // 'modules_dir' => _MODULE_DIR_,
                // 'base_url' => __PS_BASE_URI__ ,
                // 'version_ps' => _PS_VERSION_,
                'nb_reviews' => $num_reviews,
                'rate' =>   round($reviews['rate'], 1),
                'product_id' =>  $id_product,
                'customized_star_color' => '#' . $customized_star_color
            );
        }

        public function getReviewsDetails($id_product, $context, $params = null)
        {
            $display_prod_reviews = configuration::get('AV_DISPLAYPRODREVIEWS' . $this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
            $url_certificat = Configuration::get('AV_URLCERTIFICAT' . $this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
            $avisverifies_nb_reviews = (int)Configuration::get('AV_NBOFREVIEWS', null, null, $this->netreviews->current_shop_id);
            $avisverifies_nb_reviews = (!empty($avisverifies_nb_reviews)) ? $avisverifies_nb_reviews : 10;

            $local_id_website = Configuration::get('AV_IDWEBSITE' . $this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
            $local_secure_key = Configuration::get('AV_CLESECRETE' . $this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
            $shop_name = Configuration::get('PS_SHOP_NAME');
            $o_av = new NetReviewsModel();

            $stats_product = (!isset($this->netreviews->stats_product) || empty($this->netreviews->stats_product)) ? $o_av->getStatsProduct($id_product, $this->netreviews->group_name, $this->netreviews->current_shop_id) : $this->netreviews->stats_product;

            if ($this->netreviews->all_multishops_reviews || $this->netreviews->all_languages_reviews) { // override rate & nb_reviews
                $language_group = ($this->netreviews->all_languages_reviews) ? null : $this->netreviews->group_name;
                $shop_id_group = ($this->netreviews->all_multishops_reviews) ? null : $this->netreviews->current_shop_id;
                $reviews = $o_av->getProductReviews($id_product, $language_group, $shop_id_group, 0, 1, 'horodate_DESC', 0, false);
                $num_reviews = count($reviews);
                if ($num_reviews < 1 || $display_prod_reviews != 'yes') {
                    return false;
                }

                $somme_rate = 0;

                foreach ($reviews as $review) {
                    $somme_rate += $review['rate'];
                }

                $stats_product['rate'] = $somme_rate / $num_reviews;
                $stats_product['nb_reviews'] = $num_reviews;
            } else {
                $language_group = $this->netreviews->group_name;
                $shop_id_group = $this->netreviews->current_shop_id;
            }

            if (!empty($stats_product['nb_reviews']) && $display_prod_reviews == 'yes') {
                $reviews_list = array();
                $my_review = array();

                if (!$params) {
                    $reviews = $o_av->getProductReviews($id_product, $language_group, $shop_id_group, $avisverifies_nb_reviews, 1, 'horodate_DESC', 0, false);
                } else {
                    $reviews = $o_av->getProductReviews($id_product, $language_group, $shop_id_group, $avisverifies_nb_reviews, $params['current_page'], $params['filter_option'], 0, false);
                }

                $reviews_count = $stats_product['nb_reviews'];

                if ($avisverifies_nb_reviews < 1 || empty($avisverifies_nb_reviews)) {
                    $avisverifies_nb_reviews = 10;
                }
                $reviews_max_pages = floor($reviews_count / $avisverifies_nb_reviews) + ($reviews_count % $avisverifies_nb_reviews > 0 ? 1 : 0);

                foreach ($reviews as $review) {
                    //Create variable for template engine

                    $my_review['ref_produit'] = $review['ref_product'];
                    $my_review['id_product_av'] = $review['id_product_av'];
                    $my_review['sign'] = sha1($local_id_website.$review['id_product_av'].$local_secure_key);
                    if (!isset($review['helpful']) && !isset($review['helpless'])) {
                        Db::getInstance()->Execute('ALTER TABLE `'._DB_PREFIX_.'av_products_reviews`
                       ADD `helpful` int(7) DEFAULT 0,
                       ADD `helpless` int(7) DEFAULT 0');
                    } else {
                        $my_review['helpful'] = $review['helpful'];
                        $my_review['helpless'] = $review['helpless'];
                    }
                    $my_review['rate'] = $review['rate'];
                    $my_review['rate_percent'] = $review['rate']*20;
                    $my_review['avis'] = html_entity_decode(urldecode($review['review']));
                    // review date
                    if (Tools::strlen($review['horodate'])=='10') {
                        $date = new DateTime();
                        $date->setTimestamp($review['horodate']);
                        $my_review['horodate'] = $date->format('d/m/Y') ;
                    } else {
                        $my_review['horodate'] = date('d/m/Y', strtotime($review['horodate']));
                    }
                    // order date
                    if (isset($review['horodate_order']) && !empty($review['horodate_order'])) {
                        $review['horodate_order'] = str_replace('"', '', $review['horodate_order']);
                        $my_review['horodate_order'] = date('d/m/Y', strtotime($review['horodate_order']));
                    } else {
                        $my_review['horodate_order'] = $my_review['horodate'];
                    }
                    // in case imported reviews which have lack of this info
                    if (!isset($review['horodate']) || empty($review['horodate'])) {
                        $my_review['horodate'] = $my_review['horodate_order'];
                    }

                    $my_review['customer_name'] = urldecode($review['customer_name']);

                    $my_review['discussion'] = array();

                    //renverser le nom et le prénom
                    $customer_name = explode(' ', urldecode($review['customer_name']));
                    $customer_name = array_values(array_filter($customer_name));
                    $customer_name = array_diff($customer_name, array("."));
                    $customer_name = array_reverse($customer_name);
                    $customer_name = implode(' ', $customer_name);

                    $my_review['customer_name'] =  $customer_name;

                    $unserialized_discussion = NetReviewsModel::avJsonDecode(NetReviewsModel::acDecodeBase64($review['discussion']), true);
                    $unserialized_discussion = (array)$unserialized_discussion;
                    if ($unserialized_discussion) {
                        foreach ($unserialized_discussion as $k_discussion => $each_discussion) {
                            $each_discussion = (array)$each_discussion;
                            $my_review['discussion'][$k_discussion] = array();
                            if (Tools::strlen($each_discussion['horodate'])=='10') {
                                $date = new DateTime();
                                $date->setTimestamp($each_discussion['horodate']);
                                $my_review['discussion'][$k_discussion]['horodate'] = $date->format('d/m/Y') ;
                            } else {
                                $my_review['discussion'][$k_discussion]['horodate'] = date('d/m/Y', strtotime($each_discussion['horodate']));
                            }
                            $my_review['discussion'][$k_discussion]['commentaire'] = $each_discussion['commentaire'];
                            if ($each_discussion['origine'] == 'ecommercant') {
                                $my_review['discussion'][$k_discussion]['origine'] = $shop_name;
                            } elseif ($each_discussion['origine'] == 'internaute') {
                                $my_review['discussion'][$k_discussion]['origine'] = $my_review['customer_name'];
                            } else {
                                $my_review['discussion'][$k_discussion]['origine'] = $this->netreviews->l('Moderator');
                            }
                        }
                    }
                    // Media infos
                    $my_review['media_content'] = array();
                    if (isset($review['media_full'])) {
                        $review_images_result =  (array) NetReviewsModel::avJsonDecode(html_entity_decode($review['media_full']), true);
                        if (isset($review_images_result) && !empty($review_images_result) && count($review_images_result) > 1) {
                            foreach ($review_images_result as $k_media => $each_media) {
                                $my_review['media_content'][$k_media] = (array) $each_media;
                            }
                        }
                    }

                    array_push($reviews_list, $my_review);
                }

                $customized_star_color = (Configuration::get('AV_STARCOLOR', null, null, $this->netreviews->current_shop_id)) ? Configuration::get('AV_STARCOLOR', null, null, $this->netreviews->current_shop_id) : "FFCD00"; //default #FFCD00
                $hidehelpful = Configuration::get('AV_HELPFULHIDE', null, null, $this->netreviews->current_shop_id) ? 1 : 0; // 0 or null in defaut
                $hidemedia = Configuration::get('AV_MEDIAHIDE', null, null, $this->netreviews->current_shop_id); // 0 or null in defaut

                $av_urlcertificat = Configuration::get('AV_URLCERTIFICAT'.$this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
                $url_platform = explode('/', $av_urlcertificat);
                $platform = Tools::substr($url_platform[2], 4);
                $url_cgv =  "https://www.".$platform."/index.php?page=mod_conditions_utilisation";

                $logolang = $context->language->iso_code;
                $languagespack_av = array("de", "en", "es", "fr", "gb", "it", "pt");
                $logolang =(in_array($logolang, $languagespack_av)) ? $logolang : "en";

                $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $this->netreviews->current_shop_id);
                $use_image = false;

                $av_ajax_translation = array();
                $av_ajax_translation['a'] = $this->netreviews->l('published');
                $av_ajax_translation['b'] = $this->netreviews->l('the');
                $av_ajax_translation['c'] = $this->netreviews->l('following an order made on');
                $av_ajax_translation['d'] = $this->netreviews->l('Comment from');
                $av_ajax_translation['e'] = $this->netreviews->l('Show exchanges');
                $av_ajax_translation['f'] = $this->netreviews->l('Hide exchanges');
                $av_ajax_translation['g'] = $this->netreviews->l('Did you find this helpful?');
                $av_ajax_translation['h'] = $this->netreviews->l('Yes');
                $av_ajax_translation['i'] = $this->netreviews->l('No');
                $av_ajax_translation['j'] = $this->netreviews->l('More reviews...');

                $av_ajax_translation['k'] = $this->netreviews->l('View the trust certificate', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['l'] = $this->netreviews->l('Reviews subject to control', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['m'] = $this->netreviews->l('For further information on the nature of the review controls, as well as the possibility of contacting the author of the review please consult our', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['n'] = $this->netreviews->l('No inducements have been provided for these reviews', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['o'] = $this->netreviews->l('Reviews are published and kept for a period of five years', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['p'] = $this->netreviews->l('Reviews can not be modified: If a customer wishes to modify their review then they can do so by contacting Verified Reviews directly to remove the existing review and publish an amended one', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['q'] = $this->netreviews->l('The reasons for deletion of reviews are available', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['r'] = $this->netreviews->l('here', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['s'] = $this->netreviews->l('Based on', 'AVISVERIFIES-TAB-CONTENT');
                $av_ajax_translation['t'] = $this->netreviews->l('customer reviews', 'AVISVERIFIES-TAB-CONTENT');

                return array(
                    'url_cgv' => $url_cgv,
                    'use_image' => $use_image,
                    'idwebsite' => $local_id_website,
                    'reviews' => $reviews_list,
                    'count_reviews' => (int) $reviews_count,
                    'average_rate' => round($stats_product['rate'], 1),
                    'rate_percent_int' =>  (float)$stats_product['rate'] * 20,
                    'url_certificat' => $url_certificat,
                    'reviews_max_pages' => ($reviews_max_pages) ? (int)$reviews_max_pages : "",
                    'hidehelpful' =>  $hidehelpful,
                    'hidemedia' =>  $hidemedia,
                    'current_page' =>  $params ? $params['current_page'] : 1,
                    'ajax_translation' => $av_ajax_translation,
                    'logo_link' => _PS_BASE_URL_ . _MODULE_DIR_ . 'netreviews/views/img/logo_full_' . $logolang . '.png',
                    'customized_star_color' => '#' . $customized_star_color,
                    'product_id' =>  $id_product,
                );
            } else {
                return false;
            }
        }

        public function sendRequestToNetReviewsWebSite($params, $context)
        {
            $av_urlcertificat = Configuration::get('AV_URLCERTIFICAT'.$this->netreviews->group_name, null, null, $this->netreviews->current_shop_id);
            $url_platform = explode('/', $av_urlcertificat);
            $platform = Tools::substr($url_platform[2], 4);
            $avHelpfulURL =  "https://www.".$platform."/index.php?action=act_api_product_reviews_helpful";

            $method = $params['method'];
            $idWebsite = $params['idWebsite'];
            $idProduct = $params['idProduct'];
            $isHelpful = $params['isHelpful'];
            $fingerPrint = $params['fingerPrint'];
            $sign = $params['sign'];

            $data = array('method' => $method, 'idWebsite' => $idWebsite, 'idProduct' => $idProduct, 'isHelpful' => $isHelpful, 'fingerPrint' => $fingerPrint, 'sign' => $sign);
            $options = array(
                'http' => array(
                    'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
                    'method'  => 'POST',
                    'content' => http_build_query($data)
                )
            );
            $context  = stream_context_create($options);
            $result = Tools::file_get_contents($avHelpfulURL, false, $context);

            if ($result === false) {
                return false;
            }

            $obj = json_decode($result);

            return $obj;
        }
    }
}
