<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;

if (Module::isInstalled('pm_advancedsearch4') && Module::isEnabled('pm_advancedsearch4')) {
    class PrestAppPmadvancedsearch4CustomizationController
    {
        protected $pm_advancedsearch4;
        protected $criterions;
        protected $searchInstance;

        public $p;
        public $n;
        protected $column;
        protected $order;

        public function __construct($p, $n, $column, $order)
        {
            $this->pm_advancedsearch4 = Module::getInstanceByName('pm_advancedsearch4');

            $_POST['p'] = $p;
            $_POST['orderby'] = Tools::strtolower($column);
            $_POST['orderway'] = Tools::strtolower($order);
            $_POST['n'] = $n;

            $this->p = $p;
            $this->n = $n;
            $this->column = $column;
            $this->order = $order;
        }

        public function getFilters($context, $id_category, $controller = null)
        {
            $continue = false;

            if (version_compare(_PS_VERSION_, '1.7', '<') && file_exists(_PS_MODULE_DIR_ . 'pm_advancedsearch4/controllers/front/searchresults-16.php')) {
                include_once _PS_MODULE_DIR_ . 'pm_advancedsearch4/controllers/front/searchresults-16.php';
                $_POST['module'] = 'pm_advancedsearch4';
                $continue = true;
            } elseif (version_compare(_PS_VERSION_, '1.7', '>=') && file_exists(_PS_MODULE_DIR_ . 'pm_advancedsearch4/controllers/front/searchresults-17.php')) {
                include_once _PS_MODULE_DIR_ . 'pm_advancedsearch4/controllers/front/searchresults-17.php';
                $continue = true;
            }

            if ($id_category) {
                $_POST['id_category'] = $id_category;
            } else {
                if ($controller == 'prices-drop') {
                    $context->smarty->assign(array(
                        'page' => array('page_name' => 'prices-drop')
                    ));
                }
            }

            // check if the module can be loaded
            if ($continue) {
                $pm_advancedsearch4searchresultsModuleFrontController = new pm_advancedsearch4searchresultsModuleFrontController();
                $selected_criterion = $pm_advancedsearch4searchresultsModuleFrontController->getSelectedCriterions();

                $hook_list = As4SearchEngine::$valid_hooks;

                if (isset($hook_list) && count($hook_list)) {
                    if ($id_category) {
                        $context->controller->php_self = 'category';
                    } else {
                        $context->controller->php_self = $controller;
                    }

                    $found_searchs = false;

                    foreach ($hook_list as $hook) {
                        $newHookName = Hook::getRetroHookName($hook);
                        if ($newHookName == false) {
                            $newHookName = $hook;
                        }

                        $searchs = As4SearchEngine::getSearchsFromHook($newHookName, (int)$context->language->id, false);

                        if ($searchs) {
                            $found_searchs = $searchs;
                        }
                    }

                    if (!$found_searchs) {
                        $searchs = As4SearchEngine::getAllSearchs((int)$context->language->id);

                        foreach($searchs as $search) {
                            if ($search['filter_by_emplacement'] == true && $search['id_category_root'] == 0) {
                                if (!$found_searchs) {
                                    $found_searchs = array();
                                }
                                $found_searchs[] = $search;
                            }
                        }
                    }

                    if ($found_searchs) {
                        $context->controller = new CategoryController();

                        if ($controller == 'manufacturer') {
                            $context->controller = new ManufacturerController();
                        }

                        $searchs = $this->pm_advancedsearch4->getCriterionsGroupsAndCriterionsForSearch($found_searchs, (int)$context->language->id, $selected_criterion, false);
                        return array(
                            'as_searchs' => $searchs,
                            'as_selected_criterion' => $selected_criterion,
                            'criteria_group_type_interal_name' => array(1 => 'select', 2 => 'image', 3 => 'link', 4 => 'checkbox', 5 => 'slider', 7 => 'colorsquare', 8 => 'range', 9 => 'level_depth')

                        );
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            } else {
                return false;
            }
        }

        public function getProducts($context, $populate, $filters, $id_category, $controller = null)
        {
            if (_PS_VERSION_ < '1.7') {

                $filters = json_decode($filters);
                $id_search = $filters->id_search;

                $this->searchInstance = new AdvancedSearchClass((int)$id_search, (int)$context->language->id);
                $this->criterions = $this->setCriterions($filters, $id_search, $context);

                As4SearchEngine::$pm_spa_is_active = false;
                $searchs = As4SearchEngine::getSearch($id_search, (int)$context->language->id);
                $searchs = $this->pm_advancedsearch4->getCriterionsGroupsAndCriterionsForSearch($searchs, (int)$context->language->id, $this->criterions, true, false);
                
                $result = $searchs[0];
                $products = $result['products'];
    
                $products_in_layer = array();
                foreach ($products as $product) {
                    $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                    if ($product && $product != 404 && $product != 403) {
                        array_push($products_in_layer, $product);
                    }
                }

                $layer_object = $this->getLayerObject($this->searchInstance, $context, $this->criterions);

                $data = array(
                    'products' => $products_in_layer,
                    'layer' => $layer_object,
                    'nb_products' => $result['total_products']
                );

                return $data;

            } else {

                $filters = json_decode($filters);
                $idSearch = $filters->id_search;
    
                if ($controller) {
                    if ($controller == 'prices-drop') {
                        As4SearchEngine::getPricesDropProductsIds();
                    } elseif ($controller == 'manufacturer') {
                        $context->controller->php_self = 'manufacturer';
                    }
                }
    
                $this->criterions = $this->setCriterions($filters, $idSearch, $context);
    
                $result = $this->getQueryResult($context, true);
    
                $products = $result->getProducts();
    
                $products_in_layer = array();
                foreach ($products as $product) {
                    $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                    if ($product && $product != 404 && $product != 403) {
                        array_push($products_in_layer, $product);
                    }
                }
    
                $layer_object = $this->getLayerObject($this->searchInstance, $context, $this->criterions);
    
                $bulk_sort_orders = $result->getAvailableSortOrders();
                $parsed_sort_orders = array();
    
                foreach ($bulk_sort_orders as $sort_order) {
                    array_push($parsed_sort_orders, array(
                        'field' => $sort_order->getField(),
                        'direction' => $sort_order->getDirection(),
                        'label' => $sort_order->getLabel()
                    ));
                }
    
                $data = array(
                    'products' => $products_in_layer,
                    'layer' => $layer_object,
                    'sort_orders' => $parsed_sort_orders,
                    'nb_products' => $result->getTotalProductsCount()
                );
    
                return $data;
            }
        }

        public function getAvailableFilters($context, $filters)
        {
            $filters = json_decode($filters);
            $idSearch = $filters->id_search;

            $this->criterions = $this->setCriterions($filters, $idSearch, $context);

            $layer_object = $this->getLayerObject($this->searchInstance, $context, $this->criterions);

            return $layer_object;
        }

        // Override

        protected function getQueryResult($context, $for_products)
        {
            $search_context = $this->getProductSearchContext($context);

            $query = new ProductSearchQuery();
            $provider = null;
            if (null === $provider) {
                $provider = $this->getDefaultProductSearchProvider();
            }

            if ($for_products) {
                $resultsPerPage = (int) $this->n;
                if ($resultsPerPage <= 0) {
                    $resultsPerPage = Configuration::get('PS_PRODUCTS_PER_PAGE');
                }

                $query->setResultsPerPage($resultsPerPage)->setPage(max((int) $this->p, 1));

                if (($encodedSortOrder = 'product.'.$this->column.'.'.$this->order)) {
                    $query->setSortOrder(SortOrder::newFromString(
                        $encodedSortOrder
                    ));
                }
            }

            $encodedFacets = array();
            $query->setEncodedFacets($encodedFacets);

            $result = $provider->runQuery(
                $search_context,
                $query
            );

            if (!$result->getCurrentSortOrder()) {
                $result->setCurrentSortOrder($query->getSortOrder());
            }

            return $result;
        }

        protected function getProductSearchContext($context)
        {
            return (new ProductSearchContext())
                ->setIdShop($context->shop->id)
                ->setIdLang($context->language->id)
                ->setIdCurrency($context->currency->id)
                ->setIdCustomer(
                    $context->customer ?
                        $context->customer->id :
                        null
                );
        }

        protected function getDefaultProductSearchProvider()
        {
            return new As4SearchProvider(
                $this->pm_advancedsearch4,
                Context::getContext()->getTranslator(),
                $this->searchInstance,
                $this->criterions
            );
        }

        private function getLayerObject(AdvancedSearchClass $searchEngine, $context, $selectedCriterions = array())
        {
            $with_product = false;
            $searchs = As4SearchEngine::getSearch($searchEngine->id, (int)$context->language->id);
            $searchs = $this->pm_advancedsearch4->getCriterionsGroupsAndCriterionsForSearch($searchs, (int)$context->language->id, $selectedCriterions, $with_product, false);

            return array(
                'hideAS4Form' => (((empty(0) && isset($searchs[0]) && isset($searchs[0]['id_hook']) && $searchs[0]['id_hook'] != -1) || 0 == 'home') ? false : false),
                'as_searchs' => $searchs,
                'as_search' => current($searchs),
                'as_selected_criterion' => $selectedCriterions,
                'criteria_group_type_interal_name' => array(1 => 'select', 2 => 'image', 3 => 'link', 4 => 'checkbox', 5 => 'slider', 7 => 'colorsquare', 8 => 'range', 9 => 'level_depth')
            );
        }

        private function setCriterions($filters, $idSearch, $context)
        {
            $this->searchInstance = new AdvancedSearchClass((int)$idSearch, (int)$context->language->id);

            $criterions = $filters->as4c;

            if (is_array($criterions)) {
                $criterions = As4SearchEngine::cleanArrayCriterion($criterions);
            } else {
                $criterions = array();
            }

            $criterions_hidden = $filters->as4c_hidden;
            if (is_array($criterions_hidden)) {
                $criterions_hidden = As4SearchEngine::cleanArrayCriterion($criterions_hidden);
            } else {
                $criterions_hidden = array();
            }

            $reset = (int)$filters->reset;
            $reset_group = (int)$filters->reset_group;
            if ($reset) {
                $criterions = array();
            }

            if ($reset_group && isset($criterions[$reset_group])) {
                unset($criterions[$reset_group]);
                if ($this->searchInstance->step_search) {
                    $criterionsGroups = AdvancedSearchCriterionGroupClass::getCriterionsGroupsFromIdSearch((int)$idSearch, (int)$context->language->id, false);
                    if (AdvancedSearchCoreClass::_isFilledArray($criterionsGroups)) {
                        $deleteAfter = false;
                        foreach ($criterionsGroups as $criterionGroup) {
                            if ((int)$criterionGroup['id_criterion_group'] == $reset_group) {
                                $deleteAfter = true;
                            }
                            if ($deleteAfter && isset($criterions[(int)$criterionGroup['id_criterion_group']])) {
                                unset($criterions[(int)$criterionGroup['id_criterion_group']]);
                            }
                        }
                    }
                }
            }

            if ($this->searchInstance->filter_by_emplacement) {
                $criterionsFromEmplacement = As4SearchEngine::getCriteriaFromEmplacement($this->searchInstance->id, $this->searchInstance->id_category_root);
                foreach ($criterionsFromEmplacement as $idCriterionGroup => $idCriterionList) {
                    if (!isset($criterions[$idCriterionGroup])) {
                        $criterions[$idCriterionGroup] = $idCriterionList;
                    } elseif (is_array($criterions[$idCriterionGroup]) && !sizeof($criterions[$idCriterionGroup])) {
                        $criterions[$idCriterionGroup] = $idCriterionList;
                    }
                }
            }

            return $criterions;
        }
    }
}
