<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (Module::isInstalled('pm_crosssellingoncart') && Module::isEnabled('pm_crosssellingoncart')) {
    class PrestAppPmcrosssellingoncartCustomizationController
    {
        protected $pm_crosssellingoncart;

        public function __construct()
        {
            $this->pm_crosssellingoncart = Module::getInstanceByName('pm_crosssellingoncart');
        }

        public function getData($context)
        {
            $data = new StdClass();
            $data->name = 'pm_crosssellingoncart';
            $data->active = true;
            if ($this->pm_crosssellingoncart->prefixFieldsOptions != 'PM_MC_CSOC' && !Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IN_CART')) {
                $data->display_in_cart = false;
            } else {
                $data->display_in_cart = true;
            }

            if ($this->pm_crosssellingoncart->prefixFieldsOptions != 'PM_MC_CSOC' && !Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IN_PRODUCT')) {
                $data->display_in_product = false;
            } else {
                $data->display_in_product = true;
            }

            return $data;
        }

        public function getCartProducts($context, $id_product = null)
        {
            if ($this->pm_crosssellingoncart->prefixFieldsOptions != 'PM_MC_CSOC' && !Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IN_CART')) {
                return false;
            } else {
                return $this->getProducts($context, null);
            }
        }

        public function getProductProducts($id_product, $context)
        {
            if (Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IN_PRODUCT')) {
                return $this->getCartProducts($context, $id_product);
            } else {
                return false;
            }
        }

        public function getProducts($context, $id_product = null)
        {
            $cart = new Cart($context->cart->id);
            if (isset($cart) && $cart) {
                $cart_products = $cart->getProducts(true);
            } else {
                return false;
            }

            $csoc_display = Configuration::getMultiple(array(
                'PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IMG',
                'PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_TITLE',
                'PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_PRICE',
                'PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_BUTTON',
                'PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_AVAILABILITY',
            ));

            if (!$csoc_display['PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_IMG']
                && !$csoc_display['PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_TITLE']
                && !$csoc_display['PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_PRICE']
                && !$csoc_display['PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_BUTTON']
                && !$csoc_display['PM_' . pm_crosssellingoncart::$modulePrefix . '_DISPLAY_AVAILABILITY']
            ) {
                return false;
            }

            $displayProductAlreadyInCart = (bool)Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_IDENTICAL_PRODUCTY');

            // Nombre de produits à afficher simultanément -> Sur ordinateurs (page produit)
            $nbProductSelection = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_PRODUCT');
            // Nombre de produits à afficher simultanément -> Sur ordinateurs (page panier)
            $nbProductSelectionCart = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_PRODUCT_CART');
            // Nombre de produits à afficher simultanément -> Sur tablettes
            $nbProductSelectionTablet = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_PRODUCT_TABLET');
            // Nombre de produits à afficher simultanément -> Sur mobiles
            $nbProductSelectionMobile = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_PRODUCT_MOBILE');
            // Accessoires -> Nombre de produits à afficher au maximum
            $nbAccessories = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_ACCESSORIES');
            if (!$nbAccessories) {
                $nbAccessories = (int)pm_crosssellingoncart::DEFAULT_NB_ACCESSORIES;
            }
            // Produits fréquemment achetés ensemble -> Nombre de produits à afficher au maximum
            $nbCrossSelling = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_NB_CROSSSELLING');
            if (!$nbCrossSelling) {
                $nbCrossSelling = (int)pm_crosssellingoncart::DEFAULT_NB_CROSSSELLING;
            }

            $nbDays = Configuration::get($this->pm_crosssellingoncart->prefixFieldsOptions.'_CROSSSELLING_NB_DAYS');
            $imageSize = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_IMAGE_SIZE');
            $blockTitle = Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_TITLE_BLOC', $context->language->id);
            $postProductSelection = @array_filter(unserialize(Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_PRODUCT_SELECTION')));
            $productSelection = array();
            foreach ($postProductSelection as $k => $idProduct) {
                if (version_compare(_PS_VERSION_, '1.6.0.12', '>=')) {
                    if (!Product::checkAccessStatic((int)$idProduct, false)) {
                        unset($postProductSelection[$k]);
                    }
                } else {
                    $product = new Product((int)$idProduct);
                    if (!$product->checkAccess(false)) {
                        unset($postProductSelection[$k]);
                    }
                }
            }

            $productsOnCart = array();
            foreach ($cart_products as $k => $v) {
                $productsOnCart[] = $v['id_product'];
            }
            $productArray = array();
            if (isset($id_product)) {
                $productArray[] = (int)$id_product;
                $productsOnCart[] = (int)$id_product;
            }
            $productsOnCart = array_unique($productsOnCart);
            $ForceShowAccessories = true;
            if ((isset($id_product)) && $id_product && !(int)Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_ACCESSORIES_IN_PRODUCT')) {
                $ForceShowAccessories = false;
            }

            if ($ForceShowAccessories && Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_ACCESSORIES')) {
                if (isset($id_product) && $id_product) {
                    $productAccessories = $this->getAccessoriesLight($productArray, $nbAccessories);
                } else {
                    $productAccessories = $this->getAccessoriesLight($productsOnCart, $nbAccessories, $productArray);
                }
                if (pm_crosssellingoncart::isFilledArray($productAccessories)) {
                    foreach ($productAccessories as $product) {
                        $postProductSelection[] = (int)$product['id_product'];
                    }
                }
            }

            if (Configuration::get('PM_' . pm_crosssellingoncart::$modulePrefix . '_CROSSSELLING')) {
                $productsCrossSelling = $this->getCrossSellingLight($productsOnCart, $nbCrossSelling, $nbDays);
                if (pm_crosssellingoncart::isFilledArray($productsCrossSelling)) {
                    foreach ($productsCrossSelling as $product) {
                        $postProductSelection[] = (int)$product['product_id'];
                    }
                }
            }

            if (pm_crosssellingoncart::isFilledArray($postProductSelection)) {
                $postProductSelection = array_unique($postProductSelection);
                if (!$displayProductAlreadyInCart) {
                    $postProductSelection = array_diff($postProductSelection, $productsOnCart);
                }
                if (isset($id_product) && $id_product) {
                    $key = array_search((int)$id_product, $postProductSelection);
                    if ($key !== false) {
                        unset($postProductSelection[(int)$key]);
                    }
                }
                if (pm_crosssellingoncart::isFilledArray($postProductSelection)) {
                    $productSelection = $this->pm_crosssellingoncart->getElementProducts($postProductSelection, $context->language->id);
                }
            }
            if (version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
                foreach ($productSelection as $id => $product) {
                    $productSelection[$id]['price'] = Tools::displayPrice($product['price']);
                }
            }
            if (count($productSelection) <= 0) {
                return false;
            }

            $to_send_products = array();

            foreach ($productSelection as $product_selected) {
                $populate = '["price","description","images","stock"]';
                $product = PrestAppProductController::getProduct((int) $product_selected['id_product'], $populate, $context);
                array_push($to_send_products, $product);
            }

            return array(
                'product_selection' => $to_send_products,
                'bloc_title' => $blockTitle,
                'tax_enabled' => Configuration::get('PS_TAX'),
                'csoc_static_token' => Tools::getToken(false),
                'display' => $csoc_display,
                'on_product_page' => (isset($id_product) ? $id_product : false),
            );
        }

        protected function getAccessoriesLight($productList, $limit, $ignoreList = array())
        {
            if (!pm_crosssellingoncart::isFilledArray($productList)) {
                return false;
            }
            return Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS('
            SELECT `id_product_2` AS id_product
            FROM `'._DB_PREFIX_.'accessory` a
            LEFT JOIN `'._DB_PREFIX_.'product` p ON (p.`id_product` = a.`id_product_2`)
            LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (cp.`id_category` = p.`id_category_default` AND cp.`id_product` = a.`id_product_2`)
            LEFT JOIN `'._DB_PREFIX_.'category_group` ctg ON (ctg.`id_category` = cp.`id_category` AND ctg.`id_group` = '.(int)Group::getCurrent()->id.')
            WHERE `id_product_1` IN ('.(implode(',', array_map('intval', $productList))).')
            '.(pm_crosssellingoncart::isFilledArray($ignoreList) ? 'AND `id_product_2` NOT IN ('.(implode(',', array_map('intval', $ignoreList))).')' : '').'
            AND ctg.id_group IS NOT NULL
            ORDER BY RAND()
            LIMIT '.(int)$limit);
        }

        protected function getCrossSellingLight($productList, $limit, $nbDays)
        {
            if (!pm_crosssellingoncart::isFilledArray($productList)) {
                return false;
            }
            return Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS('
                SELECT DISTINCT od.product_id
                FROM '._DB_PREFIX_.'order_detail od
                LEFT JOIN `'._DB_PREFIX_.'product` p ON (p.`id_product` = od.`product_id`)
                LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (cp.`id_category` = p.`id_category_default` AND cp.`id_product` = od.`product_id`)
                LEFT JOIN `'._DB_PREFIX_.'category_group` ctg ON (ctg.`id_category` = cp.`id_category` AND ctg.`id_group` = '.(int)Group::getCurrent()->id.')
                WHERE od.product_id NOT IN ('.implode(',', array_map('intval', $productList)).') AND od.id_order IN (
                    SELECT od.id_order FROM '._DB_PREFIX_.'orders o
                    JOIN '._DB_PREFIX_.'order_detail od ON (od.id_order = o.id_order)
                    WHERE o.valid = 1 AND od.product_id IN ('.implode(',', array_map('intval', $productList)).') AND TO_DAYS("'.date('Y-m-d').' 00:00:00") - TO_DAYS(o.date_add) <= '.$nbDays.'
                )
                AND ctg.id_group IS NOT NULL
                LIMIT '.(int)$limit);
        }
    }
}
