<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

// require_once _PS_MODULE_DIR_ . 'ps_facetedsearch/src/Ps_FacetedsearchFiltersConverter.php';

use PrestaShop\Module\FacetedSearch\Filters\Block;
use PrestaShop\Module\FacetedSearch\Filters\Converter;
use PrestaShop\Module\FacetedSearch\Filters\DataAccessor;
use PrestaShop\Module\FacetedSearch\Filters\Provider;
use PrestaShop\Module\FacetedSearch\Product\Search;
use PrestaShop\Module\FacetedSearch\URLSerializer;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchProviderInterface;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

if (Module::isInstalled('ps_facetedsearch') && Module::isEnabled('ps_facetedsearch')) {
    class PrestAppPsFacecedSearchCustomizationController extends ProductPresentingFrontController
    {
        protected $blocklayered;
        public $p;
        public $n;
        protected $column;
        protected $order;

        public function __construct($p, $n, $column, $order)
        {
            if ('' == $order) {
                $order = 'ASC';
            }

            $_POST['p'] = $p;
            $_POST['orderby'] = Tools::strtolower($column);
            $_POST['orderway'] = Tools::strtolower($order);
            $_POST['n'] = $n;

            $this->p = $p;
            $this->n = $n;
            $this->column = $column;
            $this->order = $order;

            $this->blocklayered = Module::getInstanceByName('ps_facetedsearch');
        }

        public function getFilters($id_category, $filters)
        {
            $_POST['id_category'] = $id_category;

            $blocklayered = Module::getInstanceByName('ps_facetedsearch');

            if ($filter_block = $this->blocklayered->getFilterBlock($filters)) {
                if ($filter_block['nbr_filterBlocks'] == 0) {
                    return false;
                }
                return $filter_block;
            } else {
                return false;
            }
        }

        public function getAvailableFilters($context, $filters, $id_category)
        {
            $_POST['id_category'] = $id_category;

            $result = $this->getQueryResult($context, $filters, $id_category, false);

            $layer_object = $this->getLayerObject($result, $context, $id_category);

            return $layer_object;
        }

        public function getProducts($context, $populate, $selected_filters, $id_category)
        {
            $_POST['id_category'] = $id_category;

            $result = $this->getQueryResult($context, $selected_filters, $id_category, true);

            $products = $result->getProducts();

            $products_in_layer = array();
            foreach ($products as $product) {
                $product = PrestAppProductController::getProduct((int) $product['id_product'], $populate, $context);
                if ($product && $product != 404 && $product != 403) {
                    array_push($products_in_layer, $product);
                }
            }

            $layer_object = $this->getLayerObject($result, $context, $id_category);

            $data = array(
                'layer' => $layer_object,
                'products' => $products_in_layer,
            );

            return $data;
        }

        public function getLayerObject($result, $context)
        {
            $facets = $result->getFacetCollection()->getFacets();

            $parsed_facets = array();

            foreach ($facets as $facet_key => $facet_value) {
                $parsed_facet = array();

                $parsed_facet['name'] = $facets[$facet_key]->getLabel();
                $parsed_facet['type'] = $facets[$facet_key]->getType() == 'attribute_group' ? 'id_attribute_group' : ($facets[$facet_key]->getType() == 'feature' ? 'id_feature' : $facets[$facet_key]->getType());
                $parsed_facet['id_key'] = (int) $facets[$facet_key]->getProperty('id_attribute_group');

                if ($facets[$facet_key]->getProperty('id_attribute_group')) {
                    $parsed_facet['id_key'] = (int) $facets[$facet_key]->getProperty('id_attribute_group');
                } elseif ($facets[$facet_key]->getProperty('id_feature')) {
                    $parsed_facet['id_key'] = (int) $facets[$facet_key]->getProperty('id_feature');
                } else {
                    $parsed_facet['id_key'] = 0;
                }

                $widget_type = $facets[$facet_key]->getWidgetType();

                if ($widget_type == 'slider') {
                    $parsed_facet['slider'] = true;
                    $parsed_facet['min'] = (int) $facets[$facet_key]->getProperty('min');
                    $parsed_facet['max'] = (int) $facets[$facet_key]->getProperty('max');
                }

                $filters = $facets[$facet_key]->getFilters();

                $parsed_filters = array();

                foreach ($filters as $filter_key => $filter_value) {
                    $parsed_filter = array();
                    $parsed_filter['name'] = $filters[$filter_key]->getLabel();
                    $parsed_filter['nbr'] = $filters[$filter_key]->getMagnitude();
                    $value = $filters[$filter_key]->getValue();

                    if (isset($value)) {
                        $parsed_filters[$value] = $parsed_filter;
                    } else {
                        array_push($parsed_filters, $parsed_filter);
                    }
                }

                $parsed_facet['values'] = $parsed_filters;

                array_push($parsed_facets, $parsed_facet);
            }

            $layer_object = array(
                'filters' => $parsed_facets,
                'filters_products_number' => $result->getTotalProductsCount(),
                'nbr_filterBlocks' => count($parsed_facets),
                'selected_filters' => $this->convertEncodedFacetsToData($result->getEncodedFacets(), $context, $id_category),
            );

            return $layer_object;
        }

        private function convertEncodedFacetsToData($encoded_facets, $context, $id_category)
        {
            $query = new ProductSearchQuery();
            $query->setIdCategory((int) $id_category)->setSortOrder(new SortOrder('product', $this->column, $this->order));

            $facets_list = $this->getFacetsList($context, $query);

            $encoded_facets_array = preg_split('~(?<!\/)/(?!\/)~', $encoded_facets);

            foreach ($encoded_facets_array as $key => $value) {
                $encoded_facets_array[$key] = $this->deserialize($value);
            }

            $data_facets = new stdClass();

            foreach ($encoded_facets_array as $encoded_facet_key => $encoded_facet_value) {
                $encoded_facet_attribute_value_array = explode('-', $encoded_facets_array[$encoded_facet_key]);

                foreach ($facets_list as $facet_key => $facet_value) {
                    if ($facets_list[$facet_key]->getLabel() == $encoded_facet_attribute_value_array[0]) {
                        if ($facets_list[$facet_key]->getType() == 'attribute_group') {
                            foreach ($encoded_facet_attribute_value_array as $encoded_facet_attribute_value_array_key => $encoded_facet_attribute_value_array_value) {
                                if ($encoded_facet_attribute_value_array_key != 0) {
                                    foreach ($facets_list[$facet_key]->getFilters() as $facet_value_filter) {
                                        if ($facet_value_filter->getLabel() == $encoded_facet_attribute_value_array[$encoded_facet_attribute_value_array_key]) {
                                            if (!property_exists($data_facets, 'id_attribute_group')) {
                                                $type = 'id_attribute_group';
                                                $data_facets->$type = array();
                                            }
                                            array_push($data_facets->$type, $facets_list[$facet_key]->getProperty('id_attribute_group') . '_' . $facet_value_filter->getValue());
                                        }
                                    }
                                }
                            }
                        } elseif ($facets_list[$facet_key]->getType() == 'feature') {
                            foreach ($encoded_facet_attribute_value_array as $encoded_facet_attribute_value_array_key => $encoded_facet_attribute_value_array_value) {
                                if ($encoded_facet_attribute_value_array_key != 0) {
                                    foreach ($facets_list[$facet_key]->getFilters() as $facet_value_filter) {
                                        if ($facet_value_filter->getLabel() == $encoded_facet_attribute_value_array[$encoded_facet_attribute_value_array_key]) {
                                            if (!property_exists($data_facets, 'id_feature')) {
                                                $type = 'id_feature';
                                                $data_facets->$type = array();
                                            }
                                            array_push($data_facets->$type, $facets_list[$facet_key]->getProperty('id_feature') . '_' . $facet_value_filter->getValue());
                                        }
                                    }
                                }
                            }
                        } elseif ($facets_list[$facet_key]->getType() == 'price') {
                            if (!property_exists($data_facets, 'price')) {
                                $type = 'price';
                                $data_facets->$type = array();
                            }
                            array_push($data_facets->$type, (int) $encoded_facet_attribute_value_array[2]);
                            array_push($data_facets->$type, (int) $encoded_facet_attribute_value_array[3]);
                        } else {
                            foreach ($encoded_facet_attribute_value_array as $encoded_facet_attribute_value_array_key => $encoded_facet_attribute_value_array_value) {
                                if ($encoded_facet_attribute_value_array_key != 0) {
                                    foreach ($facets_list[$facet_key]->getFilters() as $facet_value_filter) {
                                        if ($facet_value_filter->getLabel() == $encoded_facet_attribute_value_array[$encoded_facet_attribute_value_array_key]) {
                                            if (!property_exists($data_facets, $facet_value_filter->getType())) {
                                                $type = $facet_value_filter->getType();
                                                $data_facets->$type = array();
                                            }
                                            array_push($data_facets->$type, (string) $facet_value_filter->getValue());
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return $data_facets;
        }

        private function getQueryResult($context, $selected_filters, $id_category, $for_products)
        {
            $search_context = $this->getProductSearchContext($context);

            $query = new ProductSearchQuery();

            if ($for_products) {
                $query->setIdCategory((int) $id_category)->setSortOrder(new SortOrder('product', $this->column, $this->order));
            } else {
                $query->setIdCategory((int) $id_category);
            }

            $provider = $this->getProductSearchProviderFromModules($query);

            if ($for_products) {
                $resultsPerPage = (int) $this->n;
                if ($resultsPerPage <= 0) {
                    $resultsPerPage = Configuration::get('PS_PRODUCTS_PER_PAGE');
                }

                $query->setResultsPerPage($resultsPerPage)->setPage(max((int) $this->p, 1));

                if (($encodedSortOrder = 'product.' . $this->column . '.' . $this->order)) {
                    $query->setSortOrder(SortOrder::newFromString(
                        $encodedSortOrder
                    ));
                }
            }

            $facets_list = $this->getFacetsList($context, $query);
            $encoded_facets_array = array();

            if ($selected_filters) {
                $filters_array = json_decode($selected_filters);
            } else {
                $filters_array = new StdClass();
            }

            foreach ($filters_array as $filter_key => $filter_value) {
                if ($filter_key == 'id_attribute_group') {
                    $filters_array->attribute_group = $filter_value;
                    unset($filters_array->id_attribute_group);
                }

                if ($filter_key == 'id_feature') {
                    $filters_array->feature = $filter_value;
                    unset($filters_array->id_attribute_group);
                }

                foreach ($facets_list as $facet_key => $facet_value) {
                    if ($filter_key == 'attribute_group') {
                        foreach ($filter_value as $filter_value_item) {
                            if ($filter_key === $facet_value->getType()) {
                                $parsed_filter_value_item = explode('_', $filter_value_item);
                                $attribute_group_id = $facet_value->getProperty('id_attribute_group');
                                if ($attribute_group_id) {
                                    if ($attribute_group_id == $parsed_filter_value_item[0]) {
                                        if (!isset($encoded_facets_array[$facets_list[$facet_key]->getLabel()])) {
                                            $encoded_facets_array[$facets_list[$facet_key]->getLabel()] = array();
                                        }

                                        foreach ($facets_list[$facet_key]->getFilters() as $facet_filter_key => $facet_filter_value) {
                                            if ($facet_filter_value->getValue() == $parsed_filter_value_item[1]) {
                                                array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $facet_filter_value->getLabel());
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } elseif ($filter_key == 'feature') {
                        foreach ($filter_value as $filter_value_item) {
                            if ($filter_key === $facet_value->getType()) {
                                $parsed_filter_value_item = explode('_', $filter_value_item);
                                $attribute_group_id = $facet_value->getProperty('id_feature');
                                if ($attribute_group_id) {
                                    if ($attribute_group_id == $parsed_filter_value_item[0]) {
                                        if (!isset($encoded_facets_array[$facets_list[$facet_key]->getLabel()])) {
                                            $encoded_facets_array[$facets_list[$facet_key]->getLabel()] = array();
                                        }

                                        foreach ($facets_list[$facet_key]->getFilters() as $facet_filter_key => $facet_filter_value) {
                                            if ($facet_filter_value->getValue() == $parsed_filter_value_item[1]) {
                                                array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $this->serialize($facet_filter_value->getLabel()));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } elseif ($filter_key == 'price') {
                        if ($filter_key === $facet_value->getType()) {
                            $currency_symbol = $facet_value->getProperty('unit');
                            if (!isset($encoded_facets_array[$facets_list[$facet_key]->getLabel()])) {
                                $encoded_facets_array[$facets_list[$facet_key]->getLabel()] = array();
                            }
                            array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $currency_symbol);
                            array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $filters_array->$filter_key[0]);
                            array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $filters_array->$filter_key[1]);
                        }
                    } else {
                        if ($filter_key === $facet_value->getType()) {
                            if (!isset($encoded_facets_array[$facets_list[$facet_key]->getLabel()])) {
                                $encoded_facets_array[$facets_list[$facet_key]->getLabel()] = array();
                            }
                            foreach ($facets_list[$facet_key]->getFilters() as $facet_filter_key => $facet_filter_value) {
                                foreach ($filters_array->$filter_key as $filter_array_key => $filter_array_value) {
                                    if ($facet_filter_value->getValue() == $filter_array_value) {
                                        array_push($encoded_facets_array[$facets_list[$facet_key]->getLabel()], $facet_filter_value->getLabel());
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $filters_string = "";

            foreach ($encoded_facets_array as $key => $encoded_facet) {
                if (count($encoded_facet) > 0) {
                    $filters_string .= $key;
                    foreach ($encoded_facet as $encoded_facet_value) {
                        $filters_string .= '-' . $encoded_facet_value;
                    }
                    $filters_string .= '/';
                }
            }

            if ($filters_string) {
                if (Tools::substr($filters_string, -1) == '/') {
                    $filters_string = Tools::substr($filters_string, 0, -1);
                }
                $selected_filters = $filters_string;
            } else {
                $selected_filters = false;
            }

            $encodedFacets = $selected_filters ? $selected_filters : false;

            $query->setEncodedFacets($encodedFacets);

            $result = $provider->runQuery(
                $search_context,
                $query
            );

            if (!$result->getCurrentSortOrder()) {
                $result->setCurrentSortOrder($query->getSortOrder());
            }

            return $result;
        }

        private function serialize($value)
        {
            $result = str_replace('/', '/' . '/', $value);
            return $result;
        }

        private function deserialize($value)
        {
            $list = [];
            $escaping = false;
            $separator = '/';
            $escape = '/';
            $currentString = '';
            $arrayOfCharacters = [];
            preg_match_all('/./u', $value, $arrayOfCharacters);
            $characters = $arrayOfCharacters[0];

            foreach ($characters as $character) {
                if ($escaping) {
                    if ($character === $separator || $character === $escape) {
                        $currentString .= $character;
                    } else {
                        $currentString = $character;
                    }
                    $escaping = false;
                } else {
                    if ($character === $escape) {
                        $escaping = true;
                    } elseif ($character === $separator) {
                        $currentString = '';
                    } else {
                        $currentString .= $character;
                    }
                }
            }

            return $currentString;
        }

        private function getFacetValueFromTypeAndId($facet, $filters_array_block, $type)
        {
            $string = '';

            foreach ($filters_array_block as $filter_value_key => $filter_value_value) {
                if ($type == 'attribute_group') {
                    $parsed_filter_value_item = explode('_', $filter_value_value);
                    $filters_array_block[$filter_value_key] = $parsed_filter_value_item[1];
                }
                if ($type == 'feature') {
                    $parsed_filter_value_item = explode('_', $filter_value_value);
                    $filters_array_block[$filter_value_key] = $parsed_filter_value_item[1];
                }
                $found_filter = false;
                foreach ($facet->getFilters() as $facet_filter) {
                    if ($filters_array_block[$filter_value_key] == $facet_filter->getValue()) {
                        $found_filter = $facet_filter;
                    }
                }

                if ($found_filter) {
                    $string .= $facet->getLabel() . '-' . $found_filter->getLabel() . '/';
                }
            }

            return $string;
        }

        private function getFacetsList($context, $query = null)
        {
            $ps_facetedsearch = new Ps_Facetedsearch();

            if (version_compare($ps_facetedsearch->version, '3', '>=')) {
                $search = new Search($context);

                if (version_compare($ps_facetedsearch->version, '3.9.0', '>=')) {
                    $search->setQuery($query);
                }

                $search->initSearch(array());

                if (version_compare($ps_facetedsearch->version, '3.11.1', '>=')) {
                    $provider = new Provider($ps_facetedsearch->getDatabase());
                    $dataAccessor = new DataAccessor($ps_facetedsearch->getDatabase());

                    $filterBlockSearch = new Block(
                        $search->getSearchAdapter(),
                        $context,
                        $ps_facetedsearch->getDatabase(),
                        $dataAccessor,
                        $query,
                        $provider
                    );

                } else if (version_compare($ps_facetedsearch->version, '3.9.0', '>=')) {
                    $dataAccessor = new DataAccessor($ps_facetedsearch->getDatabase());

                    $filterBlockSearch = new Filters\Block(
                        $search->getSearchAdapter(),
                        $context,
                        $ps_facetedsearch->getDatabase(),
                        $dataAccessor,
                        $query
                    );
                } else if (version_compare($ps_facetedsearch->version, '3.7.1', '>=')) {
                    $dataAccessor = new DataAccessor($ps_facetedsearch->getDatabase());

                    $filterBlockSearch = new Filters\Block(
                        $search->getSearchAdapter(),
                        $context,
                        $ps_facetedsearch->getDatabase(),
                        $dataAccessor
                    );
                } else {
                    $filterBlockSearch = new Filters\Block(
                        $search->getSearchAdapter(),
                        $context,
                        $ps_facetedsearch->getDatabase()
                    );
                }

                $filterBlock = $filterBlockSearch->getFilterBlock(10000, array());

                if (version_compare($ps_facetedsearch->version, '3.11.1', '>=')) {
                    $dataAccessor = new DataAccessor($ps_facetedsearch->getDatabase());
                    $provider = new Provider($ps_facetedsearch->getDatabase());
                    $urlSerializer = new URLSerializer();
                    $converter = new Converter(
                        $ps_facetedsearch->getContext(),
                        $ps_facetedsearch->getDatabase(),
                        $urlSerializer,
                        $dataAccessor,
                        $provider
                    );
                } else if (version_compare($ps_facetedsearch->version, '3.7.1', '>=')) {
                    $dataAccessor = new DataAccessor($ps_facetedsearch->getDatabase());
                    $urlSerializer = new URLSerializer();
                    $converter = new Filters\Converter(
                        $ps_facetedsearch->getContext(),
                        $ps_facetedsearch->getDatabase(),
                        $urlSerializer,
                        $dataAccessor
                    );
                } elseif (version_compare($ps_facetedsearch->version, '3.7', '>=')) {
                    $urlSerializer = new URLSerializer();
                    $converter = new Filters\Converter(
                        $ps_facetedsearch->getContext(),
                        $ps_facetedsearch->getDatabase(),
                        $urlSerializer
                    );
                } else {
                    $converter = new Filters\Converter(
                        $ps_facetedsearch->getContext(),
                        $ps_facetedsearch->getDatabase()
                    );
                }

                $facets = $converter->getFacetsFromFilterBlocks(
                    $filterBlock['filters']
                );
            } else {
                $ps_facetedsearch_provider = new Ps_FacetedsearchProductSearchProvider($ps_facetedsearch);
                $query = new ProductSearchQuery();
                $menu = $ps_facetedsearch_provider->getFacetCollectionFromEncodedFacets($query);
                $filtersConverter = new Ps_FacetedsearchFiltersConverter();

                $facetedSearchFilters = $filtersConverter->getFacetedSearchFiltersFromFacets(
                    $menu->getFacets()
                );

                $filterBlock = $ps_facetedsearch->getFilterBlock($facetedSearchFilters);

                $facets = $filtersConverter->getFacetsFromFacetedSearchFilters(
                    $filterBlock['filters']
                );
            }

            return $facets;
        }

        // protected function getTemplateVarSortOrders(array $sortOrders, $currentSortOrderURLParameter)
        // {
        //     return array_map(function ($sortOrder) use ($currentSortOrderURLParameter) {
        //         $order = $sortOrder->toArray();
        //         $order['current'] = $order['urlParameter'] === $currentSortOrderURLParameter;
        //         $order['url'] = $this->updateQueryString(array(
        //             'order' => $order['urlParameter'],
        //             'page' => null,
        //         ));

        //         return $order;
        //     }, $sortOrders);
        // }

        protected function getProductSearchContext($context)
        {
            return (new ProductSearchContext())
                ->setIdShop((int) $context->shop->id)
                ->setIdLang($context->language->id)
                ->setIdCurrency($context->currency->id)
                ->setIdCustomer($context->customer->id ? $context->customer->id : null)
            ;
        }

        private function getProductSearchProviderFromModules($query)
        {
            $providers = Hook::exec(
                'productSearchProvider',
                array('query' => $query),
                null,
                true
            );

            if (!is_array($providers)) {
                $providers = array();
            }

            foreach ($providers as $provider) {
                if ($provider instanceof ProductSearchProviderInterface) {
                    return $provider;
                }
            }

            return;
        }
    }
}
