<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'adyenofficial/adyenofficial.php')) {
    require_once _PS_MODULE_DIR_ . 'adyenofficial/adyenofficial.php';

    class PrestAppAdyenofficialModuleController
    {
        protected $adyenofficial;

        public function __construct()
        {
            $this->adyenofficial = Module::getInstanceByName('adyenofficial');
        }

        public function displayPayment($context)
        {
            if (!$this->adyenofficial->active) {
                return [];
            }

            $payment_options = array();
            $paymentMethods = $this->getPaymentMethods($context);

            if (!$context->customer->is_guest &&
                !empty($paymentMethods['storedPaymentMethods']) &&
                Configuration::get('ADYEN_ENABLE_STORED_PAYMENT_METHODS')
            ) {
                $storedPaymentMethods = $paymentMethods['storedPaymentMethods'];
                foreach ($storedPaymentMethods as $storedPaymentMethod) {
                    if (!empty($storedPaymentMethod)) {
                        if (!in_array('Ecommerce', $storedPaymentMethod['supportedShopperInteractions'])) {
                            continue;
                        }
                    }

                    $payment_options[] = array(
                        'logo' => 'https://checkoutshopper-live.adyen.com/checkoutshopper/images/logos/medium/' . $storedPaymentMethod['brand'] . '.png',
                        'name' => $this->adyenofficial->l('Pay by saved') . ' '. $storedPaymentMethod['name'] . ' '.  $this->adyenofficial->l('ending') . ' : '. $storedPaymentMethod['lastFour'],
                        'is_active' => true,
                        'is_url' => true,
                        'is_redirect' => true,
                        'url_params' => array(
                            'type' => $storedPaymentMethod['type'],
                            'name' => $storedPaymentMethod['name'],
                            'stored_id' => $storedPaymentMethod['id'],
                            'lastFour' => $storedPaymentMethod['lastFour'],
                            'brand' => $storedPaymentMethod['brand']
                        )
                    );
                }
            }

            if (!empty($paymentMethods['paymentMethods'])) {
                foreach ($paymentMethods['paymentMethods'] as $paymentMethod) {
                    if ($this->isUnsupportedPaymentMethod($paymentMethod['type'])) {
                        continue;
                    }

                    $logoName = $paymentMethod['type'];
                    if ($logoName === 'scheme') {
                        $logoName = 'card';
                    }

                    $payment_options[] = array(
                        'logo' => 'https://checkoutshopper-live.adyen.com/checkoutshopper/images/logos/medium/' . $logoName . '.png',
                        'name' => $paymentMethod['name'],
                        'is_active' => true,
                        'is_url' => true,
                        'is_redirect' => true,
                        'url_params' => array(
                            'type' => $paymentMethod['type'],
                            'name' => $paymentMethod['name']
                        )
                    );
                }
            }
    
            return ($payment_options && count($payment_options)) ? $payment_options : [];
        }

        public function getPaymentUrl($context, $type, $name, $stored_id, $lastFour, $brand)
        {
            $controller = '';

            // if ($type == 'scheme') {
                $controller = 'AdyenScheme';
            // }

            $params = array(
                'paymentMethodType' => $type,
                'paymentMethodName' => $name,
                'storedPaymentApiId' => $stored_id,
                'lastFour' => $lastFour,
                'brand' => $brand,
                'paymentMethodBrand' => $type == 'scheme' ? 'card' : $type,
                'cart_id' => $context->cart->id,
                'customer_id' => $context->customer->id,
                'language_id' => $context->language->id
            );

            $params_string = '?';

            foreach($params as $key => $value) {
                if ($value) {
                    $params_string .= $key . '=' . urlencode($value) . '&';
                }
            }

            $payment_url = _PS_BASE_URL_ . '/module/prestapp/' . $controller . $params_string;
            // $payment_url = 'http://shoptest16123.local/';

            return array(
                'type' => 'payment',
                'url' => $payment_url,
            );
        }

        private function getPaymentMethods($context)
        {
            $helper_data = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
                'Adyen\PrestaShop\helper\Data'
            );

            $paymentMethods = $helper_data->fetchPaymentMethods(
                $context->cart,
                $context->language
            );

            return $paymentMethods;
        }

        private function isUnsupportedPaymentMethod($paymentMethodType)
        {
            $unsupportedPaymentMethods = array(
                'bcmc_mobile_QR',
                'wechatpay',
                'wechatpay_pos',
                'wechatpaySdk',
                'wechatpayQr'
            );

            if (in_array($paymentMethodType, $unsupportedPaymentMethods)) {
                return true;
            }

            return false;
        }
    }
}
