<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'alma/alma.php')) {
    require_once _PS_MODULE_DIR_ . 'alma/alma.php';

    class PrestAppAlmaModuleController extends ModuleCore
    {
        protected $oney;

        public function __construct()
        {
            $this->alma = Module::getInstanceByName('alma');
        }

        public function displayPayment($context)
        {
            $cart = new Cart($context->cart->id);

            $diff = Alma\PrestaShop\Model\CartData::getCartExclusion($cart);
            if (!empty($diff)) {
                return [];
            }

            $installmentPlans = Alma\PrestaShop\API\EligibilityHelper::eligibilityCheck($context);

            if (empty($installmentPlans)) {
                return [];
            }

            $feePlans = json_decode(Alma\PrestaShop\Utils\Settings::getFeePlans());
            $paymentOptions = [];
            $sortOptions = [];
            $totalCart = (float) $this->almaPriceToCents(
                Tools::ps_round((float) $cart->getOrderTotal(true, Cart::BOTH), 2)
            );

            $payment_options = array();

            foreach ($installmentPlans as $plan) {
                $installment = $plan->installmentsCount;
                $key = "general_{$installment}_{$plan->deferredDays}_{$plan->deferredMonths}";
                $plans = $plan->paymentPlan;
                $disabled = false;
                $creditInfo = [
                    'totalCart' => $totalCart,
                    'costCredit' => $plan->customerTotalCostAmount,
                    'totalCredit' => $plan->customerTotalCostAmount + $totalCart,
                    'taeg' => $plan->annualInterestRate,
                ];

                $isDeferred = Alma\PrestaShop\Utils\Settings::isDeferred($plan);
                $isInstallmentAccordingToDeferred = $isDeferred ? $installment === 1 : $installment !== 1;

                if ($isInstallmentAccordingToDeferred) {
                    if (!$plan->isEligible && $feePlans->$key->enabled && Alma\PrestaShop\Utils\Settings::showDisabledButton()) {
                        $disabled = true;
                        $plans = null;
                    } elseif (!$plan->isEligible) {
                        continue;
                    }
                    $duration = Alma\PrestaShop\Utils\Settings::getDuration($plan);
                    $valueLogo = $isDeferred ? $duration : $installment;
                    $logo = _PS_BASE_URL_SSL_ . $this->getAlmaLogo($isDeferred, $valueLogo);

                    $paymentOption = array(
                        'logo' => $logo,
                        'name' => sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPnxButtonTitleByLang($context->language->id), $installment),
                        'additional_information' => sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPnxButtonDescriptionByLang($context->language->id), $installment),
                        'additional_information_type' => 'string',
                        'url_params' => array(
                            'key' => $key,
                        ),
                        'is_url' => true,
                        'is_active' => !$disabled,
                    );

                    if ($installment > 4) {
                        $paymentOption['text'] = sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPnxAirButtonTitleByLang($context->language->id), $installment);
                        $paymentOption['desc'] = sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPnxAirButtonDescriptionByLang($context->language->id), $installment);
                    }
                    if ($isDeferred) {
                        $paymentOption['duration'] = $duration;
                        $paymentOption['key'] = $key;
                        $paymentOption['text'] = sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPaymentButtonTitleDeferredByLang($context->language->id), $duration);
                        $paymentOption['desc'] = sprintf(Alma\PrestaShop\Utils\SettingsCustomFields::getPaymentButtonDescriptionDeferredByLang($context->language->id), $duration);
                    }

                    if (!$disabled) {
                        $payment_options[] = $paymentOption;
                    }
                }
            }

            return ($payment_options && count($payment_options)) ? $payment_options : [];
        }

        public function getPaymentUrl($context, $key)
        {
            if (!$this->alma->active) {
                return false;
            }

            $authorized = false;
            foreach (Module::getPaymentModules() as $module) {
                if ($module['name'] == $this->alma->name) {
                    $authorized = true;
                }
            }

            if (!$authorized) {
                return false;
            }

            if (!$this->checkCurrency($context)) {
                return false;
            }

            $feePlans = json_decode(Alma\PrestaShop\Utils\Settings::getFeePlans());
            $dataFromKey = Alma\PrestaShop\Utils\Settings::getDataFromKey($key);

            $cart = $context->cart;
            $data = Alma\PrestaShop\Model\PaymentData::dataFromCart($cart, $context, $dataFromKey, true);
            $alma = Alma\PrestaShop\API\ClientHelper::defaultInstance();

            if (!$data || !$alma) {
                return false;
            }

            // Check that the selected installments count is indeed enabled
            $disabled = !$feePlans->$key->enabled
            || $feePlans->$key->min > $data['payment']['purchase_amount']
            || $feePlans->$key->max < $data['payment']['purchase_amount'];

            if ($disabled) {
                return false;
            }

            try {
                $payment = $alma->payments->create($data);
            } catch (Alma\API\RequestError$e) {
                return false;
            }
            return array(
                'type' => 'payment',
                'url' => $payment->url,
            );
        }

        public function formatAddr($aAddr)
        {

        }

        public function almaPriceToCents($price)
        {
            return (int) (round($price * 100));
        }

        private function getAlmaLogo($isDeferred, $value)
        {
            if ($isDeferred) {
                $logoName = "${value}j_logo.svg";
            } else {
                $logoName = "p${value}x_logo.svg";
            }

            if (is_callable('Media::getMediaPath')) {
                $logo = Media::getMediaPath(
                    _PS_MODULE_DIR_ . $this->alma->name . "/views/img/logos/${logoName}"
                );
            } else {
                $logo = $this->alma->getPathUri() . "/views/img/logos/${logoName}";
            }

            return $logo;
        }

        private function checkCurrency($context)
        {
            $cart = new Cart($context->cart->id);

            $currencyOrder = new Currency($cart->id_currency);
            $currenciesModule = $this->alma->getCurrency($cart->id_currency);

            // Check if cart currency is one of the enabled currencies
            if (is_array($currenciesModule)) {
                foreach ($currenciesModule as $currencyModule) {
                    if ($currencyOrder->id == $currencyModule['id_currency']) {
                        return true;
                    }
                }
            }

            return false;
        }
    }
}
