<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'custompaymentmethod/custompaymentmethod.php')) {
    require_once _PS_MODULE_DIR_ . 'custompaymentmethod/custompaymentmethod.php';

    class PrestAppCustompaymentmethodModuleController
    {
        protected $custompaymentmethod;

        public function __construct()
        {
            $this->custompaymentmethod = Module::getInstanceByName('custompaymentmethod');
        }

        public function displayPayment($context)
        {
            ToolsModuleCMP::registerSmartyFunctions();

            $payments = CustomPayment::getCustomPaymentMethods(true);
            $options = [];

            foreach ($payments as $payment) {
                $payment = new SeoSaMockPayment($payment['id_custom_payment_method']);

                if (!class_exists('PrestaShop\PrestaShop\Core\Payment\PaymentOption')) {
                    continue;
                }
                if (!$payment->active) {
                    continue;
                }
                if ($payment->checkAvailability() !== true) {
                    continue;
                }

                $id_custom_payment = explode('_', $payment->name);

                if (is_numeric($id_custom_payment[1])) {
                    $custom_payment = new CustomPayment($id_custom_payment[1]);
                } else {
                    continue;
                }

                if (!file_exists(_PS_MODULE_DIR_.'custompaymentmethod/logos/'.$custom_payment->logo)) {
                    $custom_payment->logo = 0;
                }

                $cart = new Cart($context->cart->id);

                $commission_summary = $this::calculateCommission(
                    $cart,
                    $custom_payment,
                    $context
                );

                $commission = $commission_summary['commission'];
                $commission_wt = $commission_summary['commission_tax_excl'];
                $discount = $commission_summary['discount'];
                $discount_wt = $commission_summary['discount_tax_excl'];

                $order_total = CustomPaymentMethod::getOrderTotal(
                    $cart,
                    $custom_payment,
                    $context
                );

                $real_total = $context->cart->getOrderTotal();
                $real_total_wt = $context->cart->getOrderTotal(false);

                $total = $real_total + $commission + $discount;
                $total_wt = $real_total_wt + $commission_wt + $discount_wt;

                $result = array(
                    'payment'            => $custom_payment,
                    'commission'         => $commission,
                    'discount'           => $discount,
                    'order_total'        => $order_total,
                    'format_commission'  => Tools::displayPrice($commission),
                    'format_discount'    => Tools::displayPrice($discount),
                    'format_order_total' => Tools::displayPrice($order_total),
                    'lang' => $context->language->id,
                    'name' => $payment->name,
                    'total' => Tools::displayPrice($total),
                    'total_wt' => Tools::displayPrice($total_wt),
                    'commission_tax' => Tools::displayPrice($total - $total_wt)
                );

                $result['custompayment_title'] = $custom_payment->name[$context->language->id];
                $result['custompayment_logo'] = _PS_BASE_URL_ . '/modules/custompaymentmethod/logos/' . $custom_payment->logo;

                $result['additional_information'] = '';

                if ($commission != 0) {
                    $result['additional_information'] .= '<p>' . $this->custompaymentmethod->l('Commission total', 'PAYMENT_INTRO') . ' : ' . $result['format_commission'] . '</p>';
                }

                if ($discount != 0) {
                    $result['additional_information'] .= '<p>' . $this->custompaymentmethod->l('Discount total', 'PAYMENT_INTRO') . ' : ' . $result['format_discount'] . '</p>';
                }

                if ($commission != 0 || $discount != 0) {
                    $result['additional_information'] .= '<p>' . $this->custompaymentmethod->l('Order total', 'PAYMENT_INTRO') . ' : ' . $result['format_order_total'] . '</p>';
                }

                $result['additional_information'] .= $custom_payment->description_short[$context->language->id];

                $result['additional_information_type'] = 'html';

                $options[] = $result;
            }

            return $options;
        }

        public function getPaymentDisplayAttributes($context, $params)
        {
            if ($params && isset($params['id']) && $params['id']) {
                $cart = new Cart($context->cart->id);

                if ($id_custom_payment = (int)$params['id']) {
                    $this->custompaymentmethod = Module::getInstanceByName($this->custompaymentmethod->name.'_'.$id_custom_payment);
                    if (Validate::isLoadedObject($this->custompaymentmethod)) {
                        $custom_payment = $this->custompaymentmethod->custom_payment;
                    }
                }

                if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0
                    ||!$this->custompaymentmethod->active
                ) {
                    return false;
                }

                $authorized = false;
                foreach (Module::getPaymentModules() as $module) {
                    if ($module['name'] == 'custompaymentmethod_'.$id_custom_payment) {
                        $authorized = true;
                        break;
                    }
                }

                if (!$authorized) {
                    return false;
                }

                if (!Validate::isLoadedObject($custom_payment)) {
                    $ustom_payment = new CustomPayment(Configuration::get(
                        'PS_CPM_DEFAULT_METHOD'
                    ), $context->language->id);
                    if (!Validate::isLoadedObject($custom_payment)) {
                        return false;
                    }
                }

                if ($this->custompaymentmethod->checkAvailability() !== true) {
                    return false;
                }

                $type = $params['id'];

                if (!Validate::isLoadedObject(new CustomPayment($type, $context->language->id))) {
                    $type = Configuration::get('PS_CPM_DEFAULT_METHOD');
                }

                $total = CustomPaymentMethod::getOrderTotal(
                    $cart,
                    $custom_payment,
                    $context
                );

                if (!file_exists(_PS_MODULE_DIR_.'custompaymentmethod/logos/'.$custom_payment->logo)) {
                    $custom_payment->logo = 0;
                }

                if (!$custom_payment->confirmation_page) {
                    return array(
                        'status' => 'TO_VALIDATE',
                        'params' => array(
                            array(
                                'type' => 'input',
                                'input_type' => 'hidden',
                                'id' => 'type',
                                'input_value' => $type
                            )
                        )
                    );
                }

                $data = [];

                $data[] = array(
                    'name' => $this->custompaymentmethod->l('You select payment method'),
                    'value' => $custom_payment->name,
                    'type' => 'string'
                );

                $data[] = array(
                    'name' => $this->custompaymentmethod->l('Total order'),
                    'value' => Tools::displayPrice($total),
                    'type' => 'string'
                );

                $data[] = array(
                    'value' => _PS_BASE_URL_ . '/modules/custompaymentmethod/logos/' . $custom_payment->logo,
                    'type' => 'image'
                );

                $data[] = array(
                    'name' => $this->custompaymentmethod->l('Details:'),
                    'value' => $custom_payment->details,
                    'type' => 'html'
                );

                $data[] = array(
                    'name' => $this->custompaymentmethod->l('Description:'),
                    'value' => $custom_payment->description,
                    'type' => 'html'
                );

                $data[] = array(
                    'type' => 'input',
                    'input_type' => 'hidden',
                    'id' => 'type',
                    'input_value' => $type
                );

                if ($custom_payment->view_message_field) {
                    $name = $custom_payment->name_message_field;

                    if ($custom_payment->required_message_field) {
                        $name .= ' (' . $this->custompaymentmethod->l('required') . ')';
                    }

                    $data[] = array(
                        'name' => $name,
                        'type' => 'input',
                        'input_type' => 'textarea',
                        'id' => 'message',
                        'required' => $custom_payment->required_message_field ? true : false,
                        'error' => $custom_payment->error_message_field ? $custom_payment->error_message_field : $this->custompaymentmethod->l('Fill out the required field.')
                    );
                }

                return $data;
            } else {
                return false;
            }
        }

        public function validateOrder($context, $params)
        {
            if ($params && isset($params['type']) && $params['type']) {
                $cart = new Cart($context->cart->id);

                if ($id_custom_payment = (int)$params['type']) {
                    $this->custompaymentmethod = Module::getInstanceByName($this->custompaymentmethod->name.'_'.$id_custom_payment);
                    if (Validate::isLoadedObject($this->custompaymentmethod)) {
                        $custom_payment = $this->custompaymentmethod->custom_payment;
                    }
                }

                if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0
                    ||!$this->custompaymentmethod->active
                ) {
                    return false;
                }

                $authorized = false;
                foreach (Module::getPaymentModules() as $module) {
                    if ($module['name'] == 'custompaymentmethod_'.$id_custom_payment) {
                        $authorized = true;
                        break;
                    }
                }

                if (!$authorized) {
                    return false;
                }

                if (!Validate::isLoadedObject($custom_payment)) {
                    $ustom_payment = new CustomPayment(Configuration::get(
                        'PS_CPM_DEFAULT_METHOD'
                    ), $context->language->id);
                    if (!Validate::isLoadedObject($custom_payment)) {
                        return false;
                    }
                }

                if ($this->custompaymentmethod->checkAvailability() !== true) {
                    return false;
                }

                $customer = new Customer($cart->id_customer);

                $currency = $context->currency;
                $total_cart = (float)$cart->getOrderTotal(Configuration::get('PS_TAX'), Cart::BOTH);

                $id_order_state = Configuration::get('PS_OS_PREPARATION');

                if ($custom_payment->id_order_state) {
                    $order_state = new OrderState($custom_payment->id_order_state);
                    if (Validate::isLoadedObject($order_state)) {
                        $id_order_state = $order_state->id;
                    }
                }

                $calculations = CustomPaymentMethod::getOrderCalculations(
                    $cart,
                    $custom_payment,
                    $context
                );

                $commission = $calculations['commission'];
                $discount = $calculations['discount'];
                $total = $calculations['total'];

                $mail_vars = array(
                    '{custom_payment_name}'    => (Tools::strlen($custom_payment->name) ? $custom_payment->name
                        : 'custom payment method'),
                    '{custom_payment_details}' => $custom_payment->details,
                );

                $mail_vars['{total_paid}'] = Tools::displayPrice($total, $context->currency, false);
                $mail_vars['{total_paid_m}'] = Tools::displayPrice($total, $context->currency, false);
                $mail_vars['{commission}'] = Tools::displayPrice($commission, $context->currency, false);
                $mail_vars['{discount}'] = Tools::displayPrice($discount, $context->currency, false);

                $this->custompaymentmethod->validateOrder(
                    (int)$cart->id,
                    $id_order_state,
                    $total_cart,
                    (Tools::strlen($custom_payment->name) ? $custom_payment->name : 'custom payment method'),
                    ($params['message'] ? $params['message'] : null),
                    $mail_vars,
                    (int)$currency->id,
                    false,
                    $customer->secure_key
                );

                $order = new Order($this->custompaymentmethod->currentOrder);

                if (version_compare(_PS_VERSION_, '1.7.0.0', '>=') && Validate::isLoadedObject($order)) {
                    $old_message = Message::getMessageByCartId((int)$cart->id);
                    if ($old_message && $old_message['private']) {
                        $update_message = new Message((int)$old_message['id_message']);
                        $update_message->id_order = (int)$order->id;
                        $update_message->update();

                        // Add this message in the customer thread
                        $customer_thread = new CustomerThread();
                        $customer_thread->id_contact = 0;
                        $customer_thread->id_customer = (int)$order->id_customer;
                        $customer_thread->id_shop = (int)$context->shop->id;
                        $customer_thread->id_order = (int)$order->id;
                        $customer_thread->id_lang = (int)$context->language->id;
                        $customer_thread->email = $customer->email;
                        $customer_thread->status = 'open';
                        $customer_thread->token = Tools::passwdGen(12);
                        $customer_thread->add();

                        $customer_message = new CustomerMessage();
                        $customer_message->id_customer_thread = $customer_thread->id;
                        $customer_message->id_employee = 0;
                        $customer_message->message = $update_message->message;
                        $customer_message->private = 1;

                        $customer_message->add();
                    }
                }

                if ($custom_payment->is_send_mail) {
                    $this->custompaymentmethod->sendMail(
                        'select_method',
                        $customer->email,
                        $this->custompaymentmethod->l('Select payment', 'validation'),
                        array(
                            '{payment_name}'        => $custom_payment->name,
                            '{payment_details}'     => $custom_payment->details,
                            '{payment_description}' => $custom_payment->description,
                            '{firstname}'           => $customer->firstname,
                            '{lastname}'            => $customer->lastname,
                        )
                    );
                }

                return $order;
            } else {
                return false;
            }
        }

        private static function calculateCommission(
            Cart $cart,
            CustomPayment $payment,
            Context $context = null,
            $return = null
        ) {
            if (null === $context) {
                $context = Context::getContext();
            }

            $commission = 0;
            $commission_tax_excl = 0;
            $discount = 0;
            $discount_tax_excl = 0;

            $total = null;
            switch ((int)$payment->type_commission) {
                case CustomPayment::TYPE_COMMISSION_AMOUNT:
                    $currency_commission = new Currency($payment->currency_commission);
                    $commission_tax_excl = Tools::convertPriceFull(
                        $payment->commission_amount,
                        $currency_commission,
                        $context->currency
                    );
                    break;
                case CustomPayment::TYPE_COMMISSION_PERCENT:
                    if ((int)$payment->apply_commission === CustomPayment::APPLY_COMMISSION_PRODUCTS) {
                        $commission_base = $cart->getOrderTotal($payment->commission_use_tax_on_products, Cart::ONLY_PRODUCTS);
                    } elseif ((int)$payment->apply_commission === CustomPayment::APPLY_COMMISSION_TOTAL) {
                        $commission_base = $cart->getOrderTotal($payment->commission_use_tax_on_products, Cart::ONLY_PRODUCTS);
                        $shipping_base = $cart->getOrderTotal($payment->commission_use_tax_on_products, Cart::ONLY_SHIPPING);
                        $commission_base += $shipping_base;
                    }

                    $commission_tax_excl = ($commission_base / 100 * $payment->commission_percent);
                    break;
            }

            $commission = $commission_tax_excl * (1 + $payment->commission_tax / 100);

            switch ((int)$payment->type_discount) {
                case CustomPayment::TYPE_COMMISSION_AMOUNT:
                    $currency_discount = new Currency($payment->currency_discount);
                    $discount_tax_excl = Tools::convertPriceFull(
                        $payment->discount_amount,
                        $currency_discount,
                        $context->currency
                    );
                    break;
                case CustomPayment::TYPE_COMMISSION_PERCENT:
                    if ((int)$payment->apply_discount === CustomPayment::APPLY_COMMISSION_PRODUCTS) {
                        $discount_base = $cart->getOrderTotal($payment->discount_use_tax_on_products, Cart::ONLY_PRODUCTS);
                    } elseif ((int)$payment->apply_discount === CustomPayment::APPLY_COMMISSION_TOTAL) {
                        $discount_base = $cart->getOrderTotal($payment->discount_use_tax_on_products, Cart::ONLY_PRODUCTS);
                        $shipping = $cart->getOrderTotal($payment->discount_use_tax_on_products, Cart::ONLY_SHIPPING);
                        $discount_base += $shipping;
                    }

                    $discount_tax_excl = ($discount_base / 100 * $payment->discount_percent);
                    break;
            }

            $discount = $discount_tax_excl * (1 + $payment->discount_tax / 100);

            if ($return === 'commission') {
                return $commission;
            }
            if ($return === 'discount') {
                return -$discount;
            }

            $total = $cart->getOrderTotal(true, Cart::BOTH);
            $total += $commission;
            $total += -$discount;

            if ($return === 'total') {
                return $total;
            }

            return array(
                'total'      => $total,
                'commission' => $commission,
                'discount'   => -$discount,
                'commission_tax_excl' => $commission_tax_excl,
                'discount_tax_excl' => -$discount_tax_excl
            );
        }
    }
}
