<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . '/dpdfrance/dpdfrance.php')) {
    require _PS_MODULE_DIR_ . '/dpdfrance/dpdfrance.php';

    class PrestAppDpdfranceModuleController
    {
        protected $dpdfrance;

        public function __construct()
        {
            $this->dpdfrance = Module::getInstanceByName('dpdfrance');
        }

        public function hookExtraCarrier($context)
        {
            if ($context->cart->id_address_delivery) {
                $address = new Address((int) $context->cart->id_address_delivery);
            }

            return array(
                'DPDFRANCE_RELAIS_CARRIER_ID' => (int)Configuration::get('DPDFRANCE_RELAIS_CARRIER_ID'),
                'DPDFRANCE_PREDICT_CARRIER_ID' => (int)Configuration::get('DPDFRANCE_PREDICT_CARRIER_ID'),
                'DPDFRANCE_CLASSIC_CARRIER_ID' => (int)Configuration::get('DPDFRANCE_CLASSIC_CARRIER_ID'),
                'EXTRA_TPL' => (
                    "<div style='padding: 20px; color:#424143; font-size: 12px;'>" .
                        "<p>" . $this->dpdfrance->l('Avec Predict, soyez autonomes sur vos livraisons !') . "</p>" .
                        "<h4 style='font-weight: 700; margin-top: 40px;'>" . $this->dpdfrance->l('Le fonctionnement :') . "</h4>" .
                        "<ul>" .
                            "<li>" . $this->dpdfrance->l('Un premier SMS vous sera envoyé lors de la prise en charge de votre colis par DPD France. Ce SMS vous indiquera également la date de livraison prévue.') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Le jour de la livraison vous recevez un SMS et un email vous indiquant le créneau horaire de livraison et votre code sécurisé à 4 chiffres. Ce code vous permettra de réceptionner votre commande en toute sécurité.') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Lors de la livraison, transmettez votre code au livreur DPD France. Votre colis vous sera remis contre signature.') . "</li>" .
                        "</ul>" .
                        "<h4 style='font-weight: 700; margin-top: 40px;'>" . $this->dpdfrance->l('Le service Predict vous permet d’être autonome sur votre livraison grâce à un large choix d’options de reprogrammation jusqu’à la veille ou en cas d’absence :') . "</h4>" .
                        "<ul>" .
                            "<li>" . $this->dpdfrance->l('Mise à disposition de votre colis en points relais') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Livraison sur votre lieu de travail') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Livraison à une autre adresse') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Livraison à un voisin') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Retrait en agence DPD France') . "</li>" .
                            "<li>" . $this->dpdfrance->l('Livraison à une autre date') . "</li>" .
                        "</ul>" .
                        "<h4 style='font-weight: 700; margin-top: 40px;'>" . $this->dpdfrance->l('Bénéficiez des avantages de la livraison Predict !') . "</h4>" .
                    "</div>"
                ),
                'EXTRA_INPUT' => array(
                   'text' =>  $this->dpdfrance->l('Bénéficiez dès maintenant des avantages de la livraison Predict en renseignant votre n° de mobile ici'),
                   'input_name' => 'gsm_dest',
                   'route' => 'registerGsm',
                   'value' =>  ($address && $address->phone) ? $address->phone : '',
                   'required' => true
                )
            );
        }

        public function getPoints($params, $context)
        {
            $points = $this->dpdfrance->getPoints(array(
                'postcode' => $params['postcode'],
            ));

            return $points;
        }

        public function saveDeliveryPoint($params, $context)
        {
            $cart = new Cart($context->cart->id);

            $params['address1'] = ((isset($params['address1']) && $params['address1']) ? $params['address1'] : '');
            $params['address2'] = ((isset($params['address2']) && $params['address2']) ? $params['address2'] : '');

            Db::getInstance()->delete('dpdfrance_shipping', 'id_cart = "' . pSQL((int)$cart->id) . '"');

            $sql = 'INSERT IGNORE INTO ' . _DB_PREFIX_ . "dpdfrance_shipping
                (id_customer, id_cart, id_carrier, service, relay_id, company, address1, address2, postcode, city, id_country, gsm_dest)
                VALUES (
                '" . (int)$cart->id_customer . "',
                '" . (int)$cart->id . "',
                '" . (int)$cart->id_carrier . "',
                'REL',
                '" . pSQL($params['pudo_id']) . "',
                '" . pSQL($params['shop_name']) . "',
                '" . pSQL($params['address1']) . "',
                '" . pSQL($params['address2']) . "',
                '" . pSQL($params['postcode']) . "',
                '" . pSQL($params['city']) . "',
                '" . pSQL($params['id_country']) . "',
                ''
            )";

            Db::getInstance()->Execute($sql);

            return true;
        }

        public function registerGsm($params, $context)
        {
            $cart = new Cart($context->cart->id);
            $input_tel = $params['gsm_dest'];

            $gsm       = str_replace([' ', '.', '-', ',', ';', '/', '\\', '(', ')'], '', $input_tel);

            if (Tools::substr($gsm, 0, 2) == '00') {
                $gsm = str_replace('00', '+', $gsm);
            }

            /* All right, delete previous entry of GSM for this cart and write the new one */
            Db::getInstance()->delete('dpdfrance_shipping', 'id_cart = "' . pSQL((int)$cart->id) . '"');
            $sql = 'INSERT IGNORE INTO ' . _DB_PREFIX_ . "dpdfrance_shipping
                            (id_customer, id_cart, id_carrier, service, relay_id, company, address1, address2, postcode, city, id_country, gsm_dest)
                            VALUES (
                            '" . (int)$cart->id_customer . "',
                            '" . (int)$cart->id . "',
                            '" . (int)$cart->id_carrier . "',
                            'PRE',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '" . pSQL($gsm) . "'
                            )";
            Db::getInstance()->Execute($sql);

            return true;
        }
    }
}
