<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'invoicepayment/invoicepayment.php')) {
    require_once _PS_MODULE_DIR_ . 'invoicepayment/invoicepayment.php';

    class PrestAppInvoicepaymentModuleController
    {
        protected $invoicepayment;

        public function __construct()
        {
            $this->invoicepayment = Module::getInstanceByName('invoicepayment');
        }

        public function displayPayment($context)
        {
            if ($this->invoicepayment->active && isset($context->cart) && $this->invoicepayment->checkCart($context->cart)) {
                return array(
                    'logo' => _PS_BASE_URL_ . '/modules/invoicepayment/' . 'views/img/invoice.png',
                    'text' => $this->invoicepayment->l('Pay with bill', 'payment'),
                    'configuration' => array(
                        'displayAttributes' => 'dynamic',
                        'pageType' => 'display',
                        'logoDataAttribute' => 'logo',
                        'textDataAttribute' => 'text'
                    )
                );
            } else {
                return false;
            }
        }

        public function getPaymentDisplayAttributes($context, $params)
        {
            $data = [];

            $total = $context->cart->getOrderTotal(true, Cart::BOTH);
            $currency = Currency::getCurrency((int)$context->cart->id_currency);

            $data[] = array(
                'name' => $this->invoicepayment->l('You have chosen the billing method.', 'validation'),
                'value' => $this->invoicepayment->l('The total amount of your order is', 'validation') . ' ' . Tools::convertPrice($total, $currency) . $currency['sign'],
                'type' => 'string'
            );

            return $data;
        }

        public function validateOrder($context)
        {
            if ($context->cart->id_customer == 0
            || $context->cart->id_address_delivery == 0
            || $context->cart->id_address_invoice == 0
            || !$this->invoicepayment->active) {
                return false;
            }

            $authorized = false;
            foreach (Module::getPaymentModules() as $module) {
                if ($module['name'] == 'invoicepayment') {
                    $authorized = true;
                    break;
                }
            }

            if (!$authorized) {
                return false;
            }

            $customer = new Customer($context->cart->id_customer);
            if (!Validate::isLoadedObject($customer)) {
                return false;
            }

            $customer = new Customer((int)$context->cart->id_customer);
            $total = $context->cart->getOrderTotal(true, Cart::BOTH);

            $this->invoicepayment->validateOrder(
                (int)$context->cart->id,
                (Configuration::get('PS_DEFAULT_STATUS_ORDER_IP') ? (int)Configuration::get('PS_DEFAULT_STATUS_ORDER_IP') : (int)Configuration::get('PS_OS_PREPARATION')),
                $total,
                $this->invoicepayment->displayName,
                null,
                array(),
                null,
                false,
                $customer->secure_key
            );

            $order = new Order($this->invoicepayment->currentOrder);
            return $order;
        }
    }
}
