<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'lyra/lyra.php')) {
    require_once _PS_MODULE_DIR_ . 'lyra/lyra.php';

    class PrestAppLyraModuleController
    {
        protected $lyra;

        public function __construct()
        {
            $this->lyra = Module::getInstanceByName('lyra');
        }

        public function displayPayment($context)
        {
            // $html = '';
            $activatedPayment = array();

            if (!$this->lyra->active) {
                return false;
            }

            if (!$this->checkCurrency($context)) {
                return false;
            }

            $cart = $context->cart;

            $standard = new LyraStandardPayment();
            if ($standard->isAvailable($cart)) {
                $vars = $standard->getTplVars($cart);
                $standardObject = array();
                $standardObject['lyra_logo'] = _PS_BASE_URL_ . $vars['lyra_logo'];
                $standardObject['lyra_title'] = $vars['lyra_title'];
                $standardObject['lyra_params_link'] = 'standard';
                array_push($activatedPayment, $standardObject);
            }

            $multi = new LyraMultiPayment();
            if ($multi->isAvailable($cart)) {
                $vars = $multi->getTplVars($cart);

                $multiObject = array();
                $multiObject['lyra_logo'] = _PS_BASE_URL_ . $vars['lyra_logo'];
                $multiObject['lyra_title'] = $vars['lyra_title'];
                $multiObject['lyra_params_link'] = 'multi';
                $multiObject['lyra_multi_options'] = array();
                $multiObject['lyra_avail_cards'] = array();

                $multiObject['lyra_multi_options']['param'] = 'lyra_opt';
                $multiObject['lyra_multi_options']['text'] = $this->lyra->l('Choose your payment option');
                $multiObject['lyra_multi_options']['values'] = array();
                foreach ($vars['lyra_multi_options'] as $key => $value) {
                    array_push($multiObject['lyra_multi_options']['values'], array(
                        "opt_id" => $key,
                        "opt_value" => $value['localized_label'],
                    ));
                }

                $multiObject['lyra_avail_cards']['param'] = 'lyra_card_type';
                $multiObject['lyra_avail_cards']['text'] = $this->lyra->l('Choose your payment mean');
                $multiObject['lyra_avail_cards']['values'] = array();
                foreach ($vars['lyra_avail_cards'] as $key => $value) {
                    array_push($multiObject['lyra_avail_cards']['values'], array(
                        "opt_id" => $key,
                        "opt_value" => $value,
                    ));
                }

                array_push($activatedPayment, $multiObject);
            }

            return $activatedPayment;
        }

        public function getPaymentParams($attribute, $data, $context)
        {
            $cart = $context->cart;

            $params = array();

            if ($attribute == 'standard') {
                $standard = new LyraStandardPayment();
                $request = $standard->prepareRequest($cart);
                $params['lyra_params'] = $request->getRequestFieldsArray(false, false);
                $params['lyra_url'] = $request->get('platform_url');
            }

            if ($attribute == 'multi') {
                $multi = new LyraMultiPayment();
                $request = $multi->prepareRequest($cart, $data);
                $params['lyra_params'] = $request->getRequestFieldsArray(false, false);
                $params['lyra_url'] = $request->get('platform_url');
            }

            return $params;
        }

        private function checkCurrency($context)
        {
            $cart = $context->cart;

            $cart_currency = new Currency((int)$cart->id_currency);
            $currencies = $this->lyra->getCurrency((int)$cart->id_currency);

            if (!is_array($currencies) || empty($currencies)) {
                return false;
            }

            foreach ($currencies as $currency) {
                if ($cart_currency->id == $currency['id_currency']) {
                    // cart currency is allowed for this module
                    return LyraApi::findCurrencyByAlphaCode($cart_currency->iso_code) != null;
                }
            }

            return false;
        }
    }
}
