<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'oney/oney.php')) {
    require_once _PS_MODULE_DIR_ . 'oney/oney.php';

    class PrestAppOneyModuleController extends ModuleCore
    {
        protected $oney;

        public function __construct()
        {
            $this->oney = Module::getInstanceByName('oney');
        }

        public function displayPayment($context)
        {
            require_once _PS_MODULE_DIR_ . $this->oney->name . '/classes/OneyPhoneNumber.php';

            if (!$this->oney->active) {
                return [];
            }

            $cart = new Cart($context->cart->id);
            $addressInvoice = new Address($cart->id_address_invoice);

            if (!(OneyPhoneNumber::isValidPhoneNumber($addressInvoice->phone, $this->oney->sIsoCodeCountry, 1) ||
                OneyPhoneNumber::isValidPhoneNumber($addressInvoice->phone_mobile, $this->oney->sIsoCodeCountry, 2))) {
                return [];
            }

            $opcSelected = $this->oney->opc->getListOpc($this->oney->sIsoCodeCountry, 1);

            $orderTotal = $cart->getOrderTotal();

            foreach ($opcSelected as $key => $opcap) {
                if ($opcap['minimum_selling_price'] > $orderTotal || $opcap['maximum_selling_price'] < $orderTotal) {
                    unset($opcSelected[$key]);
                    continue;
                }

                $params = array(
                    'business_transaction_code' => $opcap['business_transaction_code'],
                    'pricesimulation' => $orderTotal,
                );

                $opcSelected[$key]['simulation'] = $this->oney->getSimulationFinancement($params, $this->oney->sIsoCodeCountry);
                $opcSelected[$key]['url'] = $context->link->getModuleLink($this->oney->name, 'paymenturl') .
                    '?opc=' . $opcap['business_transaction_code'] .
                    '&short_label=' . $opcap['business_transaction_code'];

                $opcSelected[$key]['url_params'] = array(
                    'opc' => $opcap['business_transaction_code'],
                    'short_label' => $opcap['business_transaction_code']
                );

                $opcSelected[$key]['additional_information'] = '<p>' . $this->oney->l('Apport :') . ' ' . Tools::displayPrice($opcSelected[$key]['simulation']->down_payment_amount, $context->currency) . '</p>';
                $opcSelected[$key]['additional_information'] .= '<p>' . $this->oney->l('Dont coût du financement :') . ' ' . Tools::displayPrice($opcSelected[$key]['simulation']->total_cost, $context->currency) . '</p>';

                foreach ($opcSelected[$key]['simulation']->instalments as $k => $instalment) {
                    if ($k == 0) {
                        $opcSelected[$key]['additional_information'] .= '<p>- ' . ((int)$k + 1) . $this->oney->l('ère mensualité : ');
                    } else {
                        $opcSelected[$key]['additional_information'] .= '<p>- ' . ((int)$k + 1) . $this->oney->l('éme mensualité : ');
                    }

                    $cur = new Currency($context->currency->id);

                    $opcSelected[$key]['additional_information'] .= Tools::displayPrice($instalment->instalment_amount, $cur) . '</p>';
                }

                $opcSelected[$key]['additional_information_type'] = 'html';

                $opcSelected[$key]['is_active'] = true;
                $opcSelected[$key]['is_url'] = true;

                if ($opcSelected[$key]['maximum_number_of_instalments'] == 2) {
                    $opcSelected[$key]['logo'] = _PS_BASE_URL_ . $this->oney->_path . 'views/img/logo_oney-financement_3';
                    if (isset($opcSelected[$key]['free_business_transaction']) && $opcSelected[$key]['free_business_transaction'] == 1) {
                        $opcSelected[$key]['logo'] .= '_1';
                    }
                } elseif ($opcSelected[$key]['maximum_number_of_instalments'] == 3) {
                    $opcSelected[$key]['logo'] = _PS_BASE_URL_ . $this->oney->_path . 'views/img/logo_oney-financement_4';
                    if (isset($opcSelected[$key]['free_business_transaction']) && $opcSelected[$key]['free_business_transaction'] == 1) {
                        $opcSelected[$key]['logo'] .= '_1';
                    }
                } else {
                    $opcSelected[$key]['logo'] = _PS_BASE_URL_ . $this->oney->_path . 'views/img/logo_default.jpg';
                }
            }

            return $opcSelected;
        }

        public function getPaymentUrl($context, $sSlectedOpc, $shortLabel)
        {
            $module = $this->oney;
            $cart = new Cart($context->cart->id);
            $customer = new Customer($context->cart->id_customer);
            $addressShipping = new Address($context->cart->id_address_delivery);
            $addressInvoice = new Address($context->cart->id_address_invoice);

            $countryShipping = new Country(CountryCore::getIdByName($cart->id_lang, $addressShipping->country));
            $countryInvoice = new Country(CountryCore::getIdByName($cart->id_lang, $addressInvoice->country));

            if (!$cart->id || !$sSlectedOpc) {
                return false;
            }

            $merchant_guid = Configuration::get('FACILYPAY_MERCHANT_GUID_' . $module->sIsoCodeCountry);
            $psp_guid = Configuration::get('FACILYPAY_PSP_GUID_' . $module->sIsoCodeCountry);

            $reference = $cart->id;

            $products = $cart->getProducts();
            $maint_item = array('price' => 0, 'reference' => 0);
            $item_list = array();

            foreach ($products as $product) {
                $arrayTmp = array();
                $price = Tools::ps_round(
                    (isset($product['price_with_reduction']) ? $product['price_with_reduction'] : $product['price_wt']),
                    2
                );
                if ($price > $maint_item['price']) {
                    $maint_item['reference'] = sizeof($item_list);
                    $maint_item['price'] = $price;
                }
                $arrayTmp['category_code'] = Configuration::get('FACILYPAY_CATEGORY');
                $arrayTmp['label'] = utf8_encode($product['name']);
                $arrayTmp['item_external_code'] = $product['reference'];
                $arrayTmp['quantity'] = $product['quantity'];
                $arrayTmp['price'] = $price;
                $arrayTmp['is_main_item'] = 0;
                $arrayTmp['travel']['journey'] = array();
                $item_list[] = $arrayTmp;
            }
            $item_list[$maint_item['reference']]['is_main_item'] = 1;

            $carrier_selected = $module->facilypay_carrier->getInfoByCarrier($cart->id_carrier);

            try {
                $aParamsEncode = $module->pepsWebService->paramsEncrypt(
                    array(
                        'id_cart' => $cart->id,
                        'nb_product' => count($item_list),
                        'amount' => $cart->getOrderTotal(),
                        'transaction_id' => $shortLabel
                    )
                );
            } catch (Exception $e) {
                return false;
            }

            $aShippingAddress = $this->formatAddr($addressShipping->address1  . ' ' . $addressShipping->address2);
            $aInvoiceAddress = $this->formatAddr($addressInvoice->address1  . ' ' . $addressInvoice->address2);
            $oGender = new Gender($customer->id_gender);

            $landPhone=(isset($addressShipping->phone) && !empty($addressShipping->phone)) ?
            OneyPhoneNumber::isValidPhoneNumber(
                $addressShipping->phone,
                $countryInvoice->iso_code,
                1
            ) : OneyPhoneNumber::isValidPhoneNumber(
                $addressShipping->phone_mobile,
                $countryInvoice->iso_code,
                2
            );
            $mobilePhone=(
                isset($addressInvoice->phone_mobile) && !empty($addressInvoice->phone_mobile)
            ) ? OneyPhoneNumber::isValidPhoneNumber(
                $addressInvoice->phone_mobile,
                $countryInvoice->iso_code,
                1
            ) : OneyPhoneNumber::isValidPhoneNumber(
                $addressInvoice->phone,
                $countryInvoice->iso_code,
                2
            );

            $parameters = array(
                'language_code' => $module->sIsoCodeCountry,
                'merchant_guid' => $merchant_guid,
                'merchant_request_id' => date('YmdHis') . '-' . $cart->id,
                'purchase' => array(
                    'external_reference_type' => 'CMDE',
                    'external_reference' => (string)($reference),
                    'purchase_amount' => $cart->getOrderTotal(),
                    'currency_code' => $context->currency->iso_code,

                    'delivery' => array(
                        'delivery_date' => date('Y-m-d'),
                        'delivery_mode_code' => (!empty($carrier_selected['delivery_mode'])) ?
                            $carrier_selected['delivery_mode'] : 1,
                        'delivery_option' => (!empty($carrier_selected['priority'])) ?
                            $carrier_selected['priority'] : 1,
                        'priority_delivery_code' => $carrier_selected['priority_delivery_code'],
                        'address_type' => (!empty($carrier_selected['address_type'])) ?
                            $carrier_selected['address_type'] : 1,
                        'recipient' => array(
                            'recipient_honorific_code' => (int)$oGender->type+1,
                            'surname' => $customer->lastname,
                            'first_name' => $customer->firstname,
                            'phone_number' => (!$landPhone) ? "" : $landPhone
                        ),
                        'delivery_address' => array(
                            'line1' => $aShippingAddress[0],
                            'line2' => isset($aShippingAddress[1]) ? $aShippingAddress[1] : '',
                            'line3' => isset($aShippingAddress[2]) ? $aShippingAddress[2] : '',
                            'line4' => isset($aShippingAddress[3]) ? $aShippingAddress[3] : '',
                            'postal_code' => $addressShipping->postcode,
                            'municipality' => $addressShipping->city,
                            'country_code' => FacilyPayCountry::getIsoAlpha3($countryShipping->iso_code),
                            'country_label' => $countryShipping->name[$customer->id_lang]
                        ),
                    ),
                    "number_of_items" => count($item_list),
                    'item_list' => $item_list,
                ),
                'customer' => array(
                    'customer_external_code' => (string)($customer->id),
                    'language_code' => $countryInvoice->iso_code,
                    'identity' => array(
                        'person_type' => (isset($customer->siret) && !empty($customer->siret)) ? '1' : '2',
                        'honorific_code' => 0,
                        'birth_name' => $customer->lastname,
                        'first_name' => $customer->firstname,
                    ),
                    'contact_details' => array(
                        'landline_number' => (!$landPhone) ? "" : $landPhone,
                        'mobile_phone_number' => (!$mobilePhone) ? "" : $mobilePhone,
                        'email_address' => Tools::strtolower($customer->email),
                    ),
                    'customer_address' => array(
                        'line1' => $aInvoiceAddress[0],
                        'line2' => isset($aInvoiceAddress[1]) ? $aInvoiceAddress[1] : '',
                        'line3' => isset($aInvoiceAddress[2]) ? $aInvoiceAddress[2] : '',
                        'line4' => isset($aInvoiceAddress[3]) ? $aInvoiceAddress[3] : '',
                        'postal_code' => $addressInvoice->postcode,
                        'municipality' => $addressInvoice->city,
                        'country_code' => FacilyPayCountry::getIsoAlpha3($countryInvoice->iso_code),
                        'country_label' => $countryInvoice->name[$customer->id_lang],
                    ),
                ),
                'payment' => array(
                    'merchant_guid' => $merchant_guid,
                    'payment_amount' => $cart->getOrderTotal(),
                    'currency_code' => $context->currency->iso_code,
                    'business_transaction' => array(
                        'code' => $sSlectedOpc
                    ),
                ),
                'navigation' => array(
                    'success_url' => $context->link->getModuleLink(
                        $module->name,
                        'validation',
                        array('reference' => $reference),
                        true
                    ),
                    'fail_url' => $context->link->getPageLink('order.php', true),
                    'server_response_url' => Context::getContext()->link->getModuleLink(
                        $module->name,
                        'orderUpdate',
                        array('iso_code' => $module->sIsoCodeCountry),
                        true
                    )
                ),
                'merchant_context' => $aParamsEncode[0],
                'psp_context' => $aParamsEncode[1]
            );

            if ($psp_guid) {
                if (!empty($psp_guid) || $psp_guid != 0) {
                    $parameters['psp_guid'] = $psp_guid;
                }
            }

            $result = $module->pepsWebService->paiement($parameters, $module->sIsoCodeCountry);

            $returnUrl = false;

            if (is_object($result)) {
                if (isset($result->encrypted_message) && !empty($result->encrypted_message)) {
                    $objResult = $this->pepsWebService->decryptBody(
                        $result->encrypted_message,
                        $module->sIsoCodeCountry
                    );
                    $returnUrl = $objResult->returned_url;
                } else {
                    $returnUrl = $result->returned_url;
                }
            } else {
                return false;
            }

            return array(
                'type' => 'payment',
                'url' => $returnUrl,
            );
        }

        public function formatAddr($aAddr)
        {
            $aReturn = array();
            if (Tools::strlen($aAddr) > 35) {
                $aReturn[] = Tools::substr($aAddr, 0, strrpos(Tools::substr($aAddr, 0, 35), ' '));
                $aReturn[] = Tools::substr($aAddr, strrpos(Tools::substr($aAddr, 0, 35), ' '));

                foreach ($aReturn as $key => $value) {
                    if (Tools::strlen($value) > 35) {
                        $aReturn[$key] = Tools::substr(
                            $value,
                            0,
                            strrpos(Tools::substr($value, 0, 35), ' ')
                        );
                        $aReturn[] = Tools::substr($value, strrpos(Tools::substr($value, 0, 35), ' '));
                    }
                }
            } else {
                $aReturn[] = $aAddr;
            }

            return $aReturn;
        }
    }
}
