<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'paybox/paybox.php')) {
    require_once _PS_MODULE_DIR_ . 'paybox/paybox.php';

    class PrestAppPayboxModuleController
    {
        protected $paybox;

        public function __construct()
        {
            $this->paybox = Module::getInstanceByName('paybox');
        }

        public function displayPayment($context)
        {
            $cart = $context->cart;
            $customer = new Customer((int)$cart->id_customer);

            if (version_compare($this->paybox->version, '4.5.1', '=')) {
                $this->paybox->pbx_total = $cart->getOrderTotal() * 100;
                $this->paybox->pbx = new PayboxConfiguration();
                $this->paybox->pbx_devise = $context->currency->iso_code_num;
                $this->paybox->cart = $cart;
                $this->paybox->pbx_porteur = $customer->email;
                $this->paybox->essential = new PayboxEssential();
                $this->paybox->date = date('c');
            }

            if (!$this->paybox->active) {
                return false;
            }

            $this->paybox->loadAsset(true);

            $payment_options = array();

            // 1.7 Update - One Payment
            if ((int)$this->paybox->pbx->configs['PBX_FULL_INTEGRATION'] === 0 && (int)$this->paybox->pbx->configs['PBX_SIMPLE_ADVANCED'] === 1) {
                if ($this->paybox->pbx->configs['PBX_MULTIPLE_1X_PAYMENTS'] == 1 && $this->paybox->pbx_total >= ($this->paybox->pbx->configs['PBX_1X_MINIMUM'] * 100)) {
                    if (version_compare($this->paybox->version, '4.5.1', '=')) {
                        $payment_options[] = $this->paymentOne(
                            $this->paybox->pbx->configs,
                            $this->paybox->pbx->account,
                            $this->paybox->pbx_total,
                            $this->paybox->pbx_devise,
                            $this->paybox->cart->id,
                            $this->paybox->pbx_porteur,
                            $this->paybox->essential->retour,
                            $this->paybox->date,
                            $this->paybox->essential->lang,
                            $this->paybox->essential->urls
                        );
                    } else {
                        $payment_options[] = $this->paybox->paymentOne(
                            $this->paybox->pbx->configs,
                            $this->paybox->pbx->account,
                            $this->paybox->pbx_total,
                            $this->paybox->pbx_devise,
                            $this->paybox->cart->id,
                            $this->paybox->pbx_porteur,
                            $this->paybox->essential->retour,
                            $this->paybox->date,
                            $this->paybox->essential->lang,
                            $this->paybox->essential->urls
                        );
                    }
                }
            }

            /* Direct payment */
            if ((int)$this->paybox->pbx->configs['PBX_FULL_INTEGRATION'] === 1) {
                $payment_options[] = $this->getEmbeddedPaymentOption();
            }


            $paybox_wallet = new PayboxWallet($this->paybox->customer->id);

            if ((int)$this->paybox->pbx->configs['PBX_SOLUTION'] === 2 && (int)$this->paybox->pbx->configs['PBX_WALLET'] === 1 && (int)$this->paybox->pbx->configs['PBX_FULL_INTEGRATION'] === 1 && $paybox_wallet->parameters['id_paybox_wallet'] != 0) {
                $payment_options[] = $this->paybox->getEmbeddedPaymentOptionWallet();
            }

            /* 1.7 Update
            /* Contracts - Show specific card types */
            if ((int)$this->paybox->pbx->configs['PBX_FULL_INTEGRATION'] === 0 && (int)$this->paybox->pbx->configs['PBX_SIMPLE_ADVANCED'] === 0) {
                $results = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'paybox_contracts` WHERE active = 1');
                foreach ($results as $contract) {
                    $payment_options[] = $this->paybox->paymentContracts($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls, $contract);
                }
            }

            /* Payments NX times */
            if ($this->paybox->pbx->configs['PBX_MULTIPLE_2X_PAYMENTS'] == 1 && $this->paybox->pbx_total >= ($this->paybox->pbx->configs['PBX_2X_MINIMUM'] * 100)) {
                if (version_compare($this->paybox->version, '4.5.1', '=')) {
                    $payment_options[] = $this->paymentTwo(
                        $this->paybox->pbx->configs,
                        $this->paybox->pbx->account,
                        $this->paybox->pbx_total,
                        $this->paybox->pbx_devise,
                        $this->paybox->cart->id,
                        $this->paybox->pbx_porteur,
                        $this->paybox->essential->retour,
                        $this->paybox->date,
                        $this->paybox->essential->lang,
                        $this->paybox->essential->urls
                    );
                } else {
                    $payment_options[] = $this->paybox->paymentTwo(
                        $this->paybox->pbx->configs,
                        $this->paybox->pbx->account,
                        $this->paybox->pbx_total,
                        $this->paybox->pbx_devise,
                        $this->paybox->cart->id,
                        $this->paybox->pbx_porteur,
                        $this->paybox->essential->retour,
                        $this->paybox->date,
                        $this->paybox->essential->lang,
                        $this->paybox->essential->urls
                    );
                }
            }
            if ($this->paybox->pbx->configs['PBX_MULTIPLE_3X_PAYMENTS'] == 1 && $this->paybox->pbx_total >= ($this->paybox->pbx->configs['PBX_3X_MINIMUM'] * 100)) {
                if (version_compare($this->paybox->version, '4.5.1', '=')) {
                    $payment_options[] = $this->paymentThree($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls);
                } else {
                    $payment_options[] = $this->paybox->paymentThree($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls);
                }
            }

            if ($this->paybox->pbx->configs['PBX_MULTIPLE_4X_PAYMENTS'] == 1 && $this->paybox->pbx_total >= ($this->paybox->pbx->configs['PBX_4X_MINIMUM'] * 100)) {
                if (version_compare($this->paybox->version, '4.5.1', '=')) {
                    $payment_options[] = $this->paymentFour($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls);
                } else {
                    $payment_options[] = $this->paybox->paymentFour($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls);
                }
            }

            if ($this->paybox->pbx->configs['PBX_ENABLE_SUBSCRIPTIONS'] == 1) {
                $payment_options[] = $this->paybox->paymentSubscription($this->paybox->pbx->configs, $this->paybox->pbx->account, $this->paybox->pbx_total, $this->paybox->pbx_devise, $this->paybox->cart->id, $this->paybox->pbx_porteur, $this->paybox->essential->retour, $this->paybox->date, $this->paybox->essential->lang, $this->paybox->essential->urls);
            }

            $finals = [];

            foreach ($payment_options as $option) {
                if (version_compare($this->paybox->version, '4.5.1', '=')) {
                    if ($option) {
                        $object = array();
                        $object['logo'] = _PS_BASE_URL_ . '/modules/paybox/img/' . $option['gateway_img'];
                        $object['text'] = $option['pbx_text'];
                        if (is_array($option)) {
                            unset($option['pbx_text']);
                            unset($option['gateway_img']);
                        }
                        $object['inputs'] = $option;
                        $object['action'] = $this->paybox->essential->system_url;
                    }
                    array_push($finals, $object);
                } else {
                    $payment = array();
                    $payment['logo'] = $option->getLogo();
                    $payment['text'] = $option->getCallToActionText();
                    $payment['additionalInformation'] = $option->getAdditionalInformation();
                    $payment['action'] = $option->getAction();
                    $payment['is_active'] = true;
                    $payment['is_url'] = true;
                    $payment['url_present'] = true;

                    $payment['inputs'] = array();

                    foreach ($option->getInputs() as $input) {
                        $payment['inputs'][$input['name']] = (string) $input['value'];
                    }

                    $gateway = $this->getGateway();
                    $demo_mode = (int)Configuration::get('PBX_DEMO_MODE');

                    if ((int)$demo_mode === 0) {
                        $payment['action'] = 'https://preprod-tpeweb.'.$gateway.'/cgi/ChoixPaiementMobile.cgi';
                    } else {
                        $serveurs = array('tpeweb.'.$gateway, //serveur primaire
                                        'tpeweb1.'.$gateway); //serveur secondaire

                        $serveur_ok = '';
                        foreach ($serveurs as $serveur) {
                            $doc = new DOMDocument();
                            $doc->loadHTMLFile('https://'.$serveur.'/load.html');
                            $server_status = '';
                            $element = $doc->getElementById('server_status');
                            if ($element) {
                                $server_status = $element->textContent;
                            }
                            if ($server_status == 'OK') {
                                $serveur_ok = $serveur;
                                break;
                            }
                        }
                        if (!$serveur_ok) {
                            $payment['action'] = 'Error with Paybox server';
                        }

                        $payment['action'] = 'https://'.$serveur_ok.'/cgi/ChoixPaiementMobile.cgi';
                    }

                    array_push($finals, $payment);
                }
            }

            return $finals;
        }

        public function getGateway()
        {
            $gateway = Configuration::get('PBX_GATEWAY');
            if ($gateway === 'paybox') {
                return "paybox.com";
            }
            return "e-transactions.fr";
        }

        private function paymentFour($configs, $account, $pbx_total, $pbx_devise, $id_cart, $pbx_porteur, $pbx_retour, $date, $pbx_langue, $urls)
        {
            if ($configs['PBX_MULTIPLE_4X_PAYMENTS'] == 1 && $pbx_total >= ($configs['PBX_4X_MINIMUM'] * 100)) {
                $additional3DSv2Details = $this->paybox->get3DSv2Details(new Cart((int) $id_cart));
                $urls['conf'] .= '&payment_type=paymentFour';
                $amount1 = round((int)$pbx_total / 4, 0);
                $amount2 = (int)$pbx_total - $amount1 - $amount1 - $amount1;
                $amount3 = (int)$pbx_total - $amount1 - $amount1 - $amount2;
                $amount4 = (int)$pbx_total - $amount1 - $amount2 - $amount3;

                $date1 = strftime('%d/%m/%Y', strtotime('+'.$configs['PBX_DELAY'].' day'));
                $date2 = strftime('%d/%m/%Y', strtotime('+'.($configs['PBX_DELAY'] * 2).' day'));
                $date3 = strftime('%d/%m/%Y', strtotime('+'.($configs['PBX_DELAY'] * 3).' day'));

                $msg = 'PBX_SITE='.$account['PBX_SITE']
                .'&PBX_RANG='.$account['PBX_RANG']
                .'&PBX_IDENTIFIANT='.$account['PBX_ID']
                .'&PBX_TOTAL='.$amount1
                .'&PBX_DEVISE='.$pbx_devise
                .'&PBX_CMD=4x'.$id_cart
                .'&PBX_PORTEUR='.$pbx_porteur
                .'&PBX_RETOUR='.$pbx_retour
                .'&PBX_HASH=SHA512&PBX_TIME='.$date
                .'&PBX_LANGUE='.$pbx_langue
                .'&PBX_ANNULE='.$urls['cancel']
                .'&PBX_EFFECTUE='.$urls['conf']
                .'&PBX_REFUSE='.$urls['conf']
                .'&PBX_REPONDRE_A='.$urls['repondre_a']
                .'&PBX_SHOPPINGCART='.$additional3DSv2Details['PBX_SHOPPINGCART']
                .'&PBX_BILLING='.$additional3DSv2Details['PBX_BILLING']
                .'&PBX_SOURCE=RWD'
                .'&PBX_2MONT1='.(string)$amount2
                .'&PBX_DATE1='.(string)$date1
                .'&PBX_2MONT2='.(string)$amount3
                .'&PBX_DATE2='.(string)$date2
                .'&PBX_2MONT3='.(string)$amount4
                .'&PBX_DATE3='.(string)$date3;

                $params = array();

                if ($configs['PBX_DISABLE_3D'] == 1 && $pbx_total < ($configs['PBX_3D_MINIMUM'] * 100)) {
                    $msg .= '&PBX_3DS=N';
                }

                $gateway = "Crédit Agricole";

                $params['PBX_SITE'] = $account['PBX_SITE'];
                $params['PBX_RANG'] = $account['PBX_RANG'];
                $params['PBX_IDENTIFIANT'] = $account['PBX_ID'];
                $params['PBX_TOTAL'] = $amount1;
                $params['PBX_DEVISE'] = $pbx_devise;
                $params['PBX_CMD'] = '4x'.(int)$id_cart;
                $params['PBX_PORTEUR'] = $pbx_porteur;
                $params['PBX_RETOUR'] = $pbx_retour;
                $params['PBX_HASH'] = 'SHA512';
                $params['PBX_TIME'] = $date;
                $params['PBX_LANGUE'] = $pbx_langue;
                $params['PBX_ANNULE'] = $urls['cancel'];
                $params['PBX_EFFECTUE'] = $urls['conf'];
                $params['PBX_REFUSE'] = $urls['conf'];
                $params['PBX_REPONDRE_A'] = $urls['repondre_a'];
                $params['PBX_SHOPPINGCART'] = $additional3DSv2Details['PBX_SHOPPINGCART'];
                $params['PBX_BILLING'] = $additional3DSv2Details['PBX_BILLING'];
                $params['PBX_SOURCE'] = 'RWD';
                $params['PBX_2MONT1'] = $amount2;
                $params['PBX_DATE1'] = (string)$date1;
                $params['PBX_2MONT2'] = $amount3;
                $params['PBX_DATE2'] = (string)$date2;
                $params['PBX_2MONT3'] = $amount3;
                $params['PBX_DATE3'] = (string)$date3;
                $params['PBX_HMAC'] = Tools::strtoupper(hash_hmac('sha512', $msg, $account['bin_key']));

                // $params['PBX_NB_INSTALMENT'] = 1;
                $params['pbx_text'] = sprintf($this->paybox->l('Pay by credit card over 4 installments'));

                $gateway = Configuration::get('PBX_GATEWAY');
                $params['gateway_img'] = 'paybox.png';
                if ($gateway == 'ca') {
                    $params['gateway_img'] = 'paybox_ca.png';
                }

                return $params;
            }

            return false;
        }

        private function paymentThree($configs, $account, $pbx_total, $pbx_devise, $id_cart, $pbx_porteur, $pbx_retour, $date, $pbx_langue, $urls)
        {
            if ($configs['PBX_MULTIPLE_3X_PAYMENTS'] == 1 && $pbx_total >= ($configs['PBX_3X_MINIMUM'] * 100)) {
                $additional3DSv2Details = $this->paybox->get3DSv2Details(new Cart((int) $id_cart));
                $urls['conf'] .= '&payment_type=paymentThree';
                $amount1 = round((int)$pbx_total / 3, 0);
                $amount2 = (int)$pbx_total - $amount1 - $amount1;
                $amount3 = (int)$pbx_total - $amount1 - $amount2;

                $date1 = strftime('%d/%m/%Y', strtotime('+'.$configs['PBX_DELAY'].' day'));
                $date2 = strftime('%d/%m/%Y', strtotime('+'.($configs['PBX_DELAY'] * 2).' day'));
                $msg = 'PBX_SITE='.$account['PBX_SITE']
                .'&PBX_RANG='.$account['PBX_RANG']
                .'&PBX_IDENTIFIANT='.$account['PBX_ID']
                .'&PBX_TOTAL='.$amount1
                .'&PBX_DEVISE='.$pbx_devise
                .'&PBX_CMD=3x'.$id_cart
                .'&PBX_PORTEUR='.$pbx_porteur
                .'&PBX_RETOUR='.$pbx_retour
                .'&PBX_HASH=SHA512&PBX_TIME='.$date
                .'&PBX_LANGUE='.$pbx_langue
                .'&PBX_ANNULE='.$urls['cancel']
                .'&PBX_EFFECTUE='.$urls['conf']
                .'&PBX_REFUSE='.$urls['conf']
                .'&PBX_REPONDRE_A='.$urls['repondre_a']
                .'&PBX_SHOPPINGCART='.$additional3DSv2Details['PBX_SHOPPINGCART']
                .'&PBX_BILLING='.$additional3DSv2Details['PBX_BILLING']
                .'&PBX_SOURCE=RWD'
                .'&PBX_2MONT1='.(string)$amount2
                .'&PBX_DATE1='.(string)$date1
                .'&PBX_2MONT2='.(string)$amount3
                .'&PBX_DATE2='.(string)$date2;

                $params = array();

                if ($configs['PBX_DISABLE_3D'] == 1 && $pbx_total < ($configs['PBX_3D_MINIMUM'] * 100)) {
                    $msg .= '&PBX_3DS=N';
                }

                $gateway = "Crédit Agricole";

                $params['PBX_SITE'] = $account['PBX_SITE'];
                $params['PBX_RANG'] = $account['PBX_RANG'];
                $params['PBX_IDENTIFIANT'] = $account['PBX_ID'];
                $params['PBX_TOTAL'] = $amount1;
                $params['PBX_DEVISE'] = $pbx_devise;
                $params['PBX_CMD'] = '3x'.(int)$id_cart;
                $params['PBX_PORTEUR'] = $pbx_porteur;
                $params['PBX_RETOUR'] = $pbx_retour;
                $params['PBX_HASH'] = 'SHA512';
                $params['PBX_TIME'] = $date;
                $params['PBX_LANGUE'] = $pbx_langue;
                $params['PBX_ANNULE'] = $urls['cancel'];
                $params['PBX_EFFECTUE'] = $urls['conf'];
                $params['PBX_REFUSE'] = $urls['conf'];
                $params['PBX_REPONDRE_A'] = $urls['repondre_a'];
                $params['PBX_SHOPPINGCART'] = $additional3DSv2Details['PBX_SHOPPINGCART'];
                $params['PBX_BILLING'] = $additional3DSv2Details['PBX_BILLING'];
                $params['PBX_SOURCE'] = 'RWD';
                $params['PBX_2MONT1'] = $amount2;
                $params['PBX_DATE1'] = (string)$date1;
                $params['PBX_2MONT2'] = $amount3;
                $params['PBX_DATE2'] = (string)$date2;
                $params['PBX_HMAC'] = Tools::strtoupper(hash_hmac('sha512', $msg, $account['bin_key']));

                // $params['PBX_NB_INSTALMENT'] = 1;
                $params['pbx_text'] = sprintf($this->paybox->l('Pay by credit card over 3 installments'));

                $gateway = Configuration::get('PBX_GATEWAY');
                $params['gateway_img'] = 'paybox.png';
                if ($gateway == 'ca') {
                    $params['gateway_img'] = 'paybox_ca.png';
                }

                return $params;
            }

            return false;
        }

        private function paymentTwo($configs, $account, $pbx_total, $pbx_devise, $id_cart, $pbx_porteur, $pbx_retour, $date, $pbx_langue, $urls)
        {
            if ($configs['PBX_MULTIPLE_2X_PAYMENTS'] == 1 && $pbx_total >= ($configs['PBX_2X_MINIMUM'] * 100)) {
                $additional3DSv2Details = $this->paybox->get3DSv2Details(new Cart((int) $id_cart));
                $urls['conf'] .= '&payment_type=paymentTwo';
                $amount1 = round((int)$pbx_total / 2, 0);
                $amount2 = (int)$pbx_total - $amount1;

                $date1 = strftime('%d/%m/%Y', strtotime('+'.$configs['PBX_DELAY'].' day'));
                $msg = 'PBX_SITE='.$account['PBX_SITE']
                .'&PBX_RANG='.$account['PBX_RANG']
                .'&PBX_IDENTIFIANT='.$account['PBX_ID']
                .'&PBX_TOTAL='.$amount1
                .'&PBX_DEVISE='.$pbx_devise
                .'&PBX_CMD=2x'.$id_cart
                .'&PBX_PORTEUR='.$pbx_porteur
                .'&PBX_RETOUR='.$pbx_retour
                .'&PBX_HASH=SHA512&PBX_TIME='.$date
                .'&PBX_LANGUE='.$pbx_langue
                .'&PBX_ANNULE='.$urls['cancel']
                .'&PBX_EFFECTUE='.$urls['conf']
                .'&PBX_REFUSE='.$urls['conf']
                .'&PBX_REPONDRE_A='.$urls['repondre_a']
                .'&PBX_SHOPPINGCART='.$additional3DSv2Details['PBX_SHOPPINGCART']
                .'&PBX_BILLING='.$additional3DSv2Details['PBX_BILLING']
                .'&PBX_SOURCE=RWD'
                .'&PBX_2MONT1='.(string)$amount2
                .'&PBX_DATE1='.(string)$date1;

                $params = array();

                if ($configs['PBX_DISABLE_3D'] == 1 && $pbx_total < ($configs['PBX_3D_MINIMUM'] * 100)) {
                    $msg .= '&PBX_3DS=N';
                }

                $gateway = "Crédit Agricole";

                $params['PBX_SITE'] = $account['PBX_SITE'];
                $params['PBX_RANG'] = $account['PBX_RANG'];
                $params['PBX_IDENTIFIANT'] = $account['PBX_ID'];
                $params['PBX_TOTAL'] = $amount1;
                $params['PBX_DEVISE'] = $pbx_devise;
                $params['PBX_CMD'] = '2x'.(int)$id_cart;
                $params['PBX_PORTEUR'] = $pbx_porteur;
                $params['PBX_RETOUR'] = $pbx_retour;
                $params['PBX_HASH'] = 'SHA512';
                $params['PBX_TIME'] = $date;
                $params['PBX_LANGUE'] = $pbx_langue;
                $params['PBX_ANNULE'] = $urls['cancel'];
                $params['PBX_EFFECTUE'] = $urls['conf'];
                $params['PBX_REFUSE'] = $urls['conf'];
                $params['PBX_REPONDRE_A'] = $urls['repondre_a'];
                $params['PBX_SHOPPINGCART'] = $additional3DSv2Details['PBX_SHOPPINGCART'];
                $params['PBX_BILLING'] = $additional3DSv2Details['PBX_BILLING'];
                $params['PBX_SOURCE'] = 'RWD';
                $params['PBX_2MONT1'] = $amount2;
                $params['PBX_DATE1'] = (string)$date1;
                $params['PBX_HMAC'] = Tools::strtoupper(hash_hmac('sha512', $msg, $account['bin_key']));

                // $params['PBX_NB_INSTALMENT'] = 1;
                $params['pbx_text'] = sprintf($this->paybox->l('Pay by credit card with over 2 installments'));

                $gateway = Configuration::get('PBX_GATEWAY');
                $params['gateway_img'] = 'paybox.png';
                if ($gateway == 'ca') {
                    $params['gateway_img'] = 'paybox_ca.png';
                }

                return $params;
            }

            return false;
        }

        private function paymentOne($configs, $account, $pbx_total, $pbx_devise, $id_cart, $pbx_porteur, $pbx_retour, $date, $pbx_langue, $urls)
        {
            if ($configs['PBX_MULTIPLE_1X_PAYMENTS'] == 1 && $pbx_total >= ($configs['PBX_1X_MINIMUM'] * 100)) {
                $additional3DSv2Details = $this->paybox->get3DSv2Details(new Cart((int) $id_cart));
                $urls['conf'] .= '&payment_type=paymentOne';

                $msg = 'PBX_SITE='.$account['PBX_SITE']
                .'&PBX_RANG='.$account['PBX_RANG']
                .'&PBX_IDENTIFIANT='.$account['PBX_ID']
                .'&PBX_TOTAL='.$pbx_total
                .'&PBX_DEVISE='.$pbx_devise
                .'&PBX_CMD=1x'.$id_cart
                .'&PBX_PORTEUR='.$pbx_porteur
                .'&PBX_RETOUR='.$pbx_retour
                .'&PBX_HASH=SHA512&PBX_TIME='.$date
                .'&PBX_LANGUE='.$pbx_langue
                .'&PBX_ANNULE='.$urls['cancel']
                .'&PBX_EFFECTUE='.$urls['conf']
                .'&PBX_REFUSE='.$urls['conf']
                .'&PBX_REPONDRE_A='.$urls['repondre_a']
                .'&PBX_SHOPPINGCART='.$additional3DSv2Details['PBX_SHOPPINGCART']
                .'&PBX_BILLING='.$additional3DSv2Details['PBX_BILLING']
                .'&PBX_SOURCE=RWD';

                $params = array();

                if ($configs['PBX_DISABLE_3D'] == 1 && $pbx_total < ($configs['PBX_3D_MINIMUM'] * 100)) {
                    $msg .= '&PBX_3DS=N';
                }

                if ($configs['PBX_CAPTURE_PAY_TYPE'] == 'capture_delayed' && $configs['PBX_CAPTURE_DELAY_DAYS'] != '00') {
                    $msg .= '&PBX_DIFF='.$configs['PBX_CAPTURE_DELAY_DAYS'];
                }
                if (Configuration::get('PBX_CAPTURE_PAY_TYPE') == 'capture_manual') {
                    $msg .= '&PBX_AUTOSEULE=O';
                }

                if (Configuration::get('PBX_CAPTURE_PAY_TYPE') == 'capture_manual') {
                    $msg .= '&PBX_AUTOSEULE=O';
                    $params['PBX_AUTOSEULE'] = 'O';
                }

                $gateway = "Crédit Agricole";

                $params['PBX_SITE'] = $account['PBX_SITE'];
                $params['PBX_RANG'] = $account['PBX_RANG'];
                $params['PBX_IDENTIFIANT'] = $account['PBX_ID'];
                $params['PBX_TOTAL'] = $pbx_total;
                $params['PBX_DEVISE'] = $pbx_devise;
                $params['PBX_CMD'] = '1x'.(int)$id_cart;
                $params['PBX_PORTEUR'] = $pbx_porteur;
                $params['PBX_RETOUR'] = $pbx_retour;
                $params['PBX_HASH'] = 'SHA512';
                $params['PBX_TIME'] = $date;
                $params['PBX_LANGUE'] = $pbx_langue;
                $params['PBX_ANNULE'] = $urls['cancel'];
                $params['PBX_EFFECTUE'] = $urls['conf'];
                $params['PBX_REFUSE'] = $urls['conf'];
                $params['PBX_REPONDRE_A'] = $urls['repondre_a'];
                $params['PBX_SHOPPINGCART'] = $additional3DSv2Details['PBX_SHOPPINGCART'];
                $params['PBX_BILLING'] = $additional3DSv2Details['PBX_BILLING'];
                $params['PBX_SOURCE'] = 'RWD';
                $params['PBX_HMAC'] = Tools::strtoupper(hash_hmac('sha512', $msg, $account['bin_key']));

                // $params['PBX_NB_INSTALMENT'] = 1;
                $params['pbx_text'] = sprintf($this->paybox->l('Pay by credit card'));

                $gateway = Configuration::get('PBX_GATEWAY');
                $params['gateway_img'] = 'paybox.png';
                if ($gateway == 'ca') {
                    $params['gateway_img'] = 'paybox_ca.png';
                }

                return $params;
            }

            return false;
        }
    }
}
