<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'payplug/payplug.php')) {
    require_once _PS_MODULE_DIR_ . 'payplug/payplug.php';

    class PrestAppPayplugModuleController
    {
        protected $payplug;
        public $module_version;

        public function __construct()
        {
            $this->payplug = Module::getInstanceByName('payplug');
            $this->module_version = $this->payplug->version;
        }

        public function getData()
        {
            $data = new StdClass();
            $data->name = 'payplug';
            if (version_compare($this->payplug->version, '2.27.0', '<')) {
                $data->cards_active = $this->isCardActive();
            } elseif (version_compare($this->payplug->version, '3.5.0', '>=')) {
                $data->cards_active = $this->isCardActive2270();
            } else {
                $data->cards_active = $this->isCardActive2270();
            }
            return $data;
        }

        public function isCardActive()
        {
            if (!$this->payplug->active) {
                return false;
            } elseif (PayplugBackward::getConfiguration('PAYPLUG_SHOW') == 0) {
                return false;
            } else {
                return true;
            }
        }

        public function isCardActive2270()
        {
            if (!$this->payplug->active) {
                return false;
            } elseif (Configuration::get('PAYPLUG_SHOW') == 0) {
                return false;
            } else {
                return true;
            }
        }

        public function displayPayment($context)
        {
            $is_active = true;

            if (!$this->payplug->active) {
                $is_active = false;
            } elseif (PayplugBackward::getConfiguration('PAYPLUG_SHOW') == 0) {
                $is_active = false;
            } elseif (!$this->payplug->checkCurrency($context->cart)) {
                $is_active = false;
            } elseif (!$this->payplug->checkAmount($context->cart)) {
                $is_active = false;
            }

            $payplug_cards = $this->payplug->getCardsByCustomer($context->customer->id);
            $one_click = (int) PayplugBackward::getConfiguration('PAYPLUG_ONE_CLICK');

            $data = array(
                'is_active' => $is_active,
                'is_url' => true,
            );

            if (!empty($payplug_cards) && $one_click == 1) {
                $data['cards'] = $payplug_cards;
                $data['one_click'] = true;
            }

            return $data;
        }

        public function displayPayment2270($context)
        {
            $cart = new Cart($context->cart->id);
            $img_lang = $context->language->iso_code === 'it' ? 'it' : 'default';

            $is_active = true;

            if (!$this->payplug->active) {
                $is_active = false;
            } elseif (Configuration::get('PAYPLUG_SHOW') == 0) {
                $is_active = false;
            }

            $options = $this->payplug->getAvailableOptions($cart);
            $payplug_cards = $options['one_click'] ? $this->payplug->getCardsByCustomer((int) $cart->id_customer, true) : [];

            $payment_list = [];

            if ($options['one_click'] && !empty($payplug_cards)) {
                foreach ($payplug_cards as $card) {
                    $brand = $card['brand'] != 'none' ? Tools::ucfirst($card['brand']) : $this->payplug->l('Card');
                    $object = array(
                        'payplug_params' => array(
                            'pc' => (int) $card['id_payplug_card'],
                            'pay' => 1,
                            'id_cart' => (int) $context->cart->id,
                            'method' => 'one_click',
                            'def' => (int) $options['deferred'],
                        ),
                        'is_active' => $is_active,
                        'is_url' => true,
                        'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/' . Tools::strtolower($card['brand']) . '.png'),
                        'payplug_title' => $brand . ' **** **** **** ' . $card['last4'] . ' - ' . $this->payplug->l('Expiry date') . ': ' . $card['expiry_date'],
                    );

                    $payment_list[] = $object;
                }
            }

            $object = array(
                'payplug_params' => array(
                    'pc' => 'new_card',
                    'pay' => 1,
                    'id_cart' => (int) $context->cart->id,
                    'method' => 'standard',
                    'def' => (int) $options['deferred'],
                ),
                'is_active' => $is_active,
                'is_url' => true,
                'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/' . (count($payplug_cards) > 0 ? 'none' : 'logos_schemes_' . $img_lang) . '.png'),
                'payplug_title' => count($payplug_cards) > 0 ? $this->payplug->l('Pay with a different card') : $this->payplug->l('Pay with a credit card'),
            );

            $payment_list[] = $object;

            if ($options['installment']) {
                $object = array(
                    'payplug_params' => array('pc' => 'new_card',
                        'pay' => 1,
                        'id_cart' => (int) $context->cart->id,
                        'method' => 'installment',
                        'def' => (int) $options['deferred'],
                    ),
                    'is_active' => $is_active,
                    'is_url' => true,
                    'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/logos_schemes_installment_' . Configuration::get('PAYPLUG_INST_MODE') . '_' . $img_lang . '.png'),
                    'payplug_title' => $this->payplug->l('Pay by card in') . ' ' . Configuration::get('PAYPLUG_INST_MODE') . ' ' . $this->payplug->l('installments'),
                );

                $payment_list[] = $object;
            }

            return $payment_list;
        }

        public function displayPaymentSupEqual350($context)
        {
            $is_active = true;

            if (!Module::isEnabled('payplug') || !Configuration::get('PAYPLUG_SHOW')) {
                $is_active = false;
                return false;
            }

            $cart = new Cart($context->cart->id);

            $use_taxes = Configuration::get('PS_TAX');
            $base_total_tax_inc = $cart->getOrderTotal(true);
            $base_total_tax_exc = $cart->getOrderTotal(false);

            if ($use_taxes) {
                $price2display = $base_total_tax_inc;
            } else {
                $price2display = $base_total_tax_exc;
            }

            $currency = $cart->id_currency;
            $result_currency = Currency::getCurrency($currency);
            $supported_currencies = explode(';', Configuration::get('PAYPLUG_CURRENCIES'));

            if (!in_array($result_currency['iso_code'], $supported_currencies, true)) {
                return false;
            }

            if (version_compare($this->module_version, '3.12.0', '>=')) {
                $payment_options = $this->getPaymentOptions3120($cart, $context);
            } else if (version_compare($this->module_version, '3.7.1', '>=')) {
                $payment_options = $this->getPaymentOptions371($cart, $context);
            } else {
                $payment_options = $this->getPaymentOptions350($cart, $context);
            }

            $payment_list = [];

            foreach ($payment_options as $key => $option) {
                $object = array(
                    'payplug_params' => array(
                        'pc' => $option['inputs']['pc']['value'],
                        'pay' => $option['inputs']['pay']['value'],
                        'id_cart' => (int) $context->cart->id,
                        'method' => $option['inputs']['method']['value'],
                    ),
                    'is_active' => $is_active,
                    'is_url' => true,
                    'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . $option['logo'],
                    'payplug_title' => $option['callToActionText'],
                );

                $payment_list[] = $object;
            }

            return $payment_list;
        }

        public function getPaymentOptions3120($cart, $context)
        {
            $dependencies = new PayPlug\classes\DependenciesClass();
            $options = $dependencies->paymentClass->getPaymentOptions();

            // var_dump($options);exit();

            $configClass = new \PayPlug\classes\ConfigClass($this->payplug->module);

            if (Configuration::get('PAYPLUG_STANDARD')) {
                if ($options['one_click'] && !empty($payplug_cards)) {
                    foreach ($payplug_cards as $card) {
                        $brand = ($card['brand'] != 'none')
                        ? Tools::ucfirst($card['brand'])
                        : $this->payplug->l('payplug.getPaymentOptions.card');
                        $payment_key = 'one_click_' . $card['id_payplug_card'];
                        $paymentOption[$payment_key]['name'] = 'one_click';
                        $paymentOption[$payment_key]['inputs'] = [
                            'pc' => [
                                'name' => 'pc',
                                'type' => 'hidden',
                                'value' => (int) $card['id_payplug_card'],
                            ],
                            'pay' => [
                                'name' => 'pay',
                                'type' => 'hidden',
                                'value' => '1',
                            ],
                            'id_cart' => [
                                'name' => 'id_cart',
                                'type' => 'hidden',
                                'value' => (int) $context->cart->id,
                            ],
                            'method' => [
                                'name' => 'method',
                                'type' => 'hidden',
                                'value' => 'one_click',
                            ],
                        ];
                        $paymentOption[$payment_key]['tpl'] = 'one_click.tpl';
                        $paymentOption[$payment_key]['payment_controller_url'] =
                        $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'payment',
                            [],
                            true
                        );
                        $paymentOption[$payment_key]['logo'] = $card['brand'] != 'none' ? Media::getMediaPath(
                            _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/' . Tools::strtolower($card['brand']) . '.svg'
                        ) : '';
                        $paymentOption[$payment_key]['callToActionText'] = $brand .
                            ' **** **** **** ' . $card['last4'];
                        $paymentOption[$payment_key]['expiry_date_card'] =
                        $this->payplug->l('payplug.getPaymentOptions.expiryDate') . ': ' . $card['expiry_date'];
                        $paymentOption[$payment_key]['action'] = $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'dispatcher',
                            ['def' => (int) $options['deferred']],
                            true
                        );
                        $paymentOption[$payment_key]['moduleName'] = 'payplug';
                    }
                }

                // Standard Payment or new card from one-click
                $paymentOption['standard']['name'] = 'standard';
                $paymentOption['standard']['inputs'] = [
                    'pc' => [
                        'name' => 'pc',
                        'type' => 'hidden',
                        'value' => 'new_card',
                    ],
                    'pay' => [
                        'name' => 'pay',
                        'type' => 'hidden',
                        'value' => '1',
                    ],
                    'id_cart' => [
                        'name' => 'id_cart',
                        'type' => 'hidden',
                        'value' => (int) $context->cart->id,
                    ],
                    'method' => [
                        'name' => 'method',
                        'type' => 'hidden',
                        'value' => 'standard',
                    ],
                ];
                $paymentOption['standard']['tpl'] = 'standard.tpl';
                $paymentOption['standard']['extra_classes'] = 'payplug default';
                $paymentOption['standard']['payment_controller_url'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'payment',
                    ['type' => 'standard']
                );

                $paymentOption['standard']['logo'] = Media::getMediaPath(
                    _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_' . $configClass->getImgLang() . '.svg'
                );
                if (count($payplug_cards) > 0) {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payDifferentCard', 'paymentclass');
                } else {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payCreditCard', 'paymentclass');
                }
                $paymentOption['standard']['action'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'dispatcher',
                    ['def' => (int) $options['deferred']],
                    true
                );
                $paymentOption['standard']['moduleName'] = 'payplug';
            }

            // Installment Payment
            if ($options['installment']) {
                $use_taxes = (bool) Configuration::get('PS_TAX');
                $cart_amount = $context->cart->getOrderTotal($use_taxes);
                if ($cart_amount >= Configuration::get('PAYPLUG_INST_MIN_AMOUNT')) {
                    $installment_mode = Configuration::get('PAYPLUG_INST_MODE');
                    $paymentOption['installment']['name'] = 'installment';
                    $paymentOption['installment']['inputs'] = [
                        'pc' => [
                            'name' => 'pc',
                            'type' => 'hidden',
                            'value' => 'new_card',
                        ],
                        'pay' => [
                            'name' => 'pay',
                            'type' => 'hidden',
                            'value' => '1',
                        ],
                        'id_cart' => [
                            'name' => 'id_cart',
                            'type' => 'hidden',
                            'value' => (int) $context->cart->id,
                        ],
                        'method' => [
                            'name' => 'method',
                            'type' => 'hidden',
                            'value' => 'installment',
                        ],
                    ];
                    $paymentOption['installment']['tpl'] = 'installment.tpl';
                    $paymentOption['installment']['payment_controller_url'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'payment',
                        ['type' => 'installment', 'i' => 1],
                        true
                    );
                    $paymentOption['installment']['logo'] = Media::getMediaPath(
                        _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_installment_' .
                        Configuration::get('PAYPLUG_INST_MODE') . '_' . $configClass->getImgLang() . '.png'
                    );
                    $paymentOption['installment']['callToActionText'] = sprintf(
                        $this->payplug->l('payplug.getPaymentOptions.payByCardInstallment', 'paymentclass'),
                        Configuration::get('PAYPLUG_INST_MODE')
                    );
                    $paymentOption['installment']['action'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'dispatcher',
                        ['def' => (int) $options['deferred']],
                        true
                    );
                    $paymentOption['installment']['moduleName'] = 'payplug';
                }
            }

            return $paymentOption;
        }

        public function getPaymentOptions371($cart, $context)
        {
            $options = \PayPlug\classes\ConfigClass::getAvailableOptions($cart);
            $this_card = $this->payplug->module->getPlugin()->getCard();
            $configClass = new \PayPlug\classes\ConfigClass($this->payplug->module);

            $id_customer = (isset($cart->id_customer)) ? $cart->id_customer : $cart['cart']->id_customer;
            $payplug_cards = $options['one_click'] ? $this_card->getByCustomer((int) $id_customer, true) : [];
            $paymentOption = [];

            // OneClick Payment
            if (Configuration::get('PAYPLUG_STANDARD')) {
                if ($options['one_click'] && !empty($payplug_cards)) {
                    foreach ($payplug_cards as $card) {
                        $brand = ($card['brand'] != 'none')
                        ? Tools::ucfirst($card['brand'])
                        : $this->payplug->l('payplug.getPaymentOptions.card');
                        $payment_key = 'one_click_' . $card['id_payplug_card'];
                        $paymentOption[$payment_key]['name'] = 'one_click';
                        $paymentOption[$payment_key]['inputs'] = [
                            'pc' => [
                                'name' => 'pc',
                                'type' => 'hidden',
                                'value' => (int) $card['id_payplug_card'],
                            ],
                            'pay' => [
                                'name' => 'pay',
                                'type' => 'hidden',
                                'value' => '1',
                            ],
                            'id_cart' => [
                                'name' => 'id_cart',
                                'type' => 'hidden',
                                'value' => (int) $context->cart->id,
                            ],
                            'method' => [
                                'name' => 'method',
                                'type' => 'hidden',
                                'value' => 'one_click',
                            ],
                        ];
                        $paymentOption[$payment_key]['tpl'] = 'one_click.tpl';
                        $paymentOption[$payment_key]['payment_controller_url'] =
                        $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'payment',
                            [],
                            true
                        );
                        $paymentOption[$payment_key]['logo'] = $card['brand'] != 'none' ? Media::getMediaPath(
                            _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/' . Tools::strtolower($card['brand']) . '.svg'
                        ) : '';
                        $paymentOption[$payment_key]['callToActionText'] = $brand .
                            ' **** **** **** ' . $card['last4'];
                        $paymentOption[$payment_key]['expiry_date_card'] =
                        $this->payplug->l('payplug.getPaymentOptions.expiryDate') . ': ' . $card['expiry_date'];
                        $paymentOption[$payment_key]['action'] = $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'dispatcher',
                            ['def' => (int) $options['deferred']],
                            true
                        );
                        $paymentOption[$payment_key]['moduleName'] = 'payplug';
                    }
                }

                // Standard Payment or new card from one-click
                $paymentOption['standard']['name'] = 'standard';
                $paymentOption['standard']['inputs'] = [
                    'pc' => [
                        'name' => 'pc',
                        'type' => 'hidden',
                        'value' => 'new_card',
                    ],
                    'pay' => [
                        'name' => 'pay',
                        'type' => 'hidden',
                        'value' => '1',
                    ],
                    'id_cart' => [
                        'name' => 'id_cart',
                        'type' => 'hidden',
                        'value' => (int) $context->cart->id,
                    ],
                    'method' => [
                        'name' => 'method',
                        'type' => 'hidden',
                        'value' => 'standard',
                    ],
                ];
                $paymentOption['standard']['tpl'] = 'standard.tpl';
                $paymentOption['standard']['extra_classes'] = 'payplug default';
                $paymentOption['standard']['payment_controller_url'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'payment',
                    ['type' => 'standard']
                );

                $paymentOption['standard']['logo'] = Media::getMediaPath(
                    _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_' . $configClass->getImgLang() . '.svg'
                );
                if (count($payplug_cards) > 0) {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payDifferentCard', 'payplugclass');
                } else {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payCreditCard', 'payplugclass');
                }
                $paymentOption['standard']['action'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'dispatcher',
                    ['def' => (int) $options['deferred']],
                    true
                );
                $paymentOption['standard']['moduleName'] = 'payplug';
            }

            // Installment Payment
            if ($options['installment']) {
                $use_taxes = (bool) Configuration::get('PS_TAX');
                $cart_amount = $context->cart->getOrderTotal($use_taxes);
                if ($cart_amount >= Configuration::get('PAYPLUG_INST_MIN_AMOUNT')) {
                    $installment_mode = Configuration::get('PAYPLUG_INST_MODE');
                    $paymentOption['installment']['name'] = 'installment';
                    $paymentOption['installment']['inputs'] = [
                        'pc' => [
                            'name' => 'pc',
                            'type' => 'hidden',
                            'value' => 'new_card',
                        ],
                        'pay' => [
                            'name' => 'pay',
                            'type' => 'hidden',
                            'value' => '1',
                        ],
                        'id_cart' => [
                            'name' => 'id_cart',
                            'type' => 'hidden',
                            'value' => (int) $context->cart->id,
                        ],
                        'method' => [
                            'name' => 'method',
                            'type' => 'hidden',
                            'value' => 'installment',
                        ],
                    ];
                    $paymentOption['installment']['tpl'] = 'installment.tpl';
                    $paymentOption['installment']['payment_controller_url'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'payment',
                        ['type' => 'installment', 'i' => 1],
                        true
                    );
                    $paymentOption['installment']['logo'] = Media::getMediaPath(
                        _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_installment_' .
                        Configuration::get('PAYPLUG_INST_MODE') . '_' . $configClass->getImgLang() . '.png'
                    );
                    $paymentOption['installment']['callToActionText'] = sprintf(
                        $this->payplug->l('payplug.getPaymentOptions.payByCardInstallment', 'payplugclass'),
                        Configuration::get('PAYPLUG_INST_MODE')
                    );
                    $paymentOption['installment']['action'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'dispatcher',
                        ['def' => (int) $options['deferred']],
                        true
                    );
                    $paymentOption['installment']['moduleName'] = 'payplug';
                }
            }

            return $paymentOption;
        }

        public function getPaymentOptions350($cart, $context)
        {
            $options = \PayPlug\classes\ConfigClass::getAvailableOptions($cart);
            $this_card = $this->payplug->module->getPlugin()->getCard();
            $configClass = new \PayPlug\classes\ConfigClass($this->payplug->module);

            $id_customer = (isset($cart->id_customer)) ? $cart->id_customer : $cart['cart']->id_customer;
            $payplug_cards = $options['one_click'] ? $this_card->getByCustomer((int) $id_customer, true) : [];
            $paymentOption = [];

            // OneClick Payment
            if (Configuration::get('PAYPLUG_STANDARD')) {
                if ($options['one_click'] && !empty($payplug_cards)) {
                    foreach ($payplug_cards as $card) {
                        $brand = ($card['brand'] != 'none')
                        ? Tools::ucfirst($card['brand'])
                        : $this->payplug->l('payplug.getPaymentOptions.card');
                        $payment_key = 'one_click_' . $card['id_payplug_card'];
                        $paymentOption[$payment_key]['name'] = 'one_click';
                        $paymentOption[$payment_key]['inputs'] = [
                            'pc' => [
                                'name' => 'pc',
                                'type' => 'hidden',
                                'value' => (int) $card['id_payplug_card'],
                            ],
                            'pay' => [
                                'name' => 'pay',
                                'type' => 'hidden',
                                'value' => '1',
                            ],
                            'id_cart' => [
                                'name' => 'id_cart',
                                'type' => 'hidden',
                                'value' => (int) $context->cart->id,
                            ],
                            'method' => [
                                'name' => 'method',
                                'type' => 'hidden',
                                'value' => 'one_click',
                            ],
                        ];
                        $paymentOption[$payment_key]['tpl'] = 'one_click.tpl';
                        $paymentOption[$payment_key]['payment_controller_url'] =
                        $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'payment',
                            [],
                            true
                        );
                        $paymentOption[$payment_key]['logo'] = $card['brand'] != 'none' ? Media::getMediaPath(
                            _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/' . Tools::strtolower($card['brand']) . '.svg'
                        ) : '';
                        $paymentOption[$payment_key]['callToActionText'] = $brand .
                            ' **** **** **** ' . $card['last4'];
                        $paymentOption[$payment_key]['expiry_date_card'] =
                        $this->payplug->l('payplug.getPaymentOptions.expiryDate') . ': ' . $card['expiry_date'];
                        $paymentOption[$payment_key]['action'] = $context->link->getModuleLink(
                            $this->payplug->module->name,
                            'dispatcher',
                            ['def' => (int) $options['deferred']],
                            true
                        );
                        $paymentOption[$payment_key]['moduleName'] = 'payplug';
                    }
                }

                // Standard Payment or new card from one-click
                $paymentOption['standard']['name'] = 'standard';
                $paymentOption['standard']['inputs'] = [
                    'pc' => [
                        'name' => 'pc',
                        'type' => 'hidden',
                        'value' => 'new_card',
                    ],
                    'pay' => [
                        'name' => 'pay',
                        'type' => 'hidden',
                        'value' => '1',
                    ],
                    'id_cart' => [
                        'name' => 'id_cart',
                        'type' => 'hidden',
                        'value' => (int) $context->cart->id,
                    ],
                    'method' => [
                        'name' => 'method',
                        'type' => 'hidden',
                        'value' => 'standard',
                    ],
                ];
                $paymentOption['standard']['tpl'] = 'standard.tpl';
                $paymentOption['standard']['extra_classes'] = 'payplug default';
                $paymentOption['standard']['payment_controller_url'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'payment',
                    ['type' => 'standard']
                );

                $paymentOption['standard']['logo'] = Media::getMediaPath(
                    _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_' . $configClass->getImgLang() . '.svg'
                );
                if (count($payplug_cards) > 0) {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payDifferentCard');
                } else {
                    $paymentOption['standard']['callToActionText'] = $this->payplug->l('payplug.getPaymentOptions.payCreditCard');
                }
                $paymentOption['standard']['action'] = $context->link->getModuleLink(
                    $this->payplug->module->name,
                    'dispatcher',
                    ['def' => (int) $options['deferred']],
                    true
                );
                $paymentOption['standard']['moduleName'] = 'payplug';
            }

            // Installment Payment
            if ($options['installment']) {
                $use_taxes = (bool) Configuration::get('PS_TAX');
                $cart_amount = $context->cart->getOrderTotal($use_taxes);
                if ($cart_amount >= Configuration::get('PAYPLUG_INST_MIN_AMOUNT')) {
                    $installment_mode = Configuration::get('PAYPLUG_INST_MODE');
                    $paymentOption['installment']['name'] = 'installment';
                    $paymentOption['installment']['inputs'] = [
                        'pc' => [
                            'name' => 'pc',
                            'type' => 'hidden',
                            'value' => 'new_card',
                        ],
                        'pay' => [
                            'name' => 'pay',
                            'type' => 'hidden',
                            'value' => '1',
                        ],
                        'id_cart' => [
                            'name' => 'id_cart',
                            'type' => 'hidden',
                            'value' => (int) $context->cart->id,
                        ],
                        'method' => [
                            'name' => 'method',
                            'type' => 'hidden',
                            'value' => 'installment',
                        ],
                    ];
                    $paymentOption['installment']['tpl'] = 'installment.tpl';
                    $paymentOption['installment']['payment_controller_url'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'payment',
                        ['type' => 'installment', 'i' => 1],
                        true
                    );
                    $paymentOption['installment']['logo'] = Media::getMediaPath(
                        _PS_MODULE_DIR_ . $this->payplug->module->name . '/views/img/logos_schemes_installment_' .
                        Configuration::get('PAYPLUG_INST_MODE') . '_' . $configClass->getImgLang() . '.png'
                    );
                    $paymentOption['installment']['callToActionText'] = sprintf(
                        $this->payplug->l('payplug.getPaymentOptions.payByCardInstallment'),
                        Configuration::get('PAYPLUG_INST_MODE')
                    );
                    $paymentOption['installment']['action'] = $context->link->getModuleLink(
                        $this->payplug->module->name,
                        'dispatcher',
                        ['def' => (int) $options['deferred']],
                        true
                    );
                    $paymentOption['installment']['moduleName'] = 'payplug';
                }
            }

            return $paymentOption;
        }

        public function displayPaymentSup2270($context)
        {
            $is_active = true;

            if (!$this->payplug->isAllowed()) {
                $is_active = false;
                return false;
            }

            $img_lang = $context->language->iso_code === 'it' ? 'it' : 'default';

            $cart = new Cart($context->cart->id);

            $options = $this->payplug->getAllowedPaymentOptions($cart);

            $payment_list = [];

            if ($this->payplug->getConfiguration('PAYPLUG_ONE_CLICK')) {
                $payplug_card = new PayPlugCard();
                $payplug_cards = $payplug_card->getByCustomer($cart->id_customer, true);

                if ($options['oneclick'] && !empty($payplug_cards)) {
                    foreach ($payplug_cards as $card) {
                        $brand = $card['brand'] != 'none' ? Tools::ucfirst($card['brand']) : $this->payplug->l('Card');
                        $object = array(
                            'payplug_params' => array(
                                'pc' => (int) $card['id_payplug_card'],
                                'pay' => 1,
                                'id_cart' => (int) $context->cart->id,
                                'type' => 'oneclick',
                            ),
                            'is_active' => $is_active,
                            'is_url' => true,
                            'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/' . Tools::strtolower($card['brand']) . '.png'),
                            'payplug_title' => $brand . ' **** **** **** ' . $card['last4'] . ' - ' . $this->payplug->l('Expiry date') . ': ' . $card['expiry_date'],
                        );

                        $payment_list[] = $object;
                    }
                }
            }

            $object = array(
                'payplug_params' => array(
                    'pc' => 'new_card',
                    'pay' => 1,
                    'id_cart' => (int) $context->cart->id,
                    'type' => 'standard',
                ),
                'is_active' => $is_active,
                'is_url' => true,
                'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/' . (count($payplug_cards) > 0 ? 'none' : 'logos_schemes_' . $img_lang) . '.png'),
                'payplug_title' => $this->payplug->l('Pay with credit card'),
            );

            $payment_list[] = $object;

            if ($options['installment']) {
                $object = array(
                    'payplug_params' => array('pc' => 'new_card',
                        'pay' => 1,
                        'id_cart' => (int) $context->cart->id,
                        'type' => 'installment',
                    ),
                    'is_active' => $is_active,
                    'is_url' => true,
                    'payplug_logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . Media::getMediaPath(_PS_MODULE_DIR_ . $this->payplug->name . '/views/img/logos_schemes_installment_' . Configuration::get('PAYPLUG_INST_MODE') . '_' . $img_lang . '.png'),
                    'payplug_title' => $this->payplug->l('Pay by card in') . ' ' . Configuration::get('PAYPLUG_INST_MODE') . ' ' . $this->payplug->l('installments'),
                );

                $payment_list[] = $object;
            }

            return $payment_list;
        }

        public function getPaymentUrl($context, $id_card)
        {
            $payplug_url = $this->processPayment($context, $id_card);

            //////////////////////////////////////////
            if ($payplug_url === false || (isset($payplug_url['result']) && $payplug_url['result'] === false)) {
                return false;
            } elseif (isset($payplug_url['result']) && $payplug_url['result'] !== false) {
                if ($payplug_url['result'] === 'new_card') {
                    return array(
                        'type' => 'payment',
                        'url' => $payplug_url['payment_url'],
                    );
                } elseif ($payplug_url['result'] === true) {
                    return array(
                        'type' => 'validation',
                        'url' => $payplug_url['validation_url'],
                    );
                }
            } else {
                return array(
                    'type' => 'payment',
                    'url' => $payplug_url,
                );
            }
        }

        public function getPaymentUrlSupEqual3120($context, $pc, $pay, $id_cart, $method, $def)
        {
            // $payplug = new \PayPlug\classes\PaymentClass();

            $dependencies = new PayPlug\classes\DependenciesClass();

            $is_installment = $method == 'installment' ? true : false;
            $is_deferred = Configuration::get('PAYPLUG_DEFERRED') == 1;
            $is_oney = false;
            $options = [
                'id_card' => $pc,
                'is_installment' => $is_installment,
                'is_deferred' => $is_deferred,
                'is_oney' => $is_oney,
                '_ajax' => 1,
            ];

            $payment = $dependencies->paymentClass->preparePayment($options);

            switch ($method) {
                case 'one_click':
                    $redirect = $payment['redirect'];
                    return array(
                        'type' => $redirect ? 'validation' : 'payment',
                        'url' => $payment['return_url'],
                    );
                    break;
                    break;
                case 'standard':
                case 'installment':
                default:
                    return array(
                        'type' => 'payment',
                        'url' => $payment['return_url'],
                    );
                    break;
            }
        }

        public function getPaymentUrlSupEqual350($context, $pc, $pay, $id_cart, $method, $def)
        {
            if (version_compare(_PS_VERSION_, '1.7', '<')) {
                $payplug = new \PayPlug\classes\PayPlugClass();

                $is_installment = $method == 'installment' ? true : false;
                $is_deferred = Configuration::get('PAYPLUG_DEFERRED') == 1;
                $is_oney = false;
                $options = [
                    'id_card' => $pc,
                    'is_installment' => $is_installment,
                    'is_deferred' => $is_deferred,
                    'is_oney' => $is_oney,
                    '_ajax' => 1,
                ];

                $payment = $payplug->preparePayment($options);

                switch ($method) {
                    case 'one_click':
                        $redirect = $payment['redirect'];
                        return array(
                            'type' => $redirect ? 'validation' : 'payment',
                            'url' => $payment['return_url'],
                        );
                        break;
                        break;
                    case 'standard':
                    case 'installment':
                    default:
                        return array(
                            'type' => 'payment',
                            'url' => $payment['return_url'],
                        );
                        break;
                }
            } else {
                $payplug = new \PayPlug\classes\PayPlugClass();

                $is_installment = $method == 'installment' ? true : false;
                $is_deferred = Configuration::get('PAYPLUG_DEFERRED') == 1;
                $is_oney = false;
                $options = [
                    'id_card' => $pc,
                    'is_installment' => $is_installment,
                    'is_deferred' => $is_deferred,
                    'is_oney' => $is_oney,
                    '_ajax' => 1,
                ];

                $payment = $payplug->preparePayment($options);

                switch ($method) {
                    case 'one_click':
                        $redirect = $payment['redirect'];
                        return array(
                            'type' => $redirect ? 'validation' : 'payment',
                            'url' => $payment['return_url'],
                        );
                        break;
                        break;
                    case 'standard':
                    case 'installment':
                    default:
                        return array(
                            'type' => 'payment',
                            'url' => $payment['return_url'],
                        );
                        break;
                }
            }
        }

        public function getPaymentUrlSup2270($context, $pc, $pay, $id_cart, $method, $def)
        {
            $cart = new Cart($id_cart);
            if (!Validate::isLoadedObject($cart)) {
                return false;
            }

            if ($this->payplug->isPaymentPending($id_cart)) {
                return false;
            }

            $deferred = $this->payplug->getConfiguration('PAYPLUG_DEFERRED');
            $options = [];
            if ($deferred) {
                $options['deferred'] = true;
            }

            $payplug_method_name = $this->payplug->getCurrentPaymentMethod($pc);
            $payplug_payment = new $payplug_method_name($pc, $options);

            try {
                $result = $payplug_payment->create();
                $payment = $result['resource'];
                if (!$payment) {
                    return false;
                } elseif (!$payplug_payment->isValidPayment($payment)) {
                    return false;
                }
            } catch (Exception $e) {
                return false;
            }

            $payplug_payment->register($payment->id);

            switch ($payplug_payment->type) {
                case 'oneclick':
                    $redirect = $payment->is_paid;
                    if (!$redirect && $deferred) {
                        $redirect = (bool) $payment->authorization->authorized_at;
                    }
                    return array(
                        'type' => $redirect ? 'validation' : 'payment',
                        'url' => $redirect ? $payplug_payment->payment_url['return'] : $payment->hosted_payment->payment_url,
                    );
                    break;
                case 'standard':
                case 'installment':
                default:
                    return array(
                        'type' => 'payment',
                        'url' => $payment->hosted_payment->payment_url,
                    );
                    break;
            }
        }

        public function getPaymentUrl2270($context, $pc, $pay, $id_cart, $method, $def)
        {
            $cart = new Cart($id_cart);
            if (!Validate::isLoadedObject($cart)) {
                return false;
            }

            $options = $this->payplug->getAvailableOptions($cart);

            if (!$options['embedded'] && $method != 'one_click') {
                $payment = $this->payplug->preparePayment($id_cart, $pc, $method == 'installment', $def);
                if (!$payment['result']) {
                    return false;
                } else {
                    return array(
                        'type' => 'payment',
                        'url' => $payment['return_url'],
                    );
                }
            } else {
                if (!$this->payplug->active) {
                    return false;
                }
                if (Configuration::get('PAYPLUG_SHOW') == 0) {
                    return false;
                }

                $id_card = $pc ? $pc : 'new_card';
                $payment = $this->payplug->preparePayment($cart, $id_card, $method == 'installment', $def);

                if ($payment['result']) {
                    if ($payment['redirect']) {
                        return array(
                            'type' => 'validation',
                            'url' => $payment['return_url'],
                        );
                    } else {
                        return array(
                            'type' => 'payment',
                            'url' => $payment['return_url'],
                        );
                    }
                } else {
                    return false;
                }
            }
        }

        public function processPayment($context, $id_card)
        {
            $cart = new Cart((int) $context->cart->id);
            if (!Validate::isLoadedObject($cart)) {
                return false;
            }

            $one_click = (int) PayplugBackward::getConfiguration('PAYPLUG_ONE_CLICK');
            $embedded_mode = (int) PayplugBackward::getConfiguration('PAYPLUG_EMBEDDED_MODE');

            $current_card = null;
            if ($id_card != null && $id_card != 'new_card') {
                $current_card = $this->payplug->getCardId(
                    (int) $cart->id_customer,
                    $id_card,
                    (int) PayplugBackward::getConfiguration('PAYPLUG_COMPANY_ID')
                );
            }

            //currency
            $result_currency = array();
            if (version_compare(_PS_VERSION_, '1.3', '<')) {
                $result_currency['iso_code'] = Tools::setCurrency()->iso_code;
            } elseif (version_compare(_PS_VERSION_, '1.5', '<')) {
                $result_currency['iso_code'] = Currency::getCurrent()->iso_code;
            } else {
                $currency = $cart->id_currency;
                $result_currency = Currency::getCurrency($currency);
            }
            $supported_currencies = explode(';', PayplugBackward::getConfiguration('PAYPLUG_CURRENCIES'));

            if (!in_array($result_currency['iso_code'], $supported_currencies)) {
                return false;
            }

            $currency = $result_currency['iso_code'];

            //amount
            if (version_compare(_PS_VERSION_, '1.4', '<')) {
                $amount = $cart->getOrderTotal(true, 3);
            } else {
                $amount = $cart->getOrderTotal(true, Cart::BOTH);
            }

            $amount = (int) round(($amount * 100), PHP_ROUND_HALF_UP);
            $current_amounts = Payplug::getAmountsByCurrency($currency);
            $current_min_amount = $current_amounts['min_amount'];
            $current_max_amount = $current_amounts['max_amount'];

            if ($amount < $current_min_amount || $amount > $current_max_amount) {
                return false;
            }

            //customer
            $customer = new Customer((int) $cart->id_customer);
            //$address_invoice = new Address((int)$cart->id_address_invoice);
            $address_delivery = new Address((int) $cart->id_address_delivery);
            //$country = new Country((int)$address_invoice->id_country);
            $country = new Country((int) $address_delivery->id_country);
            $country_iso_code = $this->payplug->getIsoCodeByCountryId((int) $country->id);

            $payment_customer = array(
                'first_name' => !empty($customer->firstname) ? $customer->firstname : null,
                'last_name' => !empty($customer->lastname) ? $customer->lastname : null,
                'email' => $customer->email,
                'address1' => !empty($address_delivery->address1) ? $address_delivery->address1 : null,
                'address2' => !empty($address_delivery->address2) ? $address_delivery->address2 : null,
                'postcode' => !empty($address_delivery->postcode) ? $address_delivery->postcode : null,
                'city' => !empty($address_delivery->city) ? $address_delivery->city : null,
                'country' => $country_iso_code,
            );

            //hosted payment
            $return_url = PayplugBackward::getModuleLink(
                $this->payplug->name,
                'validation',
                array('ps' => 1, 'cartid' => (int) $cart->id),
                true
            );
            $cancel_url = PayplugBackward::getModuleLink(
                $this->payplug->name,
                'validation',
                array('ps' => 2, 'cartid' => (int) $cart->id),
                true
            );
            if ($one_click != 1 || ($one_click == 1 && ($id_card == null || $id_card == 'new_card'))) {
                $hosted_payment = array(
                    'return_url' => $return_url,
                    'cancel_url' => $cancel_url,
                );
            }

            //notification
            $notification_url = PayplugBackward::getModuleLink($this->payplug->name, 'ipn', array(), true);

            //payment method
            if ($one_click == 1 && $current_card != null && $id_card != 'new_card') {
                $payment_method = $current_card;
            }

            //force 3ds
            $force_3ds = false;

            //save card
            $allow_save_card = false;
            if ($one_click == 1) {
                $allow_save_card = true;
            }

            //meta data
            $ssl = (Configuration::get('PS_SSL_ENABLED') && Configuration::get('PS_SSL_ENABLED_EVERYWHERE'));
            $baseurl = $ssl ? Tools::getShopDomainSsl(true) : Tools::getShopDomain(true);
            $metadata = array(
                'customer_id' => (int) $customer->id,
                'cart_id' => (int) $cart->id,
                'website' => $baseurl,
            );

            //payment
            $payment_tab = array(
                'amount' => $amount,
                'currency' => $currency,
                'customer' => array(
                    'email' => $payment_customer['email'],
                    'first_name' => $payment_customer['first_name'],
                    'last_name' => $payment_customer['last_name'],
                    'address1' => $payment_customer['address1'],
                    'address2' => $payment_customer['address2'],
                    'postcode' => $payment_customer['postcode'],
                    'city' => $payment_customer['city'],
                    'country' => $payment_customer['country'],
                ),
                'notification_url' => $notification_url,
                'force_3ds' => $force_3ds,
                'metadata' => array(
                    'ID Client' => $metadata['customer_id'],
                    'ID Cart' => $metadata['cart_id'],
                    'Website' => $metadata['website'],
                ),
            );

            if ($one_click == 1 && $current_card != null && $id_card != 'new_card') {
                $payment_tab['payment_method'] = $payment_method;
                $payment_tab['allow_save_card'] = false;
            } else {
                $payment_tab['hosted_payment'] = array(
                    'return_url' => $hosted_payment['return_url'],
                    'cancel_url' => $hosted_payment['cancel_url'],
                );
                $payment_tab['allow_save_card'] = $allow_save_card;
            }

            try {
                if (PayplugBackward::getConfiguration('PAYPLUG_DEBUG_MODE')) {
                    $log = new MyLogPHP(_PS_MODULE_DIR_ . $this->payplug->name . '/log/prepare_payment.csv');
                    $log->info('Starting payment.');
                    foreach ($payment_tab as $key => $value) {
                        if (is_array($value)) {
                            foreach ($value as $n_key => $n_value) {
                                $log->info($n_key . ' : ' . $n_value);
                            }
                        } else {
                            $log->info($key . ' : ' . $value);
                        }
                    }
                }
                $payment = \Payplug\Payment::create($payment_tab);
            } catch (Exception $e) {
                $data = array(
                    'result' => false,
                    'response' => $e->__toString(),
                );
                if (version_compare(_PS_VERSION_, '1.7', '<')) {
                    return $data;
                } else {
                    return ($data);
                }
            }

            $this->payplug->storePayment($payment->id, (int) $cart->id);
            if ($one_click == 1 && $current_card != null && $id_card != 'new_card') {
                $data = array(
                    'result' => true,
                    'validation_url' => $return_url,
                );
                if (version_compare(_PS_VERSION_, '1.7', '<')) {
                    return $data;
                } else {
                    return ($data);
                }
            } elseif (($one_click == 1 && $id_card == 'new_card') || ($one_click != 1 && $id_card == 'new_card')) {
                $data = array(
                    'result' => 'new_card',
                    'embedded_mode' => (int) $embedded_mode,
                    'payment_url' => $payment->hosted_payment->payment_url,
                );
                return $data;
            } else {
                $payment_url = $payment->hosted_payment->payment_url;

                return $payment_url;
            }
        }

        public function getCardsList($context)
        {
            $customer = $context->customer;
            $payplug_cards = $this->payplug->getCardsByCustomer($customer->id);

            $data = $payplug_cards ? $payplug_cards : array();

            return $data;
        }

        public function getCardsListSupEqual350($context)
        {
            $customer = $context->customer;

            $payplug = new \PayPlug\classes\PayPlugClass();
            $plugin = $payplug->getPlugin();
            $card = $plugin->getCard();

            $payplug_cards = $card->getByCustomer((int) $customer->id);

            $data = $payplug_cards ? $payplug_cards : array();

            return $data;
        }

        public function deleteCard($context, $id_card)
        {
            $valid_key = Payplug::setAPIKey();
            return $this->payplug->deleteCard($context->customer->id, $id_card, $valid_key);
        }

        public function deleteCardSupEqual350($context, $id_card)
        {
            $payplug = new \PayPlug\classes\PayPlugClass();
            $plugin = $payplug->getPlugin();
            $card = $plugin->getCard();
            return $card->deleteCard((int) $context->customer->id, (int) $id_card);
        }

        private function getAmountsByCurrency($iso_code)
        {
            $min_amounts = array();
            $max_amounts = array();
            foreach (explode(';', Configuration::get('PAYPLUG_MIN_AMOUNTS')) as $amount_cur) {
                $cur = array();
                preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
                $min_amounts[$cur[1]] = (int) $cur[2];
            }
            foreach (explode(';', Configuration::get('PAYPLUG_MAX_AMOUNTS')) as $amount_cur) {
                $cur = array();
                preg_match('/^([A-Z]{3}):([0-9]*)$/', $amount_cur, $cur);
                $max_amounts[$cur[1]] = (int) $cur[2];
            }
            $current_min_amount = $min_amounts[$iso_code];
            $current_max_amount = $max_amounts[$iso_code];

            return array('min_amount' => $current_min_amount, 'max_amount' => $current_max_amount);
        }

        private function getIsoCodeByCountryId($country_id)
        {
            $iso_code_list = $this->getIsoCodeList();
            if (!is_array($iso_code_list) || empty($iso_code_list) || !count($iso_code_list)) {
                return false;
            }
            if (!Validate::isInt($country_id)) {
                return false;
            }
            $country = new Country((int) $country_id);
            if (!Validate::isLoadedObject($country)) {
                return false;
            }
            if (!in_array(Tools::strtoupper($country->iso_code), $iso_code_list)) {
                return false;
            } else {
                return Tools::strtoupper($country->iso_code);
            }
        }

        private function getIsoCodeList()
        {
            $country_list_path = _PS_MODULE_DIR_ . 'payplug/lib/iso_3166-1_alpha-2/data.csv';
            $iso_code_list = array();
            if (($handle = fopen($country_list_path, 'r')) !== false) {
                while (($data = fgetcsv($handle, 1000, ',')) !== false) {
                    $iso_code_list[] = Tools::strtoupper($data[0]);
                }
                fclose($handle);
                return $iso_code_list;
            } else {
                return null;
            }
        }

        private function getCardId($id_customer, $id_payplug_card, $id_company)
        {
            $is_sandbox = (int) Configuration::get('PAYPLUG_SANDBOX_MODE');

            $req_card_id = new DbQuery();
            $req_card_id->select('pc.id_card');
            $req_card_id->from('payplug_card', 'pc');
            $req_card_id->where('pc.id_customer = ' . (int) $id_customer);
            $req_card_id->where('pc.id_payplug_card = ' . (int) $id_payplug_card);
            $req_card_id->where('pc.id_company = ' . (int) $id_company);
            $req_card_id->where('pc.is_sandbox = ' . (int) $is_sandbox);
            $res_card_id = Db::getInstance()->getValue($req_card_id);

            if (!$res_card_id) {
                return false;
            } else {
                return $res_card_id;
            }
        }

        private function storePayment($pay_id, $id_cart)
        {
            if ($inst_id = $this->payplug->getInstallmentByCart($id_cart)) {
                $this->payplug->deleteInstallment($inst_id, $id_cart);
            }

            $req_payment_cart_exists = new DbQuery();
            $req_payment_cart_exists->select('*');
            $req_payment_cart_exists->from('payplug_payment_cart', 'ppc');
            $req_payment_cart_exists->where('ppc.id_cart = ' . (int) $id_cart);
            $res_payment_cart_exists = Db::getInstance()->getRow($req_payment_cart_exists);

            if (!$res_payment_cart_exists) {
                //insert
                $req_payment_cart = '
                    INSERT INTO ' . _DB_PREFIX_ . 'payplug_payment_cart (id_payment, id_cart)
                    VALUES (\'' . pSQL($pay_id) . '\', ' . (int) $id_cart . ')';
                $res_payment_cart = Db::getInstance()->execute($req_payment_cart);
                if (!$res_payment_cart) {
                    return false;
                }
            } else {
                //update
                $req_payment_cart = '
                    UPDATE ' . _DB_PREFIX_ . 'payplug_payment_cart ppc
                    SET ppc.id_payment = \'' . pSQL($pay_id) . '\'
                    WHERE ppc.id_cart = ' . (int) $id_cart;
                $res_payment_cart = Db::getInstance()->execute($req_payment_cart);
                if (!$res_payment_cart) {
                    return false;
                }
            }

            return true;
        }
    }
}
