<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'postfinance/postfinance.php')) {
    require_once _PS_MODULE_DIR_ . 'postfinance/postfinance.php';

    class PrestAppPostfinanceModuleController
    {
        protected $postfinance;

        public function __construct()
        {
            $this->postfinance = Module::getInstanceByName('postfinance');
        }

        public function displayPayment($context)
        {
            if (version_compare($this->postfinance->version, '2', '>=') === true && $this->postfinance->author == 'Prestashop') {
                return $this->displayPayment2Prestashop($context);
            }
            if (version_compare($this->postfinance->version, '2', '>=') === true) {
                return $this->displayPayment2($context);
            } else {
                return $this->displayPayment1($context);
            }
        }

        public function displayPayment1($context)
        {
            $options = array();

            $live_mode = (int)Configuration::get($this->postfinance->name.'_LIVE_MODE');
            if ($live_mode === 0) {
                $link = $this->postfinance->payment_link_test;
            } else {
                $link = $this->postfinance->payment_link;
            }

            $id_logo = (int)Configuration::get($this->postfinance->name.'_LOGO');

            array_push($options, array(
                'url' => $link,
                'postfinance_title' => $this->postfinance->l('Pay with').' '.$this->postfinance->displayName,
                'postfinance_logo' => trim($this->postfinance->ps_url.'modules/'.$this->postfinance->name.'/views/img/payment_logo'.$id_logo.'.png'),
                'params' => $this->postfinance->getParams()
            ));

            return array(
                'isMulti' => true,
                'options' => $options,
                'override' => array(
                    'pageType' => 'form',
                    'formAttributes' => array(
                        'urlAttribute' => 'url',
                        'parametersRetrieveMethod' => 'webservice',
                        'parametersWebservice' => 'params'
                    )
                )
            );
        }

        public function displayPayment2Prestashop($context)
        {
            $options = array();

            $live_mode = (int)Configuration::get($this->postfinance->name.'_LIVE_MODE');
            if ($live_mode === 0) {
                $link = $this->postfinance->payment_link_test;
            } else {
                $link = $this->postfinance->payment_link;
            }

            $id_logo = (int)Configuration::get($this->postfinance->name.'_LOGO');

            $params = $this->postfinance->getParams();
            $parsed_params = array();

            foreach ($params as $key => $value) {
                $parsed_params[$key] = $params[$key]['value'];
            }

            array_push($options, array(
                'url' => $link,
                'postfinance_title' => $this->postfinance->l('Pay with').' '.$this->postfinance->displayName,
                'postfinance_logo' => trim($this->postfinance->ps_url.'modules/'.$this->postfinance->name.'/views/img/payment_logo'.$id_logo.'.png'),
                'params' => $parsed_params
            ));

            return array(
                'isMulti' => true,
                'options' => $options,
                'override' => array(
                    'pageType' => 'form',
                    'formAttributes' => array(
                        'urlAttribute' => 'url',
                        'parametersRetrieveMethod' => 'webservice',
                        'parametersWebservice' => 'params'
                    )
                )
            );
        }

        public function displayPayment2($context)
        {
            $data = new StdClass();
            $data->name = 'postfinance';

            if (!$this->postfinance->active) {
                return false;
            }

            if (!$this->postfinance->checkCurrency($context->cart)) {
                return false;
            }

            $logo = _PS_BASE_URL_ . '/modules/postfinance/views/img/' . Configuration::get('POSTFINANCE_LOGO');

            $conf_cards = unserialize(Configuration::get('POSTFINANCE_CARDS'));

            $methods = array();

            if (Configuration::get('POSTFINANCE_DISPLAY') == 1 && !empty($conf_cards)) {
                $payments_method = array();
                $list_cards = '';
                foreach ($conf_cards as $card) {
                    $url_payment = $this->getUrlPayment($context, $context->cart, $card);
                    $logo = _PS_BASE_URL_ . '/modules/postfinance/views/img/' . $card;
                    $externalOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
                    $externalOption->setCallToActionText($this->postfinance->cards[$card])
                                   ->setAction($url_payment)
                                   ->setLogo($logo);
                    $list_cards .= $this->postfinance->cards[$card].', ';
                    $payments_method[] = $externalOption;
                }
                $list_cards = ' ('.Tools::substr($list_cards, 0, -2).')';


                foreach ($payments_method as $method) {
                    $object = array(
                        'postfinance_logo' => $method->getLogo(),
                        'postfinance_title' => $method->getCallToActionText(),
                        'postfinance_url' => $method->getAction(),
                        'is_active' => true,
                        'is_url' => true
                    );

                    array_push($methods, $object);
                }
            } else {
                $url_payment = $this->getUrlPayment($context, $context->cart);
                $logo = false;
                $list_cards = '';
                if (!empty($conf_cards)) {
                    foreach ($conf_cards as $card) {
                        $list_cards .= $this->postfinance->cards[$card].', ';
                    }
                    $list_cards = ' ('.Tools::substr($list_cards, 0, -2).')';
                }

                $externalOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
                $externalOption->setCallToActionText($this->postfinance->l('Pay by postfinance').$list_cards)
                               ->setAction($url_payment)
                               ->setLogo($logo);

                $object = array(
                    'postfinance_logo' => $externalOption->getLogo(),
                    'postfinance_title' => $externalOption->getCallToActionText(),
                    'postfinance_url' => $externalOption->getAction(),
                    'is_active' => true,
                    'is_url' => true
                );

                array_push($methods, $object);
            }

            return array(
                'isMulti' => true,
                'options' => $methods
            );
        }

        public function getUrlPayment($context, $cart, $card = null)
        {
            $iso_code = Language::getIsoById($context->language->id);
            if ($iso_code=='fr') {
                $iso_code = 'fr_FR';
            } elseif ($iso_code=='de' || $iso_code=='dh') {
                $iso_code = 'de_DE';
            } elseif ($iso_code=='it') {
                $iso_code = 'it_IT';
            } elseif ($iso_code=='es') {
                $iso_code = 'es_ES';
            } else {
                $iso_code = 'en_US';
            }

            // adresse
            $Address = new Address($cart->id_address_invoice);
            $address = $Address->getFields();
            // country
            $country = Country::getIsoById($address['id_country']);
            // n° panier
            $order_no = $context->cart->id;

            $Currency = new Currency($context->currency->id);
            $currency = $Currency->getFields();
            $currency_iso_code = $currency['iso_code'];

            $total = $cart->getOrderTotal(true, 3);
            if ($Currency->decimals==0) {
                $total = Tools::ps_round($total, $Currency->decimals);
            }
            $total = $total*100;

            $order_no = 'cart_no:'.$order_no.'-('.date('d/m/Y-H:i:s').')';
            $Customer = new Customer($context->customer->id);
            $customer_CN = $this->postfinance->cleanValuePostfinance($Customer->firstname).' '.$this->postfinance->cleanValuePostfinance($Customer->lastname);

            $url_shop = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').$_SERVER['HTTP_HOST'].__PS_BASE_URI__;

            $fields_crypt = array(
                'PSPID'=>$this->postfinance->pspid,
                'AMOUNT'=>$total,
                'CURRENCY'=>$currency_iso_code,
                'ORDERID'=>$order_no,
                'LANGUAGE'=>$iso_code,
                'CN'=>$customer_CN,
                'EMAIL'=>$context->customer->email,
                'OWNERZIP'=>$this->postfinance->cleanValuePostfinance($address['postcode']),
                'OWNERADDRESS'=>'-',/*$this->cleanValuePostfinance($address['address1'])*/
                'OWNERCTY'=>$this->postfinance->cleanValuePostfinance($country),
                'OWNERTOWN'=>$this->postfinance->cleanValuePostfinance($address['city']),
                'ACCEPTURL'=>$url_shop.'module/postfinance/validation',
                'DECLINEURL'=>$url_shop.'module/postfinance/error',
                'EXCEPTIONURL'=>$url_shop.'module/postfinance/error',
                'CANCELURL'=>$url_shop.'?controller=order',
            );

            if (Configuration::get('POSTFINANCE_DISPLAY')==1) {
                if (isset($this->postfinance->cards_brand[$card]['PM'])) {
                    ;
                    $fields_crypt['PM'] = $this->postfinance->cards_brand[$card]['PM'];
                    $fields_crypt['BRAND'] = $this->postfinance->cards_brand[$card]['BRAND'];
                }
            }

            // alias manager ?
            if ($this->postfinance->alias_manager) {
                $fields_alias = array('ALIAS'=>$context->customer->id,
                                       'ALIASUSAGE'=>$this->postfinance->l('Keeping your information'));
                $fields_crypt = array_merge($fields_crypt, $fields_alias);
            }

            // logo
            $https_logo = Configuration::get('POSTFINANCE_HTTPS_LOGO');
            if (!empty($https_logo)) {
                $field_logo = array('LOGO'=>Configuration::get('POSTFINANCE_HTTPS_LOGO'));
                $fields_crypt = array_merge($fields_crypt, $field_logo);
            }

            // apparence
            foreach ($this->postfinance->fields_color as $fc) {
                $color_hex = Configuration::get('POSTFINANCE_'.$fc['name']);
                if (!empty($color_hex)) {
                    $color_hex = str_replace('#', '', $color_hex);
                    $field_theme = array($fc['name']=>$color_hex);
                    $fields_crypt = array_merge($fields_crypt, $field_theme);
                }
            }

            ksort($fields_crypt); // important ordre alphabétique

            $url_payment = '';
            $string_crypt = '';
            foreach ($fields_crypt as $key => $v) {
                $string_crypt .= $key.'='.$v.$this->postfinance->sha_in;
                $url_payment .= $key.'='.$v.'&';
            }
            $url_payment .= 'SHASIGN='.sha1($string_crypt);
            $url_payment = $this->postfinance->postfinance_url.'?'.$url_payment;
            return $url_payment;
        }
    }
}
