<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'postfinancecheckout/postfinancecheckout.php')) {
    require_once _PS_MODULE_DIR_ . 'postfinancecheckout/postfinancecheckout.php';

    class PrestAppPostfinancecheckoutModuleController
    {
        protected $postfinancecheckout;

        public function __construct()
        {
            $this->postfinancecheckout = Module::getInstanceByName('postfinancecheckout');
        }

        public function displayPaymentPs16($context)
        {
            if (!$this->postfinancecheckout->active) {
                return false;
            }

            if (!isset($context->cart) || ! ($context->cart instanceof Cart)) {
                return false;
            }

            $uniqueId = PostFinanceCheckoutHelper::generateUUID();
            $context->cookie->pfc_device_id = $uniqueId;

            $cart = $context->cart;

            try {
                $possiblePaymentMethods = PostFinanceCheckoutServiceTransaction::instance()->getPossiblePaymentMethods($cart);
            } catch (PostFinanceCheckoutExceptionInvalidtransactionamount $e) {
                return false;
            } catch (Exception $e) {
                return false;
            }

            $shopId = $cart->id_shop;
            $language = $context->language->language_code;
            $methods_list = array();
            $methods = array();

            foreach ($possiblePaymentMethods as $possible) {
                $methodConfiguration = PostFinanceCheckoutModelMethodconfiguration::loadByConfigurationAndShop(
                    $possible->getSpaceId(),
                    $possible->getId(),
                    $shopId
                );
                if (!$methodConfiguration->isActive()) {
                    continue;
                }
                $methods[] = $methodConfiguration;
            }

            foreach (PostFinanceCheckoutHelper::sortMethodConfiguration($methods) as $methodConfiguration) {
                $templateVars = PostFinanceCheckoutBasemodule::getParametersFromMethodConfiguration($this->postfinancecheckout, $methodConfiguration, $cart, $shopId, $language);

                $object = array(
                    'postfinance_logo' => $templateVars['image'],
                    'postfinance_title' => $templateVars['name'],
                    'is_active' => true,
                    'is_url' => true,
                    'is_redirect' => true,
                    'url_params' => array(
                        'method_id' => $templateVars['methodId'],
                        'pfc_device_id' => $uniqueId,
                        'configuration_id' => $templateVars['configurationId']
                    )
                );

                array_push($methods_list, $object);
            }

            return array(
                'isMulti' => true,
                'options' => $methods_list
            );
        }

        public function displayPayment($context)
        {
            if (!$this->postfinancecheckout->active) {
                return false;
            }

            if (!isset($context->cart) || ! ($context->cart instanceof Cart)) {
                return false;
            }

            $uniqueId = PostFinanceCheckoutHelper::generateUUID();
            $context->cookie->pfc_device_id = $uniqueId;

            $cart = $context->cart;

            try {
                $possiblePaymentMethods = PostFinanceCheckoutServiceTransaction::instance()->getPossiblePaymentMethods($cart);
            } catch (PostFinanceCheckoutExceptionInvalidtransactionamount $e) {
                return false;
            }

            $shopId = $cart->id_shop;
            $language = $context->language->language_code;
            $methods_list = array();
            $methods = array();

            foreach ($possiblePaymentMethods as $possible) {
                $methodConfiguration = PostFinanceCheckoutModelMethodconfiguration::loadByConfigurationAndShop(
                    $possible->getSpaceId(),
                    $possible->getId(),
                    $shopId
                );
                if (!$methodConfiguration->isActive()) {
                    continue;
                }
                $methods[] = $methodConfiguration;
            }

            $result = array();

            foreach (PostFinanceCheckoutHelper::sortMethodConfiguration($methods) as $methodConfiguration) {
                $parameters = PostFinanceCheckoutBasemodule::getParametersFromMethodConfiguration($this->postfinancecheckout, $methodConfiguration, $cart, $shopId, $language);
                $parameters['priceDisplayTax'] = Group::getPriceDisplayMethod(Group::getCurrent()->id);
                $parameters['orderUrl'] = $context->link->getModuleLink(
                    'postfinancecheckout',
                    'order',
                    array(),
                    true
                );
                $paymentOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
                $paymentOption->setCallToActionText($parameters['name']);
                $paymentOption->setLogo($parameters['image']);
                $paymentOption->setAction($parameters['link']);
                $paymentOption->setModuleName($this->postfinancecheckout->name);
                $result[] = array('option' => $paymentOption, 'parameters' => $parameters);
            }

            foreach ($result as $method) {
                $object = array(
                    'postfinance_logo' => $method['option']->getLogo(),
                    'postfinance_title' => $method['option']->getCallToActionText(),
                    'is_active' => true,
                    'is_url' => true,
                    'is_redirect' => true,
                    'url_params' => array(
                        'method_id' => $method['parameters']['methodId'],
                        'pfc_device_id' => $uniqueId,
                        'configuration_id' => $method['parameters']['configurationId']
                    )
                );

                array_push($methods_list, $object);
            }

            return array(
                'isMulti' => true,
                'options' => $methods_list
            );
        }

        public function getUrl($method_id, $pfc_device_id, $configuration_id, $context)
        {
            $checkoutResult = $this->checkout($method_id, $pfc_device_id, $context);

            if ($checkoutResult) {
                $orderResult = $this->order($method_id, $checkoutResult, $context);
                if ($orderResult) {
                    return array(
                        'type' => 'payment',
                        'url' => $orderResult,
                    );
                }
            }

            return false;
        }

        private function checkout($methodId, $pfc_device_id, $context)
        {
            $cart = $context->cart;

            try {
                $context->cookie->pfc_device_id = $pfc_device_id;

                PostFinanceCheckoutFeehelper::removeFeeSurchargeProductsFromCart($cart);

                if ($methodId !== null) {
                    PostFinanceCheckoutFeehelper::addSurchargeProductToCart($cart);
                    $methodConfiguration = new PostFinanceCheckoutModelMethodconfiguration($methodId);
                    PostFinanceCheckoutFeehelper::addFeeProductToCart($methodConfiguration, $cart);
                    $transaction = PostFinanceCheckoutServiceTransaction::instance()->getTransactionFromCart($cart);
                }
                $cartHash = PostFinanceCheckoutHelper::calculateCartHash($cart);

                $response = array(
                    'result' => 'success',
                    'cartHash' => $cartHash,
                    'transaction' => $transaction
                );

                return $response;
            } catch (Exception $e) {
                return false;
            }
        }

        private function order($methodId, $checkoutResult, $context)
        {
            $cartHash = $checkoutResult['cartHash'];

            if ($methodId == null || $cartHash == null) {
                return false;
            }

            $cart = $context->cart;
            if (!$this->checkAvailablility($cart)) {
                return false;
            }

            $spaceId = Configuration::get(PostFinanceCheckoutBasemodule::CK_SPACE_ID, null, null, $cart->id_shop);
            $methodConfiguration = new PostFinanceCheckoutModelMethodconfiguration($methodId);

            if (!$methodConfiguration->isActive() || $methodConfiguration->getSpaceId() != $spaceId) {
                return false;
            }
            // Ensure Fees are correct
            PostFinanceCheckoutFeehelper::removeFeeSurchargeProductsFromCart($cart);
            PostFinanceCheckoutFeehelper::addSurchargeProductToCart($cart);
            PostFinanceCheckoutFeehelper::addFeeProductToCart($methodConfiguration, $cart);

            if ($cartHash != PostFinanceCheckoutHelper::calculateCartHash($cart)) {
                return false;
            }
            $orderState = PostFinanceCheckoutOrderstatus::getRedirectOrderStatus();

            try {
                $customer = new Customer((int) $cart->id_customer);
                $this->postfinancecheckout->validateOrder(
                    $cart->id,
                    $orderState->id,
                    $cart->getOrderTotal(true, Cart::BOTH, null, null, false),
                    'postfinancecheckout_' . $methodId,
                    null,
                    array(),
                    null,
                    false,
                    $customer->secure_key
                );

                $url = PostFinanceCheckoutServiceTransaction::instance()->getPaymentPageUrl(
                    $GLOBALS['postfinancecheckoutTransactionIds']['spaceId'],
                    $GLOBALS['postfinancecheckoutTransactionIds']['transactionId']
                );

                return $url;
            } catch (Exception $e) {
                return false;
            }
        }

        protected function checkAvailablility(Cart $cart)
        {
            if ($cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || ! $this->postfinancecheckout->active || !Validate::isLoadedObject(new Customer($cart->id_customer))) {
                return false;
            }

            $authorized = false;
            foreach (Module::getPaymentModules() as $module) {
                if ($module['name'] == 'postfinancecheckout') {
                    $authorized = true;
                    break;
                }
            }
            if (!$authorized) {
                return false;
            }

            if (!$this->postfinancecheckout instanceof PostFinanceCheckout) {
                return false;
            }

            return true;
        }
    }
}
