<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon
 * @copyright Copyright (c) 2017 - 2018 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

class PrestAppPrelevementSepaModuleController extends PaymentModule
{
    protected $prelevementsepa;

    public function __construct()
    {
        $this->prelevementsepa = Module::getInstanceByName('prelevementsepa');
    }

    public function displayPayment($context)
    {
        $cart = $context->cart;
        $total = sprintf(
            $this->prelevementsepa->trans('%1$s (tax incl.)', array(), 'Modules.PrelevementSEPA.Shop'),
            Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH))
        );

        $this->prelevementsepa->smarty->assign(
            array(
                'this_path' => _PS_BASE_URL_ . $this->prelevementsepa->getPathUri(),
                'this_path_sepa' => _PS_BASE_URL_ . $this->prelevementsepa->getPathUri(),
                'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->prelevementsepa->name.'/',
                'total' => $total
            )
        );

        $tpl = $this->prelevementsepa->fetch('module:prelevementsepa/views/templates/front/1.7/payment_sepa_infos.tpl');

        return array(
            'tpl' => $tpl
        );
    }

    public function validateSepaOrder($context)
    {
        $cart = $context->cart;

        if ($cart->id_customer == 0
            || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0
            || !$this->prelevementsepa->active
        ) {
            return false;
        }

        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'prelevementsepa') {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            return false;
        }
        $customer = new Customer($cart->id_customer);

        if (!Validate::isLoadedObject($customer)) {
            return false;
        }

        $currency = $context->currency;
        $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

        $this->prelevementsepa->validateOrder(
            (int)$cart->id,
            (int)$this->prelevementsepa->conf_keys['state'],
            $total,
            $this->prelevementsepa->displayName,
            null,
            array(),
            (int)$currency->id,
            false,
            $customer->secure_key
        );

        $order = new Order($this->prelevementsepa->currentOrder);
        return $order;
    }

    public function getMandateData($context)
    {
        require_once(dirname(__FILE__).'/../../../../prelevementsepa/classes/PrevSepa.php');

        $data = [];

        $prev_obj = new PrevSepa();
        $id_sepa = $prev_obj->existCustomer((int)$context->customer->id);

        if (!is_null($id_sepa)) {
            $prel_obj = new PrelevementSEPA();
            if ($prel_obj->isConnected() && $prel_obj->sms_solde > 0) {
                $data['smsvalidation'] = 1;
            }
        }

        $sepa = new PrevSepa((int)$id_sepa);
        $data['id_customer'] = (int)$context->customer->id;
        $data['customer_country'] = $context->country->iso_code;
        $data['sepa'] = $sepa;
        $data['mandate_url'] = _PS_BASE_URL_ . $this->prelevementsepa->getPathUri();

        return $data;
    }

    public function saveSepa($params, $context)
    {
        require_once(dirname(__FILE__).'/../../../../prelevementsepa/classes/PrevSepa.php');
        require_once($this->prelevementsepa->getLocalPath().'libs/oophp-iban.php');

        $prev_obj = new PrevSepa();
        $id_sepa = $prev_obj->existCustomer((int)$context->customer->id);

        if (!is_null($id_sepa)) {
            $prev_obj = new PrevSepa($id_sepa);
        } else {
            $prev_obj->etat = 0;
        }

        if (Tools::strlen($prev_obj->rum) < 3) {
            $prev_obj->rum = 'UI'.date('Ymd').Tools::passwdGen(12, 'NUMERIC');
        }

        if (empty($params['sepa_name'])) {
            return false;
        } else {
            $prev_obj->name = $params['sepa_name'];
        }

        if (empty($params['sepa_address'])) {
            return false;
        } else {
            $prev_obj->address = $params['sepa_address'];
        }

        $iban = str_replace(' ', '', Tools::strtoupper($params['sepa_iban']));
        $bicbnc = str_replace(' ', '', Tools::strtoupper($params['sepa_bic']));

        if (Tools::strlen($iban) > 3) {
            $ibanObj = new IBAN($iban);
            if ($ibanObj->Verify()) {
                $prev_obj->iban = $iban;
            } else {
                return false;
            }
        } else {
            return false;
        }

        if (preg_match('#^([a-zA-Z]){4}([a-zA-Z]){2}([0-9a-zA-Z]){2}([0-9a-zA-Z]{3})?$#', $bicbnc)) {
            $prev_obj->bic = $bicbnc;
        } else {
            return false;
        }

        $prev_obj->id_customer = (int)$context->customer->id;
        $prev_obj->save();

        return true;
    }

    public function downloadMandate($context)
    {
        require_once(dirname(__FILE__).'/../../../../prelevementsepa/classes/PrevSepa.php');
        require_once(dirname(__FILE__).'/../../../../prelevementsepa/classes/HTMLTemplatePrelevementSepaPdf.php');

        $prev_obj = new PrevSepa();
        $id_sepa = $prev_obj->existCustomer((int)$context->customer->id);
        $prev_obj = new PrevSepa((int)$id_sepa);
        $prel_obj = new PrelevementSEPA();

        $pdf_data = array(
            'sepa' => $prev_obj,
            'conf' => $prel_obj->conf_keys,
            'this_path_sepa' => $this->prelevementsepa->getPathUri(),
            'shop_name' => $context->shop->name,
        );

        Context::getContext()->smarty->assign(
            $pdf_data
        );

        $template = $this->prelevementsepa->getLocalPath().'views/templates/front/pdf/'.$this->prelevementsepa->conf_keys['template'].'.tpl';
        if (file_exists(_PS_THEME_DIR_.'modules/'.$this->prelevementsepa->name.'/views/templates/front/pdf/'.$this->prelevementsepa->conf_keys['template'].'.tpl')) {
            $template = _PS_THEME_DIR_.'modules/'.$this->prelevementsepa->name.'/views/templates/front/pdf/'.$this->prelevementsepa->conf_keys['template'].'.tpl';
        }

        $prev_obj->html = $context->smarty->fetch($template);

        $pdf = new PDF($prev_obj, 'PrelevementSepaPdf', Context::getContext()->smarty, 'P');
        $pdf->render();
        die();
    }
}
