<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'ps_checkout/ps_checkout.php')) {
    require_once _PS_MODULE_DIR_ . 'ps_checkout/ps_checkout.php';

    class PrestAppPsCheckoutModuleController
    {
        protected $ps_checkout;

        public function __construct()
        {
            $this->ps_checkout = Module::getInstanceByName('ps_checkout');
        }

        public function displayPayment($context) {

            $cart = new Cart((int) $context->cart->id);

            if (false === Validate::isLoadedObject($cart)
                || false === $this->ps_checkout->checkCurrency($cart)
                || false === $this->ps_checkout->merchantIsValid()
            ) {
                return [];
            }

            $paypalAccountRepository = $this->ps_checkout->getService('ps_checkout.repository.paypal.account');
            $fundingSourceProvider = $this->ps_checkout->getService('ps_checkout.funding_source.provider');

            $payment_options = array();

            foreach ($fundingSourceProvider->getAll() as $fundingSource) {

                if (!$fundingSource->isEnabled) {
                    continue;
                }

                if (count($fundingSource->countries)) {
                    $country_code = getUserCurrentCountry();
                    if ($country_code) {
                        if (!in_array($country_code, $fundingSource->countries)) {
                            continue;
                        }
                    } else {
                        continue;
                    }
                }

                $payment_options[] = array(
                    'logo' => Tools::getHttpHost(true) . __PS_BASE_URI__ . '/modules/' . $this->ps_checkout->name . '/views/img/' . $fundingSource->name . '.svg',
                    'name' => $fundingSource->label,
                    'is_active' => true,
                    'is_url' => true,
                    'is_redirect' => true,
                    'url_params' => array(
                        'paymentOptionName' => $fundingSource->label,
                        'fundingSource' => $fundingSource->name
                    )
                );
            }

            return ($payment_options && count($payment_options)) ? $payment_options : [];
        }

        public function getPaymentUrl($context, $fundingSource, $paymentOptionName)
        {
            $params = array(
                'fundingSource' => $fundingSource,
                'paymentOptionName' => $paymentOptionName,
                'cart_id' => $context->cart->id
            );

            $params_string = '?';

            foreach($params as $key => $value) {
                if ($value) {
                    $params_string .= $key . '=' . urlencode($value) . '&';
                }
            }

            $payment_url = _PS_BASE_URL_ . '/module/prestapp/' . 'PsCheckout' . $params_string;

            return array(
                'type' => 'payment',
                'url' => $payment_url,
            );
        }
    }
}