<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . '/sonicepro/sonicepro.php')) {
    require _PS_MODULE_DIR_ . '/sonicepro/sonicepro.php';

    class PrestAppSoniceproModuleController
    {
        protected $sonicepro;

        protected $carrier_conf_keys = array(
            'SONICEPRO_DOM_ID',
            'SONICEPRO_DOS_ID',
            'SONICEPRO_RLY_ID',
            'SONICEPRO_COM_ID',
            'SONICEPRO_CDS_ID',
            'SONICEPRO_ECO_ID',
            'SONICEPRO_COLI_ID'
        );

        public function _construct()
        {
            $this->sonicepro = Module::getInstanceByName('sonicepro');
        }

        public function displayCarrier($context)
        {
            $address = new Address($context->cart->id_address_delivery);
            if (!Validate::isLoadedObject($address)) {
                return false;
            }

            $country_iso = Tools::strtoupper(Country::getIsoById($address->id_country));
            $phone = Tools::strlen($address->phone) ? $address->phone : $address->phone_mobile;

            $data = array(
                'address_to_search' => sprintf(
                    '%s, %s %s, %s',
                    $address->address1,
                    $address->postcode,
                    $address->city,
                    $address->country
                ),
                'sp_address_company' => Tools::strtoupper($address->company),
                'sp_address_street' => Tools::strtoupper($address->address1).',',
                'sp_address_postcode' => Tools::strtoupper($address->postcode).',',
                'sp_address_city' => Tools::strtoupper($address->city).',',
                'sp_address_postcode_city' => Tools::strtoupper(sprintf(
                    '%s %s',
                    $address->postcode,
                    $address->city
                )),
                'sp_address_country' => $address->country,
                'sp_address_country_iso' => $country_iso,
                'sp_customer_name' => Tools::strtoupper(sprintf(
                    '%s %s',
                    $address->lastname,
                    $address->firstname
                )),
                'id_address_delivery' => $context->cart->id_address_delivery,
                'id_customer' => $context->cart->id_customer,
                'id_cart' => $context->cart->id,
                'sp_customer_email' => $context->customer->email,
                'sp_customer_phone_mobile' => SoNiceProTools::formatPhoneNumber($phone, $country_iso),
                'sp_otp' => SoNiceProOTP::createOTPStatic($this->context->cart->id_customer)
            );

            $carrier_id_refs = array_filter(array_map('intval', Configuration::getMultiple(
                $this->carrier_conf_keys,
                null,
                null,
                null
            )));

            $sp_carriers_id = SoNiceProCarrier::getSoNiceProCarriersId($carrier_id_refs);

            if (isset($sp_carriers_id) && count($sp_carriers_id)) {
                foreach ($sp_carriers_id as $key => $value) {
                    $data[$key] = $value;
                }
            }

            $data['sp_get_delivery_points_url'] = Tools::getHttpHost(true).__PS_BASE_URI__. 'modules/'.$this->sonicepro->name.'/functions/get_delivery_points.php';
            $data['sp_save_delivery_point_url'] = Tools::getHttpHost(true).__PS_BASE_URI__. 'modules/'.$this->sonicepro->name.'/functions/save_delivery_point.php';
            $data['sp_update_delivery_info_url'] = Tools::getHttpHost(true).__PS_BASE_URI__. 'modules/'.$this->sonicepro->name.'/functions/update_delivery_info.php';
            $data['sp_img_dir'] = $this->sonicepro->_path.'views/img/';

            return $data;
        }

        public function getPoints($context, $params)
        {
            $tariff = new SoNiceProDeliveryTariff();
            $countries = array_map(array('Tools', 'strtoupper'), SoNiceProTools::arrayColumn(
                Country::getCountries($context->language->id, true),
                'iso_code'
            ));

            if (!$tariff->countryIsAvailableForRelay($params['sp_address_country_iso']) || !in_array($params['sp_address_country_iso'], $countries)) {
                return false;
            }

            $address = new Address();
            $address->address1 = $params['sp_address_street'];
            $address->postcode = $params['sp_address_postcode'];
            $address->city = $params['sp_address_city'];
            $address->id_country = Country::getByIso($params['sp_address_country_iso']);

            $weight = 1;
            $cart = new Cart((int)params['id_cart']);
            if (Validate::isLoadedObject($cart)) {
                $weight = max($weight, $cart->getTotalWeight());

                if (Tools::strtolower(Configuration::get('PS_WEIGHT_UNIT')) == 'kg') {
                    $weight *= 1000;
                }
            }

            $configuration = Tools::unSerialize(Configuration::get('SONICEPRO_CONFIGURATION'));

            $api = new SoNiceProDeliveryAPI(
                $configuration['delivery']['login'] ?: '123456',
                $configuration['delivery']['pwd'] ?: '123456',
                $configuration['delivery']['ws'] ?: 'https://ws.colissimo.fr/pointretrait-ws-cxf/PointRetraitServiceWS/2.0?wsdl'
            );

            $response = $api->getDeliveryPoints($address, $weight);

            return $response;
        }

        public function saveDeliveryPoint($params)
        {
            $message = '';
            $id_cart = $params['id_cart'];
            $type = $params['infoPoint']['typeDePoint'];
            $prid = $params['infoPoint']['identifiant'];
            $name = $params['infoPoint']['nom'];
            $address1 = $params['infoPoint']['adresse1'];
            $city = $params['infoPoint']['localite'];
            $zip = $params['infoPoint']['codePostal'];
            $iso_country = $params['infoPoint']['codePays'];
            try {
                Db::getInstance()->insert(
                    'sonicepro_delivery',
                    array(
                        'id_cart' => (int)$id_cart,
                        'type' => pSQL($type),
                        'prid' => pSQL($prid),
                        'name' => pSQL($name),
                        'address1' => pSQL($address1),
                        'city' => pSQL($city),
                        'zip' => pSQL($zip),
                        'iso_country' => pSQL($iso_country)
                    ),
                    '',
                    false,
                    Db::REPLACE
                );
                return true;
            } catch (Exception $e) {
                return false;
            }
        }

        public function saveInfoCommande($params, $context)
        {
            $id_cart = $params['cart'];
            $email = $params['email'];
            $phone = $params['phone'];

            $record_exists = Db::getInstance()->getRow(
                'SELECT * FROM `'._DB_PREFIX_.'sonicepro_delivery` WHERE `id_cart` = '.(int)$id_cart
            );

            try {
                if (!is_array($record_exists) || !count($record_exists)) {
                    Db::getInstance()->insert(
                        'sonicepro_delivery',
                        array(
                            'id_cart' => (int)$id_cart,
                            'type' => '',
                            'prid' => ''
                        )
                    );
                }

                Db::getInstance()->update(
                    'sonicepro_delivery',
                    array(
                        'email' => pSQL($email),
                        'phone' => pSQL($phone)
                    ),
                    '`id_cart` = '.(int)$id_cart
                );

                return true;
            } catch (Exception $e) {
                return false;
            }
        }
    }
}
