<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'stripe_official/stripe_official.php')) {
    require_once _PS_MODULE_DIR_ . 'stripe_official/stripe_official.php';

    class PrestAppStripeofficialModuleController
    {
        protected $stripe;

        public function __construct()
        {
            $this->stripe = Module::getInstanceByName('stripe_official');
        }

        public function displayPayment($intent_id, $context)
        {
            $response = $this->hookPayment($intent_id, $context);
            return $response;
        }

        public function hookPayment($intent_id, $context)
        {
            if (!$this->stripe->active) {
                return false;
            }

            if (!$this->checkApiConnection()) {
                return false;
            }

            $currency = new Currency($context->cart->id_currency);
            $currency_iso_code = Tools::strtolower($currency->iso_code);
            $address = new Address($context->cart->id_address_invoice);
            $amount = $context->cart->getOrderTotal();
            $amount = Tools::ps_round($amount, 2);
            $amount = $this->stripe->isZeroDecimalCurrency($currency_iso_code) ? $amount : $amount * 100;

            $intent = $this->retrievePaymentIntent($amount, $currency_iso_code, $intent_id, $context);

            // var_dump(Configuration::get(Stripe_official::POSTCODE));exit();

            if (Configuration::get(Stripe_official::POSTCODE) == null) {
                $stripe_postcode_enabled = 'off';
            } else {
                $stripe_postcode_enabled = Configuration::get(Stripe_official::POSTCODE);
            }

            if (Configuration::get(Stripe_official::REINSURANCE) == null) {
                $stripe_reinsurance_enabled = 'off';
            } else {
                $stripe_reinsurance_enabled = Configuration::get(Stripe_official::REINSURANCE);
            }

            // var_dump($amount);exit();


            // $address = new Address($context->cart->id_address_invoice);
            // $country = Country::getIsoById($address->id_country);
            // $currency = Tools::strtolower($context->currency->iso_code);

            // \Stripe\Stripe::setApiKey($this->stripe->getSecretKey());

            // $intent = \Stripe\PaymentIntent::create([
            //     'amount' => 200, // $amount
            //     'currency' => $currency,
            // ]);

            $module_dir = Media::getMediaPath(_PS_MODULE_DIR_.$this->stripe->name);

            $logos = array();

            if ($stripe_reinsurance_enabled == 'on') {
                $payment_methods = $this->stripe->getPaymentMethods();
                if ($payment_methods) {
                    foreach ($payment_methods as $payment_method) {
                        $logos[] =  _PS_BASE_URL_SSL_ . $module_dir . '/views/img/logo_'.$payment_method['name'].'.png';
                    }
                }
            }

            return array(
                'stripe_payment_id' => $intent->id,
                'stripe_client_secret' => $intent->client_secret,
                'currency' => $currency,
                'stripe_address_country_code' => Country::getIsoById($address->id_country),
                'stripe_email' => $context->customer->email,
                'stripe_locale' => $context->language->iso_code,
                'stripe_pk' => $this->stripe->getPublishableKey(),
                'stripe_amount' => Tools::ps_round($amount, 0),
                'applepay_googlepay' => Configuration::get(Stripe_official::ENABLE_APPLEPAY_GOOGLEPAY),
                'stripe_postcode_enabled' => $stripe_postcode_enabled,
                'stripe_cardholdername_enabled' => Configuration::get(Stripe_official::CARDHOLDERNAME),
                'stripe_reinsurance_enabled' => $stripe_reinsurance_enabled,
                'stripe_payment_methods' => $this->stripe->getPaymentMethods(),
                'customer_name' => $address->firstname . ' ' . $address->lastname,
                'stripe_css' => '{"base": {"iconColor": "#666ee8","color": "#31325f","fontWeight": 400,"fontFamily": "-apple-system, BlinkMacSystemFont, Segoe UI, Roboto, Oxygen-Sans, Ubuntu, Cantarell, Helvetica Neue, sans-serif","fontSmoothing": "antialiased","fontSize": "15px","::placeholder": { "color": "#aab7c4" },":-webkit-autofill": { "color": "#666ee8" }}}',
                'img_powered' => $module_dir . '/views/img/powered_by_stripe.png',
                'logos' => $logos
            );
        }

        protected function checkApiConnection($secretKey = null)
        {
            if (!$secretKey) {
                $secretKey = $this->stripe->getSecretKey();
            }

            try {
                \Stripe\Stripe::setApiKey($secretKey);
                \Stripe\Account::retrieve();
            } catch (Exception $e) {
                error_log($e->getMessage());
                $this->errors[] = $e->getMessage();
                return false;
            }
            return true;
        }

        protected function retrievePaymentIntent($amount, $currency, $intent_id, $context)
        {
            if (isset($intent_id)
                && !empty($intent_id)) {
                try {
                    $intent = \Stripe\PaymentIntent::retrieve($intent_id);

                    // Check that the amount is still correct
                    if ($intent->amount != $amount) {
                        $intent->update(
                            $intent_id,
                            array(
                                "amount" => $amount
                            )
                        );
                    }

                    return $intent;
                } catch (Exception $e) {
                }
            }

            $address = new Address($context->cart->id_address_invoice);
            $country = Country::getIsoById($address->id_country);
            $currency = Tools::strtolower($context->currency->iso_code);

            $options = array();
            foreach (Stripe_official::$paymentMethods as $name => $paymentMethod) {
                // Check if the payment method is enabled
                if ($paymentMethod['enable'] !== true && Configuration::get($paymentMethod['enable']) != 'on') {
                    continue;
                }

                // Check for country support
                if (isset($paymentMethod['countries']) && !in_array($country, $paymentMethod['countries'])) {
                    continue;
                }

                // Check for currency support
                if (isset($paymentMethod['currencies']) && !in_array($currency, $paymentMethod['currencies'])) {
                    continue;
                }

                $options[] = $name;
            }

            try {
                $intent = \Stripe\PaymentIntent::create(array(
                    "amount" => $amount,
                    "currency" => $currency,
                    "payment_method_types" => array($options),
                ));

                $paymentIntent = new StripePaymentIntent();
                $paymentIntent->setIdPaymentIntent($intent->id);
                $paymentIntent->setStatus($intent->status);
                $paymentIntent->setAmount($intent->amount);
                $paymentIntent->setCurrency($intent->currency);
                $paymentIntent->setDateAdd(date("Y-m-d H:i:s", $intent->created));
                $paymentIntent->setDateUpd(date("Y-m-d H:i:s", $intent->created));
                $paymentIntent->save(false, false);

                return $intent;
            } catch (Exception $e) {
                return false;
            }
        }

        public function validateOrder($response, $context)
        {
            $response = (object) $response;
            // var_dump($response);exit();
            $intent = \Stripe\PaymentIntent::retrieve($response->id);
            $charges = $intent->charges->data;

            $conveyor = array();

            $conveyor['token'] = $response->source;
            $conveyor['id_payment_intent'] = $response->id;
            $conveyor['status'] = $response->status;
            $conveyor['chargeId'] = $charges[0]->id;

            if ($this->stripe->isZeroDecimalCurrency($context->currency->iso_code)) {
                $conveyor['amount'] = $charges[0]->amount;
            } else {
                $conveyor['amount'] = $charges[0]->amount / 100;
            }

            $amount = $conveyor['amount'];
            if (strstr($conveyor['chargeId'], 'ch_')) {
                $amount = $amount*100;
            }

            $paymentIntent = new StripePaymentIntent();
            $paymentIntent->findByIdPaymentIntent($conveyor['id_payment_intent']);
            $paymentIntent->setAmount($amount);
            $paymentIntent->setStatus($conveyor['status']);
            $paymentIntent->setDateUpd(date("Y-m-d H:i:s"));

            $paymentIntent->update();

            $conveyor['paymentIntent'] = $paymentIntent;

            if ($conveyor['status'] != 'succeeded' && $conveyor['status'] != 'pending') {
                return false;
            }

            $message = 'Stripe Transaction ID: '.$conveyor['id_payment_intent'];

            $conveyor['source'] = \Stripe\Source::retrieve($conveyor['token']);

            if (isset($context->customer->secure_key)) {
                $conveyor['secure_key'] = $context->customer->secure_key;
            } else {
                $conveyor['secure_key'] = false;
            }

            if ($this->stripe->isZeroDecimalCurrency($conveyor['paymentIntent']->getCurrency())) {
                $paid = $conveyor['paymentIntent']->getAmount()*100;
            } else {
                $paid = $conveyor['paymentIntent']->getAmount();
            }

            if (!empty($conveyor['source']->type)
                && $conveyor['source']->type == 'sofort'
                && $conveyor['status'] == 'pending') {
                $orderStatus = Configuration::get('STRIPE_OS_SOFORT_WAITING');
                $conveyor['result'] = 4;
            } else {
                $orderStatus = Configuration::get('PS_OS_PAYMENT');
                $conveyor['result'] = 1;
            }

            $conveyor['cart'] = $context->cart;

            try {
                $this->stripe->validateOrder(
                    (int)$conveyor['cart']->id,
                    (int)$orderStatus,
                    $paid,
                    $this->stripe->l('Payment by Stripe', 'ValidationOrderActions'),
                    $message,
                    array(),
                    null,
                    false,
                    $conveyor['secure_key']
                );
            } catch (PrestaShopException $e) {
                return false;
            }

            if ($conveyor['source']->type == 'American Express') {
                $conveyor['source']->type = 'amex';
            } elseif ($conveyor['source']->type == 'Diners Club') {
                $conveyor['source']->type = 'diners';
            }

            if (!$this->stripe->isZeroDecimalCurrency($conveyor['source']->currency)) {
                $conveyor['source']->amount /= 100;
            }

            $cardType = $conveyor['source']->type;
            if (isset($conveyor['source']->card)) {
                $cardType = $conveyor['source']->card->brand;
            }

            $stripePayment = new StripePayment();
            $stripePayment->setIdStripe($conveyor['chargeId']);
            $stripePayment->setIdPaymentIntent($conveyor['id_payment_intent']);
            $stripePayment->setName($conveyor['source']->owner->name);
            $stripePayment->setIdCart((int)$context->cart->id);
            $stripePayment->setType(Tools::strtolower($cardType));
            $stripePayment->setAmount($conveyor['amount']);
            $stripePayment->setRefund((int)0);
            $stripePayment->setCurrency(Tools::strtolower($context->currency->iso_code));
            $stripePayment->setResult((int)$conveyor['result']);
            $stripePayment->setState((int)Configuration::get('STRIPE_MODE'));
            $stripePayment->setDateAdd(date("Y-m-d H:i:s"));
            $stripePayment->save();

            if (Tools::strtolower($cardType) != 'sofort') {
                $orderId = Order::getOrderByCartId((int)$context->cart->id);
                $orderPaymentDatas = OrderPayment::getByOrderId($orderId);

                $orderPayment = new OrderPayment($orderPaymentDatas[0]->id);
                $orderPayment->transaction_id = $conveyor['chargeId'];
                $orderPayment->save();
            }

            $id_order = Order::getOrderByCartId($context->cart->id);
            $order = new Order($id_order);

            return $order;
        }
    }
}
