<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (file_exists(_PS_MODULE_DIR_ . 'tggatos/tggatos.php')) {
    require_once _PS_MODULE_DIR_ . 'tggatos/tggatos.php';

    class PrestAppTggatosModuleController
    {
        protected $tggatos;

        public function __construct()
        {
            $this->tggatos = Module::getInstanceByName('tggatos');
        }

        public function displayPayment($context)
        {
            if (!$this->tggatos->canProcess()) {
                return false;
            }

            $id_cart = $context->cart->id;
            $cart = new Cart($id_cart);

            if ($this->tggatos->get(TggAtos::CNF_SKIP_REDIRECTION_CONTROLLER)) {
                return false;
            // to do
            } else {
                $methods = array();

                $modeSingleId = TggAtos::MODE_SINGLE;
                $mode2Id = TggAtos::MODE_2TPAYMENT;
                $mode3Id = TggAtos::MODE_3TPAYMENT;
                $modeSingleData = $this->getModeData($modeSingleId, $cart, $context);
                $mode2Data = $this->getModeData($mode2Id, $cart, $context);
                $mode3Data = $this->getModeData($mode3Id, $cart, $context);

                if ($modeSingleData) {
                    array_push($methods, $modeSingleData);
                }

                if ($mode2Data) {
                    array_push($methods, $mode2Data);
                }

                if ($mode3Data) {
                    array_push($methods, $mode3Data);
                }

                return $methods;
            }
        }

        private function getModeData($mode, $cart, $context)
        {
            if (!$this->tggatos->canProcess($mode, $cart)) {
                return false;
            }

            $currency = Currency::getCurrencyInstance((int)$context->currency->id);

            $cartAmount = $cart->getOrderTotal();
            $feesAmount = $this->tggatos->getPaymentFees($cartAmount, $currency, $mode);
            $totalAmount = $cartAmount + $feesAmount;

            $form = $this->tggatos->getPaymentRedirectionForm(
                $totalAmount,
                $currency,
                $mode,
                array(
                    'customer_id' => $context->customer->id,
                    'order_id' => $context->cart->id
                )
            );

            $module_dir = str_replace('/', '\/', _MODULE_DIR_);
            // $reg_exUrl = "/(\/)([a-zA-Z0-9-_]+\/)+([a-zA-Z0-9-_]+)([.])([a-zA-Z]{2,})/";
            $reg_exUrl = "/(".$module_dir.")([a-zA-Z0-9-_]+\/)+([a-zA-Z0-9-_]+)([.])([a-zA-Z]{2,})/";
            $base_uri = _PS_BASE_URL_.__PS_BASE_URI__;
            $base_uri = Tools::substr($base_uri, 0, -1);

            if (preg_match($reg_exUrl, $form)) {
                $form = preg_replace($reg_exUrl, $base_uri."\\0", $form);
            }

            $text = $this->tggatos->l('Pay by credit card');
            $img = $base_uri._MODULE_DIR_.$this->tggatos->name."/views/img/bank_logo/{$this->tggatos->get('BANK')}.gif";

            if ($mode == 1) {
                $text = $this->tggatos->l('Pay by credit card', 'payment');
            } elseif ($mode == 2) {
                $text = $this->tggatos->l('Pay by credit card in two payments', 'payment');
            } elseif ($mode == 3) {
                $text = $this->tggatos->l('Pay by credit card in three payments', 'payment');
            }

            return array(
                'tggatos_cartAmount' => $cartAmount,
                'tggatos_feesAmount' => $feesAmount,
                'tggatos_totalAmount' => $totalAmount,
                'tggatos_mode' => $mode,
                'tggatos_text' => $text,
                'tggatos_img' => $img,
                'tggatos_paymentCurrency' => $currency,
                'tggatos_pathURI' => $this->tggatos->getPathUri(),
                'tggatos_form' => $form
            );
        }
    }
}
