<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

$app->get('/customization/ets_marketplace/test', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->test($context);
        $response = sendOk($response, $data);
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/review', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $title = $request->getParam('title');
    $content = $request->getParam('content');
    $grade = $request->getParam('grade');
    $id_order = $request->getParam('id_order');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->addSellerReview($title, $content, $grade, $id_order, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/reviews', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');
    $page = $request->getParam('page');
    $limit = $request->getParam('limit');

    if (!$page) {
        $page = 1;
    }

    if (!$limit) {
        $limit = 5;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {
        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerReviews($id_seller, $page, $limit, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/follow', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->followSeller($id_seller, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/unfollow', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->unfollowSeller($id_seller, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSeller($id_seller, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/products/[{populate}]', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');
    $tab = $request->getParam('tab');
    $page = $request->getParam('page');
    $limit = $request->getParam('limit');
    $populate = $request->getAttribute('populate');

    if (!$page) {
        $page = 1;
    }

    if (!$limit) {
        $limit = 12;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerProducts($id_seller, $tab, $page, $limit, $populate, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/report', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');
    $title = $request->getParam('title');
    $content = $request->getParam('content');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->reportSeller($id_seller, $title, $content, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/messages/notifications', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerMessagesNotifications($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/messages', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerMessagesList($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/messages/send', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_contact = $request->getParam('id_contact');
    $id_seller = $request->getParam('id_seller');
    $id_product = $request->getParam('id_product');
    $message = $request->getParam('message');
    $email = $request->getParam('email');
    $reference = $request->getParam('reference');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->sendSellerMessage($id_contact, $id_seller, $id_product, $email, $reference, $message, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/orders', function (Request $request, Response $response) {
    $context = $request->getAttribute('context');
    $id_seller = $request->getParam('id_seller');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerOrders($id_seller, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/seller/messages/contact', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_contact = $request->getParam('id_contact');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getSellerMessage($id_contact, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/seller/messages/delete', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_contact = $request->getParam('id_contact');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->deleteSellerMessage($id_contact, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/top-sellers', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $limit = $request->getParam('limit');

    if (!$limit) {
        $limit = 6;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getTopSellers($limit, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/followed-sellers', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $limit = $request->getParam('limit');

    if (!$limit) {
        $limit = 6;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getFollowedSellers($limit, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/followed-sellers-products/[{populate}]', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $limit = $request->getParam('limit');
    $populate = $request->getAttribute('populate');

    if (!$limit) {
        $limit = 6;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getFollowedSellersProducts($limit, $populate, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/popular-products/[{populate}]', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $limit = $request->getParam('limit');
    $populate = $request->getAttribute('populate');

    if (!$limit) {
        $limit = 6;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getPopularProducts($limit, $populate, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/popular-categories/[{populate}]', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $limit_categories = $request->getParam('limit_categories');
    $limit = $request->getParam('limit');
    $populate = $request->getAttribute('populate');

    if (!$limit) {
        $limit = 6;
    }

    if (!$limit_categories) {
        $limit_categories = 2;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getPopularCategories($limit, $limit_categories, $populate, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});




//////////////////////////////////////////////////////////////////////
/////////////////////////// ADMIN ////////////////////////////////////
//////////////////////////////////////////////////////////////////////

$app->get('/customization/ets_marketplace/admin/messages', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getAdminMessagesList($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/admin/messages/send', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_contact = $request->getParam('id_contact');
    $id_order = $request->getParam('id_order');
    $id_customer = $request->getParam('id_customer');
    $message = $request->getParam('message');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->sendAdminMessage($id_contact, $id_order, $id_customer, $message, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/admin/messages/contact', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_contact = $request->getParam('id_contact');
    $id_order = $request->getParam('id_order');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getAdminMessage($id_contact, $id_order, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/admin/product', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->addProduct($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/admin/manufacturer', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->addManufacturer($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->put('/customization/ets_marketplace/admin/manufacturer', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->editManufacturer($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/admin/vacation', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->setVacationMode($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->post('/customization/ets_marketplace/admin/profile', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->editProfile($context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/admin/orders', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $page = $request->getParam('page');
    $limit = $request->getParam('limit');

    if (!$page) {
        $page = 1;
    }

    if (!$limit) {
        $limit = 5;
    }

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getOrdersList($page, $limit, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});

$app->get('/customization/ets_marketplace/admin/order', function (Request $request, Response $response) {

    $context = $request->getAttribute('context');
    $id_order = $request->getParam('id_order');

    if (PrestAppCustomizationController::checkModuleAvailability('ets_marketplace')) {

        $ets_marketplace = new PrestAppEtsMarketPlaceCustomizationController();
        $data = $ets_marketplace->getOrderDetails($id_order, $context);

        if ($data && $data['status']) {
            $response = sendOk($response, $data);
        } else {
            $response = sendBadRequest($response, array(
                'code' => $data['message']
            ));
        }
    } else {
        $response = sendBadRequest($response, 'ERROR');
    }

    return $response;
});
