<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

/*
 * GET /modules/payments/payplug/display
 */
$app->get('/modules/payments/payplug/display', function (Request $request, Response $response) {
    $payplug = new PrestAppPayplugModuleController();
    $context = $request->getAttribute('context');
    if (version_compare($payplug->module_version, '2.27.0', '<')) {
        $front = $payplug->displayPayment($context);

        if (!$front['is_active']) {
            $response = sendNotFound($response, 'NOT_FOUND');
        } else {
            $data = array(
                'data' => $front,
            );

            $response = sendOk($response, $data);
        }
    } elseif (version_compare($payplug->module_version, '3.5.0', '>=')) {
        $front = $payplug->displayPaymentSupEqual350($context);
        if (!$front) {
            $response = sendNotFound($response, 'NOT_FOUND');
        } else {
            $data = array(
                'options' => $front,
                'isMulti' => true,
            );
            $response = sendOk($response, $data);
        }
    } elseif (version_compare($payplug->module_version, '2.27.0', '>')) {
        $front = $payplug->displayPaymentSup2270($context);
        if (!$front) {
            $response = sendNotFound($response, 'NOT_FOUND');
        } else {
            $data = array(
                'options' => $front,
                'isMulti' => true,
            );
            $response = sendOk($response, $data);
        }
    } else {
        $front = $payplug->displayPayment2270($context);
        $data = array(
            'options' => $front,
            'isMulti' => true,
        );
        $response = sendOk($response, $data);
    }

    return $response;
});

$app->post('/modules/payments/payplug/url', function (Request $request, Response $response) {
    $payplug = new PrestAppPayplugModuleController();
    $context = $request->getAttribute('context');
    $id_card = $request->getParam('id_card');

    if (version_compare($payplug->module_version, '2.27.0', '<')) {
        $front = $payplug->getPaymentUrl($context, $id_card);
    } elseif (version_compare($payplug->module_version, '3.12.0', '>=')) {
        $pc = $request->getParam('pc');
        $pay = $request->getParam('pay');
        $id_cart = $request->getParam('id_cart');
        $method = $request->getParam('method');
        $def = $request->getParam('def');

        $front = $payplug->getPaymentUrlSupEqual3120($context, $pc, $pay, $id_cart, $method, $def);
    } elseif (version_compare($payplug->module_version, '3.5.0', '>=')) {
        $pc = $request->getParam('pc');
        $pay = $request->getParam('pay');
        $id_cart = $request->getParam('id_cart');
        $method = $request->getParam('method');
        $def = $request->getParam('def');

        $front = $payplug->getPaymentUrlSupEqual350($context, $pc, $pay, $id_cart, $method, $def);
    } elseif (version_compare($payplug->module_version, '2.27.0', '>')) {
        $pc = $request->getParam('pc');
        $pay = $request->getParam('pay');
        $id_cart = $request->getParam('id_cart');
        $method = $request->getParam('method');
        $def = $request->getParam('def');

        $front = $payplug->getPaymentUrlSup2270($context, $pc, $pay, $id_cart, $method, $def);
    } else {
        $pc = $request->getParam('pc');
        $pay = $request->getParam('pay');
        $id_cart = $request->getParam('id_cart');
        $method = $request->getParam('method');
        $def = $request->getParam('def');

        $front = $payplug->getPaymentUrl2270($context, $pc, $pay, $id_cart, $method, $def);
    }

    if (!$front) {
        $response = sendNotFound($response, 'NOT_FOUND');
    } else {
        $data = array(
            'data' => $front,
        );

        $response = sendOk($response, $data);
    }

    return $response;
});

$app->get('/modules/payments/payplug/cards', function (Request $request, Response $response) {
    $payplug = new PrestAppPayplugModuleController();
    $context = $request->getAttribute('context');

    if (version_compare($payplug->module_version, '3.5.0', '>=')) {
        $front = $payplug->getCardsListSupEqual350($context);
    } else {
        $front = $payplug->getCardsList($context);
    }

    $data = array(
        'data' => $front,
    );

    $response = sendOk($response, $data);

    return $response;
});

$app->post('/modules/payments/payplug/deletecard', function (Request $request, Response $response) {
    $payplug = new PrestAppPayplugModuleController();
    $context = $request->getAttribute('context');
    $id_card = $request->getParam('id_card');

    if (version_compare($payplug->module_version, '3.5.0', '>=')) {
        $front = $payplug->deleteCardSupEqual350($context, $id_card);
    } else {
        $front = $payplug->deleteCard($context, $id_card);
    }

    $data = array(
        'data' => $front,
    );

    $response = sendOk($response, $data);

    return $response;
});
