<?php

class prestappAdyenSchemeModuleFrontController extends FrontController
{

    public $type_name = 'prestapp_adyen_scheme';
    protected $adyenofficial;
    private $currencyUtil;
    private $languageAdapter;
    private $versionChecker;
    private $configuration;
    private $helper_data;
    private $countryAdapter;

    public function __construct()
    {
        parent::__construct();
        $this->adyenofficial = Module::getInstanceByName('adyenofficial');
        $this->currencyUtil = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            '\Adyen\Util\Currency'
        );
        $this->languageAdapter = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            'Adyen\PrestaShop\service\adapter\classes\Language'
        );
        $this->versionChecker = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            'Adyen\PrestaShop\application\VersionChecker'
        );
        $this->configuration = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            'Adyen\PrestaShop\service\adapter\classes\Configuration'
        );
        $this->helper_data = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            'Adyen\PrestaShop\helper\Data'
        );
        $this->countryAdapter = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get(
            'Adyen\PrestaShop\service\adapter\classes\Country'
        );
    }

    public function init()
    {
        $this->display_header = false; //hides header;
        $this->display_footer = false; // hides footer;

        $this->setMedia();

        parent::init();

        $this->context->cookie->id_cart = $_GET["cart_id"];
        $this->context->cookie->id_customer = $_GET["customer_id"];
    }

    public function initContent()
    {
        parent::initContent();

        $controller = $this->context->controller;
        $this->registerAdyenAssets($controller);

        $smartyVariables = $this->getCheckoutComponentInitData();
        $this->context->smarty->assign($smartyVariables);

        if($_GET["storedPaymentApiId"]) {
            $this->context->smarty->assign(array(
                'number' => $_GET["lastFour"],
                'name' => $_GET["paymentMethodName"],
                'logoBrand' => $_GET["brand"],
                'storedPaymentApiId' => $_GET["storedPaymentApiId"]
            ));
        } else {
            $this->context->smarty->assign(array(
                'paymentMethodType' => $_GET["paymentMethodType"],
                'paymentMethodName' => $_GET["paymentMethodName"],
                'paymentMethodBrand' => $_GET["paymentMethodBrand"],
            ));
        }

        $this->displayPaymentTopTpl();

        if($_GET["storedPaymentApiId"]) {
            $this->setTemplate(_PS_MODULE_DIR_ . 'prestapp/views/templates/front/adyenofficial/stored-payment-method.tpl');
        } else {
            $this->setTemplate(_PS_MODULE_DIR_ . 'prestapp/views/templates/front/adyenofficial/payment-method.tpl');
        }
    }

    private function displayPaymentTopTpl() {

        $paymentMethods = $this->getPaymentMethods();

        $cart = new Cart($_GET["cart_id"]);

        $selectedDeliveryAddressId = null;
        if ($cart->id_address_delivery) {
            $selectedDeliveryAddressId = $cart->id_address_delivery;
        }

        $selectedInvoiceAddressId = $selectedDeliveryAddressId;
        if ($cart->id_address_invoice) {
            $selectedInvoiceAddressId = $cart->id_address_invoice;
        }

        $selectedInvoiceAddressArray = array();

        /** @var AddressCore $selectedInvoiceAddress */
        $selectedInvoiceAddress = AddressCore::initialize($selectedInvoiceAddressId);
        if (\Validate::isLoadedObject($selectedInvoiceAddress)) {
            // Format the address in a way that frontend can use it
            $selectedInvoiceAddressArray = array(
                'city' => $selectedInvoiceAddress->city,
                'country' => $this->countryAdapter->getIsoById($selectedInvoiceAddress->id_country),
                'houseNumberOrName' => $selectedInvoiceAddress->address2,
                'postalCode' => $selectedInvoiceAddress->postcode,
                'street' => $selectedInvoiceAddress->address1
            );
        }

        $smartyVariables = array(
            'paymentMethodsResponse' => json_encode($paymentMethods),
            'selectedDeliveryAddressId' => $selectedDeliveryAddressId,
            'selectedInvoiceAddressId' => $selectedInvoiceAddressId,
            'selectedInvoiceAddress' => json_encode($selectedInvoiceAddressArray)
        );

        // Add checkout component default configuration parameters for smarty variables
        $smartyVariables = array_merge($smartyVariables, $this->getCheckoutComponentInitData());

        $js_files = array();

        foreach($this->js_files as $js_file) {
            if(
                strpos($js_file, 'global.js') === false &&
                strpos($js_file, 'ybc_instagram/views/js/instagram.js') === false &&
                strpos($js_file, 'eo-theme/js/eo.js') === false &&
                strpos($js_file, 'voyado/views/js/front.js') === false
            ) {
                $js_files[] = $js_file;
            }
        }

        // Assign variables to frontend
        $this->context->smarty->assign($smartyVariables);
        $this->context->smarty->assign(array(
            'css_files'      => $this->css_files,
            'js_def'         => false,
            'new_js_files'       => ($this->getLayout() && (bool)Configuration::get('PS_JS_DEFER')) ? array() : $js_files,
            'js_defer'       => (bool)Configuration::get('PS_JS_DEFER'),
        ));

        $main_tpl = $this->context->smarty->fetch(_PS_MODULE_DIR_ . $this->adyenofficial->name . '/views/templates/front/adyencheckout.tpl');

        $this->context->smarty->assign(array(
            'main_tpl' => $main_tpl
        ));
    }

    private function getCheckoutComponentInitData()
    {
        $cart = new Cart($_GET["cart_id"]);
        $customer = new Customer($_GET["customer_id"]);
        $language = new Language($_GET["language_id"]);

        $currencyData = Currency::getCurrency($cart->id_currency);

        $currencyIsoCode = '';
        if (!empty($currencyData['iso_code'])) {
            $currencyIsoCode = $currencyData['iso_code'];
        }

        $totalAmountInMinorUnits = $this->currencyUtil->sanitize(
            $cart->getOrderTotal(),
            $currencyIsoCode
        );

        // List of payment methods that needs to show the pay button from the component
        $paymentMethodsWithPayButtonFromComponent = json_encode(array('paywithgoogle', 'applepay', 'paypal'));

        // All payment method specific configuration
        $paymentMethodsConfigurations = json_encode(
            array(
                'applePayMerchantName' => Configuration::get('ADYEN_APPLE_PAY_MERCHANT_NAME'),
                'applePayMerchantIdentifier' => Configuration::get('ADYEN_APPLE_PAY_MERCHANT_IDENTIFIER'),
                'googlePayGatewayMerchantId' => Configuration::get('ADYEN_GOOGLE_PAY_GATEWAY_MERCHANT_ID'),
                'googlePayMerchantIdentifier' => Configuration::get('ADYEN_GOOGLE_PAY_MERCHANT_IDENTIFIER')
            )
        );

        return array(
            'locale' => $this->languageAdapter->getLocaleCode($language),
            'clientKey' => $this->configuration->clientKey,
            'environment' => Configuration::get('ADYEN_MODE'),
            'isUserLoggedIn' => !$customer->is_guest,
            'paymentProcessUrl' => $this->context->link->getModuleLink($this->adyenofficial->name, 'Payment', array(), true),
            'paymentsDetailsUrl' => $this->context->link->getModuleLink($this->adyenofficial->name, 'PaymentsDetails', array(), true),
            'isPrestaShop16' => $this->versionChecker->isPrestaShop16() ? true : false,
            'currencyIsoCode' => $currencyIsoCode,
            'totalAmountInMinorUnits' => $totalAmountInMinorUnits,
            'paymentMethodsConfigurations' => $paymentMethodsConfigurations,
            'paymentMethodsWithPayButtonFromComponent' => $paymentMethodsWithPayButtonFromComponent,
            'enableStoredPaymentMethods' => Configuration::get('ADYEN_ENABLE_STORED_PAYMENT_METHODS') ? true : false
        );
    }

    private function registerAdyenAssets($controller)
    {
        $controller = $this->context->controller;
        $controllerAdapter = \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get('Adyen\PrestaShop\service\adapter\classes\Controller');
        $controllerAdapter->setController($controller);

        $controllerAdapter->registerJavascript(
            'adyen-checkout-component', // Unique ID
            'modules/' . $this->adyenofficial->name . '/views/js/bundle.js', // JS path
            array('position' => 'bottom', 'priority' => 150) // Arguments
        );

        $controllerAdapter->registerJavascript(
            'adyen-polyfill',
            'modules/' . $this->adyenofficial->name . '/views/js/polyfill.js',
            array('position' => 'bottom', 'priority' => 140)
        );

        $controllerAdapter->registerJavascript(
            'adyen-component-renderer',
            'modules/' . $this->adyenofficial->name . '/views/js/checkout-component-renderer.js',
            array('position' => 'bottom', 'priority' => 170)
        );

        $enableAdyenCss = Configuration::get('ADYEN_ENABLE_CHECKOUT_STYLING');

        if ($enableAdyenCss) {
            $controllerAdapter->registerStylesheet(
                'adyen-adyencss',
                'modules/' . $this->adyenofficial->name . '/views/css/adyen.css'
            );
        }

        if ($this->versionChecker->isPrestaShop16()) {
            $controller->addJqueryPlugin('fancybox');
        }
    }

    private function getService($serviceName)
    {
        return \Adyen\PrestaShop\service\adapter\classes\ServiceLocator::get($serviceName);
    }

    private function getPaymentMethods()
    {
        $cart = new Cart($_GET["cart_id"]);
        $language = new Language($_GET["language_id"]);

        $paymentMethods = $this->helper_data->fetchPaymentMethods(
            $cart,
            $language
        );

        return $paymentMethods;
    }
}