<?php

use PrestaShop\Module\PrestashopCheckout\Environment\PaypalEnv;

class prestappPsCheckoutModuleFrontController extends FrontController
{

    const BASE_LINK = 'https://www.paypal.com/sdk/js';

    public $type_name = 'prestapp_ps_checkout';
    protected $ps_checkout;

    public function __construct()
    {
        parent::__construct();
        $this->ps_checkout = Module::getInstanceByName('ps_checkout');
    }

    public function init()
    {
        $this->display_header = false; //hides header;
        $this->display_footer = false; // hides footer;
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $this->template = _PS_MODULE_DIR_ . 'prestapp/views/templates/front/placeholder.tpl';
        }

        $this->setMedia();

        parent::init();
    }

    public function initContent()
    {
        parent::initContent();
        $this->addMedias();

        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $this->context->smarty->assign(array(
                'page_id' => 'order',
                'is17' => false
            ));
        } else {
            $this->context->smarty->assign(array(
                'page_id' => 'checkout',
                'is17' => true
            ));
        }

        $paymentOptionName = $_GET["paymentOptionName"];
        $fundingSource = $_GET["fundingSource"];

        $this->context->smarty->assign('js_def', 'js_def');

        if (method_exists($this->context->controller, 'registerJavascript')) {
            $this->context->controller->registerJavascript(
                $this->ps_checkout->name . 'Front',
                '/modules/' . $this->ps_checkout->name . '/views/js/front.js?version=' . $this->ps_checkout->version,
                [
                    'position' => 'bottom',
                    'priority' => 201,
                    'server' => 'remote',
                ]
            );
        } else {
            $this->context->controller->addJS(
                '/modules/' . $this->ps_checkout->name . '/views/js/front.js?version=' . $this->ps_checkout->version,
                false
            );
        }

        if (method_exists($this->context->controller, 'registerStylesheet')) {
            $this->context->controller->registerStylesheet(
                'ps-checkout-css-paymentOptions',
                '/modules/' . $this->ps_checkout->name . '/views/css/payments.css?version=' . $this->ps_checkout->version,
                [
                    'server' => 'remote',
                ]
            );
        } else {
            $this->context->controller->addCss(
                '/modules/' . $this->ps_checkout->name . '/views/css/payments16.css?version=' . $this->ps_checkout->version,
                'all',
                null,
                false
            );
        }

        $js_files = array();
        $css_files = array();

        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            foreach($this->js_files as $js_file) {
                if(strpos($js_file, 'global.js') === false){
                    $js_files[] = $js_file;
                }
            } 
            $css_files = $this->css_files;
        } else {
            $javascript = $this->getJavascript();
            foreach($javascript['bottom']['external'] as $js_file) {
                if(strpos($js_file['uri'], '/verifycustomer/views/js/ps17/vcGlobal.js') === false) {
                    $js_files[] = $js_file['uri'];
                }
            }
            $stylesheets = $this->getStylesheets();
            foreach($stylesheets['external'] as $css_file) {
                $css_files[$css_file['uri']] = $css_file['media'];
            }
        }

        $this->context->smarty->assign(array(
            'css_files'      => $css_files,
            // 'js_def'         => Media::getJsDef(),
            'new_js_files'       => ($this->getLayout() && (bool)Configuration::get('PS_JS_DEFER')) ? array() : $js_files,
            'js_defer'       => (bool)Configuration::get('PS_JS_DEFER'),
        ));

        $paypalAccountRepository = $this->ps_checkout->getService('ps_checkout.repository.paypal.account');

        $this->context->smarty->assign(array(
            'paymentOptionName' => $paymentOptionName,
            'fundingSource' => $fundingSource,
            'shoppingCartWarningPath' => $this->ps_checkout->getPathUri() . 'views/img/shopping-cart-warning.svg',
            'spinnerPath' => $this->ps_checkout->getPathUri() . 'views/img/tail-spin.svg',
            'isHostedFieldsAvailable' => $paypalAccountRepository->cardHostedFieldsIsAvailable(),
            'modulePath' => $this->ps_checkout->getPathUri(),
            'warningTranslatedText' => $this->ps_checkout->l('Warning'),
            'loaderTranslatedText' => $this->ps_checkout->l('Please wait, loading additional payment methods.'),
        ));

        $html = $this->context->smarty->fetch(_PS_MODULE_DIR_ . 'prestapp/views/templates/front/ps_checkout/payment.tpl');

        $dom_available = extension_loaded('dom') ? true : false;
        $defer = (bool)Configuration::get('PS_JS_DEFER');
        if ($defer && $dom_available) {
            $html = Media::deferInlineScripts($html);
        }
        $html = trim(str_replace(array('</body>', '</html>'), '', $html))."\n";

        $this->context->smarty->assign(array(
            'js_def' => Media::getJsDef()
        ));

        $javascript = $this->context->smarty->fetch(_PS_ALL_THEMES_DIR_.'javascript.tpl');

        if ($defer && (!isset($this->ajax) || ! $this->ajax)) {
            echo $html.$javascript;
        } else {
            echo preg_replace('/(?<!\$)'.'js_def'.'/', $javascript, $html);
        }

        // $this->setTemplate(_PS_MODULE_DIR_ . 'prestapp/views/templates/front/ps_checkout/payment.tpl');
    }

    private function addMedias()
    {
        // handle translations

        $this->context->smarty->assign(array('translations' => array(
            'I confirm my order' => $this->ps_checkout->l('I confirm my order', 'ps_checkout')
        )));

        $cart = new Cart((int) $_GET["cart_id"]);

        $this->context->cookie->id_cart = $_GET["cart_id"];
        $this->context->cookie->id_customer = $cart->id_customer;

        $payPalSdkLinkBuilder = $this->ps_checkout->getService('ps_checkout.sdk.paypal.linkbuilder');
        $paypalAccountRepository = $this->ps_checkout->getService('ps_checkout.repository.paypal.account');
        $expressCheckoutConfiguration = $this->ps_checkout->getService('ps_checkout.express_checkout.configuration');
        $payPalConfiguration = $this->ps_checkout->getService('ps_checkout.paypal.configuration');
        $fundingSourceProvider = $this->ps_checkout->getService('ps_checkout.funding_source.provider');
        try {
            $payLaterConfiguration = $this->ps_checkout->getService('ps_checkout.pay_later.configuration');
        } catch (Exception $e) {

        }

        $fundingSourcesSorted = [];
        $payWithTranslations = [];
        $isCardAvailable = false;

        foreach ($fundingSourceProvider->getAll() as $fundingSource) {
            $fundingSourcesSorted[] = $fundingSource->name;
            $payWithTranslations[$fundingSource->name] = $fundingSource->label;

            if ('card' === $fundingSource->name) {
                $isCardAvailable = $fundingSource->isEnabled;
            }
        }

        $payPalClientToken = '';
        $payPalOrderId = '';
        $cartFundingSource = 'paypal';
        $psCheckoutCart = false;
        $cartProductCount = 0;

        if (Validate::isLoadedObject($cart)) {
            $cartProductCount = (int) $cart->nbProducts();
            $psCheckoutCartRepository = $this->ps_checkout->getService('ps_checkout.repository.pscheckoutcart');
            $psCheckoutCart = $psCheckoutCartRepository->findOneByCartId((int) $cart->id);
        }

        if (false !== $psCheckoutCart
            && false === empty($psCheckoutCart->paypal_order)
            && in_array($psCheckoutCart->paypal_status, [PsCheckoutCart::STATUS_CREATED, PsCheckoutCart::STATUS_APPROVED], true)
            && false === empty($psCheckoutCart->paypal_token_expire)
            && strtotime($psCheckoutCart->paypal_token_expire) > time()
        ) {
            $payPalOrderId = $psCheckoutCart->paypal_order;
            $payPalClientToken = $psCheckoutCart->paypal_token;
            $cartFundingSource = $psCheckoutCart->paypal_funding;
        }

        Media::addJsDef(array_merge(isset($payLaterConfiguration) ? [
            $this->ps_checkout->name . 'PayLaterProductPageMessageEnabled' => $payLaterConfiguration->isProductPageMessageActive(),
            $this->ps_checkout->name . 'PayLaterOrderPageMessageEnabled' => $payLaterConfiguration->isOrderPageMessageActive(),
            $this->ps_checkout->name . 'PayLaterHomePageBannerEnabled' => $payLaterConfiguration->isHomePageBannerActive(),
            $this->ps_checkout->name . 'PayLaterCategoryPageBannerEnabled' => $payLaterConfiguration->isCategoryPageBannerActive(),
            $this->ps_checkout->name . 'PayLaterProductPageBannerEnabled' => $payLaterConfiguration->isProductPageBannerActive(),
            $this->ps_checkout->name . 'PayLaterOrderPageBannerEnabled' => $payLaterConfiguration->isOrderPageBannerActive(),
            $this->ps_checkout->name . 'PayLaterProductPageButtonEnabled' => $payLaterConfiguration->isProductPageButtonActive(),
            $this->ps_checkout->name . 'PayLaterCartPageButtonEnabled' => $payLaterConfiguration->isCartPageButtonActive(),
            $this->ps_checkout->name . 'PayLaterOrderPageButtonEnabled' => $payLaterConfiguration->isOrderPageButtonActive(),
            ] : []), [
            $this->ps_checkout->name . 'Version' => $this->ps_checkout->version,
            $this->ps_checkout->name . 'AutoRenderDisabled' => (bool) Configuration::get('PS_CHECKOUT_AUTO_RENDER_DISABLED'),
            $this->ps_checkout->name . 'LoaderImage' => $this->ps_checkout->getPathUri() . 'views/img/loader.svg',
            $this->ps_checkout->name . 'PayPalButtonConfiguration' => $payPalConfiguration->getButtonConfiguration(),
            $this->ps_checkout->name . 'CardFundingSourceImg' => Media::getMediaPath(_PS_MODULE_DIR_ . $this->ps_checkout->name . '/views/img/payment-cards.png'),
            $this->ps_checkout->name . 'GetTokenURL' => $this->context->link->getModuleLink($this->ps_checkout->name, 'token', [], true),
            $this->ps_checkout->name . 'CreateUrl' => $this->context->link->getModuleLink($this->ps_checkout->name, 'create', [], true),
            $this->ps_checkout->name . 'CheckUrl' => $this->context->link->getModuleLink($this->ps_checkout->name, 'check', [], true),
            $this->ps_checkout->name . 'ValidateUrl' => $this->context->link->getModuleLink($this->ps_checkout->name, 'validate', [], true),
            $this->ps_checkout->name . 'CancelUrl' => $this->context->link->getModuleLink($this->ps_checkout->name, 'cancel', [], true),
            $this->ps_checkout->name . 'ExpressCheckoutUrl' => $this->context->link->getModuleLink($this->ps_checkout->name, 'ExpressCheckout', [], true),
            $this->ps_checkout->name . 'CheckoutUrl' => $this->getCheckoutPageUrl(),
            $this->ps_checkout->name . 'ConfirmUrl' => $this->context->link->getPageLink('order-confirmation', true, (int) $this->context->language->id),
            $this->ps_checkout->name . 'PayPalSdkUrl' => $this->buildLink(),
            $this->ps_checkout->name . 'PayPalClientToken' => $payPalClientToken,
            $this->ps_checkout->name . 'PayPalOrderId' => $payPalOrderId,
            $this->ps_checkout->name . 'FundingSource' => $cartFundingSource,
            $this->ps_checkout->name . 'HostedFieldsEnabled' => $isCardAvailable && $payPalConfiguration->isCardPaymentEnabled() && $paypalAccountRepository->cardHostedFieldsIsAllowed(),
            $this->ps_checkout->name . 'HostedFieldsSelected' => false !== $psCheckoutCart ? (bool) $psCheckoutCart->isHostedFields : false,
            $this->ps_checkout->name . 'ExpressCheckoutSelected' => false !== $psCheckoutCart ? (bool) $psCheckoutCart->isExpressCheckout : false,
            $this->ps_checkout->name . 'ExpressCheckoutProductEnabled' => $expressCheckoutConfiguration->isProductPageEnabled(),
            $this->ps_checkout->name . 'ExpressCheckoutCartEnabled' => $expressCheckoutConfiguration->isOrderPageEnabled(),
            $this->ps_checkout->name . 'ExpressCheckoutOrderEnabled' => $expressCheckoutConfiguration->isCheckoutPageEnabled(),
            $this->ps_checkout->name . '3dsEnabled' => $payPalConfiguration->is3dSecureEnabled(),
            $this->ps_checkout->name . 'CspNonce' => $payPalConfiguration->getCSPNonce(),
            $this->ps_checkout->name . 'CartProductCount' => $cartProductCount,
            $this->ps_checkout->name . 'FundingSourcesSorted' => $fundingSourcesSorted,
            $this->ps_checkout->name . 'PayWithTranslations' => $payWithTranslations,
            $this->ps_checkout->name . 'CheckoutTranslations' => [
                'checkout.go.back.link.title' => $this->ps_checkout->l('Go back to the Checkout'),
                'checkout.go.back.label' => $this->ps_checkout->l('Checkout'),
                'checkout.card.payment' => $this->ps_checkout->l('Card payment'),
                'checkout.page.heading' => $this->ps_checkout->l('Order summary'),
                'checkout.cart.empty' => $this->ps_checkout->l('Your shopping cart is empty.'),
                'checkout.page.subheading.card' => $this->ps_checkout->l('Card'),
                'checkout.page.subheading.paypal' => $this->ps_checkout->l('PayPal'),
                'checkout.payment.by.card' => $this->ps_checkout->l('You have chosen to pay by Card.'),
                'checkout.payment.by.paypal' => $this->ps_checkout->l('You have chosen to pay by PayPal.'),
                'checkout.order.summary' => $this->ps_checkout->l('Here is a short summary of your order:'),
                'checkout.order.amount.total' => $this->ps_checkout->l('The total amount of your order comes to'),
                'checkout.order.included.tax' => $this->ps_checkout->l('(tax incl.)'),
                'checkout.order.confirm.label' => $this->ps_checkout->l('Please confirm your order by clicking "I confirm my order".'),
                'paypal.hosted-fields.label.card-number' => $this->ps_checkout->l('Card number'),
                'paypal.hosted-fields.placeholder.card-number' => $this->ps_checkout->l('Card number'),
                'paypal.hosted-fields.label.expiration-date' => $this->ps_checkout->l('Expiry date'),
                'paypal.hosted-fields.placeholder.expiration-date' => $this->ps_checkout->l('MM/YY'),
                'paypal.hosted-fields.label.cvv' => $this->ps_checkout->l('CVC'),
                'paypal.hosted-fields.placeholder.cvv' => $this->ps_checkout->l('XXX'),
                'express-button.cart.separator' => $this->ps_checkout->l('or'),
                'express-button.checkout.express-checkout' => $this->ps_checkout->l('Express Checkout'),
                'error.paypal-sdk' => $this->ps_checkout->l('No PayPal Javascript SDK Instance'),
                'checkout.payment.others.link.label' => $this->ps_checkout->l('Other payment methods'),
                'checkout.payment.others.confirm.button.label' => $this->ps_checkout->l('I confirm my order'),
                'checkout.form.error.label' => $this->ps_checkout->l('There was an error during the payment. Please try again or contact the support.'),
                'loader-component.label.header' => $this->ps_checkout->l('Thanks for your purchase!'),
                'loader-component.label.body' => $this->ps_checkout->l('Please wait, we are processing your payment'),
                'error.paypal-sdk.contingency.cancel' => $this->ps_checkout->l('Card holder authentication canceled, please choose another payment method or try again.'),
                'error.paypal-sdk.contingency.error' => $this->ps_checkout->l('An error occurred on card holder authentication, please choose another payment method or try again.'),
                'error.paypal-sdk.contingency.failure' => $this->ps_checkout->l('Card holder authentication failed, please choose another payment method or try again.'),
                'error.paypal-sdk.contingency.unknown' => $this->ps_checkout->l('Card holder authentication cannot be checked, please choose another payment method or try again.'),
            ],
        ]);
    }

    private function getCheckoutPageUrl()
    {
        /** @var \PrestaShop\Module\PrestashopCheckout\ShopContext $shopContext */
        $shopContext = $this->ps_checkout->getService('ps_checkout.context.shop');

        if ($shopContext->isShop17()) {
            return $this->context->link->getPageLink(
                'order',
                true,
                (int) $this->context->language->id
            );
        }

        // PrestaShop 1.6 legacy native one page checkout
        if (1 === (int) Configuration::get('PS_ORDER_PROCESS_TYPE')) {
            return $this->context->link->getPageLink(
                'order-opc',
                true,
                (int) $this->context->language->id
            );
        }

        // PrestaShop 1.6 standard checkout
        return $this->context->link->getPageLink(
            'order',
            true,
            (int) $this->context->language->id,
            [
                'step' => 1,
            ]
        );
    }

    private function buildLink()
    {
        $components = [
            'marks',
            'funding-eligibility',
        ];

        $components[] = 'buttons';
        $components[] = 'hosted-fields';

        $paypalAccountRepository = $this->ps_checkout->getService('ps_checkout.repository.paypal.account');
        $payPalConfiguration = $this->ps_checkout->getService('ps_checkout.paypal.configuration');

        $params = [
            'components' => implode(',', $components),
            'client-id' => (new PaypalEnv())->getPaypalClientId(),
            'merchant-id' => $paypalAccountRepository->getMerchantId(),
            'currency' => \Context::getContext()->currency->iso_code,
            'intent' => strtolower($payPalConfiguration->getIntent()),
            'commit' => 'true',
            'vault' => 'false',
            'integration-date' => $payPalConfiguration->getIntegrationDate(),
        ];

        if ('SANDBOX' === $payPalConfiguration->getPaymentMode()) {
            $params['debug'] = 'true';
        }

        $fundingSourcesDisabled = $this->getFundingSourcesDisabled();

        if (false === empty($fundingSourcesDisabled)) {
            $params['disable-funding'] = implode(',', $fundingSourcesDisabled);
        }

        if ($this->isPayLaterEnabled()) {
            $params['enable-funding'] = 'paylater';
        }

        return self::BASE_LINK . '?' . urldecode(http_build_query($params));
    }

    private function getFundingSourcesDisabled()
    {
        $fundingSourcesDisabled = [];
        $fundingSourceConfigurationRepository = $this->ps_checkout->getService('ps_checkout.funding_source.configuration.repository');
        $fundingSources = $fundingSourceConfigurationRepository->getAll();

        if (empty($fundingSources)) {
            return $fundingSourcesDisabled;
        }

        foreach ($fundingSources as $fundingSource) {
            if (!$fundingSource['active']) {
                $fundingSourcesDisabled[] = $fundingSource['name'];
            }
        }

        return $fundingSourcesDisabled;
    }

    private function isPayLaterEnabled()
    {
        $fundingSourceConfigurationRepository = $this->ps_checkout->getService('ps_checkout.funding_source.configuration.repository');
        $payLaterConfig = $fundingSourceConfigurationRepository->get('paylater');

        return $payLaterConfig === null || !empty($payLaterConfig) && (int) $payLaterConfig['active'] === 1;
    }
}