<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to a commercial license from Aquil'App.
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL is strictly forbidden.
 * In order to obtain a license, please contact us: contact@prestapp.eu
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe Aquil'App.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la Aquil'App est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter Aquil'App a l'adresse: contact@prestapp.eu
 * ...........................................................................
 *
 * @author    Maxime Léon <maxime.leon@prestapp.eu>
 * @copyright Copyright (c) 2017 - 2019 Aquil'App 2 Rue D'espalion 44800 Saint-Herblain France
 * @license   Commercial license
 * Support by mail  :  contact@prestapp.eu
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Prestapp extends PaymentModule
{
    protected $api_url;

    public function __construct()
    {
        $this->name = 'prestapp';
        $this->tab = 'mobile';
        $this->version = '1.7.3';
        $this->author = 'Aquil\'App';
        $this->need_instance = 0;
        $this->module_key = '9341195e16d310293a30a7286ab73c37';
        $this->author_address = '0x3EC17b06Ad9d217884d8CAF70d25cc8454b3E81C';
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('PrestApp - Create your own mobile app and mobile first PWA');
        $this->description = $this->l('Increase your business by offering to your customers an optimized mobile and tablet experience. Create your mobile app and/or your Progressive Web App now with PrestApp!');
        $this->api_url = Tools::getHttpHost(true) . __PS_BASE_URI__ . 'modules/prestapp/api/';

        /* Backward compatibility */
        if (_PS_VERSION_ < '1.5') {
            require _PS_MODULE_DIR_ . $this->name . '/backward_compatibility/backward.php';
        }
    }

    public function install()
    {
        $length = 16;
        $token = bin2hex(openssl_random_pseudo_bytes($length));
        $secret_key = bin2hex(openssl_random_pseudo_bytes($length));

        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        if (!parent::install() ||
            !$this->registerHook('displayHeader') ||
            !$this->registerHook('actionOrderStatusUpdate') ||
            !$this->registerHook('actionValidateOrder') ||
            !$this->registerHook('displayBeforeRedirect') ||
            !Configuration::updateValue('PRESTAPP_API_KEY', 't_' . $token) ||
            !Configuration::updateValue('PRESTAPP_SECRET_KEY', 'sk_' . $secret_key) ||
            !Configuration::updateValue('PRESTAPP_REDIRECTION', 'NO_REDIRECTION') ||
            !Configuration::updateValue('PRESTAPP_REDIRECTBOTS', '0') ||
            !Configuration::updateValue('PRESTAPP_NOINDEX', '1')
        ) {
            return false;
        } else {
            return true;
        }
    }

    public function uninstall()
    {
        return parent::uninstall();
    }

    /**
     * Load the configuration form.
     */
    public function getContent()
    {

        $this->context->controller->addJS($this->_path . 'views/js/back.js');
        $protocol = Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://';

        $status = array();
        $isError = false;

        $version = explode('.', PHP_VERSION);
        $php_version = $version[0] . '.' . $version[1];
        $php_version_numeric = $version[0] * 10000 + $version[1] * 100 + (is_numeric($version[2]) ? $version[2] : 0);

        // Check PHP version
        if (!defined('PHP_VERSION_ID')) {
            define('PHP_VERSION_ID', $php_version_numeric);
        }

        if (!function_exists('curl_version')) {
            $isError = true;
            $status['curl'] = "CURL_DISABLED";
        } else {
            $curl = curl_init();

            //send the event and say to our app that the order changed status
            curl_setopt_array($curl, array(
                CURLOPT_URL => $this->api_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => "GET",
                CURLOPT_POSTFIELDS => "",
                CURLOPT_HTTPHEADER => array(
                    "Authorization: " . Configuration::get('PRESTAPP_API_KEY'),
                    "User-Agent: PrestApp PrestaShop Module",
                ),
            ));

            $response = curl_exec($curl);
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $err = curl_error($curl);
            curl_close($curl);

            if (!isset($response) || $httpcode !== 200 || $err !== "") {
                $status['response_code'] = "NO_CONTACT_WITH_OUR_API";
            }

            //server error
            if ($httpcode == 500) {
                $data = json_decode($response);
                if ($data) {
                    if ($data->mod_rewrite) {
                        $isError = true;
                        $status['mod_rewrite'] = $data->mod_rewrite;
                    }
                    if ($data->mod_headers) {
                        $isError = true;
                        $status['mod_headers'] = $data->mod_headers;
                    }
                    if ($data->php) {
                        $isError = true;
                        $status['php'] = $data->php;
                    }
                    if ($data->prestashop) {
                        $isError = true;
                        $status['prestashop'] = $data->prestashop;
                    }
                }
            }
        }

        $prestapp_paypal_error = null;

        if (Tools::isSubmit('submitPrestAppPaypal') && Module::isEnabled('paypal')) {
            $PRESTAPP_PAYPAL_SECRET_KEY = (string) Tools::getValue('PRESTAPP_PAYPAL_SECRET_KEY');
            $PRESTAPP_PAYPAL_CLIENT_ID = (string) Tools::getValue('PRESTAPP_PAYPAL_CLIENT_ID');

            if (!$PRESTAPP_PAYPAL_SECRET_KEY ||
                empty($PRESTAPP_PAYPAL_SECRET_KEY) ||
                !Validate::isGenericName($PRESTAPP_PAYPAL_SECRET_KEY) ||
                !$PRESTAPP_PAYPAL_CLIENT_ID ||
                empty($PRESTAPP_PAYPAL_CLIENT_ID) ||
                !Validate::isGenericName($PRESTAPP_PAYPAL_CLIENT_ID)
            ) {
                $prestapp_paypal_error = true;
            } else {
                $prestapp_paypal_error = false;
                Configuration::updateValue('PRESTAPP_PAYPAL_SECRET_KEY', $PRESTAPP_PAYPAL_SECRET_KEY);
                Configuration::updateValue('PRESTAPP_PAYPAL_CLIENT_ID', $PRESTAPP_PAYPAL_CLIENT_ID);
            }
        }

        if (Tools::isSubmit('submitGDPR')) {
            $PRESTAPP_CUSTOM_GDPR_TEXT = (string) Tools::getValue('gdpr_full_languages');
            if (Tools::getIsset("PRESTAPP_CUSTOM_GDPR")) {
                $PRESTAPP_CUSTOM_GDPR = (string) Tools::getValue('PRESTAPP_CUSTOM_GDPR');
            } else {
                $PRESTAPP_CUSTOM_GDPR = "0";
            }

            Configuration::updateValue('PRESTAPP_CUSTOM_GDPR_TEXT', trim($PRESTAPP_CUSTOM_GDPR_TEXT));
            Configuration::updateValue('PRESTAPP_CUSTOM_GDPR', $PRESTAPP_CUSTOM_GDPR);
        }

        if (Tools::isSubmit('submitAppPromotion')) {
            $PRESTAPP_CUSTOM_APP_PROMOTION_TEXT = (string) Tools::getValue('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT');
            $PRESTAPP_CUSTOM_APP_PROMOTION_COLOR = (string) Tools::getValue('PRESTAPP_CUSTOM_APP_PROMOTION_COLOR');
            $PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR = (string) Tools::getValue('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR');
            $PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR = (string) Tools::getValue('PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR');

            if (Tools::getIsset("PRESTAPP_CUSTOM_APP_PROMOTION")) {
                $PRESTAPP_CUSTOM_APP_PROMOTION = (string) Tools::getValue('PRESTAPP_CUSTOM_APP_PROMOTION');
            } else {
                $PRESTAPP_CUSTOM_APP_PROMOTION = "0";
            }

            Configuration::updateValue('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT', trim($PRESTAPP_CUSTOM_APP_PROMOTION_TEXT), true);
            Configuration::updateValue('PRESTAPP_CUSTOM_APP_PROMOTION_COLOR', trim($PRESTAPP_CUSTOM_APP_PROMOTION_COLOR));
            Configuration::updateValue('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR', trim($PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR));
            Configuration::updateValue('PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR', trim($PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR));
            Configuration::updateValue('PRESTAPP_CUSTOM_APP_PROMOTION', $PRESTAPP_CUSTOM_APP_PROMOTION);
        }

        if (Tools::isSubmit('submitRedirectionSettings')) {
            Configuration::updateValue('PRESTAPP_REDIRECTION', (string) Tools::getValue('PRESTAPP_REDIRECTION'));
            Configuration::updateValue('PRESTAPP_PWA_URL', (string) Tools::getValue('PRESTAPP_PWA_URL'));
        }

        if (Tools::isSubmit('submitSecurityHeaders')) {
            if (Tools::getIsset("PRESTAPP_HEADER_ALLOW_ORIGIN")) {
                $PRESTAPP_HEADER_ALLOW_ORIGIN = '1';
            } else {
                $PRESTAPP_HEADER_ALLOW_ORIGIN = "0";
            }

            Configuration::updateValue('PRESTAPP_HEADER_ALLOW_ORIGIN', $PRESTAPP_HEADER_ALLOW_ORIGIN);
        }

        if (Tools::isSubmit('submitSeo')) {
            if (Tools::getIsset("PRESTAPP_REDIRECTBOTS")) {
                $PRESTAPP_REDIRECTBOTS = '1';
            } else {
                $PRESTAPP_REDIRECTBOTS = "0";
            }

            if (Tools::getIsset("PRESTAPP_NOINDEX")) {
                $PRESTAPP_NOINDEX = '1';
            } else {
                $PRESTAPP_NOINDEX = "0";
            }

            Configuration::updateValue('PRESTAPP_NOINDEX', $PRESTAPP_NOINDEX);
            Configuration::updateValue('PRESTAPP_REDIRECTBOTS', $PRESTAPP_REDIRECTBOTS);
        }

        $employees = Employee::getEmployees(true);

        foreach ($employees as $employee_key => $employee_value) {
            $employee = new Employee($employee_value['id_employee']);
            $employees[$employee_key]['email'] = $employee->email;
        }

        $languages = Language::getLanguages();
        $gdpr_languages_encoded = array();

        $PRESTAPP_CUSTOM_GDPR_TEXT = Configuration::get('PRESTAPP_CUSTOM_GDPR_TEXT', null);

        if (json_decode($PRESTAPP_CUSTOM_GDPR_TEXT)) {
            foreach ($languages as $key => $value) {
                foreach (json_decode($PRESTAPP_CUSTOM_GDPR_TEXT) as $key2 => $value2) {
                    if ($value['id_lang'] == $value2->id_lang) {
                        $languages[$key]['gdpr_text'] = $value2->value;
                        $gdpr_languages_encoded[] = array(
                            'id_lang' => $value2->id_lang,
                            'value' => $value2->value,
                        );
                    }
                }
            }
        } elseif ($PRESTAPP_CUSTOM_GDPR_TEXT) {
            foreach ($languages as $key => $value) {
                $languages[$key]['gdpr_text'] = $PRESTAPP_CUSTOM_GDPR_TEXT;
                $gdpr_languages_encoded[] = array(
                    'id_lang' => $languages[$key]['id_lang'],
                    'value' => $PRESTAPP_CUSTOM_GDPR_TEXT,
                );
            }
        } else {
            foreach ($languages as $key => $value) {
                $languages[$key]['gdpr_text'] = '';
                $gdpr_languages_encoded[] = array(
                    'id_lang' => $languages[$key]['id_lang'],
                    'value' => '',
                );
            }
        }

        $this->context->smarty->assign(array(
            'PRESTAPP_API_PROTOCOL' => $protocol,
            'PRESTAPP_API_URL' => Tools::getHttpHost(false),
            'PRESTAPP_API_KEY' => Configuration::get('PRESTAPP_API_KEY', null),
            'PRESTAPP_SECRET_KEY' => Configuration::get('PRESTAPP_SECRET_KEY', null),
            'PRESTAPP_PAYPAL_SECRET_KEY' => Configuration::get('PRESTAPP_PAYPAL_SECRET_KEY', null),
            'PRESTAPP_PAYPAL_CLIENT_ID' => Configuration::get('PRESTAPP_PAYPAL_CLIENT_ID', null),
            'PRESTAPP_CUSTOM_GDPR_TEXT' => Configuration::get('PRESTAPP_CUSTOM_GDPR_TEXT', null),
            'PRESTAPP_CUSTOM_GDPR' => Configuration::get('PRESTAPP_CUSTOM_GDPR', null),
            'PRESTAPP_CUSTOM_APP_PROMOTION_TEXT' => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT', null),
            'PRESTAPP_CUSTOM_APP_PROMOTION_COLOR' => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_COLOR', null),
            'PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR' => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR', null),
            'PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR' => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR', null),
            'PRESTAPP_CUSTOM_APP_PROMOTION' => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION', null),
            'PRESTAPP_HEADER_ALLOW_ORIGIN' => Configuration::get('PRESTAPP_HEADER_ALLOW_ORIGIN', null),
            'PRESTAPP_REDIRECTBOTS' => Configuration::get('PRESTAPP_REDIRECTBOTS', null),
            'PRESTAPP_NOINDEX' => Configuration::get('PRESTAPP_NOINDEX', null),
            'PRESTAPP_REDIRECTION' => Configuration::get('PRESTAPP_REDIRECTION', null),
            'PRESTAPP_PWA_URL' => Configuration::get('PRESTAPP_PWA_URL', null),
            'PRESTAPP_PAYPAL_ENABLED' => Module::isEnabled('paypal'),
            'PRESTAPP_PAYPAL_ERROR' => $prestapp_paypal_error,
            'API_URL' => $this->api_url,
            'PHP_VERSION' => $php_version,
            'PRESTASHOP_VERSION' => _PS_VERSION_,
            'IS_ERROR' => $isError,
            'STATUS' => $status,
            'MODULE_VERSION' => $this->version,
            'EMPLOYEES' => $employees,
            'LANGUAGES' => $languages,
            'GDPR_LANGUAGES_ENCODED' => $gdpr_languages_encoded,
        ));

        return ($this->context->smarty->fetch(dirname(__FILE__) . '/views/templates/admin/configuration.tpl'));
    }

    /**
     * Send event to application when order is changing status
     *
     * @return void
     */
    public function hookActionOrderStatusUpdate($params)
    {
        // improve : detect only orders passed with PrestApp
        // the problem is : bank and paypal request directly the shop for the confirmation
        $blackListTriggerStatus = array(1, 3, 4, 5, 7, 10, 13); //no need to trigger request for theses statuses
        $id_status = $params['newOrderStatus']->id;

        if (!in_array($id_status, $blackListTriggerStatus)) {
            $id_order = (int) $params['id_order'];
            $order = new Order($id_order);
            $id_cart = (int) $params['cart']->id;

            $curl = curl_init();

            $url = 'https://mobileapi.prestapp.eu/sendEvent';

            //send the event and say to our app that the order changed status
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => "POST",
                CURLOPT_POSTFIELDS => array(
                    "id_cart" => $id_cart,
                    "id_order" => $id_order,
                    "id_status" => $id_status,
                    "webservice_key" => Configuration::get('PRESTAPP_API_KEY'),
                ),
            ));

            $response = curl_exec($curl);
            $err = curl_error($curl);
            curl_close($curl);
        }
    }

    public function hookActionValidateOrder($params)
    {

        //////////////////////
        //   ecrelaypoint   //
        //////////////////////

        if (Module::isInstalled('ecrelaypoint') && Module::isEnabled('ecrelaypoint')) {
            $ec_carrier_id = (int) Configuration::get('ECRELAYPOINT_TRANSPORTEUR', null, $this->context->shop->id_shop_group, (int) $this->context->shop->id);
            if (isset($ec_carrier_id)) {
                if ($ec_carrier_id) {
                    if ((int) $params['order']->id_carrier == (int) $ec_carrier_id) {
                        $result = Db::getInstance()->getRow('SELECT max(id_temp), id_relaypoint FROM ' . _DB_PREFIX_ . 'ecrelaypoint_temp WHERE id_cart =' . (int) $params['cart']->id);
                        if (!empty($result['id_relaypoint'])) {
                            $relaypoint = Db::getInstance()->getRow('SELECT * FROM ' . _DB_PREFIX_ . 'ecrelaypoint WHERE id_relaypoint =' . (int) $result['id_relaypoint']);
                            Db::getInstance()->insert('address', array(
                                'id_country' => (int) $relaypoint['id_country'],
                                'id_state' => (int) $relaypoint['id_state'],
                                'id_customer' => (int) $params['customer']->id, //ajout par ThF
                                'alias' => pSQL($relaypoint['name']),
                                'firstname' => pSQL($params['customer']->firstname),
                                'lastname' => pSQL($params['customer']->lastname),
                                'company' => pSQL($relaypoint['name']), //ajout par ThF
                                'address1' => pSQL($relaypoint['address1']),
                                'address2' => pSQL($relaypoint['address2']),
                                'postcode' => pSQL($relaypoint['postcode']),
                                'city' => pSQL($relaypoint['city']),
                                'phone' => pSQL($relaypoint['phone']),
                            ));
                            $newidaddr = Db::getInstance()->getRow('SELECT id_address FROM ' . _DB_PREFIX_ . 'address order by id_address desc ');

                            $params['order']->id_address_delivery = (int) $newidaddr['id_address'];
                            $params['order']->save();
                        } else {
                            $sql = 'DELETE FROM ' . _DB_PREFIX_ . 'ecrelaypoint_temp WHERE id_cart = ' . (int) $params['cart']->id . ' and id_shop=' . (int) $this->context->shop->id;
                            Db::getInstance()->execute($sql);
                        }
                    } else {
                        $sql = 'DELETE FROM ' . _DB_PREFIX_ . 'ecrelaypoint_temp WHERE id_cart = ' . (int) $params['cart']->id . ' and id_shop=' . (int) $this->context->shop->id;
                        Db::getInstance()->execute($sql);
                    }
                } else {
                    $sql = 'DELETE FROM ' . _DB_PREFIX_ . 'ecrelaypoint_temp WHERE id_cart = ' . (int) $params['cart']->id . ' and id_shop=' . (int) $this->context->shop->id;
                    Db::getInstance()->execute($sql);
                }
            } else {
                $sql = 'DELETE FROM ' . _DB_PREFIX_ . 'ecrelaypoint_temp WHERE id_cart = ' . (int) $params['cart']->id . ' and id_shop=' . (int) $this->context->shop->id;
                Db::getInstance()->execute($sql);
            }
        }
    }

    public function hookDisplayBeforeRedirect()
    {
        $redirection_type = Configuration::get('PRESTAPP_REDIRECTION');

        if (Context::getContext()->isMobile()) {
            if (isset($redirection_type) && $redirection_type == "ALWAYS_REDIRECT") {
                $controller = Tools::getValue('controller');
                if ($controller === "orderconfirmation") {
                    if ($this->isCrawler() && Configuration::get('PRESTAPP_REDIRECTBOTS') == "1") {
                        $this->doRedirection();
                    } elseif (!$this->isCrawler()) {
                        $this->doRedirection();
                    }
                }
            }
        }
    }

    public function hookDisplayHeader()
    {
        $redirection_type = Configuration::get('PRESTAPP_REDIRECTION');

        if (Context::getContext()->isMobile()) {
            if (isset($redirection_type) && $redirection_type == "ALWAYS_REDIRECT") {
                //check if crawlers are redirected
                if ($this->isCrawler() && Configuration::get('PRESTAPP_REDIRECTBOTS') == "1") {
                    $this->doRedirection();
                } elseif (!$this->isCrawler()) {
                    $this->doRedirection();
                }
            } elseif (isset($redirection_type) && $redirection_type == "NO_REDIRECTION" && Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION')) {
                if (Context::getContext() && Context::getContext()->controller) {
                    Context::getContext()->controller->addCSS($this->_path . '/views/css/front.css');
                    Context::getContext()->controller->addJS($this->_path . 'views/js/jquery.cookie.min.js');
                    Context::getContext()->controller->addJS($this->_path . 'views/js/front.js');

                    $this->context->smarty->assign(array(
                        "mobile_promotion_banner_text" => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT'),
                        "mobile_promotion_banner_background_color" => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_COLOR'),
                        "mobile_promotion_banner_text_color" => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_TEXT_COLOR'),
                        "mobile_promotion_banner_button_color" => Configuration::get('PRESTAPP_CUSTOM_APP_PROMOTION_BUTTON_COLOR'),
                    ));
                    return $this->display(__FILE__, 'mobile-promotion-banner.tpl');
                }
            }
        }
    }

    /**
     * Get status code
     *
     * @return int
     */
    protected function getHttpResponseCode($URL)
    {
        $headers = get_headers($URL);
        return Tools::substr($headers[0], 9, 3);
    }

    /**
     * Check if the given user agent string is one of a crawler, spider, or bot.
     *
     * @param string $user_agent
     *   A user agent string (e.g. Googlebot/2.1 (+http://www.google.com/bot.html))
     *
     * @return bool
     *   TRUE if the user agent is a bot, FALSE if not.
     */

    protected function isCrawler()
    {
        // User lowercase string for comparison.
        $user_agent = Tools::strtolower($_SERVER['HTTP_USER_AGENT']);

        // A list of some common words used only for bots and crawlers.
        $bot_identifiers = array(
            'bot',
            'slurp',
            'crawler',
            'spider',
            'curl',
            'facebook',
            'fetch',
        );

        // See if one of the identifiers is in the UA string.
        foreach ($bot_identifiers as $identifier) {
            if (strpos($user_agent, $identifier) !== false) {
                return true;
            }
        }

        return false;
    }

    protected function doRedirection()
    {

        $url = Configuration::get('PRESTAPP_PWA_URL');
        $controller = Tools::getValue('controller');
        $header = [];

        if (isset($url) && isset($controller)) {
            if (Configuration::get('PRESTAPP_NOINDEX') == "1") {
                array_push($header, "X-Robots-Tag: noindex, nofollow");
            }

            if ($controller === "index") {
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            } elseif ($controller === "category") {
                $url = $url . "/category/" . $this->context->controller->getCategory()->id;
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            } elseif ($controller === "product") {
                $url = $url . "/product/" . $this->context->controller->getProduct()->id;
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            } elseif ($controller === "pricesdrop") {
                $url = $url . "/special/prices-drop";
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            } elseif ($controller === "manufacturer") {
                if ($id_manufacturer = Tools::getValue('id_manufacturer')) {
                    $url = $url . "/special/manufacturer/" . $id_manufacturer;
                    Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
                } else {
                    $url = $url . "/special/manufacturer";
                    Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
                }
            } elseif ($controller === "orderconfirmation") {
                if (Tools::getValue('id_order')) {
                    $url = $url . "/checkout";
                    Tools::redirect($url . '?lang=' . $this->context->language->language_code . '&step=confirmation&id_cart=' . Tools::getValue('id_cart') . '&id_order=' . Tools::getValue('id_order'), null, null, $header);
                } else {
                    $url = $url . "/account/orders";
                    Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
                }
            } elseif ($controller === "order") {
                $url = $url . "/checkout";
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            } elseif ($controller === 'search') {
                $search_string = Tools::getValue('s');
                if (!$search_string) {
                    $search_string = Tools::getValue('search_query');
                }
                $url = $url . "/search/" . $search_string;
                Tools::redirect($url . '?lang=' . $this->context->language->language_code, null, null, $header);
            }
        }
    }
}
